@@ -28,21 +28,21 @@
 #endif
 #include <rx\rx.h>
 #include <rx\rx_null.h>
+#include <rx\rxstat.h>
 #include <WINNT/syscfg.h>
 #include <WINNT/afsreg.h>
+#include <afs\afscbint.h>
 
 #include "smb.h"
 #include "cm_rpc.h"
 #include "lanahelper.h"
 #include <strsafe.h>
 #include "cm_memmap.h"
+#include "msrpc.h"
 #ifdef DEBUG
 #include <crtdbg.h>
 #endif
 
-extern int RXAFSCB_ExecuteRequest(struct rx_call *z_call);
-extern int RXSTATS_ExecuteRequest(struct rx_call *z_call);
-
 extern afs_uint32 cryptall;
 extern afs_uint32 cm_anonvldb;
 extern int cm_enableServerLocks;
@@ -68,6 +68,7 @@ DWORD cm_mountRootLen;
 clientchar_t cm_mountRootC[1024];
 DWORD cm_mountRootCLen;
 
+int cm_readonlyVolumeVersioning = 0;
 int cm_logChunkSize;
 int cm_chunkSize;
 
@@ -86,8 +87,8 @@ char cm_HostName[200];
 long cm_HostAddr;
 unsigned short cm_callbackport = CM_DEFAULT_CALLBACKPORT;
 
-char cm_NetbiosName[MAX_NB_NAME_LENGTH] = "";
-clientchar_t cm_NetbiosNameC[MAX_NB_NAME_LENGTH] = _C("");
+char cm_NetbiosName[MAX_NB_NAME_LENGTH] = "NOT.YET.SET";
+clientchar_t cm_NetbiosNameC[MAX_NB_NAME_LENGTH] = _C("NOT.YET.SET");
 
 char cm_CachePath[MAX_PATH];
 DWORD cm_ValidateCache = 1;
@@ -112,9 +113,7 @@ DWORD TraceOption = 0;
 
 HANDLE afsi_file;
 
-#ifdef AFS_AFSDB_ENV
 int cm_dnsEnabled = 1;
-#endif
 
 
 static int afsi_log_useTimestamp = 1;
@@ -240,159 +239,6 @@ void afsd_ForceTrace(BOOL flush)
     CloseHandle(handle);
 }
 
-static void
-configureBackConnectionHostNames(void)
-{
-    /* On Windows XP SP2, Windows 2003 SP1, and all future Windows operating systems
-     * there is a restriction on the use of SMB authentication on loopback connections.
-     * There are two work arounds available:
-     * 
-     *   (1) We can disable the check for matching host names.  This does not
-     *   require a reboot:
-     *   [HKEY_LOCAL_MACHINE\SYSTEM\CurrentControlSet\Control\Lsa]
-     *     "DisableLoopbackCheck"=dword:00000001
-     *
-     *   (2) We can add the AFS SMB/CIFS service name to an approved list.  This
-     *   does require a reboot:
-     *   [HKEY_LOCAL_MACHINE\SYSTEM\CurrentControlSet\Control\Lsa\MSV1_0]
-     *     "BackConnectionHostNames"=multi-sz
-     *
-     * The algorithm will be:
-     *   (1) Check to see if cm_NetbiosName exists in the BackConnectionHostNames list
-     *   (2a) If not, add it to the list.  (This will not take effect until the next reboot.)
-     *   (2b1)    and check to see if DisableLoopbackCheck is set.
-     *   (2b2)    If not set, set the DisableLoopbackCheck value to 0x1 
-     *   (2b3)                and create HKLM\SOFTWARE\OpenAFS\Client  UnsetDisableLoopbackCheck
-     *   (2c) else If cm_NetbiosName exists in the BackConnectionHostNames list,
-     *             check for the UnsetDisableLoopbackCheck value.  
-     *             If set, set the DisableLoopbackCheck flag to 0x0 
-     *             and delete the UnsetDisableLoopbackCheck value
-     *
-     * Starting in Longhorn Beta 1, an entry in the BackConnectionHostNames value will
-     * force Windows to use the loopback authentication mechanism for the specified 
-     * services.
-     */
-    HKEY hkLsa;
-    HKEY hkMSV10;
-    HKEY hkClient;
-    DWORD dwType;
-    DWORD dwSize, dwAllocSize;
-    DWORD dwValue;
-    PBYTE pHostNames = NULL, pName = NULL;
-    BOOL  bNameFound = FALSE;   
-
-    if ( RegOpenKeyEx( HKEY_LOCAL_MACHINE, 
-                       "SYSTEM\\CurrentControlSet\\Control\\Lsa\\MSV1_0",
-                       0,
-                       KEY_READ|KEY_WRITE,
-                       &hkMSV10) == ERROR_SUCCESS )
-    {
-        if ((RegQueryValueEx( hkMSV10, "BackConnectionHostNames", 0, 
-			     &dwType, NULL, &dwAllocSize) == ERROR_SUCCESS) &&
-            (dwType == REG_MULTI_SZ)) 
-        {
-	    dwAllocSize += 1 /* in case the source string is not nul terminated */
-		+ strlen(cm_NetbiosName) + 2;
-	    pHostNames = malloc(dwAllocSize);
-	    dwSize = dwAllocSize;
-            if (RegQueryValueEx( hkMSV10, "BackConnectionHostNames", 0, &dwType, 
-				 pHostNames, &dwSize) == ERROR_SUCCESS) 
-            {
-		for (pName = pHostNames; 
-		     (pName - pHostNames < (int) dwSize) && *pName ; 
-		     pName += strlen(pName) + 1)
-		{
-		    if ( !stricmp(pName, cm_NetbiosName) ) {
-			bNameFound = TRUE;
-			break;
-		    }   
-		}
-	    }
-        }
-             
-        if ( !bNameFound ) {
-            size_t size = strlen(cm_NetbiosName) + 2;
-            if ( !pHostNames ) {
-                pHostNames = malloc(size);
-		pName = pHostNames;
-            }
-            StringCbCopyA(pName, size, cm_NetbiosName);
-            pName += size - 1;
-            *pName = '\0';  /* add a second nul terminator */
-
-            dwType = REG_MULTI_SZ;
-	    dwSize = pName - pHostNames + 1;
-            RegSetValueEx( hkMSV10, "BackConnectionHostNames", 0, dwType, pHostNames, dwSize);
-
-            if ( RegOpenKeyEx( HKEY_LOCAL_MACHINE, 
-                               "SYSTEM\\CurrentControlSet\\Control\\Lsa",
-                               0,
-                               KEY_READ|KEY_WRITE,
-                               &hkLsa) == ERROR_SUCCESS )
-            {
-                dwSize = sizeof(DWORD);
-                if ( RegQueryValueEx( hkLsa, "DisableLoopbackCheck", 0, &dwType, (LPBYTE)&dwValue, &dwSize) != ERROR_SUCCESS ||
-                     dwValue == 0 ) {
-                    dwType = REG_DWORD;
-                    dwSize = sizeof(DWORD);
-                    dwValue = 1;
-                    RegSetValueEx( hkLsa, "DisableLoopbackCheck", 0, dwType, (LPBYTE)&dwValue, dwSize);
-
-                    if (RegCreateKeyEx( HKEY_LOCAL_MACHINE, 
-                                        AFSREG_CLT_OPENAFS_SUBKEY,
-                                        0,
-                                        NULL,
-                                        REG_OPTION_NON_VOLATILE,
-                                        KEY_READ|KEY_WRITE,
-                                        NULL,
-                                        &hkClient,
-                                        NULL) == ERROR_SUCCESS) {
-
-                        dwType = REG_DWORD;
-                        dwSize = sizeof(DWORD);
-                        dwValue = 1;
-                        RegSetValueEx( hkClient, "RemoveDisableLoopbackCheck", 0, dwType, (LPBYTE)&dwValue, dwSize);
-                        RegCloseKey(hkClient);
-                    }
-                    RegCloseKey(hkLsa);
-                }
-            }
-        } else {
-            if (RegCreateKeyEx( HKEY_LOCAL_MACHINE, 
-                                AFSREG_CLT_OPENAFS_SUBKEY,
-                                0,
-                                NULL,
-                                REG_OPTION_NON_VOLATILE,
-                                KEY_READ|KEY_WRITE,
-                                NULL,
-                                &hkClient,
-                                NULL) == ERROR_SUCCESS) {
-
-                dwSize = sizeof(DWORD);
-                if ( RegQueryValueEx( hkClient, "RemoveDisableLoopbackCheck", 0, &dwType, (LPBYTE)&dwValue, &dwSize) == ERROR_SUCCESS &&
-                     dwValue == 1 ) {
-                    if ( RegOpenKeyEx( HKEY_LOCAL_MACHINE, 
-                                       "SYSTEM\\CurrentControlSet\\Control\\Lsa",
-                                       0,
-                                       KEY_READ|KEY_WRITE,
-                                       &hkLsa) == ERROR_SUCCESS )
-                    {
-                        RegDeleteValue(hkLsa, "DisableLoopbackCheck");
-                        RegCloseKey(hkLsa);
-                    }
-                }
-                RegDeleteValue(hkClient, "RemoveDisableLoopbackCheck");
-                RegCloseKey(hkClient);
-            }
-        }
-        RegCloseKey(hkMSV10);
-    }
-
-    if (pHostNames)
-	free(pHostNames);
-}
-
-
 static void afsd_InitServerPreferences(void)
 {
     HKEY hkPrefs = 0;
@@ -450,13 +296,18 @@ static void afsd_InitServerPreferences(void)
 
                 saddr.sin_addr.S_un.S_addr = *(unsigned long *)pEntry->h_addr;
             }
+            saddr.sin_port = htons(7003);
             saddr.sin_family = AF_INET;
             dwRank += (rand() & 0x000f);
 
             tsp = cm_FindServer(&saddr, CM_SERVER_VLDB);
             if ( tsp )		/* an existing server - ref count increased */
             {
-                tsp->ipRank = (USHORT)dwRank; /* no need to protect by mutex*/
+                lock_ObtainMutex(&tsp->mx);
+                tsp->ipRank = (USHORT)dwRank;
+                tsp->flags |= CM_SERVERFLAG_PREF_SET;
+		tsp->adminRank = tsp->ipRank;
+                lock_ReleaseMutex(&tsp->mx);
 
                 /* set preferences for an existing vlserver */
                 cm_ChangeRankCellVLServer(tsp);
@@ -464,8 +315,12 @@ static void afsd_InitServerPreferences(void)
             }
             else	/* add a new server without a cell */
             {
-                tsp = cm_NewServer(&saddr, CM_SERVER_VLDB, NULL, CM_FLAG_NOPROBE); /* refcount = 1 */
+                tsp = cm_NewServer(&saddr, CM_SERVER_VLDB, NULL, NULL, CM_FLAG_NOPROBE); /* refcount = 1 */
+                lock_ObtainMutex(&tsp->mx);
                 tsp->ipRank = (USHORT)dwRank;
+                tsp->flags |= CM_SERVERFLAG_PREF_SET;
+		tsp->adminRank = tsp->ipRank;
+                lock_ReleaseMutex(&tsp->mx);
             }
         }
 
@@ -517,13 +372,18 @@ static void afsd_InitServerPreferences(void)
 
                 saddr.sin_addr.S_un.S_addr = *(unsigned long *)pEntry->h_addr;
             }
+            saddr.sin_port = htons(7000);
             saddr.sin_family = AF_INET;
             dwRank += (rand() & 0x000f);
 
             tsp = cm_FindServer(&saddr, CM_SERVER_FILE);
             if ( tsp )		/* an existing server - ref count increased */
             {
-                tsp->ipRank = (USHORT)dwRank; /* no need to protect by mutex*/
+                lock_ObtainMutex(&tsp->mx);
+                tsp->ipRank = (USHORT)dwRank;
+		tsp->flags |= CM_SERVERFLAG_PREF_SET;
+		tsp->adminRank = tsp->ipRank;
+                lock_ReleaseMutex(&tsp->mx);
 
                 /* find volumes which might have RO copy 
                 /* on server and change the ordering of 
@@ -534,8 +394,12 @@ static void afsd_InitServerPreferences(void)
             }
             else	/* add a new server without a cell */
             {
-                tsp = cm_NewServer(&saddr, CM_SERVER_FILE, NULL, CM_FLAG_NOPROBE); /* refcount = 1 */
+                tsp = cm_NewServer(&saddr, CM_SERVER_FILE, NULL, NULL, CM_FLAG_NOPROBE); /* refcount = 1 */
+                lock_ObtainMutex(&tsp->mx);
                 tsp->ipRank = (USHORT)dwRank;
+                tsp->flags |= CM_SERVERFLAG_PREF_SET;
+		tsp->adminRank = tsp->ipRank;
+                lock_ReleaseMutex(&tsp->mx);
             }
         }
 
@@ -543,11 +407,84 @@ static void afsd_InitServerPreferences(void)
     }
 }
 
+
+#ifndef _WIN64
+typedef BOOL (WINAPI *LPFN_ISWOW64PROCESS) (HANDLE, PBOOL);
+
+static BOOL
+is_wow64(void)
+{
+    static BOOL bChecked = FALSE;
+    static BOOL bIsWow64 = FALSE;
+
+    if (!bChecked)
+    {
+        HANDLE h1 = NULL;
+        LPFN_ISWOW64PROCESS fnIsWow64Process = NULL;
+
+        h1 = GetModuleHandle("kernel32.dll");
+        fnIsWow64Process =
+            (LPFN_ISWOW64PROCESS)GetProcAddress(h1, "IsWow64Process");
+
+        /* If we don't find the fnIsWow64Process function then we
+         * are not running in a broken Wow64
+         */
+        if (fnIsWow64Process)
+            fnIsWow64Process(GetCurrentProcess(), &bIsWow64);
+
+        bChecked = TRUE;
+    }
+
+    return bIsWow64;
+}
+#endif /* _WIN64 */
+
 /*
  * AFSD Initialization
  */
 
-int afsd_InitCM(char **reasonP)
+static int
+afsd_InitRoot(char **reasonP)
+{
+    long code;
+    cm_req_t req;
+
+    cm_InitReq(&req);
+
+    if (cm_freelanceEnabled) {
+        cm_FakeRootFid(&cm_data.rootFid);
+    } else {
+	int attempts = 10;
+
+        osi_Log0(afsd_logp, "Loading Root Volume from cell");
+	do {
+	    code = cm_FindVolumeByName(cm_data.rootCellp, cm_rootVolumeName, cm_rootUserp,
+				       &req, CM_GETVOL_FLAG_CREATE, &cm_data.rootVolumep);
+	    afsi_log("cm_FindVolumeByName code %x root vol %x", code,
+		      (code ? (cm_volume_t *)-1 : cm_data.rootVolumep));
+	} while (code && --attempts);
+        if (code != 0) {
+            *reasonP = "can't find root volume in root cell";
+            return -1;
+        }
+
+        /* compute the root fid */
+        cm_SetFid(&cm_data.rootFid, cm_data.rootCellp->cellID, cm_GetROVolumeID(cm_data.rootVolumep), 1, 1);
+    }
+
+    code = cm_GetSCache(&cm_data.rootFid, &cm_data.rootSCachep, cm_rootUserp, &req);
+    afsi_log("cm_GetSCache code %x scache %x", code,
+             (code ? (cm_scache_t *)-1 : cm_data.rootSCachep));
+    if (code != 0) {
+        *reasonP = "unknown error";
+        return -1;
+    }
+
+    return 0;
+}
+
+int
+afsd_InitCM(char **reasonP)
 {
     osi_uid_t debugID;
     afs_uint64 cacheBlocks;
@@ -560,10 +497,15 @@ int afsd_InitCM(char **reasonP)
     DWORD dwValue;
     DWORD rx_enable_peer_stats;
     DWORD rx_enable_process_stats;
+    DWORD rx_udpbufsize = -1;
+    DWORD lockOrderValidation;
     long traceBufSize;
     long maxcpus;
     long ltt, ltto;
     long rx_nojumbo;
+    int  rx_max_rwin_size;
+    int  rx_max_swin_size;
+    int  rx_min_peer_timeout;
     long virtualCache = 0;
     fschar_t rootCellName[256];
     struct rx_service *serverp;
@@ -583,16 +525,27 @@ int afsd_InitCM(char **reasonP)
     int cm_SubnetMask[CM_MAXINTERFACE_ADDR];/* client's subnet mask in host order*/
     int cm_NetMtu[CM_MAXINTERFACE_ADDR];    /* client's MTU sizes */
     int cm_NetFlags[CM_MAXINTERFACE_ADDR];  /* network flags */
+    DWORD dwPriority;
 
     WSAStartup(0x0101, &WSAjunk);
 
     init_et_to_sys_error();
 
+    cm_utilsInit();
+
     /* setup osidebug server at RPC slot 1000 */
     osi_LongToUID(1000, &debugID);
     code = osi_InitDebug(&debugID);
     afsi_log("osi_InitDebug code %d", code);
 
+#ifndef _WIN64
+    if (is_wow64())
+    {
+        *reasonP = "32-bit OpenAFS Service is incompatible with the WOW64 environment";
+        return -1;
+    }
+#endif
+
     //	osi_LockTypeSetDefault("stat");	/* comment this out for speed */
     if (code != 0) {
         if (code == RPC_S_NO_PROTSEQS)
@@ -624,6 +577,29 @@ int afsd_InitCM(char **reasonP)
         osi_panic(buf, __FILE__, __LINE__);
     }
 
+    dummyLen = sizeof(dwPriority);
+    code = RegQueryValueEx(parmKey, "PriorityClass", NULL, NULL,
+                            (BYTE *) &dwPriority, &dummyLen);
+    if (code != ERROR_SUCCESS || dwPriority == 0) {
+        dwPriority = HIGH_PRIORITY_CLASS;
+    }
+    if (dwPriority != GetPriorityClass(GetCurrentProcess()))
+        SetPriorityClass(GetCurrentProcess(), dwPriority);
+    afsi_log("PriorityClass 0x%x", GetPriorityClass(GetCurrentProcess()));
+
+    dummyLen = sizeof(lockOrderValidation);
+    code = RegQueryValueEx(parmKey, "LockOrderValidation", NULL, NULL,
+                            (BYTE *) &lockOrderValidation, &dummyLen);
+    if (code != ERROR_SUCCESS) {
+#ifdef DEBUG
+        lockOrderValidation = 1;
+#else
+        lockOrderValidation = 0;
+#endif
+    }
+    osi_SetLockOrderValidation(lockOrderValidation);
+    afsi_log("Lock Order Validation %s", lockOrderValidation ? "On" : "Off");
+
     dummyLen = sizeof(maxcpus);
     code = RegQueryValueEx(parmKey, "MaxCPUs", NULL, NULL,
                             (BYTE *) &maxcpus, &dummyLen);
@@ -689,6 +665,11 @@ int afsd_InitCM(char **reasonP)
 
     osi_Log0(afsd_logp, "Log init");
 
+    dummyLen = sizeof(smb_monitorReqs);
+    code = RegQueryValueEx(parmKey, "SMBRequestMonitor", NULL, NULL,
+                           (BYTE *) &smb_monitorReqs, &dummyLen);
+    afsi_log("SMB request monitoring is %s", (smb_monitorReqs != 0)? "enabled": "disabled");
+
     dummyLen = sizeof(cacheSize);
     code = RegQueryValueEx(parmKey, "CacheSize", NULL, NULL,
                             (BYTE *) &cacheSize, &dummyLen);
@@ -834,15 +815,15 @@ int afsd_InitCM(char **reasonP)
                             (LPBYTE) cm_mountRootC, &cm_mountRootCLen);
     if (code == ERROR_SUCCESS) {
         afsi_log("Mount root %S", cm_mountRootC);
-        cm_mountRootCLen = cm_ClientStrLen(cm_mountRootC);
+        cm_mountRootCLen = (DWORD)cm_ClientStrLen(cm_mountRootC);
     } else {
         cm_ClientStrCpy(cm_mountRootC, lengthof(cm_mountRootC), _C("/afs"));
-        cm_mountRootCLen = cm_ClientStrLen(cm_mountRootC);
+        cm_mountRootCLen = (DWORD)cm_ClientStrLen(cm_mountRootC);
         /* Don't log */
     }
 
     cm_ClientStringToFsString(cm_mountRootC, -1, cm_mountRoot, lengthof(cm_mountRoot));
-    cm_mountRootLen = cm_FsStrLen(cm_mountRoot);
+    cm_mountRootLen = (DWORD)cm_FsStrLen(cm_mountRoot);
 
     dummyLen = sizeof(buf);
     code = RegQueryValueEx(parmKey, "CachePath", NULL, &regType,
@@ -955,14 +936,16 @@ int afsd_InitCM(char **reasonP)
     code = RegQueryValueEx(parmKey, "SecurityLevel", NULL, NULL,
                            (BYTE *) &cryptall, &dummyLen);
     if (code == ERROR_SUCCESS) {
-        afsi_log("SecurityLevel is %s", cryptall?"crypt":"clear");
+        afsi_log("SecurityLevel is %s", cryptall == 1?"crypt": cryptall == 2?"auth":"clear");
     } else {
         cryptall = 0;
         afsi_log("Default SecurityLevel is clear");
     }
 
-    if (cryptall)
+    if (cryptall == 1)
 	LogEvent(EVENTLOG_INFORMATION_TYPE, MSG_CRYPT_ON);
+    else if (cryptall == 2)
+	LogEvent(EVENTLOG_INFORMATION_TYPE, MSG_CRYPT_AUTH);
     else
 	LogEvent(EVENTLOG_INFORMATION_TYPE, MSG_CRYPT_OFF);
 
@@ -971,7 +954,6 @@ int afsd_InitCM(char **reasonP)
                             (BYTE *) &cm_anonvldb, &dummyLen);
     afsi_log("CM ForceAnonVLDB is %s", cm_anonvldb ? "on" : "off");
 
-#ifdef AFS_AFSDB_ENV
     dummyLen = sizeof(cm_dnsEnabled);
     code = RegQueryValueEx(parmKey, "UseDNS", NULL, NULL,
                             (BYTE *) &cm_dnsEnabled, &dummyLen);
@@ -983,9 +965,6 @@ int afsd_InitCM(char **reasonP)
         cm_dnsEnabled = 1;   /* default on */
         afsi_log("Default to use DNS to find AFS cell servers");
     }
-#else /* AFS_AFSDB_ENV */
-    afsi_log("AFS not built with DNS support to find AFS cell servers");
-#endif /* AFS_AFSDB_ENV */
 
 #ifdef AFS_FREELANCE_CLIENT
     dummyLen = sizeof(cm_freelanceEnabled);
@@ -1018,6 +997,22 @@ int afsd_InitCM(char **reasonP)
     afsi_log("Dot files/dirs will %sbe marked hidden",
               smb_hideDotFiles ? "" : "not ");
 
+    dummyLen = sizeof(dwValue);
+    code = RegQueryValueEx(parmKey, "UnixModeFileDefault", NULL, NULL,
+                           (BYTE *) &dwValue, &dummyLen);
+    if (code == ERROR_SUCCESS) {
+        smb_unixModeDefaultFile = (dwValue & 07777);
+    }
+    afsi_log("Default unix mode bits for files is 0%04o", smb_unixModeDefaultFile);
+
+    dummyLen = sizeof(dwValue);
+    code = RegQueryValueEx(parmKey, "UnixModeDirDefault", NULL, NULL,
+                           (BYTE *) &dwValue, &dummyLen);
+    if (code == ERROR_SUCCESS) {
+        smb_unixModeDefaultDir = (dwValue & 07777);
+    }
+    afsi_log("Default unix mode bits for directories is 0%04o", smb_unixModeDefaultDir);
+
     dummyLen = sizeof(smb_maxMpxRequests);
     code = RegQueryValueEx(parmKey, "MaxMpxRequests", NULL, NULL,
                            (BYTE *) &smb_maxMpxRequests, &dummyLen);
@@ -1044,24 +1039,64 @@ int afsd_InitCM(char **reasonP)
     }
     afsi_log("SMB authentication type is %s", ((smb_authType == SMB_AUTH_NONE)?"NONE":((smb_authType == SMB_AUTH_EXTENDED)?"EXTENDED":"NTLM")));
 
+    dummyLen = sizeof(rx_max_rwin_size);
+    code = RegQueryValueEx(parmKey, "RxMaxRecvWinSize", NULL, NULL,
+                           (BYTE *) &rx_max_rwin_size, &dummyLen);
+    if (code == ERROR_SUCCESS)
+        rx_SetMaxReceiveWindow(rx_max_rwin_size);
+    afsi_log("Rx Maximum Receive Window Size is %d", rx_GetMaxReceiveWindow());
+
+    dummyLen = sizeof(rx_max_swin_size);
+    code = RegQueryValueEx(parmKey, "RxMaxSendWinSize", NULL, NULL,
+                           (BYTE *) &rx_max_swin_size, &dummyLen);
+    if (code == ERROR_SUCCESS)
+        rx_SetMaxSendWindow(rx_max_swin_size);
+    afsi_log("Rx Maximum Send Window Size is %d", rx_GetMaxSendWindow());
+
+    dummyLen = sizeof(rx_min_peer_timeout);
+    code = RegQueryValueEx(parmKey, "RxMinPeerTimeout", NULL, NULL,
+                           (BYTE *) &rx_min_peer_timeout, &dummyLen);
+    if (code == ERROR_SUCCESS)
+        rx_SetMinPeerTimeout(rx_min_peer_timeout);
+    afsi_log("Rx Minimum Peer Timeout is %d ms", rx_GetMinPeerTimeout());
+
     dummyLen = sizeof(rx_nojumbo);
     code = RegQueryValueEx(parmKey, "RxNoJumbo", NULL, NULL,
                            (BYTE *) &rx_nojumbo, &dummyLen);
     if (code != ERROR_SUCCESS) {
-        rx_nojumbo = 0;
+        DWORD jumbo;
+        dummyLen = sizeof(jumbo);
+        code = RegQueryValueEx(parmKey, "RxJumbo", NULL, NULL,
+                                (BYTE *) &jumbo, &dummyLen);
+        if (code != ERROR_SUCCESS) {
+            rx_nojumbo = 1;
+        } else {
+            rx_nojumbo = !jumbo;
+        }
     }
     if (rx_nojumbo)
         afsi_log("RX Jumbograms are disabled");
+    else
+        afsi_log("RX Jumbograms are enabled");
 
     dummyLen = sizeof(rx_extraPackets);
     code = RegQueryValueEx(parmKey, "RxExtraPackets", NULL, NULL,
                            (BYTE *) &rx_extraPackets, &dummyLen);
     if (code != ERROR_SUCCESS) {
-        rx_extraPackets = 120;
+        rx_extraPackets = (numBkgD + numSvThreads + 5) * 64;
     }
     if (rx_extraPackets)
         afsi_log("RX extraPackets is %d", rx_extraPackets);
 
+    dummyLen = sizeof(rx_udpbufsize);
+    code = RegQueryValueEx(parmKey, "RxUdpBufSize", NULL, NULL,
+                           (BYTE *) &rx_udpbufsize, &dummyLen);
+    if (code != ERROR_SUCCESS) {
+        rx_udpbufsize = 256*1024;
+    }
+    if (rx_udpbufsize != -1)
+        afsi_log("RX udpbufsize is %d", rx_udpbufsize);
+
     dummyLen = sizeof(rx_mtu);
     code = RegQueryValueEx(parmKey, "RxMaxMTU", NULL, NULL,
                            (BYTE *) &rx_mtu, &dummyLen);
@@ -1211,6 +1246,22 @@ int afsd_InitCM(char **reasonP)
     } 
     afsi_log("CM FollowBackupPath is %u", cm_followBackupPath);
 
+    dummyLen = sizeof(DWORD);
+    code = RegQueryValueEx(parmKey, "PerFileAccessCheck", NULL, NULL,
+                           (BYTE *) &dwValue, &dummyLen);
+    if (code == ERROR_SUCCESS) {
+        cm_accessPerFileCheck = (int) dwValue;
+    } 
+    afsi_log("CM PerFileAccessCheck is %d", cm_accessPerFileCheck);
+
+    dummyLen = sizeof(DWORD);
+    code = RegQueryValueEx(parmKey, "ReadOnlyVolumeVersioning", NULL, NULL,
+                           (BYTE *) &dwValue, &dummyLen);
+    if (code == ERROR_SUCCESS) {
+        cm_readonlyVolumeVersioning = (unsigned short) dwValue;
+    }
+    afsi_log("CM ReadOnlyVolumeVersioning is %u", cm_readonlyVolumeVersioning);
+
     RegCloseKey (parmKey);
 
     cacheBlocks = ((afs_uint64)cacheSize * 1024) / blockSize;
@@ -1240,9 +1291,6 @@ int afsd_InitCM(char **reasonP)
     cm_initParams.setTime = 0;
     cm_initParams.memCache = 1;
 
-    /* Ensure the AFS Netbios Name is registered to allow loopback access */
-    configureBackConnectionHostNames();
-
     /* init user daemon, and other packages */
     cm_InitUser();
 
@@ -1265,12 +1313,10 @@ int afsd_InitCM(char **reasonP)
         return -1;
     }
 
-#ifdef AFS_AFSDB_ENV
 #if !defined(_WIN32_WINNT) || (_WIN32_WINNT < 0x0500)
     if (cm_InitDNS(cm_dnsEnabled) == -1)
         cm_dnsEnabled = 0;  /* init failed, so deactivate */
     afsi_log("cm_InitDNS %d", cm_dnsEnabled);
-#endif
 #endif
 
     /* Set RX parameters before initializing RX */
@@ -1286,6 +1332,11 @@ int afsd_InitCM(char **reasonP)
         afsi_log("rx_SetMaxMTU %d successful", rx_mtu);
     }
 
+    if ( rx_udpbufsize != -1 ) {
+        rx_SetUdpBufSize(rx_udpbufsize);
+        afsi_log("rx_SetUdpBufSize %d", rx_udpbufsize);
+    }
+
     /* initialize RX, and tell it to listen to the callbackport, 
      * which is used for callback RPC messages.
      */
@@ -1309,6 +1360,9 @@ int afsd_InitCM(char **reasonP)
         *reasonP = "unknown error";
         return -1;
     }
+    rx_SetMinProcs(serverp, 2);
+    rx_SetMaxProcs(serverp, 4);
+    rx_SetCheckReach(serverp, 1);
 
     nullServerSecurityClassp = rxnull_NewServerSecurityObject();
     serverp = rx_NewService(0, RX_STATS_SERVICE_ID, "rpcstats",
@@ -1318,6 +1372,8 @@ int afsd_InitCM(char **reasonP)
         *reasonP = "unknown error";
         return -1;
     }
+    rx_SetMinProcs(serverp, 2);
+    rx_SetMaxProcs(serverp, 4);
         
     /* start server threads, *not* donating this one to the pool */
     rx_StartServer(0);
@@ -1359,57 +1415,31 @@ int afsd_InitCM(char **reasonP)
     /* Initialize the RPC server for session keys */
     RpcInit();
 
+    /* Initialize the RPC server for pipe services */
+    MSRPC_Init();
+
     afsd_InitServerPreferences();
-    return 0;
+
+    code = afsd_InitRoot(reasonP);
+
+    return code;
 }
 
 int afsd_ShutdownCM(void)
 {
+    MSRPC_Shutdown();
+
     cm_ReleaseSCache(cm_data.rootSCachep);
 
+    cm_utilsCleanup();
+
+    cm_shutdown = 1;
+
     return 0;
 }
 
 int afsd_InitDaemons(char **reasonP)
 {
-    long code;
-    cm_req_t req;
-
-    cm_InitReq(&req);
-
-    /* this should really be in an init daemon from here on down */
-
-    if (!cm_freelanceEnabled) {
-	int attempts = 10;
-
-        osi_Log0(afsd_logp, "Loading Root Volume from cell");
-	do {
-	    code = cm_FindVolumeByName(cm_data.rootCellp, cm_rootVolumeName, cm_rootUserp,
-				       &req, CM_GETVOL_FLAG_CREATE, &cm_data.rootVolumep);
-	    afsi_log("cm_FindVolumeByName code %x root vol %x", code,
-		      (code ? (cm_volume_t *)-1 : cm_data.rootVolumep));
-	} while (code && --attempts);
-        if (code != 0) {
-            *reasonP = "can't find root volume in root cell";
-            return -1;
-        }
-    }
-
-    /* compute the root fid */
-    if (!cm_freelanceEnabled) {
-        cm_SetFid(&cm_data.rootFid, cm_data.rootCellp->cellID, cm_GetROVolumeID(cm_data.rootVolumep), 1, 1);
-    }
-    else
-        cm_FakeRootFid(&cm_data.rootFid);
-        
-    code = cm_GetSCache(&cm_data.rootFid, &cm_data.rootSCachep, cm_rootUserp, &req);
-    afsi_log("cm_GetSCache code %x scache %x", code,
-             (code ? (cm_scache_t *)-1 : cm_data.rootSCachep));
-    if (code != 0) {
-        *reasonP = "unknown error";
-        return -1;
-    }
-
     cm_InitDaemon(numBkgD);
     afsi_log("cm_InitDaemon complete");
 
@@ -1649,16 +1679,21 @@ static BOOL (WINAPI *pMiniDumpWriteDump)(HANDLE hProcess,DWORD ProcessId,HANDLE
 static HANDLE
 OpenDumpFile(void)
 {
+    char tmp[256];
     char wd[256];
+    SYSTEMTIME st;
     DWORD code;
 
-    code = GetEnvironmentVariable("TEMP", wd, sizeof(wd));
-    if ( code == 0 || code > sizeof(wd) )
+    code = GetEnvironmentVariable("TEMP", tmp, sizeof(tmp));
+    if ( code == 0 || code > sizeof(tmp) )
     {
-        if (!GetWindowsDirectory(wd, sizeof(wd)))
+        if (!GetWindowsDirectory(tmp, sizeof(tmp)))
             return NULL;
     }
-    StringCbCatA(wd, sizeof(wd), "\\afsd.dmp");
+    GetLocalTime(&st);
+    StringCbPrintfA(wd, sizeof(wd),
+                    "%s\\afsd-%04d-%02d-%02d-%02d_%02d_%02d.dmp", tmp,
+                    st.wYear, st.wMonth, st.wDay, st.wHour, st.wMinute, st.wSecond);
     return CreateFile( wd, GENERIC_WRITE, FILE_SHARE_READ, NULL,
                             CREATE_ALWAYS, FILE_FLAG_WRITE_THROUGH, NULL);
 }
@@ -1666,8 +1701,8 @@ OpenDumpFile(void)
 void 
 GenerateMiniDump(PEXCEPTION_POINTERS ep)
 {
-	if (IsDebuggerPresent())
-		return;
+    if (IsDebuggerPresent())
+        return;
 
     if (ep == NULL) 
     {
