@@ -100,6 +100,9 @@
 #define CM_ERROR_NOSUCHDEVICE           (CM_ERROR_BASE+58)
 #define CM_ERROR_LOCK_NOT_GRANTED       (CM_ERROR_BASE+59)
 #define CM_ERROR_NOTINCACHE             (CM_ERROR_BASE+60)
+#define CM_ERROR_FORCE_DNS_LOOKUP       (CM_ERROR_BASE+61)
+#define CM_ERROR_BADFORMAT              (CM_ERROR_BASE+62)
+#define CM_ERROR_RPC_MOREDATA		(CM_ERROR_BASE+63)
 
 /* Used by cm_FollowMountPoint and cm_FindVolumeByName */
 /* And as an index in cm_volume_t */
@@ -121,6 +124,7 @@
 #define LOCK_HIERARCHY_SMB_RCT_GLOBAL          110
 #define LOCK_HIERARCHY_SMB_USERNAME            115
 #define LOCK_HIERARCHY_SMB_VC                  120
+#define LOCK_HIERARCHY_SMB_MONITOR             125
 
 
 #define LOCK_HIERARCHY_DAEMON_GLOBAL           400
