@@ -76,7 +76,7 @@ typedef struct cm_buf {
     afs_uint32 waitRequests;    /* num of thread wait requests */
 
     afs_uint32 dirty_offset;    /* offset from beginning of buffer containing dirty bytes */
-    afs_uint32 dirty_length;    /* number of dirty bytes within the buffer */
+    afs_uint32 dirty_length;      /* number of dirty bytes within the buffer */
 
 #ifdef DISKCACHE95
     cm_diskcache_t *dcp;        /* diskcache structure */
@@ -94,6 +94,8 @@ typedef struct cm_buf {
 #define CM_BUF_CMSTORING	2	/* storing this buffer */
 #define CM_BUF_CMFULLYFETCHED	4	/* read-while-fetching optimization */
 #define CM_BUF_CMWRITING        8       /* writing to this buffer */
+#define CM_BUF_CMBKGFETCH      16       /* background fetch queued by
+                                         * prefetch or redirector */
 /* waiting is done based on scp->flags.  Removing bits from cmFlags
    should be followed by waking the scp. */
 
@@ -101,7 +103,7 @@ typedef struct cm_buf {
 #define CM_BUF_WRITING	2	/* now writing buffer to the disk */
 #define CM_BUF_INHASH	4	/* in the hash table */
 #define CM_BUF_DIRTY	8	/* buffer is dirty */
-#define CM_BUF_INLRU	0x10	/* in lru queue */
+#define CM_BUF_INLRU	0x10	/* in lru queue (aka free list) */
 #define CM_BUF_ERROR	0x20	/* something went wrong on delayed write */
 #define CM_BUF_WAITING	0x40	/* someone's waiting for a flag to change */
 #define CM_BUF_INDL     0x80    /* in the dirty list */
@@ -154,15 +156,19 @@ extern cm_buf_t *buf_FindLocked(struct cm_scache *, osi_hyper_t *);
 
 extern cm_buf_t *buf_Find(struct cm_scache *, osi_hyper_t *);
 
-extern long buf_GetNewLocked(struct cm_scache *, osi_hyper_t *, cm_buf_t **);
+extern cm_buf_t *buf_FindAllLocked(struct cm_scache *, osi_hyper_t *, afs_uint32 flags);
+ 
+extern cm_buf_t *buf_FindAll(struct cm_scache *, osi_hyper_t *, afs_uint32 flags);
+ 
+extern long buf_GetNewLocked(struct cm_scache *, osi_hyper_t *, cm_req_t *, cm_buf_t **);
 
-extern long buf_Get(struct cm_scache *, osi_hyper_t *, cm_buf_t **);
+extern long buf_Get(struct cm_scache *, osi_hyper_t *, cm_req_t *, cm_buf_t **);
 
-extern long buf_GetNew(struct cm_scache *, osi_hyper_t *, cm_buf_t **);
+extern long buf_GetNew(struct cm_scache *, osi_hyper_t *, cm_req_t *, cm_buf_t **);
 
-extern long buf_CleanAsyncLocked(cm_buf_t *, cm_req_t *);
+extern afs_uint32 buf_CleanAsyncLocked(cm_buf_t *, cm_req_t *, afs_uint32 *);
 
-extern long buf_CleanAsync(cm_buf_t *, cm_req_t *);
+extern afs_uint32 buf_CleanAsync(cm_buf_t *, cm_req_t *, afs_uint32 *);
 
 extern void buf_CleanWait(cm_scache_t *, cm_buf_t *, afs_uint32 locked);
 
