@@ -13,13 +13,18 @@
 
 /* this function will continue to return cell server
    names for the given cell, ending in null */
-int getAFSServer(char *cellname, int *cellHostAddrs, char cellHostNames[][MAXHOSTCHARS], 
-                 unsigned short ipRanks[], int *numServers, int *ttl);
+int getAFSServer(const char *service, const char *protocol, const char *cellname,
+                 unsigned short afsdbPort,
+                 int *cellHostAddrs, char cellHostNames[][MAXHOSTCHARS],
+                 unsigned short ports[], unsigned short ipRanks[], int *numServers, int *ttl);
 
 /* Same as above, but using cm_unichar_t.  Note that this functon will
    only be defined for DNSAPI_ENV. */
-int getAFSServerW(cm_unichar_t *cellName, int *cellHostAddrs,
+int getAFSServerW(const cm_unichar_t *service, const cm_unichar_t *protocol, const cm_unichar_t *cellName,
+                  unsigned short afsdbPort,
+                  int *cellHostAddrs,
                   cm_unichar_t cellHostNames[][MAXHOSTCHARS], 
+                  unsigned short ports[],
                   unsigned short ipRanks[],
                   int *numServers, int *ttl);
 
