@@ -10,9 +10,7 @@
 #ifndef __SMB_H_ENV__
 #define __SMB_H_ENV__ 1
 
-#ifdef DJGPP
-#include "netbios95.h"
-#endif /* DJGPP */
+/* #define DEBUG_SMB_REFCOUNT 1 */
 
 #if _WIN32_WINNT < 0x0501
 #undef _WIN32_WINNT
@@ -20,8 +18,7 @@
 #endif
 #include <ntsecapi.h>
 
-/* Support largefiles by default */
-#define AFS_LARGEFILES
+#include <cm_nls.h>
 
 /* basic core protocol SMB structure */
 #pragma pack(push, 1)
@@ -51,7 +48,7 @@ typedef struct smb {
 #define SMB_FLAGS_CANONICAL_PATHNAMES      0x10
 #define SMB_FLAGS_REQUEST_OPLOCK           0x20
 #define SMB_FLAGS_REQUEST_BATCH_OPLOCK     0x40
-#define SMB_FLAGS_SERVER_TO_CLIENT         0x80           
+#define SMB_FLAGS_SERVER_TO_CLIENT         0x80
 
 /* flg2 values */
 
@@ -68,6 +65,7 @@ typedef struct smb {
 
 #define KNOWS_LONG_NAMES(inp) ((((smb_t *)inp)->flg2 & SMB_FLAGS2_KNOWS_LONG_NAMES)?1:0)
 #define WANTS_DFS_PATHNAMES(inp) ((((smb_t *)inp)->flg2 & SMB_FLAGS2_DFS_PATHNAMES)?1:0)
+#define WANTS_UNICODE(inp) ((((smb_t *)inp)->flg2 & SMB_FLAGS2_UNICODE)?1:0)
 
 /* Information Levels */
 #define SMB_INFO_STANDARD               1
@@ -113,6 +111,19 @@ typedef struct smb {
 #define SMB_FIND_FILE_NAMES_INFO	  0x103
 #define SMB_FIND_FILE_BOTH_DIRECTORY_INFO 0x104
 
+/* SMB_COM_TRANSACTION Named pipe operations */
+#define SMB_TRANS_SET_NMPIPE_STATE	0x0001
+#define SMB_TRANS_RAW_READ_NMPIPE	0x0011
+#define SMB_TRANS_QUERY_NMPIPE_STATE	0x0021
+#define SMB_TRANS_QUERY_NMPIPE_INFO	0x0022
+#define SMB_TRANS_PEEK_NMPIPE		0x0023
+#define SMB_TRANS_TRANSACT_NMPIPE	0x0026
+#define SMB_TRANS_RAW_WRITE_NMPIPE	0x0031
+#define SMB_TRANS_READ_NMPIPE		0x0036
+#define SMB_TRANS_WRITE_NMPIPE		0x0037
+#define SMB_TRANS_WAIT_NMPIPE		0x0053
+#define SMB_TRANS_CALL_NMPIPE		0x0054
+
 /* more defines */
 #define SMB_NOPCODES		256	/* # of opcodes in the dispatch table */
 
@@ -129,6 +140,8 @@ typedef struct smb {
 #define SMB_PACKETSIZE	32768 /* was 8400 */
 /* raw mode is considered obsolete and cannot be used with message signing */
 #define SMB_MAXRAWSIZE  65536
+/* max STRING characters per packet per request */
+#define SMB_STRINGBUFSIZE 4096
 
 /* Negotiate protocol constants */
 /* Security */
@@ -190,10 +203,7 @@ typedef struct smb_packet {
     unsigned char oddByte;
     unsigned short ncb_length;
     unsigned char flags;
-#ifdef DJGPP
-    dos_ptr dos_pkt;
-    unsigned int dos_pkt_sel;
-#endif /* DJGPP */
+    cm_space_t *stringsp;               /* decoded strings from this packet */
 } smb_packet_t;
 
 /* smb_packet flags */
@@ -206,11 +216,6 @@ typedef struct myncb {
     NCB ncb;			        /* ncb to use */
     struct myncb *nextp;		/* when on free list */
     long magic;
-#ifdef DJGPP
-    dos_ptr dos_ncb;
-    smb_packet_t *orig_pkt;
-    unsigned int dos_ncb_sel;
-#endif /* DJGPP */
 } smb_ncb_t;
 
 /* structures representing environments from kernel / SMB network.
@@ -235,7 +240,7 @@ typedef struct smb_vc {
     struct smb_user *usersp;	        /* the first child in the user session list */
     struct smb_fid *fidsp;		/* the first child in the open file list */
     unsigned char errorCount;
-    char rname[17];
+    clientchar_t rname[17];
     int lana;
     char encKey[MSV1_0_CHALLENGE_LENGTH]; /* MSV1_0_CHALLENGE_LENGTH is 8 */
     void * secCtx;                      /* security context when negotiating SMB extended auth
@@ -255,6 +260,7 @@ typedef struct smb_vc {
 #define SMB_VCFLAG_SESSX_RCVD	0x40	/* we received at least one session setups on this vc */
 #define SMB_VCFLAG_AUTH_IN_PROGRESS 0x80 /* a SMB NT extended authentication is in progress */
 #define SMB_VCFLAG_CLEAN_IN_PROGRESS 0x100
+#define SMB_VCFLAG_USEUNICODE   0x200   /* une UNICODE for STRING fields (NTLM 0.12 or later) */
 
 /* one per user session */
 typedef struct smb_user {
@@ -265,7 +271,7 @@ typedef struct smb_user {
     unsigned short userID;		/* the session identifier */
     struct smb_vc *vcp;		        /* back ptr to virtual circuit */
     struct smb_username *unp;           /* user name struct */
-    afs_uint32	delete;			/* ok to del: locked by smb_rctLock */
+    afs_uint32	deleteOk;		/* ok to del: locked by smb_rctLock */
 } smb_user_t;
 
 #define SMB_USERFLAG_DELETE	    1	/* delete struct when ref count zero */
@@ -276,8 +282,8 @@ typedef struct smb_username {
     long flags;			        /* flags; locked by mx */
     osi_mutex_t mx;
     struct cm_user *userp;		/* CM user structure */
-    char *name;			        /* user name */
-    char *machine;                      /* machine name */
+    clientchar_t *name;                 /* user name */
+    clientchar_t *machine;              /* machine name */
     time_t last_logoff_t;		/* most recent logoff time */
 } smb_username_t;
 
@@ -310,8 +316,8 @@ typedef struct smb_tid {
     struct smb_vc *vcp;		        /* back ptr */
     struct cm_user *userp;		/* user logged in at the
 					 * tree connect level (base) */
-    char *pathname;			/* pathname derived from sharename */
-    afs_uint32	delete;			/* ok to del: locked by smb_rctLock */
+    clientchar_t *pathname;             /* pathname derived from sharename */
+    afs_uint32	deleteOk;		/* ok to del: locked by smb_rctLock */
 } smb_tid_t;
 
 #define SMB_TIDFLAG_IPC		1 	/* IPC$ */
@@ -326,36 +332,12 @@ typedef struct smb_pid {
     struct smb_tid *tidp;		/* back ptr */
 } smb_pid_t;
 
-/* ioctl parameter, while being assembled and/or processed */
-typedef struct smb_ioctl {
-    /* input side */
-    char *inDatap;			/* ioctl func's current position
-					 * in input parameter block */
-    char *inAllocp;			/* allocated input parameter block */
-    afs_uint32 inCopied;			/* # of input bytes copied in so far
-					 * by write calls */
-    cm_space_t *prefix;		        /* prefix for subst drives */
-    char *tidPathp;			/* Pathname associated with Tree ID */
 
-    /* output side */
-    char *outDatap;			/* output results assembled so far */
-    char *outAllocp;		        /* output results assembled so far */
-    afs_uint32 outCopied;		/* # of output bytes copied back so far
-                                         * by read calls */
-	
-    /* flags */
-    afs_uint32 flags;
+/* Defined in smb_ioctl.h */
+struct smb_ioctl;
 
-    /* fid pointer */
-    struct smb_fid *fidp;
-
-    /* uid pointer */
-    smb_user_t *uidp;
-} smb_ioctl_t;
-
-/* flags for smb_ioctl_t */
-#define SMB_IOCTLFLAG_DATAIN	1	/* reading data from client to server */
-#define SMB_IOCTLFLAG_LOGON	2	/* got tokens from integrated logon */
+/* Defined in smb_rpc.h */
+struct smb_rpc;
 
 /* one per file ID; these are really file descriptors */
 typedef struct smb_fid {
@@ -371,20 +353,24 @@ typedef struct smb_fid {
                                            the file if session is
                                            terminated) */
     osi_hyper_t offset;			/* our file pointer */
-    smb_ioctl_t *ioctlp;		/* ptr to ioctl structure */
+    struct smb_ioctl *ioctlp;           /* ptr to ioctl structure */
 					/* Under NT, we may need to know the
 					 * parent directory and pathname used
 					 * to open the file, either to delete
 					 * the file on close, or to do a
 					 * change notification */
+    struct smb_rpc   *rpcp;	        /* ptr to RPC structure.  Used
+					   to keep track of endpoint
+					   that was opened for the
+					   RPC. */
     struct cm_scache *NTopen_dscp;	/* parent directory (NT) */
-    char *NTopen_pathp;		        /* path used in open (NT) */
-    char *NTopen_wholepathp;	        /* entire path, not just last name */
+    clientchar_t *NTopen_pathp;		/* path used in open (NT) */
+    clientchar_t *NTopen_wholepathp;	/* entire path, not just last name */
     int curr_chunk;			/* chunk being read */
     int prev_chunk;			/* previous chunk read */
     int raw_writers;		        /* pending async raw writes */
     EVENT_HANDLE raw_write_event;	/* signal this when raw_writers zero */
-    afs_uint32	delete;			/* ok to del: locked by smb_rctLock */
+    afs_uint32	deleteOk;		/* ok to del: locked by smb_rctLock */
 } smb_fid_t;
 
 #define SMB_FID_OPENREAD_LISTDIR	1	/* open for reading / listing directory */
@@ -409,6 +395,11 @@ typedef struct smb_fid {
 #define SMB_FID_SHARE_READ              0x1000
 #define SMB_FID_SHARE_WRITE             0x2000
 
+#define SMB_FID_RPC                     0x4000	/* open for MS RPC */
+#define SMB_FID_MESSAGEMODEPIPE		0x8000	/* message mode pipe */
+#define SMB_FID_BLOCKINGPIPE	       0x10000	/* blocking pipe */
+#define SMB_FID_RPC_INCALL	       0x20000	/* in an RPC call */
+
 #define SMB_FID_QLOCK_HIGH              0x7f000000
 #define SMB_FID_QLOCK_LOW               0x00000000
 #define SMB_FID_QLOCK_LENGTH            1
@@ -448,6 +439,22 @@ typedef struct smb_fid {
 #define LOCKING_ANDX_CANCEL_LOCK        0x08 /* Cancel outstanding request */
 #define LOCKING_ANDX_LARGE_FILES        0x10 /* Large file locking format */
 
+/* File type constants */
+#define SMB_FILETYPE_DISK		0x0000
+#define SMB_FILETYPE_BYTE_MODE_PIPE	0x0001
+#define SMB_FILETYPE_MESSAGE_MODE_PIPE	0x0002
+#define SMB_FILETYPE_PRINTER		0x0003
+#define SMB_FILETYPE_UNKNOWN		0xffff
+
+/* Device state constants */
+#define SMB_DEVICESTATE_READASBYTESTREAM    0x0000
+#define SMB_DEVICESTATE_READMSGFROMPIPE	    0x0100
+#define SMB_DEVICESTATE_BYTESTREAMPIPE	    0x0000
+#define SMB_DEVICESTATE_MESSAGEMODEPIPE	    0x0400
+#define SMB_DEVICESTATE_PIPECLIENTEND	    0x0000
+#define SMB_DEVICESTATE_PIPESERVEREND	    0x4000
+#define SMB_DEVICESTATE_BLOCKING	    0x8000
+
 /* for tracking in-progress directory searches */
 typedef struct smb_dirSearch {
     osi_queue_t q;			/* queue of all outstanding cookies */
@@ -460,9 +467,9 @@ typedef struct smb_dirSearch {
 					 * locked by smb_globalLock */
     unsigned short attribute;	        /* search attribute
 					 * (used for extended protocol) */
-    char tidPath[256];                  /* tid path */
-    char relPath[1024];                 /* relative path */        
-    char mask[256];			/* search mask for V3 */
+    clientchar_t tidPath[256];          /* tid path */
+    clientchar_t relPath[1024];         /* relative path */        
+    clientchar_t mask[256];	        /* search mask for V3 */
 } smb_dirSearch_t;
 
 #define SMB_DIRSEARCH_DELETE	1	/* delete struct when ref count zero */
@@ -473,7 +480,7 @@ typedef struct smb_dirSearch {
 /* type for patching directory listings */
 typedef struct smb_dirListPatch {
     osi_queue_t q;
-    char *dptr;		                /* ptr to attr, time, data, sizel, sizeh */
+    char *dptr;                      /* ptr to attr, time, data, sizel, sizeh */
     long flags;                         /* flags.  See below */
     cm_fid_t fid;
     cm_dirEntry_t *dep;                 /* temp */
@@ -495,9 +502,10 @@ typedef struct smb_waitingLock {
     int              state;
 } smb_waitingLock_t;
 
-#define SMB_WAITINGLOCKSTATE_WAITING 0
-#define SMB_WAITINGLOCKSTATE_DONE    1
-#define SMB_WAITINGLOCKSTATE_ERROR   2
+#define SMB_WAITINGLOCKSTATE_WAITING   0
+#define SMB_WAITINGLOCKSTATE_DONE      1
+#define SMB_WAITINGLOCKSTATE_ERROR     2
+#define SMB_WAITINGLOCKSTATE_CANCELLED 3
 
 /* waiting lock request */
 typedef struct smb_waitingLockRequest {
@@ -507,7 +515,8 @@ typedef struct smb_waitingLockRequest {
     smb_packet_t *inp;
     smb_packet_t *outp;
     int           lockType;
-    time_t        timeRemaining;
+    time_t        start_t;              /* osi_Time */
+    afs_uint32    msTimeout;            /* msecs, 0xFFFFFFFF = wait forever */
     smb_waitingLock_t * locks;
 } smb_waitingLockRequest_t;
 
@@ -532,44 +541,70 @@ typedef struct smb_dispatch {
 
 extern void smb_Init(osi_log_t *logp, int useV3,
 	int nThreads
-#ifndef DJGPP
         , void *aMBfunc
-#endif
   );
 
-extern void smb_LargeSearchTimeFromUnixTime(FILETIME *largeTimep, time_t unixTime);
-
-extern void smb_UnixTimeFromLargeSearchTime(time_t *unixTimep, FILETIME *largeTimep);
-
-extern void smb_SearchTimeFromUnixTime(afs_uint32 *searchTimep, time_t unixTime);
-
-extern void smb_UnixTimeFromSearchTime(time_t *unixTimep, afs_uint32 searchTime);
-
 extern void smb_DosUTimeFromUnixTime(afs_uint32 *dosUTimep, time_t unixTime);
 
 extern void smb_UnixTimeFromDosUTime(time_t *unixTimep, afs_uint32 dosUTime);
 
 extern void CompensateForSmbClientLastWriteTimeBugs(afs_uint32 *dosTimep);
 
+#ifdef DEBUG_SMB_REFCOUNT
+extern smb_vc_t *smb_FindVCDbg(unsigned short lsn, int flags, int lana, char *, long);
+#define smb_FindVC(a,b,c) smb_FindVCDbg(a,b,c,__FILE__,__LINE__);
+
+extern void smb_HoldVCDbg(smb_vc_t *vcp, char *, long);
+#define smb_HoldVC(a) smb_HoldVCDbg(a,__FILE__,__LINE__);
+
+extern void smb_HoldVCNoLockDbg(smb_vc_t *vcp, char *, long);
+#define smb_HoldVCNoLock(a) smb_HoldVCNoLockDbg(a,__FILE__,__LINE__);
+
+extern void smb_ReleaseVCDbg(smb_vc_t *vcp, char *, long);
+#define smb_ReleaseVC(a) smb_ReleaseVCDbg(a,__FILE__,__LINE__);
+
+extern void smb_ReleaseVCNoLockDbg(smb_vc_t *vcp, char *, long);
+#define smb_ReleaseVCNoLock(a) smb_ReleaseVCNoLockDbg(a,__FILE__,__LINE__);
+#else
 extern smb_vc_t *smb_FindVC(unsigned short lsn, int flags, int lana);
 
-extern void smb_CleanupDeadVC(smb_vc_t *vcp);
+extern void smb_HoldVC(smb_vc_t *vcp);
+
+extern void smb_HoldVCNoLock(smb_vc_t *vcp);
 
 extern void smb_ReleaseVC(smb_vc_t *vcp);
 
 extern void smb_ReleaseVCNoLock(smb_vc_t *vcp);
+#endif
 
+extern void smb_CleanupDeadVC(smb_vc_t *vcp);
+
+extern void smb_MarkAllVCsDead(smb_vc_t *exclude_vcp);
+
+#ifdef DEBUG_SMB_REFCOUNT
+extern smb_tid_t *smb_FindTIDDbg(smb_vc_t *vcp, unsigned short tid, int flags, char *, long);
+#define smb_FindTID(a,b,c) smb_FindTIDDbg(a,b,c,__FILE__,__LINE__);
+
+extern void smb_HoldTIDNoLockDbg(smb_tid_t *tidp, char *, long);
+#define smb_HoldTIDNoLock(a) smb_HoldTIDNoLockDbg(a,__FILE__,__LINE__);
+
+extern void smb_ReleaseTIDDbg(smb_tid_t *tidp, afs_uint32 locked, char *, long);
+#define smb_ReleaseTID(a,b) smb_ReleaseTIDDbg(a,b,__FILE__,__LINE__);
+#else
 extern smb_tid_t *smb_FindTID(smb_vc_t *vcp, unsigned short tid, int flags);
 
 extern void smb_HoldTIDNoLock(smb_tid_t *tidp);
 
 extern void smb_ReleaseTID(smb_tid_t *tidp, afs_uint32 locked);
+#endif
 
 extern smb_user_t *smb_FindUID(smb_vc_t *vcp, unsigned short uid, int flags);
 
-extern smb_username_t *smb_FindUserByName(char *usern, char *machine, afs_uint32 flags);
+extern smb_username_t *smb_FindUserByName(clientchar_t *usern, clientchar_t *machine, afs_uint32 flags);
 
-extern smb_user_t *smb_FindUserByNameThisSession(smb_vc_t *vcp, char *usern); 
+extern cm_user_t *smb_FindCMUserByName(clientchar_t *usern, clientchar_t *machine, afs_uint32 flags);
+
+extern smb_user_t *smb_FindUserByNameThisSession(smb_vc_t *vcp, clientchar_t *usern); 
 
 extern void smb_ReleaseUsername(smb_username_t *unp);
 
@@ -581,8 +616,21 @@ extern cm_user_t *smb_GetUserFromVCP(smb_vc_t *vcp, smb_packet_t *inp);
 
 extern cm_user_t *smb_GetUserFromUID(smb_user_t *uidp);
 
-extern long smb_LookupTIDPath(smb_vc_t *vcp, unsigned short tid, char ** tidPathp);
+extern long smb_LookupTIDPath(smb_vc_t *vcp, unsigned short tid, clientchar_t ** tidPathp);
 
+#ifdef DEBUG_SMB_REFCOUNT
+extern smb_fid_t *smb_FindFIDDbg(smb_vc_t *vcp, unsigned short fid, int flags, char *, long);
+#define smb_FindFID(a,b,c) smb_FindFIDDbg(a,b,c,__FILE__,__LINE__);
+
+extern smb_fid_t *smb_FindFIDByScacheDbg(smb_vc_t *vcp, cm_scache_t * scp, char *, long);
+#define smb_FindFIDByScache(a,b) smb_FindFIDByScacheDbg(a,b,__FILE__,__LINE__);
+
+extern void smb_HoldFIDNoLockDbg(smb_fid_t *fidp, char *, long);
+#define smb_HoldFIDNoLock(a) smb_HoldFIDNoLockDbg(a,__FILE__,__LINE__);
+
+extern void smb_ReleaseFIDDbg(smb_fid_t *fidp, char *, long);
+#define smb_ReleaseFID(a) smb_ReleaseFIDDbg(a,__FILE__,__LINE__);
+#else
 extern smb_fid_t *smb_FindFID(smb_vc_t *vcp, unsigned short fid, int flags);
 
 extern smb_fid_t *smb_FindFIDByScache(smb_vc_t *vcp, cm_scache_t * scp);
@@ -590,13 +638,14 @@ extern smb_fid_t *smb_FindFIDByScache(smb_vc_t *vcp, cm_scache_t * scp);
 extern void smb_HoldFIDNoLock(smb_fid_t *fidp);
 
 extern void smb_ReleaseFID(smb_fid_t *fidp);
+#endif
 
 extern long smb_CloseFID(smb_vc_t *vcp, smb_fid_t *fidp, cm_user_t *userp,
                          afs_uint32 dosTime);
 
-extern int smb_FindShare(smb_vc_t *vcp, smb_user_t *uidp, char *shareName, char **pathNamep);
+extern int smb_FindShare(smb_vc_t *vcp, smb_user_t *uidp, clientchar_t *shareName, clientchar_t **pathNamep);
 
-extern int smb_FindShareCSCPolicy(char *shareName);
+extern int smb_FindShareCSCPolicy(clientchar_t *shareName);
 
 extern smb_dirSearch_t *smb_FindDirSearchNL(long cookie);
 
@@ -632,13 +681,37 @@ extern void smb_SetSMBParmDouble(smb_packet_t *smbp, int slot, char *parmValuep)
 
 extern void smb_SetSMBParmByte(smb_packet_t *smbp, int slot, unsigned int parmValue);
 
-extern void smb_StripLastComponent(char *outPathp, char **lastComponentp,
-	char *inPathp);
+extern void smb_StripLastComponent(clientchar_t *outPathp, clientchar_t **lastComponentp,
+	clientchar_t *inPathp);
 
-extern unsigned char *smb_ParseASCIIBlock(unsigned char *inp, char **chainpp);
+#define SMB_STRF_FORCEASCII (1<<0)
+#define SMB_STRF_ANSIPATH   (1<<1)
+#define SMB_STRF_IGNORENUL  (1<<2)
+#define SMB_STRF_SRCNULTERM (1<<3)
+
+extern clientchar_t *smb_ParseASCIIBlock(smb_packet_t * pktp, unsigned char *inp,
+                                         char **chainpp, int flags);
+
+extern clientchar_t *smb_ParseString(smb_packet_t * pktp, unsigned char * inp,
+                                     char ** chainpp, int flags);
+
+extern clientchar_t *smb_ParseStringBuf(const unsigned char * bufbase,
+                                        cm_space_t ** stringspp,
+                                        unsigned char *inp, size_t *pcb_max,
+                                        char **chainpp, int flags);
+
+extern clientchar_t *smb_ParseStringCb(smb_packet_t * pktp, unsigned char * inp,
+                                       size_t cb, char ** chainpp, int flags);
+
+extern clientchar_t *smb_ParseStringCch(smb_packet_t * pktp, unsigned char * inp,
+                                        size_t cch, char ** chainpp, int flags);
+
+extern unsigned char * smb_UnparseString(smb_packet_t * pktp, unsigned char * outp,
+                                   clientchar_t * str,
+                                   size_t * plen, int flags);
 
 extern unsigned char *smb_ParseVblBlock(unsigned char *inp, char **chainpp,
-	int *lengthp);
+                                  int *lengthp);
 
 extern smb_packet_t *smb_GetResponsePacket(smb_vc_t *vcp, smb_packet_t *inp);
 
@@ -649,9 +722,7 @@ extern void smb_MapCoreError(long code, smb_vc_t *vcp, unsigned short *scodep,
 
 extern void smb_MapNTError(long code, unsigned long *NTStatusp);
 
-extern void smb_HoldVC(smb_vc_t *vcp);
-
-extern void smb_HoldVCNoLock(smb_vc_t *vcp);
+extern void smb_MapWin32Error(long code, unsigned long *Win32Ep);
 
 /* some globals, too */
 extern char *smb_localNamep;
@@ -670,7 +741,7 @@ extern int smb_maxMpxRequests; /* max # of mpx requests */
 
 extern int smb_StoreAnsiFilenames;
 extern int smb_hideDotFiles;
-extern unsigned int smb_IsDotFile(char *lastComp);
+extern unsigned int smb_IsDotFile(clientchar_t *lastComp);
 extern afs_uint32 smb_AsyncStore;
 extern afs_uint32 smb_AsyncStoreSize;
 
@@ -683,14 +754,16 @@ extern int smb_authType; /* Type of SMB authentication to be used. One from belo
 
 extern HANDLE smb_lsaHandle; /* LSA handle obtained during smb_init if using SMB auth */
 extern ULONG smb_lsaSecPackage; /* LSA security package id. Set during smb_init */
-extern char smb_ServerDomainName[];
+extern clientchar_t smb_ServerDomainName[];
 extern int smb_ServerDomainNameLength;
-extern char smb_ServerOS[];
+extern clientchar_t smb_ServerOS[];
 extern int smb_ServerOSLength;
-extern char smb_ServerLanManager[];
+extern clientchar_t smb_ServerLanManager[];
 extern int smb_ServerLanManagerLength;
 extern GUID smb_ServerGUID;
 extern LSA_STRING smb_lsaLogonOrigin;
+extern LONG smb_UseUnicode;
+extern DWORD smb_monitorReqs;
 
 /* used for getting a challenge for SMB auth */
 typedef struct _MSV1_0_LM20_CHALLENGE_REQUEST {  
@@ -703,9 +776,9 @@ typedef struct _MSV1_0_LM20_CHALLENGE_RESPONSE {
 } MSV1_0_LM20_CHALLENGE_RESPONSE, *PMSV1_0_LM20_CHALLENGE_RESPONSE;
 /**/
 
-extern long smb_AuthenticateUserLM(smb_vc_t *vcp, char * accountName, char * primaryDomain, char * ciPwd, unsigned ciPwdLength, char * csPwd, unsigned csPwdLength);
+extern long smb_AuthenticateUserLM(smb_vc_t *vcp, clientchar_t * accountName, clientchar_t * primaryDomain, char * ciPwd, unsigned ciPwdLength, char * csPwd, unsigned csPwdLength);
 
-extern long smb_GetNormalizedUsername(char * usern, const char * accountName, const char * domainName);
+extern long smb_GetNormalizedUsername(clientchar_t * usern, const clientchar_t * accountName, const clientchar_t * domainName);
 
 extern void smb_FormatResponsePacket(smb_vc_t *vcp, smb_packet_t *inp,
 	smb_packet_t *op);
@@ -716,47 +789,34 @@ extern char *myCrt_2Dispatch(int i);
 
 extern char *myCrt_RapDispatch(int i);
 
+extern char * myCrt_NmpipeDispatch(int i);
+
 extern unsigned int smb_Attributes(cm_scache_t *scp);
 
 extern int smb_ChainFID(int fid, smb_packet_t *inp);
 
 extern unsigned char *smb_ParseDataBlock(unsigned char *inp, char **chainpp, int *lengthp);
 
-extern unsigned char *smb_ParseASCIIBlock(unsigned char *inp, char **chainpp);
-
 extern unsigned char *smb_ParseVblBlock(unsigned char *inp, char **chainpp, int *lengthp);
 
 extern int smb_SUser(cm_user_t *userp);
 
-#ifndef DJGPP
-long smb_WriteData(smb_fid_t *fidp, osi_hyper_t *offsetp, long count, char *op,
+long smb_WriteData(smb_fid_t *fidp, osi_hyper_t *offsetp, afs_uint32 count, char *op,
 	cm_user_t *userp, long *writtenp);
-#else /* DJGPP */
-long smb_WriteData(smb_fid_t *fidp, osi_hyper_t *offsetp, long count, char *op,
-	cm_user_t *userp, long *writtenp, int dosflag);
-#endif /* !DJGPP */
 
-#ifndef DJGPP
-extern long smb_ReadData(smb_fid_t *fidp, osi_hyper_t *offsetp, long count,
+extern long smb_ReadData(smb_fid_t *fidp, osi_hyper_t *offsetp, afs_uint32 count,
 	char *op, cm_user_t *userp, long *readp);
-#else /* DJGPP */
-extern long smb_ReadData(smb_fid_t *fidp, osi_hyper_t *offsetp, long count,
-	char *op, cm_user_t *userp, long *readp, int dosflag);
-#endif /* !DJGPP */
 
-extern long smb_Rename(smb_vc_t *vcp, smb_packet_t *inp, char *oldPathp, char *newPathp, int attrs);
+extern long smb_Rename(smb_vc_t *vcp, smb_packet_t *inp, clientchar_t *oldPathp, clientchar_t *newPathp, int attrs);
 
-extern long smb_Link(smb_vc_t *vcp, smb_packet_t *inp, char *oldPathp, char *newPathp);
+extern long smb_Link(smb_vc_t *vcp, smb_packet_t *inp, clientchar_t *oldPathp, clientchar_t *newPathp);
 
-extern BOOL smb_IsLegalFilename(char *filename);
+extern BOOL smb_IsLegalFilename(clientchar_t *filename);
 
 extern char *smb_GetSharename(void);
 
 extern DWORD smb_ServerExceptionFilter(void);
 
-extern void smb_UpdateServerPriority(void);
-extern void smb_SetRequestStartTime(void);
-extern void smb_ResetServerPriority(void);
 extern void smb_RestartListeners(int);
 extern void smb_StopListeners(int);
 extern void smb_StopListener(NCB *ncbp, int lana, int wait);
@@ -764,6 +824,7 @@ extern long smb_IsNetworkStarted(void);
 extern void smb_LanAdapterChange(int);
 extern void smb_SetLanAdapterChangeDetected(void);
 
+extern void smb_InitReq(cm_req_t *reqp);
 
 #define SMB_LISTENER_UNINITIALIZED -1
 #define SMB_LISTENER_STOPPED 0
@@ -773,8 +834,19 @@ extern void smb_SetLanAdapterChangeDetected(void);
 #include "smb3.h"
 #include "smb_ioctl.h"
 #include "smb_iocons.h"
+#include "smb_rpc.h"
+#include "cm_vnodeops.h"
 
-cm_user_t *smb_FindOrCreateUser(smb_vc_t *vcp, char *usern);
+extern int smb_unixModeDefaultFile;
+extern int smb_unixModeDefaultDir;
+extern void smb_SetInitialModeBitsForFile(int smb_attr, cm_attr_t * attr);
+extern void smb_SetInitialModeBitsForDir(int smb_attr, cm_attr_t * attr);
+
+cm_user_t *smb_FindOrCreateUser(smb_vc_t *vcp, clientchar_t *usern);
+
+int smb_DumpVCP(FILE *outputFile, char *cookie, int lock);
+
+void smb_Shutdown(void);
 
 #ifdef NOTSERVICE
 extern void smb_LogPacket(smb_packet_t *packet);
