@@ -20,6 +20,7 @@ typedef struct smb_tran2Packet {
         int curParms;			/* current # of received parm bytes */
         int maxReturnData;		/* max # of returned data bytes */
         int maxReturnParms;		/* max # of returned parm bytes */
+    int setupCount;			/* setup count field */
         int opcode;			/* subopcode we're handling */
         long flags;			/* flags */
         smb_vc_t *vcp;			/* virtual circuit we're dealing with */
@@ -28,8 +29,14 @@ typedef struct smb_tran2Packet {
         unsigned short pid;		/* pid to remember */
         unsigned short uid;		/* uid to remember */
 	unsigned short res[6];		/* contains PidHigh */
+    unsigned int error_code;		/* CM error code for the packet */
         unsigned short *parmsp;		/* parms */
         unsigned char *datap;		/* data bytes */
+    int pipeCommand;			/* named pipe command code */
+    int pipeParam;			/* pipe parameter, if there is one */
+    clientchar_t *name;			/* contents of Name
+					   field. Only used for Named
+					   pipes */
         cm_space_t * stringsp;          /* decoded strings */
 } smb_tran2Packet_t;
 
@@ -207,6 +214,22 @@ typedef struct {
 	    unsigned long  fileNameLength;
 	    unsigned char  fileName[512];
 	} QFfileNameInfo;
+	struct {
+	    unsigned long  nextEntryOffset;
+	    unsigned long  streamNameLength;
+	    LARGE_INTEGER  streamSize;
+	    LARGE_INTEGER  streamAllocationSize;
+	    unsigned char  fileName[512]; /* STRING */
+	} QFfileStreamInfo;
+        struct {
+            LARGE_INTEGER  creationTime;
+	    LARGE_INTEGER  lastAccessTime;
+	    LARGE_INTEGER  lastWriteTime;
+	    LARGE_INTEGER  lastChangeTime;
+            LARGE_INTEGER  allocationSize;
+            LARGE_INTEGER  endOfFile;
+	    unsigned long  attributes;
+        } QFnetworkOpenInfo;
     } u;
 } smb_tran2QFileInfo_t;
 
@@ -293,6 +316,14 @@ extern smb_tran2Dispatch_t smb_tran2DispatchTable[SMB_TRAN2_NOPCODES];
 
 extern smb_tran2Dispatch_t smb_rapDispatchTable[SMB_RAP_NOPCODES];
 
+extern smb_tran2Packet_t *smb_GetTran2ResponsePacket(smb_vc_t *vcp,
+						     smb_tran2Packet_t *inp, smb_packet_t *outp,
+						     int totalParms, int totalData);
+
+extern void smb_FreeTran2Packet(smb_tran2Packet_t *t2p);
+
+extern void smb_SendTran2Packet(smb_vc_t *vcp, smb_tran2Packet_t *t2p, smb_packet_t *tp);
+
 extern long smb_ReceiveV3SessionSetupX(smb_vc_t *vcp, smb_packet_t *inp, smb_packet_t *outp);
 
 extern long smb_ReceiveV3TreeConnectX(smb_vc_t *vcp, smb_packet_t *inp, smb_packet_t *outp);
@@ -408,4 +439,9 @@ extern void smb_NegotiateExtendedSecurity(void ** secBlob, int * secBlobLength);
 #define FILE_NOTIFY_CHANGE_STREAM_NAME  0x00000200
 #define FILE_NOTIFY_CHANGE_STREAM_SIZE  0x00000400
 #define FILE_NOTIFY_CHANGE_STREAM_WRITE 0x00000800
+
+/* NT Create Device Status bit flags */
+#define NO_REPARSETAG 0x0004
+#define NO_SUBSTREAMS 0x0002
+#define NO_EAS        0x0001
 #endif /*  __SMB3_H_ENV__ */
