@@ -7,6 +7,9 @@
  * directory or online at http://www.openafs.org/dl/license10.html
  */
 
+#include <winsock2.h>
+#include <ws2tcpip.h>
+
 extern "C" {
 #include <afs/param.h>
 #include <afs/stds.h>
@@ -139,7 +142,8 @@ void Alert_EndUpdate (LPIDENT lpi, LPSERVER lpServer)
          BOOL fNeedBadCredsWarning = FALSE;
          BOOL fHaveBadCredsWarning = FALSE;
 
-         for (size_t iAlert = 0; iAlert < lpoaServer->nAlerts; ++iAlert)
+         size_t iAlert;
+         for (iAlert = 0; iAlert < lpoaServer->nAlerts; ++iAlert)
             {
             if (lpoaServer->aAlerts[ iAlert ].alert == alertSECONDARY)
                {
@@ -226,7 +230,8 @@ void Alert_RemoveSecondary (LPIDENT lpiChild)
    LPOBJECTALERTS lpoa;
    if ((lpoa = Alert_GetObjectAlerts (lpiChild, TRUE)) != NULL)
       {
-      for (size_t iAlert = 0; iAlert < lpoa->nAlerts; )
+      size_t iAlert;
+      for (iAlert = 0; iAlert < lpoa->nAlerts; )
          {
          if ( (lpoa->aAlerts[ iAlert ].alert == alertSECONDARY) &&
               (lpoa->aAlerts[ iAlert ].aiSECONDARY.lpiSecondary == lpiChild) )
@@ -342,7 +347,8 @@ void Alert_Scout_ServerStatus (LPIDENT lpi, ULONG status)
          if (lpoa->nAlerts && (lpoa->aAlerts[0].alert == alertBADCREDS))
             iInsert = 1;
 
-         for (size_t iHole = iInsert; iHole < lpoa->nAlerts; ++iHole)
+         size_t iHole;
+         for (iHole = iInsert; iHole < lpoa->nAlerts; ++iHole)
             {
             if (lpoa->aAlerts[ iHole ].alert == alertINVALID)
                break;
@@ -357,7 +363,6 @@ void Alert_Scout_ServerStatus (LPIDENT lpi, ULONG status)
             lpoa->nAlerts ++;
             lpoa->aAlerts[ iInsert ].alert = alertTIMEOUT;
             lpoa->aAlerts[ iInsert ].aiTIMEOUT.status = status;
-            lpoa->aAlerts[ iInsert ].aiTIMEOUT.stLastAttempt;
             GetSystemTime (&lpoa->aAlerts[ iInsert ].aiTIMEOUT.stLastAttempt);
 
             fChanged = TRUE;
@@ -713,7 +718,7 @@ BOOL Alert_StartScout (ULONG *pStatus)
 {
    if (hScout == 0)  // create scout?
       {
-      heScoutWakeup = CreateEvent (NULL, FALSE, FALSE, NULL);
+      heScoutWakeup = CreateEvent (NULL, FALSE, FALSE, TEXT("AfsSvrMgr Alert Scout Wakeup"));
 
       DWORD dwThreadID;
       if ((hScout = CreateThread (NULL, 0,
