@@ -12,17 +12,15 @@
 #include <afs/param.h>
 #include <afs/stds.h>
 
-#ifndef DJGPP
 #include <windows.h>
 #include <rpc.h>
-#endif /* !DJGPP */
 #include <malloc.h>
 #include "osi.h"
-#ifndef DJGPP
 #include "dbrpc.h"
-#endif /* !DJGPP */
 #include <stdio.h>
 #include <assert.h>
+#include <WINNT\afsreg.h>
+#include <strsafe.h>
 
 #define AFS_DAEMON_EVENT_NAME "TransarcAFSDaemon"
 
@@ -36,16 +34,22 @@ osi_log_t *osi_allLogsp;	/* all logs known; for use during panic */
 unsigned long osi_logFreq;	/* 0, or frequency of high perf counter */
 unsigned long osi_logTixToMicros;	/* mult. correction factor */
 
+#define TRACE_OPTION_EVENT 2
+#define TRACE_OPTION_DEBUGLOG 4
+
+#define ISCLIENTTRACE(v) ( ((v) & TRACE_OPTION_EVENT)==TRACE_OPTION_EVENT)
+#define ISCLIENTDEBUGLOG(v) (((v) & TRACE_OPTION_DEBUGLOG)==TRACE_OPTION_DEBUGLOG)
+
+DWORD osi_TraceOption=0;
+
 osi_fdOps_t osi_logFDOps = {
 	osi_LogFDCreate,
-#ifndef DJGPP
         osi_LogFDGetInfo,
-#endif
         osi_LogFDClose
 };
 
 /* create a new log, taking a name and a size in entries (not words) */
-osi_log_t *osi_LogCreate(char *namep, long size)
+osi_log_t *osi_LogCreate(char *namep, size_t size)
 {
 	osi_log_t *logp;
         osi_fdType_t *typep;
@@ -54,7 +58,6 @@ osi_log_t *osi_LogCreate(char *namep, long size)
         LARGE_INTEGER bigTemp;
         LARGE_INTEGER bigJunk;
 	
-#ifndef DJGPP
 	if (osi_Once(&osi_logOnce)) {
 		QueryPerformanceFrequency(&bigFreq);
                 if (bigFreq.LowPart == 0 && bigFreq.HighPart == 0)
@@ -76,13 +79,15 @@ osi_log_t *osi_LogCreate(char *namep, long size)
 		/* done with init */
 		osi_EndOnce(&osi_logOnce);
         }
-#endif /* !DJGPP */
 
         logp = malloc(sizeof(osi_log_t));
         memset(logp, 0, sizeof(osi_log_t));
-        logp->namep = malloc(strlen(namep)+1);
-        strcpy(logp->namep, namep);
-        
+        {
+            size_t namelen = strlen(namep) + 1;
+
+            logp->namep = malloc(namelen * sizeof(char));
+            StringCchCopyA(logp->namep, namelen, namep);
+        }
         osi_QAdd((osi_queue_t **) &osi_allLogsp, &logp->q);
 
 	/* compute size we'll use */
@@ -93,17 +98,16 @@ osi_log_t *osi_LogCreate(char *namep, long size)
         logp->datap = malloc(size * sizeof(osi_logEntry_t));
 
 	/* init strings array */
-	logp->maxstringindex = size/10;
+	logp->maxstringindex = size/3;
 	logp->stringindex = 0;
-	logp->stringsp = malloc((size/10) * OSI_LOG_STRINGSIZE);
+	logp->stringsp = malloc(logp->maxstringindex * OSI_LOG_STRINGSIZE);
  
         /* and sync */
         thrd_InitCrit(&logp->cs);
         
-	strcpy(tbuffer, "log:");
-        strcat(tbuffer, namep);
+	StringCbCopyA(tbuffer, sizeof(tbuffer), "log:");
+        StringCbCatA(tbuffer, sizeof(tbuffer), namep);
 	typep = osi_RegisterFDType(tbuffer, &osi_logFDOps, logp);
-#ifndef DJGPP
 	if (typep) {
 		/* add formatting info */
 		osi_AddFDFormatInfo(typep, OSI_DBRPC_REGIONINT, 0,
@@ -111,15 +115,14 @@ osi_log_t *osi_LogCreate(char *namep, long size)
 		osi_AddFDFormatInfo(typep, OSI_DBRPC_REGIONSTRING, 1,
 			"Time (mics)", 0);
 	}
-#endif
 	
         return logp;
 }
 
-/* we just panic'd.  Turn off all logging adding special log record
- * to all enabled logs.  Be careful not to wait for a lock.
+/* we just panic'd.  Log the error to all enabled log files.
+ * Be careful not to wait for a lock.
  */
-void osi_LogPanic(char *filep, long lineNumber)
+void osi_LogPanic(char *msgp, char *filep, size_t lineNumber)
 {
 	osi_log_t *tlp;
 
@@ -128,9 +131,9 @@ void osi_LogPanic(char *filep, long lineNumber)
 
 		/* otherwise, proceed */
 		if (filep)
-	                osi_LogAdd(tlp, "**PANIC** (file %s:%d)", (long) filep, lineNumber, 0, 0);
+	                osi_LogAdd(tlp, "**PANIC** \"%s\" (file %s:%d)", (size_t)msgp, (size_t) filep, lineNumber, 0, 0);
 		else
-			osi_LogAdd(tlp, "**PANIC**", 0, 0, 0, 0);
+			osi_LogAdd(tlp, "**PANIC** \"%s\"", (size_t)msgp, 0, 0, 0, 0);
 		
                 /* should grab lock for this, but we're in panic, and better safe than
                  * sorry.
@@ -163,7 +166,7 @@ void osi_LogFree(osi_log_t *logp)
 }
 
 /* add an element to a log */
-void osi_LogAdd(osi_log_t *logp, char *formatp, long p0, long p1, long p2, long p3)
+void osi_LogAdd(osi_log_t *logp, char *formatp, size_t p0, size_t p1, size_t p2, size_t p3, size_t p4)
 {
 	osi_logEntry_t *lep;
         long ix;
@@ -190,34 +193,41 @@ void osi_LogAdd(osi_log_t *logp, char *formatp, long p0, long p1, long p2, long
         lep->tid = thrd_Current();
 
 	/* get the time, using the high res timer if available */
-#ifndef DJGPP
         if (osi_logFreq) {
 		QueryPerformanceCounter(&bigTime);
 		lep->micros = (bigTime.LowPart / osi_logFreq) * osi_logTixToMicros;
         }
         else lep->micros = GetCurrentTime() * 1000;
-#else
-        lep->micros = gettime_us();
-#endif /* !DJGPP */                
 
         lep->formatp = formatp;
         lep->parms[0] = p0;
         lep->parms[1] = p1;
         lep->parms[2] = p2;
         lep->parms[3] = p3;
-
+        lep->parms[4] = p4;
 #ifdef NOTSERVICE
-		printf( "%9ld:", lep->micros );
-		printf( formatp, p0, p1, p2, p3);
-		printf( "\n" );
+        printf( "%9ld:", lep->micros );
+        printf( formatp, p0, p1, p2, p3, p4);
+        printf( "\n" );
 #endif
 
+        if(ISCLIENTDEBUGLOG(osi_TraceOption)) {
+	    char wholemsg[1024], msg[1000];
+
+	    StringCbPrintfA(msg, sizeof(msg), formatp,
+                            p0, p1, p2, p3, p4);
+	    StringCbPrintfA(wholemsg, sizeof(wholemsg), 
+                            "tid[%d] %s\n",
+                            lep->tid, msg);
+            OutputDebugStringA(wholemsg);
+        }
+
 	thrd_LeaveCrit(&logp->cs);
 }
 
 void osi_LogPrint(osi_log_t *logp, FILE_HANDLE handle)
 {
-	char wholemsg[1000], msg[1000];
+	char wholemsg[1024], msg[1000];
 	int i, ix, ioCount;
 	osi_logEntry_t *lep;
 
@@ -229,19 +239,17 @@ void osi_LogPrint(osi_log_t *logp, FILE_HANDLE handle)
 	     i < logp->nused;
 	     i++, ix++, (ix >= logp->alloc ? ix -= logp->alloc : 0)) {
 		lep = logp->datap + ix;		/* pointer arithmetic */
-		sprintf(msg, lep->formatp,
-			lep->parms[0], lep->parms[1],
-			lep->parms[2], lep->parms[3]);
-		sprintf(wholemsg, "time %d.%06d, pid %d %s\n",
-			lep->micros / 1000000,
-			lep->micros % 1000000,
-			lep->tid, msg);
-#ifndef DJGPP
+		StringCbPrintfA(msg, sizeof(msg), lep->formatp,
+                                lep->parms[0], lep->parms[1],
+                                lep->parms[2], lep->parms[3],
+                                lep->parms[4]);
+		StringCbPrintfA(wholemsg, sizeof(wholemsg),
+                                "time %d.%06d, tid %d %s\r\n",
+                                lep->micros / 1000000,
+                                lep->micros % 1000000,
+                                lep->tid, msg);
 		if (!WriteFile(handle, wholemsg, strlen(wholemsg),
-				&ioCount, NULL))
-#else /* DJGPP */
-                if ((ioCount = fwrite(wholemsg, 1, strlen(wholemsg), handle)) == 0)
-#endif /* !DJGPP */
+                               &ioCount, NULL))
 			break;
 	}
 
@@ -250,20 +258,63 @@ void osi_LogPrint(osi_log_t *logp, FILE_HANDLE handle)
 
 char *osi_LogSaveString(osi_log_t *logp, char *s)
 {
-	char *saveplace = logp->stringsp[logp->stringindex];
+    char *saveplace;
 
-	if (s == NULL) return NULL;
+    if (!logp) return s;
 
-	if (strlen(s) >= OSI_LOG_STRINGSIZE)
-		sprintf(saveplace, "...%s",
-			s + strlen(s) - (OSI_LOG_STRINGSIZE - 4));
-	else
-		strcpy(saveplace, s);
-	logp->stringindex++;
-	if (logp->stringindex >= logp->maxstringindex)
-	    logp->stringindex = 0;
+    if (!logp->enabled) return s;
 
-	return saveplace;
+    if (s == NULL) return NULL;
+
+    thrd_EnterCrit(&logp->cs);
+
+    saveplace = logp->stringsp[logp->stringindex];
+
+    if (strlen(s) >= OSI_LOG_STRINGSIZE)
+        StringCbPrintfA(saveplace, OSI_LOG_STRINGSIZE,
+                        "...%s",
+                        s + strlen(s) - (OSI_LOG_STRINGSIZE - 4));
+    else
+        StringCbCopyA(saveplace, OSI_LOG_STRINGSIZE, s);
+
+    logp->stringindex++;
+
+    if (logp->stringindex >= logp->maxstringindex)
+        logp->stringindex = 0;
+
+    thrd_LeaveCrit(&logp->cs);
+
+    return saveplace;
+}
+
+wchar_t *osi_LogSaveStringW(osi_log_t *logp, wchar_t *s)
+{
+    wchar_t *saveplace;
+
+    if (!logp) return s;
+
+    if (!logp->enabled) return s;
+
+    if (s == NULL) return NULL;
+
+    thrd_EnterCrit(&logp->cs);
+
+    saveplace = (wchar_t *) (logp->stringsp[logp->stringindex]);
+
+    if (wcslen(s)*sizeof(wchar_t) >= OSI_LOG_STRINGSIZE)
+        StringCbPrintfW(saveplace, OSI_LOG_STRINGSIZE, L"...%s",
+                        (s + wcslen(s) - (OSI_LOG_STRINGSIZE/sizeof(wchar_t) - 4)));
+    else
+        StringCbCopyW(saveplace, OSI_LOG_STRINGSIZE, s);
+
+    logp->stringindex++;
+
+    if (logp->stringindex >= logp->maxstringindex)
+        logp->stringindex = 0;
+
+    thrd_LeaveCrit(&logp->cs);
+
+    return saveplace;
 }
 
 long osi_LogFDCreate(osi_fdType_t *typep, osi_fd_t **outpp)
@@ -283,45 +334,43 @@ long osi_LogFDCreate(osi_fdType_t *typep, osi_fd_t **outpp)
         return 0;
 }
 
-#ifndef DJGPP
 long osi_LogFDGetInfo(osi_fd_t *ifd, osi_remGetInfoParms_t *outp)
 {
-	osi_logFD_t *lfdp;
-        osi_log_t *logp;
-        osi_logEntry_t *lep;
-        char tbuffer[256];
-        long ix;
+    osi_logFD_t *lfdp;
+    osi_log_t *logp;
+    osi_logEntry_t *lep;
+    char tbuffer[256];
+    long ix;
         
-        lfdp = (osi_logFD_t *) ifd;
-        logp = lfdp->logp;
+    lfdp = (osi_logFD_t *) ifd;
+    logp = lfdp->logp;
         
-	/* see if we're done */
-	if (lfdp->current >= lfdp->nused) return OSI_DBRPC_EOF;
+    /* see if we're done */
+    if (lfdp->current >= lfdp->nused) return OSI_DBRPC_EOF;
         
-	/* grab lock */
-	thrd_EnterCrit(&logp->cs);
+    /* grab lock */
+    thrd_EnterCrit(&logp->cs);
 
-        /* compute which one we want */
-        ix = lfdp->first + lfdp->current;
-        if (ix >= logp->alloc) ix -= logp->alloc;
-        lfdp->current++;
-        lep = logp->datap + ix;	/* ptr arith to current index */
+    /* compute which one we want */
+    ix = lfdp->first + lfdp->current;
+    if (ix >= logp->alloc) ix -= logp->alloc;
+    lfdp->current++;
+    lep = logp->datap + ix;	/* ptr arith to current index */
 
-	sprintf(tbuffer, lep->formatp, lep->parms[0], lep->parms[1],
-        	lep->parms[2], lep->parms[3]);
+    StringCbPrintfA(tbuffer, sizeof(tbuffer), lep->formatp, lep->parms[0], lep->parms[1],
+                    lep->parms[2], lep->parms[3], lep->parms[4]);
 
-	/* now copy out info */
-        strcpy(outp->sdata[0], tbuffer);
-        sprintf(tbuffer, "%5.6f", ((double)lep->micros)/1000000.0);
-        strcpy(outp->sdata[1], tbuffer);
-        outp->idata[0] = lep->tid;
-        outp->scount = 2;
-        outp->icount = 1;
+    /* now copy out info */
+    StringCbCopyA(outp->sdata[0], sizeof(outp->sdata[0]), tbuffer);
+    StringCbPrintfA(tbuffer, sizeof(tbuffer), "%5.6f", ((double)lep->micros)/1000000.0);
+    StringCbCopyA(outp->sdata[1], sizeof(outp->sdata[0]), tbuffer);
+    outp->idata[0] = lep->tid;
+    outp->scount = 2;
+    outp->icount = 1;
 
-	thrd_LeaveCrit(&logp->cs);
-        return 0;
+    thrd_LeaveCrit(&logp->cs);
+    return 0;
 }
-#endif /* !DJGPP */
 
 long osi_LogFDClose(osi_fd_t *ifdp)
 {
@@ -341,17 +390,11 @@ void osi_LogDisable(osi_log_t *logp)
 		logp->enabled = 0;
 }
 
-#define REG_CLIENT_PARMS_KEY  "SYSTEM\\CurrentControlSet\\Services\\TransarcAFSDaemon\\Parameters"
-#define TRACE_OPTION_EVENT 2
-#define ISCLIENTTRACE(v) ( ((v) & TRACE_OPTION_EVENT)==TRACE_OPTION_EVENT)
-
-DWORD osi_TraceOption=0;
-
 void osi_InitTraceOption()
 {
 	DWORD LSPtype, LSPsize;
 	HKEY NPKey;
-	(void) RegOpenKeyEx(HKEY_LOCAL_MACHINE, REG_CLIENT_PARMS_KEY,
+	(void) RegOpenKeyEx(HKEY_LOCAL_MACHINE, AFSREG_CLT_SVC_PARAM_SUBKEY,
 		    0, KEY_QUERY_VALUE, &NPKey);
 	LSPsize=sizeof(osi_TraceOption);
 	RegQueryValueEx(NPKey, "TraceOption", NULL,
@@ -381,7 +424,7 @@ void osi_LogEvent(char *a,char *b,char *c,...)
 		return;
     h = RegisterEventSource(NULL, AFS_DAEMON_EVENT_NAME);
 	va_start(marker,c);
-	_vsnprintf(buf,MAXBUF_,c,marker);
+	StringCbVPrintfA(buf,MAXBUF_,c,marker);
 	ptbuf[0] = buf;
 	ReportEvent(h, EVENTLOG_INFORMATION_TYPE, 0, 0, NULL, 1, 0, (const char **)ptbuf, NULL);\
 	DeregisterEventSource(h);
