@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2005,2006 Secure Endpoints Inc.
+ * Copyright (c) 2005,2006,2007,2008 Secure Endpoints Inc.
  *
  * Permission is hereby granted, free of charge, to any person
  * obtaining a copy of this software and associated documentation
@@ -22,7 +22,7 @@
  * SOFTWARE.
  */
 
-/* $Id: afsnewcreds.c,v 1.1.2.3 2006/10/16 13:55:35 jaltman Exp $ */
+/* $Id$ */
 
 #include<afscred.h>
 #include<commctrl.h>
@@ -377,6 +377,7 @@ afs_remove_token_from_identities(wchar_t * cell) {
         khm_size cb;
         wchar_t vbuf[1024];
         wchar_t * tbuf = NULL;
+        khm_int32 enabled = 0;
 
         kcdb_identity_create(t, 0, &h_id);
         if (h_id == NULL) {
@@ -393,6 +394,10 @@ afs_remove_token_from_identities(wchar_t * cell) {
                                       0, &csp_afs)))
             goto _cleanup_loop;
 
+        if (KHM_SUCCEEDED(khc_read_int32(csp_afs, L"AFSEnabled", &enabled)) &&
+            !enabled)
+            goto _cleanup_loop;
+
         if (khc_read_multi_string(csp_afs, L"Cells", NULL, &cb)
             != KHM_ERROR_TOO_LONG)
             goto _cleanup_loop;
@@ -472,6 +477,7 @@ afs_check_add_token_to_identity(wchar_t * cell, khm_handle ident,
         khm_size cb;
         wchar_t vbuf[1024];
         wchar_t * tbuf = NULL;
+        khm_int32 enabled = 0;
 
         kcdb_identity_create(t, 0, &h_id);
         if (h_id == NULL) {
@@ -493,6 +499,10 @@ afs_check_add_token_to_identity(wchar_t * cell, khm_handle ident,
                                       0, &csp_afs)))
             goto _cleanup_loop;
 
+        if (KHM_SUCCEEDED(khc_read_int32(csp_afs, L"AFSEnabled", &enabled)) &&
+            !enabled)
+            goto _cleanup_loop;
+
         if (khc_read_multi_string(csp_afs, L"Cells", NULL, &cb)
             != KHM_ERROR_TOO_LONG)
             goto _cleanup_loop;
@@ -535,6 +545,113 @@ afs_check_add_token_to_identity(wchar_t * cell, khm_handle ident,
     return ok_to_add;
 }
 
+static void
+add_token_to_list(afs_cred_list * l,
+                  khm_handle h_gcells,
+                  HKEY hk_gcells,
+                  const wchar_t * c_cell)
+{
+    khm_size cb;
+    int i;
+    afs_cred_row * r;
+    khm_handle h_cell = NULL;
+    HKEY       hk_cell = NULL;
+    wchar_t wbuf[MAXCELLCHARS];
+    wchar_t wmethod[KHUI_MAXCCH_NAME];
+
+    if (FAILED(StringCbLength(c_cell, (MAXCELLCHARS + 1) * sizeof(wchar_t),
+                              &cb)))
+        return;
+    cb += sizeof(wchar_t);
+
+    for (i=0; i < l->n_rows; i++) {
+        if (!_wcsicmp(l->rows[i].cell, c_cell))
+            break;
+    }
+
+    if (i < l->n_rows)
+        return;
+
+    r = afs_cred_get_new_row(l);
+
+    r->cell = PMALLOC(cb);
+    StringCbCopy(r->cell, cb, c_cell);
+
+    if (h_gcells &&
+        KHM_SUCCEEDED(khc_open_space(h_gcells, c_cell, 0, &h_cell))) {
+
+        cb = sizeof(wmethod);
+        if (KHM_SUCCEEDED(khc_read_string(h_cell, L"MethodName",
+                                          wmethod, &cb))) {
+
+            r->method = afs_get_method_id(wmethod);
+
+            /* remove the deprecated value if it is present. */
+            khc_remove_value(h_cell, L"Method", 0);
+
+        } else if (KHM_SUCCEEDED(khc_read_int32(h_cell, 
+                                                L"Method", &i))) {
+            /* the Method property is deprecated.  We detect and
+               correct this whenever possible. */
+
+            if (!afs_is_valid_method_id(i))
+                i = AFS_TOKEN_AUTO;
+
+            r->method = i;
+
+            afs_get_method_name(i, wmethod, sizeof(wmethod));
+
+            khc_write_string(h_cell, L"MethodName", wmethod);
+
+            khc_remove_value(h_cell, L"Method", 0);
+        } else {
+            r->method = AFS_TOKEN_AUTO;
+        }
+
+        cb = sizeof(wbuf);
+        if (KHM_SUCCEEDED(khc_read_string(h_cell, L"Realm", wbuf, &cb)) &&
+            cb > sizeof(wchar_t)) {
+            r->realm = PMALLOC(cb);
+            StringCbCopy(r->realm, cb, wbuf);
+        } else {
+            r->realm = NULL;
+        }
+
+        khc_close_space(h_cell);
+        h_cell = NULL;
+    } else if (hk_gcells &&
+               RegOpenKeyEx(hk_gcells, c_cell, 0, KEY_READ, &hk_cell) == ERROR_SUCCESS) {
+
+        DWORD cbd;
+
+        cbd = sizeof(wmethod);
+        if (RegQueryValueEx(hk_cell, L"MethodName", NULL, NULL, (LPBYTE) wmethod, &cbd) == ERROR_SUCCESS) {
+            r->method = afs_get_method_id(wmethod);
+        } else {
+            r->method = AFS_TOKEN_AUTO;
+        }
+
+        cbd = sizeof(wbuf);
+        if (RegQueryValueEx(hk_cell, L"Realm", NULL, NULL, (LPBYTE) wbuf, &cbd) == ERROR_SUCCESS) {
+            cbd += sizeof(wchar_t);
+            r->realm = PMALLOC(cbd);
+            ZeroMemory(r->realm, cbd);
+            StringCbCopy(r->realm, cbd, wbuf);
+        } else {
+            r->realm = NULL;
+        }
+
+        RegCloseKey(hk_cell);
+
+    } else {
+        r->realm = NULL;
+        r->method = AFS_TOKEN_AUTO;
+    }
+
+    r->flags = 0;
+}
+
+
 void 
 afs_cred_get_identity_creds(afs_cred_list * l, 
                             khm_handle ident,
@@ -587,70 +704,7 @@ afs_cred_get_identity_creds(afs_cred_list * l,
 
     s = ms;
     for(s = ms; s && *s; s = multi_string_next(s)) {
-        afs_cred_row * r;
-        size_t cb;
-        khm_handle h_cell = NULL;
-
-        /* is this a valid cell name? */
-        if(FAILED(StringCbLength(s, MAXCELLCHARS, &cb)))
-            continue;
-        cb += sizeof(wchar_t);
-
-        r = afs_cred_get_new_row(l);
-
-        r->cell = PMALLOC(cb);
-        StringCbCopy(r->cell, cb, s);
-
-        r->realm = NULL;
-        r->method = 0;
-        r->flags = 0;
-
-        if(KHM_SUCCEEDED(khc_open_space(h_cells, s, 
-                                        0, &h_cell))) {
-            khm_int32 i;
-            wchar_t wname[KHUI_MAXCCH_NAME];
-            khm_size cb;
-
-            if(khc_read_string(h_cell, L"Realm", 
-                               NULL, &cbi) == 
-               KHM_ERROR_TOO_LONG &&
-               cbi > sizeof(wchar_t)) {
-
-                r->realm = PMALLOC(cbi);
-                khc_read_string(h_cell, L"Realm", r->realm, &cbi);
-            }
-
-            i = AFS_TOKEN_AUTO;
-
-            cb = sizeof(wname);
-            if (KHM_SUCCEEDED(khc_read_string(h_cell, L"MethodName",
-                                              wname, &cb))) {
-
-                r->method = afs_get_method_id(wname);
-
-                /* remove the deprecated value if it is present. */
-                khc_remove_value(h_cell, L"Method", 0);
-
-            } else if (KHM_SUCCEEDED(khc_read_int32(h_cell, 
-                                                    L"Method", &i))) {
-                /* the Method property is deprecated.  We detect and
-                   correct this whenever possible. */
-
-                if (!afs_is_valid_method_id(i))
-                    i = AFS_TOKEN_AUTO;
-
-                r->method = i;
-
-                afs_get_method_name(i, wname, sizeof(wname));
-
-                khc_write_string(h_cell, L"MethodName",
-                                 wname);
-
-                khc_remove_value(h_cell, L"Method", 0);
-            }
-
-            khc_close_space(h_cell);
-        }
+        add_token_to_list(l, h_cells, NULL, s);
     }
 
     if(ms) {
@@ -664,55 +718,17 @@ afs_cred_get_identity_creds(afs_cred_list * l,
         /* We want to load defaults */
         char buf[MAXCELLCHARS];
         wchar_t wbuf[MAXCELLCHARS];
-        wchar_t wmethod[KHUI_MAXCCH_NAME];
         wchar_t * defcells;
         khm_size cb_defcells;
-        afs_cred_row * r;
-        khm_size sz;
 
         khc_open_space(csp_params, L"Cells", 0, &h_gcells);
 
-        if(!cm_GetRootCellName(buf)) {
+        if (!cm_GetRootCellName(buf) &&
+            afs_check_for_cell_realm_match(ident, buf)) {
             AnsiStrToUnicode(wbuf, sizeof(wbuf), buf);
 
             if (afs_check_add_token_to_identity(wbuf, ident, NULL)) {
-                khm_handle h_cell = NULL;
-
-                r = afs_cred_get_new_row(l);
-
-                StringCbLength(wbuf, sizeof(wbuf), &sz);
-                sz += sizeof(wchar_t);
-
-                r->cell = PMALLOC(sz);
-                StringCbCopy(r->cell, sz, wbuf);
-
-                if (h_gcells &&
-                    KHM_SUCCEEDED(khc_open_space(h_gcells, wbuf, 0, &h_cell))) {
-                    khm_size cb;
-
-                    cb = sizeof(wmethod);
-                    if (KHM_SUCCEEDED(khc_read_string(h_cell, L"MethodName", wmethod, &cb))) {
-                        r->method = afs_get_method_id(wmethod);
-                    } else {
-                        r->method = AFS_TOKEN_AUTO;
-                    }
-
-                    cb = sizeof(wbuf);
-                    if (KHM_SUCCEEDED(khc_read_string(h_cell, L"Realm", wbuf, &cb))) {
-                        r->realm = PMALLOC(cb);
-                        StringCbCopy(r->realm, cb, wbuf);
-                    } else {
-                        r->realm = NULL;
-                    }
-
-                    khc_close_space(h_cell);
-
-                } else {
-                    r->realm = NULL;
-                    r->method = AFS_TOKEN_AUTO;
-                }
-
-                r->flags = 0;
+                add_token_to_list(l, h_gcells, NULL, wbuf);
             }
         }
 
@@ -734,59 +750,95 @@ afs_cred_get_identity_creds(afs_cred_list * l,
             for (c_cell = defcells;
                  c_cell && *c_cell;
                  c_cell = multi_string_next(c_cell)) {
+                char cell[MAXCELLCHARS];
 
-                khm_size cb;
-                int i;
-                khm_handle h_cell = NULL;
-                afs_cred_row * r;
+                UnicodeStrToAnsi(cell, sizeof(cell), c_cell);
 
-                if (FAILED(StringCbLength(c_cell, (MAXCELLCHARS + 1) * sizeof(wchar_t),
-                                          &cb)))
-                    continue;
-                cb += sizeof(wchar_t);
-
-                for (i=0; i < l->n_rows; i++) {
-                    if (!_wcsicmp(l->rows[i].cell, c_cell))
-                        break;
-                }
-
-                if (i < l->n_rows)
+                if (!afs_check_for_cell_realm_match(ident, cell))
                     continue;
 
-                r = afs_cred_get_new_row(l);
-
-                r->cell = PMALLOC(cb);
-                StringCbCopy(r->cell, cb, c_cell);
-
-                if (h_gcells &&
-                    KHM_SUCCEEDED(khc_open_space(h_gcells, c_cell, 0, &h_cell))) {
-
-                    cb = sizeof(wmethod);
-                    if (KHM_SUCCEEDED(khc_read_string(h_cell, L"MethodName", wmethod, &cb))) {
-                        r->method = afs_get_method_id(wmethod);
-                    } else {
-                        r->method = AFS_TOKEN_AUTO;
-                    }
-
-                    cb = sizeof(wbuf);
-                    if (KHM_SUCCEEDED(khc_read_string(h_cell, L"Realm", wbuf, &cb))) {
-                        r->realm = PMALLOC(cb);
-                        StringCbCopy(r->realm, cb, wbuf);
-                    } else {
-                        r->realm = NULL;
-                    }
-
-                    khc_close_space(h_cell);
-                } else {
-                    r->realm = NULL;
-                    r->method = AFS_TOKEN_AUTO;
+                add_token_to_list(l, h_gcells, NULL, c_cell);
                 }
 
-                r->flags = 0;
-            }
-
             PFREE(defcells);
         }
+
+        /* Check HKLM\Software\OpenAFS\Client\Realms\<Realm> registry key as well */
+        {
+            wchar_t idname[KCDB_IDENT_MAXCCH_NAME];
+            wchar_t * realm;
+            wchar_t * cell=NULL;
+            khm_size cb;
+            HKEY hk_realms = NULL, hk_realm = NULL;
+
+            cb = sizeof(idname);
+            kcdb_identity_get_name(ident, idname, &cb);
+
+            realm = wcsrchr(idname, L'@');
+            if (realm == NULL || realm[1] == L'\0')
+                goto _done_realms;
+
+            realm++;
+
+            if (RegOpenKeyEx(HKEY_LOCAL_MACHINE, L"SOFTWARE\\OpenAFS\\Client\\Realms", 0, KEY_READ, &hk_realms) != ERROR_SUCCESS)
+                goto _done_realms;
+
+            if (RegOpenKeyEx(hk_realms, realm, 0, KEY_READ, &hk_realm) != ERROR_SUCCESS)
+                goto _done_realms;
+
+            if (penabled) {
+                DWORD enabled = 1;
+                DWORD cbd;
+
+                cbd = sizeof(enabled);
+
+                if (RegQueryValueEx(hk_realm, L"AFSEnabled",
+                                    NULL, NULL, (LPBYTE) &enabled, &cbd) == ERROR_SUCCESS) {
+                    *penabled = !!enabled;
+                }
+            }
+
+            {
+                DWORD dwNumCells=0, dwMaxCellLen=0, dwIndex, dwCellBufSz;
+
+                RegQueryInfoKey( hk_realm,
+                                 NULL,  /* lpClass */
+                                 NULL,  /* lpcClass */
+                                 NULL,  /* lpReserved */
+                                 &dwNumCells,  /* lpcSubKeys */
+                                 &dwMaxCellLen,  /* lpcMaxSubKeyLen */
+                                 NULL,  /* lpcMaxClassLen */
+                                 NULL, /* lpcValues */
+                                 NULL,  /* lpcMaxValueNameLen */
+                                 NULL,  /* lpcMaxValueLen */
+                                 NULL,  /* lpcbSecurityDescriptor */
+                                 NULL   /* lpftLastWriteTime */
+                                 );
+
+                dwCellBufSz = (dwMaxCellLen + 1) * sizeof(wchar_t);
+                cell = PMALLOC(dwCellBufSz);
+                ZeroMemory(cell, dwCellBufSz);
+
+                for ( dwIndex=0; dwIndex < dwNumCells; dwIndex++ ) {
+                    if (RegEnumKey( hk_realm, dwIndex, cell, dwCellBufSz) != ERROR_SUCCESS)
+                        goto _done_realms;
+
+                    if (afs_check_add_token_to_identity(cell, ident, NULL))
+                        add_token_to_list(l, NULL, hk_realm, cell);
+                }
+            }
+
+        _done_realms:
+
+            if (hk_realm)
+                RegCloseKey(hk_realm);
+
+            if (hk_realms)
+                RegCloseKey(hk_realms);
+
+            if (cell)
+                PFREE(cell);
+        }
     }
 
  _done_defaults:
@@ -841,6 +893,9 @@ nc_dlg_show_tooltip(HWND hwnd,
 
     d = (afs_dlg_data *)(LONG_PTR) GetWindowLongPtr(hwnd, DWLP_USER);
 
+    if (d == NULL)
+        return;
+
     ZeroMemory(&ti, sizeof(ti));
     ti.cbSize = sizeof(ti);
     ti.hwnd = hwnd;
@@ -879,6 +934,9 @@ nc_dlg_hide_tooltip(HWND hwnd, UINT_PTR id)
 
     d = (afs_dlg_data *)(LONG_PTR) GetWindowLongPtr(hwnd, DWLP_USER);
 
+    if (d == NULL)
+        return;
+
     if(!d->tooltip_visible)
         return;
 
@@ -962,6 +1020,9 @@ nc_dlg_del_token(HWND hwnd) {
 
     d = (afs_dlg_data *)(LONG_PTR) GetWindowLongPtr(hwnd, DWLP_USER);
 
+    if (d == NULL)
+        return;
+
     if (d->nc)
         khui_cw_find_type(d->nc, afs_credtype_id, &nct);
 
@@ -1048,6 +1109,9 @@ nc_dlg_add_token(HWND hwnd) {
 
     d = (afs_dlg_data *)(LONG_PTR) GetWindowLongPtr(hwnd, DWLP_USER);
 
+    if (d == NULL)
+        return;
+
     if (d->nc)
         khui_cw_find_type(d->nc, afs_credtype_id, &nct);
     else
@@ -1499,6 +1563,10 @@ afs_dlg_proc(HWND hwnd,
             SendDlgItemMessage(hwnd, IDC_NCAFS_OBTAIN, 
                                BM_SETCHECK, BST_CHECKED, 0);
 
+            SendDlgItemMessage(hwnd, IDC_NCAFS_CELL, CB_LIMITTEXT, MAXCELLCHARS-1, 0);
+            SendDlgItemMessage(hwnd, IDC_NCAFS_REALM, CB_LIMITTEXT, MAXKTCREALMLEN-1, 0);
+            SendDlgItemMessage(hwnd, IDC_NCAFS_METHOD, CB_LIMITTEXT, KCDB_MAXCCH_NAME-1, 0);
+
             LeaveCriticalSection(&d->cs);
 
             /* the cells and realms combo boxes need to be filled
@@ -1515,6 +1583,9 @@ afs_dlg_proc(HWND hwnd,
             d = (afs_dlg_data *)(LONG_PTR) 
                 GetWindowLongPtr(hwnd, DWLP_USER);
 
+            if (d == NULL)
+                return TRUE;
+
             EnterCriticalSection(&d->cs);
 
             if (d->nc) {
@@ -1529,6 +1600,8 @@ afs_dlg_proc(HWND hwnd,
             DeleteCriticalSection(&d->cs);
 
             PFREE(d);
+
+            SetWindowLongPtr(hwnd, DWLP_USER, 0);
         }
         return TRUE;
 
@@ -1540,6 +1613,9 @@ afs_dlg_proc(HWND hwnd,
             d = (afs_dlg_data *)(LONG_PTR) 
                 GetWindowLongPtr(hwnd, DWLP_USER);
 
+            if (d == NULL)
+                return FALSE;
+
             EnterCriticalSection(&d->cs);
 
             if (d->nc)
@@ -1594,6 +1670,9 @@ afs_dlg_proc(HWND hwnd,
             d = (afs_dlg_data *)(LONG_PTR) 
                 GetWindowLongPtr(hwnd, DWLP_USER);
 
+            if (d == NULL)
+                return TRUE;
+
             EnterCriticalSection(&d->cs);
 
             if (d->nc)
@@ -1647,7 +1726,7 @@ afs_dlg_proc(HWND hwnd,
                             }
                             SendDlgItemMessage(hwnd, IDC_NCAFS_CELL, 
                                                CB_SELECTSTRING, 
-                                               -1, (LPARAM) wbuf);
+                                               (WPARAM)-1, (LPARAM) wbuf);
                         }
                     }
 
@@ -1671,7 +1750,7 @@ afs_dlg_proc(HWND hwnd,
                                            CB_SETITEMDATA, idx, TRUE);
                         SendDlgItemMessage(hwnd, IDC_NCAFS_REALM, 
                                            CB_SELECTSTRING, 
-                                           -1, (LPARAM) wbuf);
+                                           (WPARAM)-1, (LPARAM) wbuf);
                     }
 
                     /* load the LRU realms */
@@ -1690,7 +1769,8 @@ afs_dlg_proc(HWND hwnd,
                             s = buf;
                             while(*s) {
                                 if(SendDlgItemMessage(hwnd, IDC_NCAFS_REALM, 
-                                                      CB_FINDSTRINGEXACT, -1, 
+                                                      CB_FINDSTRINGEXACT, 
+                                                      (WPARAM)-1,
                                                       (LPARAM) s) == CB_ERR) {
                                     idx = 
                                         (int)
@@ -1874,6 +1954,9 @@ afs_dlg_proc(HWND hwnd,
                 d = (afs_dlg_data *)(LONG_PTR) 
                     GetWindowLongPtr(hwnd, DWLP_USER);
 
+                if (d == NULL)
+                    return FALSE;
+
                 EnterCriticalSection(&d->cs);
 
                 hw = GetDlgItem(hwnd, IDC_NCAFS_TOKENLIST);
@@ -1905,7 +1988,8 @@ afs_dlg_proc(HWND hwnd,
                     LoadString(hResModule, IDS_NC_REALM_AUTO, wbuf, 
                                ARRAYLENGTH(wbuf));
                     idx = (int) SendDlgItemMessage(hwnd, IDC_NCAFS_REALM, 
-                                                   CB_FINDSTRINGEXACT, -1, 
+                                                   CB_FINDSTRINGEXACT, 
+                                                   (WPARAM) -1,
                                                    (LPARAM) wbuf);
                     SendDlgItemMessage(hwnd, IDC_NCAFS_REALM, CB_SETCURSEL,
                                        idx, 0);
@@ -1928,6 +2012,9 @@ afs_dlg_proc(HWND hwnd,
                 d = (afs_dlg_data *)(LONG_PTR) 
                     GetWindowLongPtr(hwnd, DWLP_USER);
 
+                if (d == NULL)
+                    return FALSE;
+
                 EnterCriticalSection(&d->cs);
 
                 pnmi = (LPNMITEMACTIVATE) lpnmh;
@@ -2059,13 +2146,13 @@ afs_adjust_token_ident_proc(khm_handle cred, void * vd)
             kcdb_cred_set_identity(cred, b->ident);
             if(l->rows[i].realm)
                 kcdb_cred_set_attr(cred, afs_attr_realm, l->rows[i].realm, 
-                                   KCDB_CBSIZE_AUTO);
+                                   (khm_size)KCDB_CBSIZE_AUTO);
             else
                 kcdb_cred_set_attr(cred, afs_attr_realm, NULL, 0);
 
             method = l->rows[i].method;
             kcdb_cred_set_attr(cred, afs_attr_method, &method, 
-                               KCDB_CBSIZE_AUTO);
+                               (khm_size)KCDB_CBSIZE_AUTO);
 
             break;
         }
@@ -2147,13 +2234,13 @@ afs_cred_write_ident_data(afs_dlg_data * d) {
                               lru_cell,
                               &cbt);
     } else {
-        cbcell = MAXCELLCHARS * sizeof(wchar_t) * 
-            l->n_rows;
-        if (cbcell > 0) {
+        cbcell = MAXCELLCHARS * sizeof(wchar_t) * l->n_rows + sizeof(wchar_t);
+        if (l->n_rows > 0) {
             lru_cell = PMALLOC(cbcell);
             ZeroMemory(lru_cell, cbcell);
         } else {
             lru_cell = NULL;
+            cbcell = 0;
         }
     }
 
@@ -2171,21 +2258,23 @@ afs_cred_write_ident_data(afs_dlg_data * d) {
                               lru_realm,
                               &cbt);
     } else {
-        cbrealm = MAXCELLCHARS * sizeof(wchar_t) * l->n_rows;
-        if (cbrealm > 0) {
+        cbrealm = MAXCELLCHARS * sizeof(wchar_t) * l->n_rows + sizeof(wchar_t);
+        if (l->n_rows > 0) {
             lru_realm = PMALLOC(cbrealm);
             ZeroMemory(lru_realm, cbrealm);
         } else {
             lru_cell = NULL;
+            cbrealm = 0;
         }
     }
 
-    cbidcell = MAXCELLCHARS * sizeof(wchar_t) * l->n_rows;
-    if (cbidcell > 0) {
+    cbidcell = MAXCELLCHARS * sizeof(wchar_t) * l->n_rows + sizeof(wchar_t);
+    if (l->n_rows > 0) {
         id_cell = PMALLOC(cbidcell);
         ZeroMemory(id_cell, cbidcell);
     } else {
         id_cell = NULL;
+        cbidcell = 0;
     }
 
     for(i=0; i < l->n_rows; i++)
@@ -2233,10 +2322,12 @@ afs_cred_write_ident_data(afs_dlg_data * d) {
                 khc_close_space(h_acell);
             }
 
-            if (h_cellmap) {
-                khc_write_string(h_cellmap,
-                                 l->rows[i].cell,
-                                 idname);
+            if (l->rows[i].flags & DLGROW_FLAG_DONE) {
+                if (h_cellmap) {
+                    khc_write_string(h_cellmap,
+                                     l->rows[i].cell,
+                                     idname);
+                }
             }
         }
 
@@ -2249,6 +2340,8 @@ afs_cred_write_ident_data(afs_dlg_data * d) {
     if (id_cell)
         khc_write_multi_string(h_afs, L"Cells",
                                id_cell);
+    else
+        khc_write_multi_string(h_afs, L"Cells", L"\0");
 
     if (d->config_dlg) {
         if (d->dirty)
@@ -2565,7 +2658,8 @@ afs_msg_newcred(khm_int32 msg_subtype,
             for(i=0; i<l->n_rows; i++) {
                 int code;
                 char cell[MAXCELLCHARS];
-                char realm[MAXCELLCHARS];
+                char realm[MAXKTCREALMLEN];
+                char linkedCell[MAXCELLCHARS]="";
                 khm_handle ctoken;
                 FILETIME ft_old;
                 FILETIME ft_new;
@@ -2573,6 +2667,9 @@ afs_msg_newcred(khm_int32 msg_subtype,
                 khm_size cb;
                 khm_int32 method = AFS_TOKEN_AUTO;
                 khm_handle csp_cell = NULL;
+                BOOL bgetLinked = 0;
+
+                _progress(i, l->n_rows);
 
                 if (l->rows[i].flags &
                     (DLGROW_FLAG_DONE | DLGROW_FLAG_DELETED))
@@ -2620,16 +2717,20 @@ afs_msg_newcred(khm_int32 msg_subtype,
                     method = l->rows[i].method;
                 }
 
+              getLinked:
                 _report_cs3(KHERR_INFO,
                             L"Getting tokens for cell %1!S! with realm %2!S! using method %3!d!",
-                            _cstr(cell),
+                            _cstr((bgetLinked ? linkedCell: cell)),
                             _cstr(realm),
                             _int32(method));
                 _resolve();
 
                 /* make the call */
-                code = afs_klog(ident, "", cell, realm, 0, 
-                                method, &new_exp);
+                code = afs_klog(ident, "", 
+                                bgetLinked ? linkedCell : cell, 
+                                realm, 0,
+                                method, &new_exp, 
+                                bgetLinked ? NULL :linkedCell);
 
                 _report_cs1(KHERR_INFO,
                             L"klog returns code %1!d!",
@@ -2680,9 +2781,16 @@ afs_msg_newcred(khm_int32 msg_subtype,
                             ident_renew_triggered = TRUE;
                         }
                     }
+
+                    if ( !bgetLinked && linkedCell[0] ) {
+                        bgetLinked = TRUE;
+                        goto getLinked;
+                    }
                 }
             }
 
+            _progress(1,1);
+
         _skip_tokens:
 
             if(failed) {
