@@ -19,12 +19,16 @@ extern afs_rwlock_t afs_xosi;
 
 /* osi_misc.c */
 extern int osi_lookupname(char *aname, enum uio_seg seg, int followlink,
-			  struct vnode **dirvpp, struct vnode **vpp);
+			  struct vnode **vpp);
 extern void *osi_fbsd_alloc(size_t size, int dropglobal);
 extern void osi_fbsd_free(void *p);
 
 /* osi_vfsops.c */
-#ifdef AFS_FBSD50_ENV
+int afs_init(struct vfsconf *vfc);
+int afs_uninit(struct vfsconf *vfc);
+#if defined(AFS_FBSD80_ENV)
+extern int afs_statfs(struct mount *mp, struct statfs *abp);
+#elif defined(AFS_FBSD50_ENV)
 extern int afs_statfs(struct mount *mp, struct statfs *abp, struct thread *th);
 #else
 extern int afs_statfs(struct mount *mp, struct statfs *abp, struct proc *p);
