@@ -65,7 +65,6 @@ afs_CopyOutAttrs(register struct vcache *avc, register struct vattr *attrs)
     }
 #if defined(AFS_DARWIN_ENV)
     {
-	extern u_int32_t afs_darwin_realmodes;
 	if (!afs_darwin_realmodes) {
 	    /* Mac OS X uses the mode bits to determine whether a file or
 	     * directory is accessible, and believes them, even though under
@@ -92,9 +91,9 @@ afs_CopyOutAttrs(register struct vcache *avc, register struct vattr *attrs)
     attrs->va_fsid = avc->v.v_vfsp->vfs_fsid.val[0];
 #elif defined(AFS_DARWIN80_ENV)
     VATTR_RETURN(attrs, va_fsid, vfs_statfs(vnode_mount(AFSTOV(avc)))->f_fsid.val[0]);
-#elif defined(AFS_DARWIN70_ENV)
+#elif defined(AFS_DARWIN_ENV)
     attrs->va_fsid = avc->v->v_mount->mnt_stat.f_fsid.val[0];
-#else /* ! AFS_DARWIN70_ENV */
+#else /* ! AFS_DARWIN_ENV */
     attrs->va_fsid = 1;
 #endif 
     if (avc->mvstat == 2) {
@@ -189,6 +188,7 @@ afs_getattr(OSI_VC_DECL(avc), struct vattr *attrs, afs_ucred_t *acred)
     afs_int32 code;
     struct vrequest treq;
     struct unixuser *au;
+    int inited = 0;
     OSI_VC_CONVERT(avc);
 
     AFS_STATCNT(afs_getattr);
@@ -212,23 +212,15 @@ afs_getattr(OSI_VC_DECL(avc), struct vattr *attrs, afs_ucred_t *acred)
 	afs_PutFakeStat(&fakestat);
 	return code;
     }
-
-#if defined(AFS_SUN5_ENV) || defined(AFS_DARWIN_ENV) && !defined(AFS_DARWIN80_ENV)
 #if defined(AFS_SUN5_ENV)
-	if (flags & ATTR_HINT)
-#else
-	if (avc->f.states & CUBCinit)
+    if (flags & ATTR_HINT) {
+	code = afs_CopyOutAttrs(avc, attrs);
+	return code;
+    }
 #endif
-    {
-	if (!(code = afs_InitReq(&treq, acred))) {
-	    if (vType(avc) != VDIR && vType(avc) != VLNK
-	        !afs_AccessOK(avc, PRSFS_READ, &treq, DONT_CHECK_MODE_BITS)) {
-
-		code = EACCES;
-	    }
-	    if (!code)
-		code = afs_CopyOutAttrs(avc, attrs);
-	}
+#if defined(AFS_DARWIN_ENV) && !defined(AFS_DARWIN80_ENV)
+    if (avc->f.states & CUBCinit) {
+	code = afs_CopyOutAttrs(avc, attrs);
 	return code;
     }
 #endif
@@ -242,11 +234,13 @@ afs_getattr(OSI_VC_DECL(avc), struct vattr *attrs, afs_ucred_t *acred)
     if (afs_shuttingdown)
 	return EIO;
 
-    code = afs_InitReq(&treq, acred);
-
-    if (code == 0 && !(avc->f.states & CStatd)) {
-	code = afs_VerifyVCache2(avc, &treq);
-    }
+    if (!(avc->f.states & CStatd)) {
+	if (!(code = afs_InitReq(&treq, acred))) {
+	    code = afs_VerifyVCache2(avc, &treq);
+	    inited = 1;
+	}
+    } else
+	code = 0;
 
 #ifdef AFS_BOZONLOCK_ENV
     if (code == 0)
@@ -254,17 +248,17 @@ afs_getattr(OSI_VC_DECL(avc), struct vattr *attrs, afs_ucred_t *acred)
     afs_BozonUnlock(&avc->pvnLock, avc);
 #endif
 
-    if (code == 0 && vType(avc) != VDIR && vType(avc) != VLNK &&
-        !afs_AccessOK(avc, PRSFS_READ, &treq, DONT_CHECK_MODE_BITS)) {
-
-	code = EACCES;
-    }
 
     if (code == 0) {
 	osi_FlushText(avc);	/* only needed to flush text if text locked last time */
 	code = afs_CopyOutAttrs(avc, attrs);
 
 	if (afs_nfsexporter) {
+	    if (!inited) {
+		if ((code = afs_InitReq(&treq, acred)))
+		    return code;
+		inited = 1;
+	    }
 	    if (AFS_NFSXLATORREQ(acred)) {
 		if ((vType(avc) != VDIR)
 		    && !afs_AccessOK(avc, PRSFS_READ, &treq,
