@@ -16,8 +16,6 @@
 #include <afsconfig.h>
 #include "afs/param.h"
 
-RCSID
-    ("$Header: /cvs/openafs/src/afs/VNOPS/afs_vnop_create.c,v 1.23.4.3 2007/12/08 17:59:06 shadow Exp $");
 
 #include "afs/sysincludes.h"	/* Standard vendor system headers */
 #include "afsincludes.h"	/* Afs-based standard headers */
@@ -34,18 +32,18 @@ RCSID
 #ifdef AFS_SGI64_ENV
 int
 afs_create(OSI_VC_DECL(adp), char *aname, struct vattr *attrs, int flags,
-	   int amode, struct vcache **avcp, struct AFS_UCRED *acred)
+	   int amode, struct vcache **avcp, afs_ucred_t *acred)
 #else /* AFS_SGI64_ENV */
 int
 afs_create(OSI_VC_DECL(adp), char *aname, struct vattr *attrs,
 	   enum vcexcl aexcl, int amode, struct vcache **avcp,
-	   struct AFS_UCRED *acred)
+	   afs_ucred_t *acred)
 #endif				/* AFS_SGI64_ENV */
 {
     afs_int32 origCBs, origZaps, finalZaps;
     struct vrequest treq;
     register afs_int32 code;
-    register struct conn *tc;
+    register struct afs_conn *tc;
     struct VenusFid newFid;
     struct AFSStoreStatus InStatus;
     struct AFSFetchStatus OutFidStatus, OutDirStatus;
@@ -84,12 +82,12 @@ afs_create(OSI_VC_DECL(adp), char *aname, struct vattr *attrs,
 
     if (strlen(aname) > AFSNAMEMAX) {
 	code = ENAMETOOLONG;
-	goto done;
+	goto done3;
     }
 
     if (!afs_ENameOK(aname)) {
 	code = EINVAL;
-	goto done;
+	goto done3;
     }
     switch (attrs->va_type) {
     case VBLK:
@@ -100,10 +98,12 @@ afs_create(OSI_VC_DECL(adp), char *aname, struct vattr *attrs,
     case VFIFO:
 	/* We don't support special devices or FIFOs */
 	code = EINVAL;
-	goto done;
+	goto done3;
     default:
 	;
     }
+    AFS_DISCON_LOCK();
+
     code = afs_EvalFakeStat(&adp, &fakestate, &treq);
     if (code)
 	goto done;
@@ -115,11 +115,16 @@ afs_create(OSI_VC_DECL(adp), char *aname, struct vattr *attrs,
     /** If the volume is read-only, return error without making an RPC to the
       * fileserver
       */
-    if (adp->states & CRO) {
+    if (adp->f.states & CRO) {
 	code = EROFS;
 	goto done;
     }
 
+    if (AFS_IS_DISCONNECTED && !AFS_IS_DISCON_RW) {
+        code = ENETDOWN;
+        goto done;
+    }
+
     tdc = afs_GetDCache(adp, (afs_size_t) 0, &treq, &offset, &len, 1);
     ObtainWriteLock(&adp->lock, 135);
     if (tdc)
@@ -129,8 +134,8 @@ afs_create(OSI_VC_DECL(adp), char *aname, struct vattr *attrs,
      * Make sure that the data in the cache is current. We may have
      * received a callback while we were waiting for the write lock.
      */
-    if (!(adp->states & CStatd)
-	|| (tdc && !hsame(adp->m.DataVersion, tdc->f.versionNo))) {
+    if (!(adp->f.states & CStatd)
+	|| (tdc && !hsame(adp->f.m.DataVersion, tdc->f.versionNo))) {
 	ReleaseWriteLock(&adp->lock);
 	if (tdc) {
 	    ReleaseSharedLock(&tdc->lock);
@@ -155,8 +160,8 @@ afs_create(OSI_VC_DECL(adp), char *aname, struct vattr *attrs,
 		goto done;
 	    }
 	    /* found the file, so use it */
-	    newFid.Cell = adp->fid.Cell;
-	    newFid.Fid.Volume = adp->fid.Fid.Volume;
+	    newFid.Cell = adp->f.fid.Cell;
+	    newFid.Fid.Volume = adp->f.fid.Fid.Volume;
 	    tvc = NULL;
 	    if (newFid.Fid.Unique == 0) {
 		tvc = afs_LookupVCache(&newFid, &treq, NULL, adp, aname);
@@ -189,8 +194,8 @@ afs_create(OSI_VC_DECL(adp), char *aname, struct vattr *attrs,
 #endif
 		{
 		    /* needed for write access check */
-		    tvc->parentVnode = adp->fid.Fid.Vnode;
-		    tvc->parentUnique = adp->fid.Fid.Unique;
+		    tvc->f.parent.vnode = adp->f.fid.Fid.Vnode;
+		    tvc->f.parent.unique = adp->f.fid.Fid.Unique;
 		    /* need write mode for these guys */
 		    if (!afs_AccessOK
 			(tvc, PRSFS_WRITE, &treq, CHECK_MODE_BITS)) {
@@ -217,6 +222,8 @@ afs_create(OSI_VC_DECL(adp), char *aname, struct vattr *attrs,
 		    VATTR_INIT(attrs);
 		    VATTR_SET_SUPPORTED(attrs, va_data_size);
 		    VATTR_SET_ACTIVE(attrs, va_data_size);
+#elif defined(UKERNEL)
+		    attrs->va_mask = ATTR_SIZE;
 #elif defined(AFS_SUN5_ENV) || defined(AFS_SGI_ENV)
 		    attrs->va_mask = AT_SIZE;
 #else
@@ -224,7 +231,7 @@ afs_create(OSI_VC_DECL(adp), char *aname, struct vattr *attrs,
 #endif
 		    attrs->va_size = len;
 		    ObtainWriteLock(&tvc->lock, 136);
-		    tvc->states |= CCreating;
+		    tvc->f.states |= CCreating;
 		    ReleaseWriteLock(&tvc->lock);
 #if defined(AFS_SUN5_ENV) || defined(AFS_SGI_ENV)
 #if defined(AFS_SGI64_ENV)
@@ -238,7 +245,7 @@ afs_create(OSI_VC_DECL(adp), char *aname, struct vattr *attrs,
 		    code = afs_setattr(tvc, attrs, acred);
 #endif /* SUN5 || SGI */
 		    ObtainWriteLock(&tvc->lock, 137);
-		    tvc->states &= ~CCreating;
+		    tvc->f.states &= ~CCreating;
 		    ReleaseWriteLock(&tvc->lock);
 		    if (code) {
 			afs_PutVCache(tvc);
@@ -252,10 +259,10 @@ afs_create(OSI_VC_DECL(adp), char *aname, struct vattr *attrs,
 	    goto done;
 	}
     }
-
+    
     /* if we create the file, we don't do any access checks, since
      * that's how O_CREAT is supposed to work */
-    if (adp->states & CForeign) {
+    if (adp->f.states & CForeign) {
 	origCBs = afs_allCBs;
 	origZaps = afs_allZaps;
     } else {
@@ -264,7 +271,7 @@ afs_create(OSI_VC_DECL(adp), char *aname, struct vattr *attrs,
     }
     InStatus.Mask = AFS_SETMODTIME | AFS_SETMODE | AFS_SETGROUP;
     InStatus.ClientModTime = osi_Time();
-    InStatus.Group = (afs_int32) acred->cr_gid;
+    InStatus.Group = (afs_int32) afs_cr_gid(acred);
     if (AFS_NFSXLATORREQ(acred)) {
 	/*
 	 * XXX The following is mainly used to fix a bug in the HP-UX
@@ -284,74 +291,89 @@ afs_create(OSI_VC_DECL(adp), char *aname, struct vattr *attrs,
 		attrs->va_mode = 0x1b6;	/* XXX default mode: rw-rw-rw XXX */
 	}
     }
-    InStatus.UnixModeBits = attrs->va_mode & 0xffff;	/* only care about protection bits */
-    do {
-	tc = afs_Conn(&adp->fid, &treq, SHARED_LOCK);
-	if (tc) {
-	    hostp = tc->srvr->server;	/* remember for callback processing */
-	    now = osi_Time();
-	    XSTATS_START_TIME(AFS_STATS_FS_RPCIDX_CREATEFILE);
-	    RX_AFS_GUNLOCK();
-	    code =
-		RXAFS_CreateFile(tc->id, (struct AFSFid *)&adp->fid.Fid,
+
+    if (!AFS_IS_DISCONNECTED) {
+	/* If not disconnected, connect to the server.*/
+
+    	InStatus.UnixModeBits = attrs->va_mode & 0xffff;	/* only care about protection bits */
+    	do {
+	    tc = afs_Conn(&adp->f.fid, &treq, SHARED_LOCK);
+	    if (tc) {
+	    	hostp = tc->srvr->server;	/* remember for callback processing */
+	    	now = osi_Time();
+	    	XSTATS_START_TIME(AFS_STATS_FS_RPCIDX_CREATEFILE);
+	    	RX_AFS_GUNLOCK();
+	    	code =
+		    RXAFS_CreateFile(tc->id, (struct AFSFid *)&adp->f.fid.Fid,
 				 aname, &InStatus, (struct AFSFid *)
 				 &newFid.Fid, &OutFidStatus, &OutDirStatus,
 				 &CallBack, &tsync);
-	    RX_AFS_GLOCK();
-	    XSTATS_END_TIME;
-	    CallBack.ExpirationTime += now;
-	} else
-	    code = -1;
-    } while (afs_Analyze
-	     (tc, code, &adp->fid, &treq, AFS_STATS_FS_RPCIDX_CREATEFILE,
-	      SHARED_LOCK, NULL));
+	    	RX_AFS_GLOCK();
+	    	XSTATS_END_TIME;
+	    	CallBack.ExpirationTime += now;
+	    } else
+	    	code = -1;
+    	} while (afs_Analyze
+	         (tc, code, &adp->f.fid, &treq, AFS_STATS_FS_RPCIDX_CREATEFILE,
+	          SHARED_LOCK, NULL));
 
-    if ((code == EEXIST || code == UAEEXIST) &&
+	if ((code == EEXIST || code == UAEEXIST) &&
 #ifdef AFS_SGI64_ENV
-    !(flags & VEXCL)
+    	!(flags & VEXCL)
 #else /* AFS_SGI64_ENV */
-    aexcl == NONEXCL
+    	aexcl == NONEXCL
 #endif
-    ) {
-	/* if we get an EEXIST in nonexcl mode, just do a lookup */
-	if (tdc) {
-	    ReleaseSharedLock(&tdc->lock);
-	    afs_PutDCache(tdc);
-	}
-	ReleaseWriteLock(&adp->lock);
-#if !(defined(AFS_OSF_ENV) || defined(AFS_DARWIN_ENV))
-#if defined(AFS_SUN5_ENV) || defined(AFS_SGI_ENV)
+    	) {
+	    /* if we get an EEXIST in nonexcl mode, just do a lookup */
+	    if (tdc) {
+	    	ReleaseSharedLock(&tdc->lock);
+	    	afs_PutDCache(tdc);
+	    }
+	    ReleaseWriteLock(&adp->lock);
+
+
 #if defined(AFS_SGI64_ENV)
-	code = afs_lookup(VNODE_TO_FIRST_BHV((vnode_t *) adp), aname, avcp, 
-			  NULL, 0, NULL, acred);
-#else
-	code = afs_lookup(adp, aname, avcp, NULL, 0, NULL, acred);
-#endif /* AFS_SGI64_ENV */
-#else /* SUN5 || SGI */
-	code = afs_lookup(adp, aname, avcp, acred);
-#endif /* SUN5 || SGI */
-#endif /* !(AFS_OSF_ENV || AFS_DARWIN_ENV) */
+	    code = afs_lookup(VNODE_TO_FIRST_BHV((vnode_t *) adp), aname, avcp,
+				  NULL, 0, NULL, acred);
+#elif defined(AFS_SUN5_ENV) || defined(AFS_SGI_ENV)
+	    code = afs_lookup(adp, aname, avcp, NULL, 0, NULL, acred);
+#elif defined(UKERNEL)
+	    code = afs_lookup(adp, aname, avcp, acred, 0);
+#elif !defined(AFS_DARWIN_ENV)
+	    code = afs_lookup(adp, aname, avcp, acred);
+#endif
 	goto done;
-    }
-    if (code) {
-	if (code < 0) {
-	    ObtainWriteLock(&afs_xcbhash, 488);
-	    afs_DequeueCallback(adp);
-	    adp->states &= ~CStatd;
-	    ReleaseWriteLock(&afs_xcbhash);
-	    osi_dnlc_purgedp(adp);
-	}
-	ReleaseWriteLock(&adp->lock);
-	if (tdc) {
-	    ReleaseSharedLock(&tdc->lock);
-	    afs_PutDCache(tdc);
-	}
+        }
+
+	if (code) {
+	    if (code < 0) {
+	    	ObtainWriteLock(&afs_xcbhash, 488);
+	    	afs_DequeueCallback(adp);
+	    	adp->f.states &= ~CStatd;
+	    	ReleaseWriteLock(&afs_xcbhash);
+	    	osi_dnlc_purgedp(adp);
+	    }
+	    ReleaseWriteLock(&adp->lock);
+	    if (tdc) {
+	    	ReleaseSharedLock(&tdc->lock);
+	    	afs_PutDCache(tdc);
+	    }
 	goto done;
-    }
+	}
+
+    } else {
+#if defined(AFS_DISCON_ENV)
+	/* Generate a fake FID for disconnected mode. */
+	newFid.Cell = adp->f.fid.Cell;
+	newFid.Fid.Volume = adp->f.fid.Fid.Volume;
+	afs_GenFakeFid(&newFid, VREG, 1);
+#endif
+    }				/* if (!AFS_IS_DISCON_RW) */
+
     /* otherwise, we should see if we can make the change to the dir locally */
     if (tdc)
 	UpgradeSToWLock(&tdc->lock, 631);
-    if (afs_LocalHero(adp, tdc, &OutDirStatus, 1)) {
+    if (AFS_IS_DISCON_RW || afs_LocalHero(adp, tdc, &OutDirStatus, 1)) {
 	/* we can do it locally */
 	ObtainWriteLock(&afs_xdcache, 291);
 	code = afs_dir_Create(tdc, aname, &newFid.Fid);
@@ -365,8 +387,11 @@ afs_create(OSI_VC_DECL(adp), char *aname, struct vattr *attrs,
 	ReleaseWriteLock(&tdc->lock);
 	afs_PutDCache(tdc);
     }
-    newFid.Cell = adp->fid.Cell;
-    newFid.Fid.Volume = adp->fid.Fid.Volume;
+    if (AFS_IS_DISCON_RW)
+	adp->f.m.LinkCount++;
+
+    newFid.Cell = adp->f.fid.Cell;
+    newFid.Fid.Volume = adp->f.fid.Fid.Volume;
     ReleaseWriteLock(&adp->lock);
     volp = afs_FindVolume(&newFid, READ_LOCK);
 
@@ -388,7 +413,7 @@ afs_create(OSI_VC_DECL(adp), char *aname, struct vattr *attrs,
      * would fail, since no call would be able to update the local vnode status after modifying
      * a file on a file server. */
     ObtainWriteLock(&afs_xvcache, 138);
-    if (adp->states & CForeign)
+    if (adp->f.states & CForeign)
 	finalZaps = afs_allZaps;	/* do this before calling newvcache */
     else
 	finalZaps = afs_evenZaps;	/* do this before calling newvcache */
@@ -407,26 +432,36 @@ afs_create(OSI_VC_DECL(adp), char *aname, struct vattr *attrs,
 	    ObtainWriteLock(&afs_xcbhash, 489);
 	    finalCBs = afs_evenCBs;
 	    /* add the callback in */
-	    if (adp->states & CForeign) {
-		tvc->states |= CForeign;
+	    if (adp->f.states & CForeign) {
+		tvc->f.states |= CForeign;
 		finalCBs = afs_allCBs;
 	    }
 	    if (origCBs == finalCBs && origZaps == finalZaps) {
-		tvc->states |= CStatd;	/* we've fake entire thing, so don't stat */
-		tvc->states &= ~CBulkFetching;
-		tvc->cbExpires = CallBack.ExpirationTime;
-		afs_QueueCallback(tvc, CBHash(CallBack.ExpirationTime), volp);
+		tvc->f.states |= CStatd;	/* we've fake entire thing, so don't stat */
+		tvc->f.states &= ~CBulkFetching;
+		if (!AFS_IS_DISCON_RW) {
+		    tvc->cbExpires = CallBack.ExpirationTime;
+		    afs_QueueCallback(tvc, CBHash(CallBack.ExpirationTime), volp);
+		}
 	    } else {
 		afs_DequeueCallback(tvc);
-		tvc->states &= ~(CStatd | CUnique);
+		tvc->f.states &= ~(CStatd | CUnique);
 		tvc->callback = 0;
-		if (tvc->fid.Fid.Vnode & 1 || (vType(tvc) == VDIR))
+		if (tvc->f.fid.Fid.Vnode & 1 || (vType(tvc) == VDIR))
 		    osi_dnlc_purgedp(tvc);
 	    }
 	    ReleaseWriteLock(&afs_xcbhash);
-	    afs_ProcessFS(tvc, &OutFidStatus, &treq);
-	    tvc->parentVnode = adp->fid.Fid.Vnode;
-	    tvc->parentUnique = adp->fid.Fid.Unique;
+	    if (AFS_IS_DISCON_RW) {
+#if defined(AFS_DISCON_ENV)
+		afs_DisconAddDirty(tvc, VDisconCreate, 0);
+		afs_GenDisconStatus(adp, tvc, &newFid, attrs, &treq, VREG);
+#endif
+	    } else {
+		afs_ProcessFS(tvc, &OutFidStatus, &treq);
+	    }
+
+	    tvc->f.parent.vnode = adp->f.fid.Fid.Vnode;
+	    tvc->f.parent.unique = adp->f.fid.Fid.Unique;
 	    ReleaseWriteLock(&tvc->lock);
 	    *avcp = tvc;
 	    code = 0;
@@ -447,6 +482,9 @@ afs_create(OSI_VC_DECL(adp), char *aname, struct vattr *attrs,
     ReleaseWriteLock(&afs_xvcache);
 
   done:
+    AFS_DISCON_UNLOCK();
+
+  done3:
     if (volp)
 	afs_PutVolume(volp, READ_LOCK);
 
@@ -455,10 +493,6 @@ afs_create(OSI_VC_DECL(adp), char *aname, struct vattr *attrs,
 	/* return the new status in vattr */
 	afs_CopyOutAttrs(*avcp, attrs);
     }
-#ifdef	AFS_OSF_ENV
-    if (!code && !strcmp(aname, "core"))
-	tvc->states |= CCore1;
-#endif
 
     afs_PutFakeStat(&fakestate);
     code = afs_CheckCode(code, &treq, 20);
@@ -489,8 +523,8 @@ afs_LocalHero(register struct vcache *avc, register struct dcache *adc,
     hset64(avers, astat->dataVersionHigh, astat->DataVersion);
     /* this *is* the version number, no matter what */
     if (adc) {
-	ok = (hsame(avc->m.DataVersion, adc->f.versionNo) && avc->callback
-	      && (avc->states & CStatd) && avc->cbExpires >= osi_Time());
+	ok = (hsame(avc->f.m.DataVersion, adc->f.versionNo) && avc->callback
+	      && (avc->f.states & CStatd) && avc->cbExpires >= osi_Time());
     } else {
 	ok = 0;
     }
@@ -499,14 +533,14 @@ afs_LocalHero(register struct vcache *avc, register struct dcache *adc,
 #endif
     /* The bulk status code used the length as a sequence number.  */
     /* Don't update the vcache entry unless the stats are current. */
-    if (avc->states & CStatd) {
-	hset(avc->m.DataVersion, avers);
+    if (avc->f.states & CStatd) {
+	hset(avc->f.m.DataVersion, avers);
 #ifdef AFS_64BIT_CLIENT
-	FillInt64(avc->m.Length, astat->Length_hi, astat->Length);
+	FillInt64(avc->f.m.Length, astat->Length_hi, astat->Length);
 #else /* AFS_64BIT_CLIENT */
-	avc->m.Length = astat->Length;
+	avc->f.m.Length = astat->Length;
 #endif /* AFS_64BIT_CLIENT */
-	avc->m.Date = astat->ClientModTime;
+	avc->f.m.Date = astat->ClientModTime;
     }
     if (ok) {
 	/* we've been tracking things correctly */
@@ -518,7 +552,7 @@ afs_LocalHero(register struct vcache *avc, register struct dcache *adc,
 	    ZapDCE(adc);
 	    DZap(adc);
 	}
-	if (avc->states & CStatd) {
+	if (avc->f.states & CStatd) {
 	    osi_dnlc_purgedp(avc);
 	}
 	return 0;
