@@ -20,8 +20,6 @@
 #include <afsconfig.h>
 #include "afs/param.h"
 
-RCSID
-    ("$Header: /cvs/openafs/src/afs/VNOPS/afs_vnop_dirops.c,v 1.21.4.3 2008/09/22 19:29:55 shadow Exp $");
 
 #include "afs/sysincludes.h"	/* Standard vendor system headers */
 #include "afsincludes.h"	/* Afs-based standard headers */
@@ -37,14 +35,16 @@ extern afs_rwlock_t afs_xcbhash;
 
 int
 afs_mkdir(OSI_VC_DECL(adp), char *aname, struct vattr *attrs, 
-     register struct vcache **avcp, struct AFS_UCRED *acred)
+     register struct vcache **avcp, afs_ucred_t *acred)
 {
     struct vrequest treq;
     register afs_int32 code;
-    register struct conn *tc;
+    register struct afs_conn *tc;
     struct VenusFid newFid;
     register struct dcache *tdc;
+#ifdef AFS_DISCON_ENV
     struct dcache *new_dc;
+#endif
     afs_size_t offset, len;
     register struct vcache *tvc;
     struct AFSStoreStatus InStatus;
@@ -86,32 +86,33 @@ afs_mkdir(OSI_VC_DECL(adp), char *aname, struct vattr *attrs,
     /** If the volume is read-only, return error without making an RPC to the
       * fileserver
       */
-    if (adp->states & CRO) {
+    if (adp->f.states & CRO) {
 	code = EROFS;
 	goto done;
     }
    
-    if (AFS_IS_DISCONNECTED && !AFS_IS_DISCON_RW)
+    if (AFS_IS_DISCONNECTED && !AFS_IS_DISCON_RW) {
 	/*printf("Network is down in afs_mkdir\n");*/
 	code = ENETDOWN;
+	goto done;
+    }
     InStatus.Mask = AFS_SETMODTIME | AFS_SETMODE | AFS_SETGROUP;
     InStatus.ClientModTime = osi_Time();
     InStatus.UnixModeBits = attrs->va_mode & 0xffff;	/* only care about protection bits */
-    InStatus.Group = (afs_int32) acred->cr_gid;
+    InStatus.Group = (afs_int32) afs_cr_gid(acred);
     tdc = afs_GetDCache(adp, (afs_size_t) 0, &treq, &offset, &len, 1);
     ObtainWriteLock(&adp->lock, 153);
 
     if (!AFS_IS_DISCON_RW) {
-
     	do {
-	    tc = afs_Conn(&adp->fid, &treq, SHARED_LOCK);
+	    tc = afs_Conn(&adp->f.fid, &treq, SHARED_LOCK);
 	    if (tc) {
 	    	XSTATS_START_TIME(AFS_STATS_FS_RPCIDX_MAKEDIR);
 	    	now = osi_Time();
 	    	RX_AFS_GUNLOCK();
 	    	code =
 		    RXAFS_MakeDir(tc->id,
-		    		(struct AFSFid *)&adp->fid.Fid,
+		    		(struct AFSFid *)&adp->f.fid.Fid,
 				aname,
 				&InStatus,
 				(struct AFSFid *)&newFid.Fid,
@@ -126,14 +127,14 @@ afs_mkdir(OSI_VC_DECL(adp), char *aname, struct vattr *attrs,
 	    } else
 	    	code = -1;
     	} while (afs_Analyze
-		    (tc, code, &adp->fid, &treq, AFS_STATS_FS_RPCIDX_MAKEDIR,
+		    (tc, code, &adp->f.fid, &treq, AFS_STATS_FS_RPCIDX_MAKEDIR,
 		     SHARED_LOCK, NULL));
 
     	if (code) {
 	    if (code < 0) {
 	    	ObtainWriteLock(&afs_xcbhash, 490);
 	    	afs_DequeueCallback(adp);
-	    	adp->states &= ~CStatd;
+	    	adp->f.states &= ~CStatd;
 	    	ReleaseWriteLock(&afs_xcbhash);
 	    	osi_dnlc_purgedp(adp);
 	    }
@@ -150,9 +151,9 @@ afs_mkdir(OSI_VC_DECL(adp), char *aname, struct vattr *attrs,
 	/* We have the dir entry now, we can use it while disconnected. */
 	if (adp->mvid == NULL) {
 	    /* If not mount point, generate a new fid. */
-	    newFid.Cell = adp->fid.Cell;
-    	    newFid.Fid.Volume = adp->fid.Fid.Volume;
-	    afs_GenFakeFid(&newFid, VDIR);
+	    newFid.Cell = adp->f.fid.Cell;
+    	    newFid.Fid.Volume = adp->f.fid.Fid.Volume;
+	    afs_GenFakeFid(&newFid, VDIR, 1);
 	}
     	/* XXX: If mount point???*/
 
@@ -182,11 +183,11 @@ afs_mkdir(OSI_VC_DECL(adp), char *aname, struct vattr *attrs,
 
     if (AFS_IS_DISCON_RW)
 	/* We will have to settle with the local link count. */
-	adp->m.LinkCount++;
+	adp->f.m.LinkCount++;
     else
-	adp->m.LinkCount = OutDirStatus.LinkCount;
-    newFid.Cell = adp->fid.Cell;
-    newFid.Fid.Volume = adp->fid.Fid.Volume;
+	adp->f.m.LinkCount = OutDirStatus.LinkCount;
+    newFid.Cell = adp->f.fid.Cell;
+    newFid.Fid.Volume = adp->f.fid.Fid.Volume;
     ReleaseWriteLock(&adp->lock);
     if (AFS_IS_DISCON_RW) {
 #if defined(AFS_DISCON_ENV)
@@ -195,7 +196,6 @@ afs_mkdir(OSI_VC_DECL(adp), char *aname, struct vattr *attrs,
 	/* Generate a new vcache and fill it. */
 	tvc = afs_NewVCache(&newFid, NULL);
 	if (tvc) {
-	    code = 0;
 	    *avcp = tvc;
 	} else {
 	    code = ENOENT;
@@ -210,34 +210,25 @@ afs_mkdir(OSI_VC_DECL(adp), char *aname, struct vattr *attrs,
 	/* Get a new dcache for it first. */
 	new_dc = afs_GetDCache(tvc, (afs_size_t) 0, &treq, &offset, &len, 1);
 	if (!new_dc) {
-	    printf("afs_mkdir: can't get new dcache for dir.\n");
+	    /* printf("afs_mkdir: can't get new dcache for dir.\n"); */
 	    code = ENOENT;
 	    goto done;
 	}
 
 	ObtainWriteLock(&afs_xdcache, 739);
 	code = afs_dir_MakeDir(new_dc,
-				(afs_int32 *) &newFid.Fid,
-				(afs_int32) &adp->fid.Fid);
+			       (afs_int32 *) &newFid.Fid,
+			       (afs_int32 *) &adp->f.fid.Fid);
 	ReleaseWriteLock(&afs_xdcache);
-	if (code)
-	    printf("afs_mkdir: afs_dirMakeDir code = %u\n", code);
+	/* if (code) printf("afs_mkdir: afs_dirMakeDir code = %u\n", code); */
 
-	/* Add to dirty list. */
-	if (!tvc->ddirty_flags ||
-		(tvc->ddirty_flags == VDisconShadowed)) {
-
-	    /* Put it in the list only if it's fresh. */
-	    ObtainWriteLock(&afs_DDirtyVCListLock, 730);
-	    AFS_DISCON_ADD_DIRTY(tvc);
-	    ReleaseWriteLock(&afs_DDirtyVCListLock);
-	}
+	afs_PutDCache(new_dc);
 
 	ObtainWriteLock(&tvc->lock, 731);
 	/* Update length in the vcache. */
-	tvc->m.Length = new_dc->f.chunkBytes;
-	/* Set create flag. */
-	tvc->ddirty_flags |= VDisconCreate;
+	tvc->f.m.Length = new_dc->f.chunkBytes;
+
+	afs_DisconAddDirty(tvc, VDisconCreate, 1);
 	ReleaseWriteLock(&tvc->lock);
 #endif				/* #ifdef AFS_DISCON_ENV */
     } else {
@@ -264,16 +255,16 @@ int
 /* don't set CDirty in here because RPC is called synchronously */
 #if	defined(AFS_SUN5_ENV) || defined(AFS_SGI_ENV)
 afs_rmdir(OSI_VC_DECL(adp), char *aname, struct vnode *cdirp, 
-	  struct AFS_UCRED *acred)
+	  afs_ucred_t *acred)
 #else
-afs_rmdir(OSI_VC_DECL(adp), char *aname, struct AFS_UCRED *acred)
+afs_rmdir(OSI_VC_DECL(adp), char *aname, afs_ucred_t *acred)
 #endif
 {
     struct vrequest treq;
     register struct dcache *tdc;
     register struct vcache *tvc = NULL;
     register afs_int32 code;
-    register struct conn *tc;
+    register struct afs_conn *tc;
     afs_size_t offset, len;
     struct AFSFetchStatus OutDirStatus;
     struct AFSVolSync tsync;
@@ -308,7 +299,7 @@ afs_rmdir(OSI_VC_DECL(adp), char *aname, struct AFS_UCRED *acred)
     /** If the volume is read-only, return error without making an RPC to the
       * fileserver
       */
-    if (adp->states & CRO) {
+    if (adp->f.states & CRO) {
 	code = EROFS;
 	goto done;
     }
@@ -323,7 +314,7 @@ afs_rmdir(OSI_VC_DECL(adp), char *aname, struct AFS_UCRED *acred)
     ObtainWriteLock(&adp->lock, 154);
     if (tdc)
 	ObtainSharedLock(&tdc->lock, 633);
-    if (tdc && (adp->states & CForeign)) {
+    if (tdc && (adp->f.states & CForeign)) {
 	struct VenusFid unlinkFid;
 
 	unlinkFid.Fid.Vnode = 0;
@@ -331,8 +322,8 @@ afs_rmdir(OSI_VC_DECL(adp), char *aname, struct AFS_UCRED *acred)
 	if (code == 0) {
 	    afs_int32 cached = 0;
 
-	    unlinkFid.Cell = adp->fid.Cell;
-	    unlinkFid.Fid.Volume = adp->fid.Fid.Volume;
+	    unlinkFid.Cell = adp->f.fid.Cell;
+	    unlinkFid.Fid.Volume = adp->f.fid.Fid.Volume;
 	    if (unlinkFid.Fid.Unique == 0) {
 		tvc =
 		    afs_LookupVCache(&unlinkFid, &treq, &cached, adp, aname);
@@ -347,13 +338,13 @@ afs_rmdir(OSI_VC_DECL(adp), char *aname, struct AFS_UCRED *acred)
     if (!AFS_IS_DISCON_RW) {
 	/* Not disconnected, can connect to server. */
     	do {
-	    tc = afs_Conn(&adp->fid, &treq, SHARED_LOCK);
+	    tc = afs_Conn(&adp->f.fid, &treq, SHARED_LOCK);
 	    if (tc) {
 	    	XSTATS_START_TIME(AFS_STATS_FS_RPCIDX_REMOVEDIR);
 	    	RX_AFS_GUNLOCK();
 	    	code =
 		    RXAFS_RemoveDir(tc->id,
-		    		(struct AFSFid *)&adp->fid.Fid,
+		    		(struct AFSFid *)&adp->f.fid.Fid,
 				aname,
 				&OutDirStatus,
 				&tsync);
@@ -362,7 +353,7 @@ afs_rmdir(OSI_VC_DECL(adp), char *aname, struct AFS_UCRED *acred)
 	    } else
 	    	code = -1;
     	} while (afs_Analyze
-	         (tc, code, &adp->fid, &treq, AFS_STATS_FS_RPCIDX_REMOVEDIR,
+	         (tc, code, &adp->f.fid, &treq, AFS_STATS_FS_RPCIDX_REMOVEDIR,
 	         SHARED_LOCK, NULL));
 
     	if (code) {
@@ -374,7 +365,7 @@ afs_rmdir(OSI_VC_DECL(adp), char *aname, struct AFS_UCRED *acred)
 	    if (code < 0) {
 	    	ObtainWriteLock(&afs_xcbhash, 491);
 	    	afs_DequeueCallback(adp);
-	    	adp->states &= ~CStatd;
+	    	adp->f.states &= ~CStatd;
 	    	ReleaseWriteLock(&afs_xcbhash);
 	    	osi_dnlc_purgedp(adp);
 	    }
@@ -383,72 +374,62 @@ afs_rmdir(OSI_VC_DECL(adp), char *aname, struct AFS_UCRED *acred)
     	}
 
     	/* here if rpc worked; update the in-core link count */
-    	adp->m.LinkCount = OutDirStatus.LinkCount;
+    	adp->f.m.LinkCount = OutDirStatus.LinkCount;
 
     } else {
 #if defined(AFS_DISCON_ENV)
     	/* Disconnected. */
 
+	if (!tdc) {
+	    ReleaseWriteLock(&adp->lock);
+	    /* printf("afs_rmdir: No local dcache!\n"); */
+	    code = ENETDOWN;
+	    goto done;
+	}
+	
 	if (!tvc) {
 	    /* Find the vcache. */
 	    struct VenusFid tfid;
 
-	    tfid.Cell = adp->fid.Cell;
-	    tfid.Fid.Volume = adp->fid.Fid.Volume;
+	    tfid.Cell = adp->f.fid.Cell;
+	    tfid.Fid.Volume = adp->f.fid.Fid.Volume;
 	    code = afs_dir_Lookup(tdc, aname, &tfid.Fid);
 
 	    ObtainSharedLock(&afs_xvcache, 764);
 	    tvc = afs_FindVCache(&tfid, 0, 1 /* do xstats */ );
 	    ReleaseSharedLock(&afs_xvcache);
-
+	    
 	    if (!tvc) {
-		printf("afs_rmdir: Can't find dir's vcache!\n");
+		/* printf("afs_rmdir: Can't find dir's vcache!\n"); */
 		ReleaseSharedLock(&tdc->lock);
-		afs_PutDCache(tdc);	/* drop ref count */
-    		ReleaseWriteLock(&adp->lock);
-	    	goto done;
+	        afs_PutDCache(tdc);	/* drop ref count */
+    	        ReleaseWriteLock(&adp->lock);
+		code = ENETDOWN;
+	        goto done;
 	    }
 	}
 
-	if (tvc->m.LinkCount > 2) {
+	if (tvc->f.m.LinkCount > 2) {
 	    /* This dir contains more than . and .., thus it can't be
 	     * deleted.
 	     */
 	    ReleaseSharedLock(&tdc->lock);
+	    afs_PutDCache(tdc);
+	    afs_PutVCache(tvc);
+	    ReleaseWriteLock(&adp->lock);
 	    code = ENOTEMPTY;
 	    goto done;
 	}
 
     	/* Make a shadow copy of the parent dir (if not done already).
-	 * There's no need to make a shadow copy of the deleted directory
-	 * because a dir must be empty in order to be rmdir'ed.
-	 * If the deleted dir has no shadow, it means that it was empty.
+	 * If we were created locally, then we don't need to have a shadow
+	 * directory (as there's no server state to remember)
 	 */
-	if (!(adp->ddirty_flags & VDisconShadowed)) {
-	    /* If tdc available, then it is locked.
-	     * afs_MakeShadowDir unlocks it.
-	     */
-	    if (tdc)
-	    	ReleaseSharedLock(&tdc->lock);
-	    afs_MakeShadowDir(adp);
-	    if (tdc)
-	    	ObtainSharedLock(&tdc->lock, 732);
+	if (!adp->f.shadow.vnode && !(adp->f.ddirty_flags & VDisconCreate)) {
+	    afs_MakeShadowDir(adp, tdc);
 	}
 
-	if (!tvc->ddirty_flags ||
-		(tvc->ddirty_flags == VDisconShadowed)) {
-	    /* Put it in the list only if it's fresh or has only been shadowed. */
-	    ObtainWriteLock(&afs_DDirtyVCListLock, 728);
-	    AFS_DISCON_ADD_DIRTY(tvc);
-	    ReleaseWriteLock(&afs_DDirtyVCListLock);
-	}
-
-	/* Now add the vcache to the dirty list. */
-	ObtainWriteLock(&tvc->lock, 727);
-	tvc->ddirty_flags |= VDisconRemove;
-	ReleaseWriteLock(&tvc->lock);
-
-	adp->m.LinkCount--;
+	adp->f.m.LinkCount--;
 #endif				/* #ifdef AFS_DISCON_ENV */
     }				/* if (!AFS_IS_DISCON_RW) */
 
@@ -467,7 +448,6 @@ afs_rmdir(OSI_VC_DECL(adp), char *aname, struct AFS_UCRED *acred)
 	afs_PutDCache(tdc);	/* drop ref count */
     }
 
-
     if (tvc)
 	osi_dnlc_purgedp(tvc);	/* get rid of any entries for this directory */
     else
@@ -475,11 +455,20 @@ afs_rmdir(OSI_VC_DECL(adp), char *aname, struct AFS_UCRED *acred)
 
     if (tvc) {
 	ObtainWriteLock(&tvc->lock, 155);
-	tvc->states &= ~CUnique;	/* For the dfs xlator */
+	tvc->f.states &= ~CUnique;	/* For the dfs xlator */
+#if defined(AFS_DISCON_ENV)
+	if (AFS_IS_DISCON_RW) {
+	    if (tvc->f.ddirty_flags & VDisconCreate) {
+		/* If we we were created whilst disconnected, removal doesn't
+		 * need to get logged. Just go away gracefully */
+		afs_DisconRemoveDirty(tvc);
+	    } else {
+		afs_DisconAddDirty(tvc, VDisconRemove, 1);
+	    }
+	}
+#endif
 	ReleaseWriteLock(&tvc->lock);
-	/* If disconnected, keep this vcache around for resync. */
-	if (!AFS_IS_DISCON_RW)
-	    afs_PutVCache(tvc);
+	afs_PutVCache(tvc);
     }
     ReleaseWriteLock(&adp->lock);
     /* don't worry about link count since dirs can not be hardlinked */
@@ -487,7 +476,6 @@ afs_rmdir(OSI_VC_DECL(adp), char *aname, struct AFS_UCRED *acred)
 
   done:
     AFS_DISCON_UNLOCK();
-  done3:
     afs_PutFakeStat(&fakestate);
     code = afs_CheckCode(code, &treq, 27);
   done2:
