@@ -17,9 +17,6 @@
 #include <afsconfig.h>
 #include "afs/param.h"
 
-RCSID
-    ("$Header: /cvs/openafs/src/afs/VNOPS/afs_vnop_lookup.c,v 1.72.2.9 2008/09/22 19:29:55 shadow Exp $");
-
 #include "afs/sysincludes.h"	/* Standard vendor system headers */
 #include "afsincludes.h"	/* Afs-based standard headers */
 #include "afs/afs_stats.h"	/* statistics */
@@ -29,11 +26,8 @@ RCSID
 #include "afs/afs_osidnlc.h"
 #include "afs/afs_dynroot.h"
 
-
-extern struct DirEntry *afs_dir_GetBlob();
 extern struct vcache *afs_globalVp;
 
-
 afs_int32 afs_bkvolpref = 0;
 afs_int32 afs_bulkStatsDone;
 static int bulkStatCounter = 0;	/* counter for bulk stat seq. numbers */
@@ -57,42 +51,41 @@ int afs_fakestat_enable = 0;	/* 1: fakestat-all, 2: fakestat-crosscell */
 static int
 EvalMountData(char type, char *data, afs_uint32 states, afs_uint32 cellnum,
               struct volume **avolpp, register struct vrequest *areq,
-	      afs_uint32 *acellidxp, afs_uint32 *avolnump, afs_uint32 *avnoidp)
+	      afs_uint32 *acellidxp, afs_uint32 *avolnump,
+	      afs_uint32 *avnoidp, afs_uint32 *auniqp)
 {
     struct volume *tvp = 0;
     struct VenusFid tfid;
     struct cell *tcell;
-    char *cpos, *volnamep, *x;
-    char *buf;
+    char *cpos, *volnamep;
+    char *buf, *endptr;
     afs_int32 prefetch;		/* 1=>None  2=>RO  3=>BK */
     afs_int32 mtptCell, assocCell = 0, hac = 0;
     afs_int32 samecell, roname, len;
-    afs_uint32 volid, cellidx, vnoid = 0;
+    afs_uint32 volid = 0, cellidx, vnoid = 0, uniq = 0;
 
+    /* Start by figuring out and finding the cell */
     cpos = afs_strchr(data, ':');	/* if cell name present */
     if (cpos) {
-	cellnum = 0;
 	volnamep = cpos + 1;
 	*cpos = 0;
-	for (x = data; *x >= '0' && *x <= '9'; x++)
-	    cellnum = (cellnum * 10) + (*x - '0');
-	if (cellnum && !*x)
+	if ((afs_strtoi_r(data, &endptr, &cellnum) == 0) &&
+	    (endptr == cpos))
 	    tcell = afs_GetCell(cellnum, READ_LOCK);
 	else {
 	    tcell = afs_GetCellByName(data, READ_LOCK);
-	               cellnum = 0;
+	    cellnum = 0;
 	}
 	*cpos = ':';
     } else if (cellnum) {
 	volnamep = data;
 	tcell = afs_GetCell(cellnum, READ_LOCK);
     } else {
-	/*printf("No cellname %s , or cellnum %d , returning ENODEV\n", 
-	       data, cellnum);*/
+	/* No cellname or cellnum; return ENODEV */
 	return ENODEV;
     }
     if (!tcell) {
-	/*printf("Lookup failed, returning ENODEV\n");*/
+	/* no cell found; return ENODEV */
 	return ENODEV;
     }
 
@@ -104,22 +97,38 @@ EvalMountData(char type, char *data, afs_uint32 states, afs_uint32 cellnum,
     }
     afs_PutCell(tcell, READ_LOCK);
 
-    cpos = afs_strrchr(volnamep, ':'); /* if vno present */
-    if (cpos) 
+    /* If there's nothing to look up, we can't proceed */
+    if (!*volnamep)
+	return ENODEV;
+
+    /* cell found. figure out volume */
+    cpos = afs_strchr(volnamep, ':');
+    if (cpos)
 	*cpos = 0;
+
     /* Look for an all-numeric volume ID */
-    volid = 0;
-    for (x = volnamep; *x >= '0' && *x <= '9'; x++)
-	volid = (volid * 10) + (*x - '0');
-    if (cpos) {
-	*cpos = ':';
-	vnoid = 0;
-	if (!*x) /* allow vno with numeric volid only */
-	    for (x = (cpos + 1); *x >= '0' && *x <= '9'; x++)
-		vnoid = (vnoid * 10) + (*x - '0');
-	if (*x)
-	    vnoid = 0;
-    }
+    if ((afs_strtoi_r(volnamep, &endptr, &volid) == 0) &&
+	((endptr == cpos) || (!*endptr)))
+    {
+	/* Ok. Is there a vnode and uniq? */
+	if (cpos) {
+	    char *vnodep = (char *)(cpos + 1);
+	    char *uniqp = NULL;
+	    if ((!*vnodep) /* no vnode after colon */
+		|| !(uniqp = afs_strchr(vnodep, ':')) /* no colon for uniq */
+		|| (!*(++uniqp)) /* no uniq after colon */
+		|| (afs_strtoi_r(vnodep, &endptr, &vnoid) != 0) /* bad vno */
+		|| (*endptr != ':') /* bad vnode field */
+		|| (afs_strtoi_r(uniqp, &endptr, &uniq) != 0) /* bad uniq */
+		|| (*endptr)) /* anything after uniq */
+	    {
+		*cpos = ':';
+		/* sorry. vnode and uniq, or nothing */
+		return ENODEV;
+	    }
+	}
+    } else
+	    volid = 0;
 
     /*
      * If the volume ID was all-numeric, and they didn't ask for a
@@ -127,14 +136,10 @@ EvalMountData(char type, char *data, afs_uint32 states, afs_uint32 cellnum,
      * as-is.  This is currently only used for handling name lookups
      * in the dynamic mount directory.
      */
-    if (!*x && !avolpp) {
-	if (acellidxp)
-	    *acellidxp = cellidx;
-	if (avolnump)
-	    *avolnump = volid;
-	if (avnoidp)
-	    *avnoidp = vnoid;
-	return 0;
+    if (volid && !avolpp) {
+	if (*cpos)
+	    *cpos = ':';
+	goto done;
     }
 
     /*
@@ -143,14 +148,14 @@ EvalMountData(char type, char *data, afs_uint32 states, afs_uint32 cellnum,
      * and don't second-guess them by forcing use of a RW volume when
      * they gave the ID of something else.
      */
-    if (!*x && type == '%') {
+    if (volid && type == '%') {
 	tfid.Fid.Volume = volid;	/* remember BK volume */
 	tfid.Cell = mtptCell;
 	tvp = afs_GetVolume(&tfid, areq, WRITE_LOCK);	/* get the new one */
-	if (!tvp) {
-	    /*printf("afs_GetVolume failed - returning ENODEV");*/
-	    return ENODEV;	/* oops, can't do it */
-	}
+	if (cpos) /* one way or another we're done */
+	    *cpos = ':';
+	if (!tvp)
+	    return ENODEV; /* afs_GetVolume failed; return ENODEV */
 	goto done;
     }
 
@@ -184,8 +189,8 @@ EvalMountData(char type, char *data, afs_uint32 states, afs_uint32 cellnum,
     /* Get the volume struct. Unless this volume name has ".readonly" or
      * ".backup" in it, this will get the volume struct for the RW volume.
      * The RO volume will be prefetched if requested (but not returned).
+     * Set up to use volname first.
      */
-    /*printf("Calling GetVolumeByName\n");*/
     tvp = afs_GetVolumeByName(volnamep, mtptCell, prefetch, areq, WRITE_LOCK);
 
     /* If no volume was found in this cell, try the associated linked cell */
@@ -213,11 +218,13 @@ EvalMountData(char type, char *data, afs_uint32 states, afs_uint32 cellnum,
 	}
 	osi_FreeSmallSpace(buf);
     }
-
-    if (!tvp) {
-	/*printf("Couldn't find the volume\n");*/
+    /* done with volname */
+    if (cpos)
+	*cpos = ':';
+    if (!tvp)
 	return ENODEV;		/* Couldn't find the volume */
-    }
+    else
+	volid = tvp->volume;
 
     /* Don't cross mountpoint from a BK to a BK volume */
     if ((states & CBackup) && (tvp->states & VBackup)) {
@@ -251,12 +258,14 @@ done:
     if (acellidxp)
 	*acellidxp = cellidx;
     if (avolnump)
-	*avolnump = tvp->volume;
+	*avolnump = volid;
     if (avnoidp)
 	*avnoidp = vnoid;
+    if (auniqp)
+	*auniqp = uniq;
     if (avolpp)
 	*avolpp = tvp;
-    else
+    else if (tvp)
 	afs_PutVolume(tvp, WRITE_LOCK);
     return 0;
 }
@@ -266,11 +275,11 @@ EvalMountPoint(register struct vcache *avc, struct vcache *advc,
 	       struct volume **avolpp, register struct vrequest *areq)
 {
     afs_int32 code;
-    afs_uint32 avnoid;
+    afs_uint32 avnoid, auniq;
 
     AFS_STATCNT(EvalMountPoint);
 #ifdef notdef
-    if (avc->mvid && (avc->states & CMValid))
+    if (avc->mvid && (avc->f.states & CMValid))
 	return 0;		/* done while racing */
 #endif
     *avolpp = NULL;
@@ -280,21 +289,24 @@ EvalMountPoint(register struct vcache *avc, struct vcache *advc,
 
     /* Determine which cell and volume the mointpoint goes to */
     code = EvalMountData(avc->linkData[0], avc->linkData + 1,
-                         avc->states, avc->fid.Cell, avolpp, areq, 0, 0,
-			 &avnoid);
+                         avc->f.states, avc->f.fid.Cell, avolpp, areq, 0, 0,
+			 &avnoid, &auniq);
     if (code) return code;
 
     if (!avnoid)
 	avnoid = 1;
 
+    if (!auniq)
+	auniq = 1;
+
     if (avc->mvid == 0)
 	avc->mvid =
 	    (struct VenusFid *)osi_AllocSmallSpace(sizeof(struct VenusFid));
     avc->mvid->Cell = (*avolpp)->cell;
     avc->mvid->Fid.Volume = (*avolpp)->volume;
     avc->mvid->Fid.Vnode = avnoid;
-    avc->mvid->Fid.Unique = 1;
-    avc->states |= CMValid;
+    avc->mvid->Fid.Unique = auniq;
+    avc->f.states |= CMValid;
 
     /* Used to: if the mount point is stored within a backup volume,
      * then we should only update the parent pointer information if
@@ -303,7 +315,7 @@ EvalMountPoint(register struct vcache *avc, struct vcache *advc,
      *
      * Next two lines used to be under this if:
      *
-     * if (!(avc->states & CBackup) || tvp->dotdot.Fid.Volume == 0)
+     * if (!(avc->f.states & CBackup) || tvp->dotdot.Fid.Volume == 0)
      *
      * Now: update mount point back pointer on every call, so that we handle
      * multiple mount points better.  This way, when du tries to go back
@@ -311,9 +323,10 @@ EvalMountPoint(register struct vcache *avc, struct vcache *advc,
      * cd'ing via a new path to a volume will reset the ".." pointer
      * to the new path.
      */
-    (*avolpp)->mtpoint = avc->fid;	/* setup back pointer to mtpoint */
+    (*avolpp)->mtpoint = avc->f.fid;	/* setup back pointer to mtpoint */
+    
     if (advc)
-	(*avolpp)->dotdot = advc->fid;
+	(*avolpp)->dotdot = advc->f.fid;
 
     return 0;
 }
@@ -364,23 +377,24 @@ afs_EvalFakeStat_int(struct vcache **avcp, struct afs_fakestat_state *state,
     if (tvc->mvstat != 1)
 	return 0;
 
-    /* Is the call to VerifyVCache really necessary? */
-    code = afs_VerifyVCache(tvc, areq);
-    if (code)
-	goto done;
     if (canblock) {
+	/* Is the call to VerifyVCache really necessary? */
+	code = afs_VerifyVCache(tvc, areq);
+	if (code)
+	    goto done;
+
 	ObtainWriteLock(&tvc->lock, 599);
 	code = EvalMountPoint(tvc, NULL, &tvolp, areq);
 	ReleaseWriteLock(&tvc->lock);
 	if (code)
 	    goto done;
 	if (tvolp) {
-	    tvolp->dotdot = tvc->fid;
-	    tvolp->dotdot.Fid.Vnode = tvc->parentVnode;
-	    tvolp->dotdot.Fid.Unique = tvc->parentUnique;
+	    tvolp->dotdot = tvc->f.fid;
+	    tvolp->dotdot.Fid.Vnode = tvc->f.parent.vnode;
+	    tvolp->dotdot.Fid.Unique = tvc->f.parent.unique;
 	}
     }
-    if (tvc->mvid && (tvc->states & CMValid)) {
+    if (tvc->mvid && (tvc->f.states & CMValid)) {
 	if (!canblock) {
 	    afs_int32 retry;
 
@@ -402,9 +416,9 @@ afs_EvalFakeStat_int(struct vcache **avcp, struct afs_fakestat_state *state,
 	    goto done;
 	}
 #ifdef AFS_DARWIN80_ENV
-        root_vp->m.Type = VDIR;
+        root_vp->f.m.Type = VDIR;
         AFS_GUNLOCK();
-        code = afs_darwin_finalizevnode(root_vp, NULL, NULL, 0);
+        code = afs_darwin_finalizevnode(root_vp, NULL, NULL, 0, 0);
         AFS_GLOCK();
         if (code) goto done;
         vnode_ref(AFSTOV(root_vp));
@@ -514,7 +528,7 @@ afs_getsysname(register struct vrequest *areq, register struct vcache *adp,
     if (!afs_nfsexporter)
 	strcpy(bufp, (*sysnamelist)[0]);
     else {
-	au = afs_GetUser(areq->uid, adp->fid.Cell, 0);
+	au = afs_GetUser(areq->uid, adp->f.fid.Cell, 0);
 	if (au->exporter) {
 	    error = EXP_SYSNAME(au->exporter, (char *)0, sysnamelist, num, 0);
 	    if (error) {
@@ -590,7 +604,7 @@ Next_AtSys(register struct vcache *avc, struct vrequest *areq,
 	*sysnamelist = afs_sysnamelist;
 
 	if (afs_nfsexporter) {
-	    au = afs_GetUser(areq->uid, avc->fid.Cell, 0);
+	    au = afs_GetUser(areq->uid, avc->f.fid.Cell, 0);
 	    if (au->exporter) {
 		error =
 		    EXP_SYSNAME(au->exporter, (char *)0, sysnamelist, &num, 0);
@@ -633,10 +647,11 @@ afs_DoBulkStat(struct vcache *adp, long dirCookie, struct vrequest *areqp)
 {
     int nentries;		/* # of entries to prefetch */
     int nskip;			/* # of slots in the LRU queue to skip */
+#ifdef AFS_DARWIN80_ENV
+    struct vnode *lruvp;
+#endif
     struct vcache *lruvcp;	/* vcache ptr of our goal pos in LRU queue */
     struct dcache *dcp;		/* chunk containing the dir block */
-    char *statMemp;		/* status memory block */
-    char *cbfMemp;		/* callback and fid memory block */
     afs_size_t temp;		/* temp for holding chunk length, &c. */
     struct AFSFid *fidsp;	/* file IDs were collecting */
     struct AFSCallBack *cbsp;	/* call back pointers */
@@ -648,7 +663,7 @@ afs_DoBulkStat(struct vcache *adp, long dirCookie, struct vrequest *areqp)
     AFSCBFids fidParm;		/* file ID parm for bulk stat */
     AFSBulkStats statParm;	/* stat info parm for bulk stat */
     int fidIndex = 0;		/* which file were stating */
-    struct conn *tcp = 0;	/* conn for call */
+    struct afs_conn *tcp = 0;	/* conn for call */
     AFSCBs cbParm;		/* callback parm for bulk stat */
     struct server *hostp = 0;	/* host we got callback from */
     long startTime;		/* time we started the call,
@@ -664,13 +679,14 @@ afs_DoBulkStat(struct vcache *adp, long dirCookie, struct vrequest *areqp)
     afs_int32 retry;		/* handle low-level SGI MP race conditions */
     long volStates;		/* flags from vol structure */
     struct volume *volp = 0;	/* volume ptr */
-    struct VenusFid dotdot;
+    struct VenusFid dotdot = {0, {0, 0, 0}};
     int flagIndex = 0;		/* First file with bulk fetch flag set */
     int inlinebulk = 0;		/* Did we use InlineBulk RPC or not? */
     XSTATS_DECLS;
-#ifdef AFS_DARWIN80_ENV
-    panic("bulkstatus doesn't work on AFS_DARWIN80_ENV. don't call it");
-#endif
+    dotdot.Cell = 0;
+    dotdot.Fid.Unique = 0;
+    dotdot.Fid.Vnode = 0;
+
     /* first compute some basic parameters.  We dont want to prefetch more
      * than a fraction of the cache in any given call, and we want to preserve
      * a portion of the LRU queue in any event, so as to avoid thrashing
@@ -694,13 +710,11 @@ afs_DoBulkStat(struct vcache *adp, long dirCookie, struct vrequest *areqp)
      * one for fids and callbacks, and one for stat info.  Well set
      * up our pointers to the memory from there, too.
      */
-    statMemp = osi_AllocLargeSpace(nentries * sizeof(AFSFetchStatus));
-    statsp = (struct AFSFetchStatus *)statMemp;
-    cbfMemp =
-	osi_AllocLargeSpace(nentries *
-			    (sizeof(AFSCallBack) + sizeof(AFSFid)));
-    fidsp = (AFSFid *) cbfMemp;
-    cbsp = (AFSCallBack *) (cbfMemp + nentries * sizeof(AFSFid));
+    statsp = (AFSFetchStatus *) 
+	    osi_Alloc(AFSCBMAX * sizeof(AFSFetchStatus));
+    fidsp = (AFSFid *) osi_AllocLargeSpace(nentries * sizeof(AFSFid));
+    cbsp = (AFSCallBack *) 
+	    osi_Alloc(AFSCBMAX * sizeof(AFSCallBack));
 
     /* next, we must iterate over the directory, starting from the specified
      * cookie offset (dirCookie), and counting out nentries file entries.
@@ -728,9 +742,9 @@ afs_DoBulkStat(struct vcache *adp, long dirCookie, struct vrequest *areqp)
      * 1. The cache data is being fetched by another process.
      * 2. The cache data is no longer valid
      */
-    while ((adp->states & CStatd)
+    while ((adp->f.states & CStatd)
 	   && (dcp->dflags & DFFetching)
-	   && hsame(adp->m.DataVersion, dcp->f.versionNo)) {
+	   && hsame(adp->f.m.DataVersion, dcp->f.versionNo)) {
 	afs_Trace4(afs_iclSetp, CM_TRACE_DCACHEWAIT, ICL_TYPE_STRING,
 		   __FILE__, ICL_TYPE_INT32, __LINE__, ICL_TYPE_POINTER, dcp,
 		   ICL_TYPE_INT32, dcp->dflags);
@@ -740,8 +754,8 @@ afs_DoBulkStat(struct vcache *adp, long dirCookie, struct vrequest *areqp)
 	ObtainReadLock(&adp->lock);
 	ObtainReadLock(&dcp->lock);
     }
-    if (!(adp->states & CStatd)
-	|| !hsame(adp->m.DataVersion, dcp->f.versionNo)) {
+    if (!(adp->f.states & CStatd)
+	|| !hsame(adp->f.m.DataVersion, dcp->f.versionNo)) {
 	ReleaseReadLock(&dcp->lock);
 	ReleaseReadLock(&adp->lock);
 	afs_PutDCache(dcp);
@@ -756,6 +770,9 @@ afs_DoBulkStat(struct vcache *adp, long dirCookie, struct vrequest *areqp)
      * and it is safe to set the status information for this file.
      */
     statSeqNo = bulkStatCounter++;
+    /* ensure against wrapping */
+    if (statSeqNo == 0)
+	statSeqNo = bulkStatCounter++;
 
     /* now we have dir data in the cache, so scan the dir page */
     fidIndex = 0;
@@ -779,7 +796,7 @@ afs_DoBulkStat(struct vcache *adp, long dirCookie, struct vrequest *areqp)
 
 	/* dont copy more than we have room for */
 	if (fidIndex >= nentries) {
-	    DRelease((struct buffer *)dirEntryp, 0);
+	    DRelease(dirEntryp, 0);
 	    break;
 	}
 
@@ -791,8 +808,8 @@ afs_DoBulkStat(struct vcache *adp, long dirCookie, struct vrequest *areqp)
 	     * also make us skip "." and probably "..", unless it has
 	     * disappeared from the cache since we did our namei call.
 	     */
-	    tfid.Cell = adp->fid.Cell;
-	    tfid.Fid.Volume = adp->fid.Fid.Volume;
+	    tfid.Cell = adp->f.fid.Cell;
+	    tfid.Fid.Volume = adp->f.fid.Fid.Volume;
 	    tfid.Fid.Vnode = ntohl(dirEntryp->fid.vnode);
 	    tfid.Fid.Unique = ntohl(dirEntryp->fid.vunique);
 	    do {
@@ -805,7 +822,7 @@ afs_DoBulkStat(struct vcache *adp, long dirCookie, struct vrequest *areqp)
 		}
 	    } while (tvcp && retry);
 	    if (!tvcp) {	/* otherwise, create manually */
-		tvcp = afs_NewVCache(&tfid, hostp);
+		tvcp = afs_NewBulkVCache(&tfid, hostp, statSeqNo);
 		if (tvcp)
 		{
 			ObtainWriteLock(&tvcp->lock, 505);
@@ -820,34 +837,29 @@ afs_DoBulkStat(struct vcache *adp, long dirCookie, struct vrequest *areqp)
 	    }
 	    if (!tvcp)
 	    {
-		DRelease((struct buffer *)dirEntryp, 0);
+		DRelease(dirEntryp, 0);
 		ReleaseReadLock(&dcp->lock);
 		ReleaseReadLock(&adp->lock);
 		afs_PutDCache(dcp);
 		goto done;	/* can happen if afs_NewVCache fails */
 	    }
 
-#ifdef AFS_DARWIN80_ENV
-            if (tvcp->states & CVInit) {
-                 /* XXX don't have status yet, so creating the vnode is
-                    not yet useful. we would get CDeadVnode set, and the
-                    upcoming PutVCache will cause the vcache to be flushed &
-                    freed, which in turn means the bulkstatus results won't 
-                    be used */
-            }
-#endif
 	    /* WARNING: afs_DoBulkStat uses the Length field to store a
 	     * sequence number for each bulk status request. Under no
 	     * circumstances should afs_DoBulkStat store a sequence number
 	     * if the new length will be ignored when afs_ProcessFS is
 	     * called with new stats. */
 #ifdef AFS_SGI_ENV
-	    if (!(tvcp->states & (CStatd | CBulkFetching))
+	    if (!(tvcp->f.states & CStatd)
+		&& (!((tvcp->f.states & CBulkFetching) &&
+		      (tvcp->f.m.Length != statSeqNo)))
 		&& (tvcp->execsOrWriters <= 0)
 		&& !afs_DirtyPages(tvcp)
 		&& !AFS_VN_MAPPED((vnode_t *) tvcp))
 #else
-	    if (!(tvcp->states & (CStatd | CBulkFetching))
+	    if (!(tvcp->f.states & CStatd)
+		&& (!((tvcp->f.states & CBulkFetching) &&
+		      (tvcp->f.m.Length != statSeqNo)))
 		&& (tvcp->execsOrWriters <= 0)
 		&& !afs_DirtyPages(tvcp))
 #endif
@@ -861,7 +873,7 @@ afs_DoBulkStat(struct vcache *adp, long dirCookie, struct vrequest *areqp)
 		 * CBulkFetching state bit and the value in the file size.
 		 * It is safe to set the status only if the CBulkFetching
 		 * flag is still set and the value in the file size does
-		 * not change.
+		 * not change. NewBulkVCache sets us up.
 		 *
 		 * Don't fetch status for dirty files. We need to
 		 * preserve the value of the file size. We could
@@ -869,8 +881,6 @@ afs_DoBulkStat(struct vcache *adp, long dirCookie, struct vrequest *areqp)
 		 */
 		memcpy((char *)(fidsp + fidIndex), (char *)&tfid.Fid,
 		       sizeof(*fidsp));
-		tvcp->states |= CBulkFetching;
-		tvcp->m.Length = statSeqNo;
 		fidIndex++;
 	    }
 	    afs_PutVCache(tvcp);
@@ -881,7 +891,7 @@ afs_DoBulkStat(struct vcache *adp, long dirCookie, struct vrequest *areqp)
 	 * used by this dir entry.
 	 */
 	temp = afs_dir_NameBlobs(dirEntryp->name) << 5;
-	DRelease((struct buffer *)dirEntryp, 0);
+	DRelease(dirEntryp, 0);
 	if (temp <= 0)
 	    break;
 	dirCookie += temp;
@@ -910,7 +920,7 @@ afs_DoBulkStat(struct vcache *adp, long dirCookie, struct vrequest *areqp)
 	/* start the timer; callback expirations are relative to this */
 	startTime = osi_Time();
 
-	tcp = afs_Conn(&adp->fid, areqp, SHARED_LOCK);
+	tcp = afs_Conn(&adp->f.fid, areqp, SHARED_LOCK);
 	if (tcp) {
 	    hostp = tcp->srvr->server;
 	    XSTATS_START_TIME(AFS_STATS_FS_RPCIDX_BULKSTATUS);
@@ -939,7 +949,7 @@ afs_DoBulkStat(struct vcache *adp, long dirCookie, struct vrequest *areqp)
 	} else
 	    code = -1;
     } while (afs_Analyze
-	     (tcp, code, &adp->fid, areqp, AFS_STATS_FS_RPCIDX_BULKSTATUS,
+	     (tcp, code, &adp->f.fid, areqp, AFS_STATS_FS_RPCIDX_BULKSTATUS,
 	      SHARED_LOCK, NULL));
 
     /* now, if we didnt get the info, bail out. */
@@ -948,7 +958,7 @@ afs_DoBulkStat(struct vcache *adp, long dirCookie, struct vrequest *areqp)
 
     /* we need vol flags to create the entries properly */
     dotdot.Fid.Volume = 0;
-    volp = afs_GetVolume(&adp->fid, areqp, READ_LOCK);
+    volp = afs_GetVolume(&adp->f.fid, areqp, READ_LOCK);
     if (volp) {
 	volStates = volp->states;
 	if (volp->dotdot.Fid.Volume != 0)
@@ -975,9 +985,13 @@ afs_DoBulkStat(struct vcache *adp, long dirCookie, struct vrequest *areqp)
 	refpanic("Bulkstat VLRU inconsistent");
     }
     for (tq = VLRU.next; tq != &VLRU; tq = QNext(tq)) {
-	if (--nskip <= 0)
-	    break;
-	else if (QNext(QPrev(tq)) != tq) {
+	if (--nskip <= 0) {
+#ifdef AFS_DARWIN80_ENV
+	    if (!(QTOV(tq)->f.states & CDeadVnode))
+#endif
+		break;
+	}
+	if (QNext(QPrev(tq)) != tq) {
 	    BStvc = QTOV(tq);
 	    refpanic("BulkStat VLRU inconsistent");
 	}
@@ -993,7 +1007,20 @@ afs_DoBulkStat(struct vcache *adp, long dirCookie, struct vrequest *areqp)
      * doesn't hurt nearly as much.
      */
     retry = 0;
+#ifdef AFS_DARWIN80_ENV
+    lruvp = AFSTOV(lruvcp);
+    if (vnode_get(lruvp))       /* this bumps ref count */
+	retry = 1;
+    else if (vnode_ref(lruvp)) {
+	AFS_GUNLOCK();
+	/* AFSTOV(lruvcp) may be NULL */
+	vnode_put(lruvp);
+	AFS_GLOCK();
+	retry = 1;
+    }
+#else
     osi_vnhold(lruvcp, &retry);
+#endif
     ReleaseReadLock(&afs_xvcache);	/* could be read lock */
     if (retry)
 	goto reskip;
@@ -1010,8 +1037,8 @@ afs_DoBulkStat(struct vcache *adp, long dirCookie, struct vrequest *areqp)
     for (i = 0; i < fidIndex; i++) {
 	if ((&statsp[i])->errorCode)
 	    continue;
-	afid.Cell = adp->fid.Cell;
-	afid.Fid.Volume = adp->fid.Fid.Volume;
+	afid.Cell = adp->f.fid.Cell;
+	afid.Fid.Volume = adp->f.fid.Fid.Volume;
 	afid.Fid.Vnode = fidsp[i].Vnode;
 	afid.Fid.Unique = fidsp[i].Unique;
 	do {
@@ -1035,9 +1062,17 @@ afs_DoBulkStat(struct vcache *adp, long dirCookie, struct vrequest *areqp)
 	 * and we may not have the latest status information for this
 	 * file.  Leave the entry alone.
 	 */
-	if (!(tvcp->states & CBulkFetching) || (tvcp->m.Length != statSeqNo)) {
+	if (!(tvcp->f.states & CBulkFetching) || (tvcp->f.m.Length != statSeqNo)) {
+#ifdef AFS_DARWIN80_ENV	    
+	    int isdead = (tvcp->f.states & CDeadVnode);
+#endif
 	    flagIndex++;
 	    ReleaseWriteLock(&tvcp->lock);
+#ifdef AFS_DARWIN80_ENV	    
+	    if (!isdead)
+		/* re-acquire the usecount that the other finalizevnode disposed of */
+		vnode_ref(AFSTOV(tvcp));
+#endif
 	    afs_PutVCache(tvcp);
 	    continue;
 	}
@@ -1086,8 +1121,13 @@ afs_DoBulkStat(struct vcache *adp, long dirCookie, struct vrequest *areqp)
 	/* We need to check the flags again. We may have missed
 	 * something while we were waiting for a lock.
 	 */
-	if (!(tvcp->states & CBulkFetching) || (tvcp->m.Length != statSeqNo)) {
+	if (!(tvcp->f.states & CBulkFetching) || (tvcp->f.m.Length != statSeqNo)) {
 	    flagIndex++;
+#ifdef AFS_DARWIN80_ENV	    
+	    if ((tvcp->f.states & CDeadVnode) == 0)
+		/* re-acquire the usecount that the other finalizevnode disposed of */
+		vnode_ref(AFSTOV(tvcp));
+#endif
 	    ReleaseWriteLock(&tvcp->lock);
 	    ReleaseWriteLock(&afs_xcbhash);
 	    afs_PutVCache(tvcp);
@@ -1106,21 +1146,21 @@ afs_DoBulkStat(struct vcache *adp, long dirCookie, struct vrequest *areqp)
 	 * loaded, so we can tell if we use it before it gets
 	 * recycled.
 	 */
-	tvcp->states |= CBulkStat;
-	tvcp->states &= ~CBulkFetching;
+	tvcp->f.states |= CBulkStat;
+	tvcp->f.states &= ~CBulkFetching;
 	flagIndex++;
 	afs_bulkStatsDone++;
 
 	/* merge in vol info */
 	if (volStates & VRO)
-	    tvcp->states |= CRO;
+	    tvcp->f.states |= CRO;
 	if (volStates & VBackup)
-	    tvcp->states |= CBackup;
+	    tvcp->f.states |= CBackup;
 	if (volStates & VForeign)
-	    tvcp->states |= CForeign;
+	    tvcp->f.states |= CForeign;
 
 	/* merge in the callback info */
-	tvcp->states |= CTruth;
+	tvcp->f.states |= CTruth;
 
 	/* get ptr to the callback we are interested in */
 	tcbp = cbsp + i;
@@ -1128,22 +1168,39 @@ afs_DoBulkStat(struct vcache *adp, long dirCookie, struct vrequest *areqp)
 	if (tcbp->ExpirationTime != 0) {
 	    tvcp->cbExpires = tcbp->ExpirationTime + startTime;
 	    tvcp->callback = hostp;
-	    tvcp->states |= CStatd;
+	    tvcp->f.states |= CStatd;
 	    afs_QueueCallback(tvcp, CBHash(tcbp->ExpirationTime), volp);
-	} else if (tvcp->states & CRO) {
+	} else if (tvcp->f.states & CRO) {
 	    /* ordinary callback on a read-only volume -- AFS 3.2 style */
 	    tvcp->cbExpires = 3600 + startTime;
 	    tvcp->callback = hostp;
-	    tvcp->states |= CStatd;
+	    tvcp->f.states |= CStatd;
 	    afs_QueueCallback(tvcp, CBHash(3600), volp);
 	} else {
 	    tvcp->callback = 0;
-	    tvcp->states &= ~(CStatd | CUnique);
+	    tvcp->f.states &= ~(CStatd | CUnique);
 	    afs_DequeueCallback(tvcp);
-	    if ((tvcp->states & CForeign) || (vType(tvcp) == VDIR))
+	    if ((tvcp->f.states & CForeign) || (vType(tvcp) == VDIR))
 		osi_dnlc_purgedp(tvcp);	/* if it (could be) a directory */
 	}
 	ReleaseWriteLock(&afs_xcbhash);
+#ifdef AFS_DARWIN80_ENV
+	/* reclaim->FlushVCache will need xcbhash */
+	if (tvcp->f.states & CDeadVnode) {
+	    /* passing in a parent hangs getting the vnode lock */
+	    code = afs_darwin_finalizevnode(tvcp, NULL, NULL, 0, 1);
+	    if (code) {
+		/* It's gonna get recycled - shouldn't happen */
+		tvcp->callback = 0;
+		tvcp->f.states &= ~(CStatd | CUnique);
+		afs_DequeueCallback(tvcp);
+		if ((tvcp->f.states & CForeign) || (vType(tvcp) == VDIR))
+		    osi_dnlc_purgedp(tvcp); /* if it (could be) a directory */
+	    } else
+		/* re-acquire the usecount that finalizevnode disposed of */
+		vnode_ref(AFSTOV(tvcp));
+	}
+#endif
 
 	ReleaseWriteLock(&tvcp->lock);
 	/* finally, we're done with the entry */
@@ -1151,13 +1208,20 @@ afs_DoBulkStat(struct vcache *adp, long dirCookie, struct vrequest *areqp)
     }				/* for all files we got back */
 
     /* finally return the pointer into the LRU queue */
+#ifdef AFS_DARWIN80_ENV
+    AFS_GUNLOCK();
+    vnode_put(lruvp);
+    vnode_rele(lruvp);
+    AFS_GLOCK();
+#else
     afs_PutVCache(lruvcp);
+#endif
 
   done:
     /* Be sure to turn off the CBulkFetching flags */
     for (i = flagIndex; i < fidIndex; i++) {
-	afid.Cell = adp->fid.Cell;
-	afid.Fid.Volume = adp->fid.Fid.Volume;
+	afid.Cell = adp->f.fid.Cell;
+	afid.Fid.Volume = adp->f.fid.Fid.Volume;
 	afid.Fid.Vnode = fidsp[i].Vnode;
 	afid.Fid.Unique = fidsp[i].Unique;
 	do {
@@ -1166,11 +1230,16 @@ afs_DoBulkStat(struct vcache *adp, long dirCookie, struct vrequest *areqp)
 	    tvcp = afs_FindVCache(&afid, &retry, 0 /* !stats&!lru */ );
 	    ReleaseReadLock(&afs_xvcache);
 	} while (tvcp && retry);
-	if (tvcp != NULL && (tvcp->states & CBulkFetching)
-	    && (tvcp->m.Length == statSeqNo)) {
-	    tvcp->states &= ~CBulkFetching;
+	if (tvcp != NULL && (tvcp->f.states & CBulkFetching)
+	    && (tvcp->f.m.Length == statSeqNo)) {
+	    tvcp->f.states &= ~CBulkFetching;
 	}
 	if (tvcp != NULL) {
+#ifdef AFS_DARWIN80_ENV	    
+	    if ((tvcp->f.states & CDeadVnode) == 0) 
+		/* re-acquire the usecount that the other finalizevnode disposed of */
+		vnode_ref(AFSTOV(tvcp));
+#endif
 	    afs_PutVCache(tvcp);
 	}
     }
@@ -1178,9 +1247,9 @@ afs_DoBulkStat(struct vcache *adp, long dirCookie, struct vrequest *areqp)
 	afs_PutVolume(volp, READ_LOCK);
 
     /* If we did the InlineBulk RPC pull out the return code */
-    if (inlinebulk) {
+    if (inlinebulk && code == 0) {
 	if ((&statsp[0])->errorCode) {
-	    afs_Analyze(tcp, (&statsp[0])->errorCode, &adp->fid, areqp,
+	    afs_Analyze(tcp, (&statsp[0])->errorCode, &adp->f.fid, areqp,
 			AFS_STATS_FS_RPCIDX_BULKSTATUS, SHARED_LOCK, NULL);
 	    code = (&statsp[0])->errorCode;
 	}
@@ -1188,27 +1257,22 @@ afs_DoBulkStat(struct vcache *adp, long dirCookie, struct vrequest *areqp)
 	code = 0;
     }
   done2:
-    osi_FreeLargeSpace(statMemp);
-    osi_FreeLargeSpace(cbfMemp);
+    osi_FreeLargeSpace((char *)fidsp);
+    osi_Free((char *)statsp, AFSCBMAX * sizeof(AFSFetchStatus));
+    osi_Free((char *)cbsp, AFSCBMAX * sizeof(AFSCallBack));
     return code;
 }
 
 /* was: (AFS_DEC_ENV) || defined(AFS_OSF30_ENV) || defined(AFS_NCR_ENV) */
-#ifdef AFS_DARWIN80_ENV
-#define AFSDOBULK 0
-#else
 static int AFSDOBULK = 1;
-#endif
 
 int
-#ifdef AFS_OSF_ENV
-afs_lookup(OSI_VC_DECL(adp), char *aname, struct vcache **avcp, struct AFS_UCRED *acred, int opflag, int wantparent)
-#elif defined(AFS_SUN5_ENV) || defined(AFS_SGI_ENV)
-afs_lookup(OSI_VC_DECL(adp), char *aname, struct vcache **avcp, struct pathname *pnp, int flags, struct vnode *rdir, struct AFS_UCRED *acred)
+#if defined(AFS_SUN5_ENV) || defined(AFS_SGI_ENV)
+afs_lookup(OSI_VC_DECL(adp), char *aname, struct vcache **avcp, struct pathname *pnp, int flags, struct vnode *rdir, afs_ucred_t *acred)
 #elif defined(UKERNEL)
-afs_lookup(OSI_VC_DECL(adp), char *aname, struct vcache **avcp, struct AFS_UCRED *acred, int flags)
+afs_lookup(OSI_VC_DECL(adp), char *aname, struct vcache **avcp, afs_ucred_t *acred, int flags)
 #else
-afs_lookup(OSI_VC_DECL(adp), char *aname, struct vcache **avcp, struct AFS_UCRED *acred)
+afs_lookup(OSI_VC_DECL(adp), char *aname, struct vcache **avcp, afs_ucred_t *acred)
 #endif
 {
     struct vrequest treq;
@@ -1236,10 +1300,6 @@ afs_lookup(OSI_VC_DECL(adp), char *aname, struct vcache **avcp, struct AFS_UCRED
     if ((code = afs_InitReq(&treq, acred)))
 	goto done;
 
-#ifdef	AFS_OSF_ENV
-    ndp->ni_dvp = AFSTOV(adp);
-#endif /* AFS_OSF_ENV */
-
     if (afs_fakestat_enable && adp->mvstat == 1) {
        if (strcmp(aname, ".directory") == 0)
            tryEvalOnly = 1;
@@ -1255,6 +1315,10 @@ afs_lookup(OSI_VC_DECL(adp), char *aname, struct vcache **avcp, struct AFS_UCRED
 	if (strcmp(aname, "Contents") == 0)
 	    tryEvalOnly = 1;
     }
+    if (afs_fakestat_enable && adp->mvstat == 2) {
+	if (strncmp(aname, "._", 2) == 0)
+	    tryEvalOnly = 1;
+    }
 #endif
 
     if (tryEvalOnly)
@@ -1278,7 +1342,7 @@ afs_lookup(OSI_VC_DECL(adp), char *aname, struct vcache **avcp, struct AFS_UCRED
     *avcp = NULL;		/* Since some callers don't initialize it */
     bulkcode = 0;
 
-    if (!(adp->states & CStatd) && !afs_InReadDir(adp)) {
+    if (!(adp->f.states & CStatd) && !afs_InReadDir(adp)) {
 	if ((code = afs_VerifyVCache2(adp, &treq))) {
 	    goto done;
 	}
@@ -1289,18 +1353,6 @@ afs_lookup(OSI_VC_DECL(adp), char *aname, struct vcache **avcp, struct AFS_UCRED
     if (adp->mvstat == 2 && aname[0] == '.' && aname[1] == '.' && !aname[2]) {
 	/* looking up ".." in root via special hacks */
 	if (adp->mvid == (struct VenusFid *)0 || adp->mvid->Fid.Volume == 0) {
-#ifdef	AFS_OSF_ENV
-	    if (adp == afs_globalVp) {
-		struct vnode *rvp = AFSTOV(adp);
-/*
-		ndp->ni_vp = rvp->v_vfsp->vfs_vnodecovered;
-		ndp->ni_dvp = ndp->ni_vp;
-		VN_HOLD(*avcp);
-*/
-		code = ENODEV;
-		goto done;
-	    }
-#endif
 	    code = ENODEV;
 	    goto done;
 	}
@@ -1384,15 +1436,26 @@ afs_lookup(OSI_VC_DECL(adp), char *aname, struct vcache **avcp, struct AFS_UCRED
      */
     if (afs_IsDynrootMount(adp)) {
 	struct VenusFid tfid;
-	afs_uint32 cellidx, volid, vnoid;
+	afs_uint32 cellidx, volid, vnoid, uniq;
 
-	code = EvalMountData('%', aname, 0, 0, NULL, &treq, &cellidx, &volid, &vnoid);
+	code = EvalMountData('%', aname, 0, 0, NULL, &treq, &cellidx, &volid, &vnoid, &uniq);
 	if (code)
 	    goto done;
-	afs_GetDynrootMountFid(&tfid);
-	tfid.Fid.Vnode = VNUM_FROM_TYPEID(VN_TYPE_MOUNT, cellidx << 2);
-	tfid.Fid.Unique = volid;
+	/* If a vnode was returned, it's not a real mount point */
+	if (vnoid > 1) {
+	    struct cell *tcell = afs_GetCellByIndex(cellidx, READ_LOCK);
+	    tfid.Cell = tcell->cellNum;
+	    afs_PutCell(tcell, READ_LOCK);
+	    tfid.Fid.Vnode = vnoid;
+	    tfid.Fid.Volume = volid;
+	    tfid.Fid.Unique = uniq;
+	} else {
+	    afs_GetDynrootMountFid(&tfid);
+	    tfid.Fid.Vnode = VNUM_FROM_TYPEID(VN_TYPE_MOUNT, cellidx << 2);
+	    tfid.Fid.Unique = volid;
+	}
 	*avcp = tvc = afs_GetVCache(&tfid, &treq, NULL, NULL);
+	code = (tvc ? 0 : ENOENT);
 	hit = 1;
 	goto done;
     }
@@ -1485,17 +1548,17 @@ afs_lookup(OSI_VC_DECL(adp), char *aname, struct vcache **avcp, struct AFS_UCRED
 	 * use that.  This eliminates several possible deadlocks.  
 	 */
 	if (!afs_InReadDir(adp)) {
-	    while ((adp->states & CStatd)
+	    while ((adp->f.states & CStatd)
 		   && (tdc->dflags & DFFetching)
-		   && hsame(adp->m.DataVersion, tdc->f.versionNo)) {
+		   && hsame(adp->f.m.DataVersion, tdc->f.versionNo)) {
 		ReleaseReadLock(&tdc->lock);
 		ReleaseReadLock(&adp->lock);
 		afs_osi_Sleep(&tdc->validPos);
 		ObtainReadLock(&adp->lock);
 		ObtainReadLock(&tdc->lock);
 	    }
-	    if (!(adp->states & CStatd)
-		|| !hsame(adp->m.DataVersion, tdc->f.versionNo)) {
+	    if (!(adp->f.states & CStatd)
+		|| !hsame(adp->f.m.DataVersion, tdc->f.versionNo)) {
 		ReleaseReadLock(&tdc->lock);
 		ReleaseReadLock(&adp->lock);
 		afs_PutDCache(tdc);
@@ -1535,8 +1598,7 @@ afs_lookup(OSI_VC_DECL(adp), char *aname, struct vcache **avcp, struct AFS_UCRED
 	ReleaseReadLock(&tdc->lock);
 	if (!afs_InReadDir(adp))
 	    afs_PutDCache(tdc);
-
-	if (code == ENOENT && afs_IsDynroot(adp) && dynrootRetry) {
+	if (code == ENOENT && afs_IsDynroot(adp) && dynrootRetry && !tryEvalOnly) {
 	    ReleaseReadLock(&adp->lock);
 	    dynrootRetry = 0;
 	    if (tname[0] == '.')
@@ -1551,15 +1613,15 @@ afs_lookup(OSI_VC_DECL(adp), char *aname, struct vcache **avcp, struct AFS_UCRED
 	}
 
 	/* new fid has same cell and volume */
-	tfid.Cell = adp->fid.Cell;
-	tfid.Fid.Volume = adp->fid.Fid.Volume;
+	tfid.Cell = adp->f.fid.Cell;
+	tfid.Fid.Volume = adp->f.fid.Fid.Volume;
 	afs_Trace4(afs_iclSetp, CM_TRACE_LOOKUP, ICL_TYPE_POINTER, adp,
 		   ICL_TYPE_STRING, tname, ICL_TYPE_FID, &tfid,
 		   ICL_TYPE_INT32, code);
 
 	if (code) {
 	    if (code != ENOENT) {
-		printf("LOOKUP dirLookupOff -> %d\n", code);
+		/*printf("LOOKUP dirLookupOff -> %d\n", code);*/
 	    }
 	    goto done;
 	}
@@ -1568,7 +1630,7 @@ afs_lookup(OSI_VC_DECL(adp), char *aname, struct vcache **avcp, struct AFS_UCRED
 	 * dirCookie tells us where to start prefetching from.
 	 */
 	if (!AFS_IS_DISCONNECTED && 
-	    AFSDOBULK && adp->opens > 0 && !(adp->states & CForeign)
+	    AFSDOBULK && adp->opens > 0 && !(adp->f.states & CForeign)
 	    && !afs_IsDynroot(adp) && !afs_InReadDir(adp)) {
 	    afs_int32 retry;
 	    /* if the entry is not in the cache, or is in the cache,
@@ -1581,15 +1643,15 @@ afs_lookup(OSI_VC_DECL(adp), char *aname, struct vcache **avcp, struct AFS_UCRED
 		ReleaseReadLock(&afs_xvcache);
 	    } while (tvc && retry);
 
-	    if (!tvc || !(tvc->states & CStatd))
+	    if (!tvc || !(tvc->f.states & CStatd))
 		bulkcode = afs_DoBulkStat(adp, dirCookie, &treq);
 	    else
 		bulkcode = 0;
 
 	    /* if the vcache isn't usable, release it */
-	    if (tvc && !(tvc->states & CStatd)) {
+	    if (tvc && !(tvc->f.states & CStatd)) {
 #ifndef  AFS_FBSD80_ENV
-	      afs_PutVCache(tvc);
+		afs_PutVCache(tvc);
 #endif
 		tvc = NULL;
 	    }
@@ -1607,7 +1669,7 @@ afs_lookup(OSI_VC_DECL(adp), char *aname, struct vcache **avcp, struct AFS_UCRED
 	 */
 	if (!tvc) {
 	    afs_int32 cached = 0;
-	    if (!tfid.Fid.Unique && (adp->states & CForeign)) {
+	    if (!tfid.Fid.Unique && (adp->f.states & CForeign)) {
 		tvc = afs_LookupVCache(&tfid, &treq, &cached, adp, tname);
 	    }
 	    if (!tvc && !bulkcode) {	/* lookup failed or wasn't called */
@@ -1617,11 +1679,11 @@ afs_lookup(OSI_VC_DECL(adp), char *aname, struct vcache **avcp, struct AFS_UCRED
     }				/* sub-block just to reduce stack usage */
 
     if (tvc) {
-	if (adp->states & CForeign)
-	    tvc->states |= CForeign;
-	tvc->parentVnode = adp->fid.Fid.Vnode;
-	tvc->parentUnique = adp->fid.Fid.Unique;
-	tvc->states &= ~CBulkStat;
+	if (adp->f.states & CForeign)
+	    tvc->f.states |= CForeign;
+	tvc->f.parent.vnode = adp->f.fid.Fid.Vnode;
+	tvc->f.parent.unique = adp->f.fid.Fid.Unique;
+	tvc->f.states &= ~CBulkStat;
 
 	if (afs_fakestat_enable == 2 && tvc->mvstat == 1) {
 	    ObtainSharedLock(&tvc->lock, 680);
@@ -1637,7 +1699,7 @@ afs_lookup(OSI_VC_DECL(adp), char *aname, struct vcache **avcp, struct AFS_UCRED
 		force_eval = 1;
 	    ReleaseReadLock(&tvc->lock);
 	}
-	if (tvc->mvstat == 1 && (tvc->states & CMValid) && tvc->mvid != NULL)
+	if (tvc->mvstat == 1 && (tvc->f.states & CMValid) && tvc->mvid != NULL)
 	  force_eval = 1; /* This is now almost for free, get it correct */
 
 #if defined(UKERNEL) && defined(AFS_WEB_ENHANCEMENTS)
@@ -1662,12 +1724,12 @@ afs_lookup(OSI_VC_DECL(adp), char *aname, struct vcache **avcp, struct AFS_UCRED
 		}
 
 		/* next, we want to continue using the target of the mt point */
-		if (tvc->mvid && (tvc->states & CMValid)) {
+		if (tvc->mvid && (tvc->f.states & CMValid)) {
 		    struct vcache *uvc;
 		    /* now lookup target, to set .. pointer */
 		    afs_Trace2(afs_iclSetp, CM_TRACE_LOOKUP1,
 			       ICL_TYPE_POINTER, tvc, ICL_TYPE_FID,
-			       &tvc->fid);
+			       &tvc->f.fid);
 		    uvc = tvc;	/* remember for later */
 
 		    if (tvolp && (tvolp->states & VForeign)) {
@@ -1726,14 +1788,14 @@ afs_lookup(OSI_VC_DECL(adp), char *aname, struct vcache **avcp, struct AFS_UCRED
 	if (!AFS_IS_DISCONNECTED) {
 	    if (pass == 0) {
 	        struct volume *tv;
-	        tv = afs_GetVolume(&adp->fid, &treq, READ_LOCK);
+	        tv = afs_GetVolume(&adp->f.fid, &treq, READ_LOCK);
 	        if (tv) {
 		    if (tv->states & VRO) {
 		        pass = 1;	/* try this *once* */
 		        ObtainWriteLock(&afs_xcbhash, 495);
 		        afs_DequeueCallback(adp);
 		        /* re-stat to get later version */
-		        adp->states &= ~CStatd;
+		        adp->f.states &= ~CStatd;
 		        ReleaseWriteLock(&afs_xcbhash);
 		        osi_dnlc_purgedp(adp);
 		        afs_PutVolume(tv, READ_LOCK);
@@ -1744,7 +1806,6 @@ afs_lookup(OSI_VC_DECL(adp), char *aname, struct vcache **avcp, struct AFS_UCRED
 	    }
 	    code = ENOENT;
 	} else {
-	    printf("Network down in afs_lookup\n");
 	    code = ENETDOWN;
 	}
     }
@@ -1754,27 +1815,16 @@ afs_lookup(OSI_VC_DECL(adp), char *aname, struct vcache **avcp, struct AFS_UCRED
     if (tname != aname && tname)
 	osi_FreeLargeSpace(tname);
     if (code == 0) {
-#ifdef	AFS_OSF_ENV
-	/* Handle RENAME; only need to check rename "."  */
-	if (opflag == RENAME && wantparent && *ndp->ni_next == 0) {
-	    if (!FidCmp(&(tvc->fid), &(adp->fid))) {
-		afs_PutVCache(*avcp);
-		*avcp = NULL;
-		afs_PutFakeStat(&fakestate);
-		return afs_CheckCode(EISDIR, &treq, 18);
-	    }
-	}
-#endif /* AFS_OSF_ENV */
 
 	if (afs_mariner)
 	    afs_AddMarinerName(aname, tvc);
 
 #if defined(UKERNEL) && defined(AFS_WEB_ENHANCEMENTS)
-	if (!(flags & AFS_LOOKUP_NOEVAL))
+	if (!(flags & AFS_LOOKUP_NOEVAL)) {
 	    /* Here we don't enter the name into the DNLC because we want the
-	     * evaluated mount dir to be there (the vcache for the mounted volume)
-	     * rather than the vc of the mount point itself.  we can still find the
-	     * mount point's vc in the vcache by its fid. */
+	     * evaluated mount dir to be there (the vcache for the mounted
+	     * volume) rather than the vc of the mount point itself.  We can
+	     * still find the mount point's vc in the vcache by its fid. */
 #endif /* UKERNEL && AFS_WEB_ENHANCEMENTS */
 	    if (!hit && force_eval) {
 		osi_dnlc_enter(adp, aname, tvc, &versionNo);
@@ -1788,6 +1838,9 @@ afs_lookup(OSI_VC_DECL(adp), char *aname, struct vcache **avcp, struct AFS_UCRED
 		return 0;	/* can't have been any errors if hit and !code */
 #endif
 	    }
+#if defined(UKERNEL) && defined(AFS_WEB_ENHANCEMENTS)
+	}
+#endif
     }
     if (bulkcode)
 	code = bulkcode;
