@@ -13,15 +13,12 @@
  * afs_MemWrite
  * afs_StoreOnLastReference
  * afs_close
- * afs_closex
  * afs_fsync
  */
 
 #include <afsconfig.h>
 #include "afs/param.h"
 
-RCSID
-    ("$Header: /cvs/openafs/src/afs/VNOPS/afs_vnop_write.c,v 1.50.2.1 2007/12/04 20:40:53 shadow Exp $");
 
 #include "afs/sysincludes.h"	/* Standard vendor system headers */
 #include "afsincludes.h"	/* Afs-based standard headers */
@@ -49,8 +46,8 @@ afs_StoreOnLastReference(register struct vcache *avc,
      * ourselves now. If we're called by the CCore clearer, the CCore
      * flag will already be clear, so we don't have to worry about
      * clearing it twice. */
-    if (avc->states & CCore) {
-	avc->states &= ~CCore;
+    if (avc->f.states & CCore) {
+	avc->f.states &= ~CCore;
 #if defined(AFS_SGI_ENV)
 	osi_Assert(avc->opens > 0 && avc->execsOrWriters > 0);
 #endif
@@ -61,37 +58,45 @@ afs_StoreOnLastReference(register struct vcache *avc,
 	avc->opens--;
 	avc->execsOrWriters--;
 	AFS_RELE(AFSTOV(avc));	/* VN_HOLD at set CCore(afs_FakeClose) */
-	crfree((struct AFS_UCRED *)avc->linkData);	/* "crheld" in afs_FakeClose */
+	crfree((afs_ucred_t *)avc->linkData);	/* "crheld" in afs_FakeClose */
 	avc->linkData = NULL;
     }
-    /* Now, send the file back.  Used to require 0 writers left, but now do
-     * it on every close for write, since two closes in a row are harmless
-     * since first will clean all chunks, and second will be noop.  Note that
-     * this will also save confusion when someone keeps a file open 
-     * inadvertently, since with old system, writes to the server would never
-     * happen again. 
-     */
-    code = afs_StoreAllSegments(avc, treq, AFS_LASTSTORE /*!sync-to-disk */ );
-    /*
-     * We have to do these after the above store in done: in some systems like
-     * aix they'll need to flush all the vm dirty pages to the disk via the
-     * strategy routine. During that all procedure (done under no avc locks)
-     * opens, refcounts would be zero, since it didn't reach the afs_{rd,wr}
-     * routines which means the vcache is a perfect candidate for flushing!
-     */
+
+    if (!AFS_IS_DISCONNECTED) {
+	/* Connected. */
+
+	/* Now, send the file back.  Used to require 0 writers left, but now do
+	 * it on every close for write, since two closes in a row are harmless
+	 * since first will clean all chunks, and second will be noop.  Note that
+	 * this will also save confusion when someone keeps a file open
+	 * inadvertently, since with old system, writes to the server would never
+	 * happen again.
+	 */
+	code = afs_StoreAllSegments(avc, treq, AFS_LASTSTORE /*!sync-to-disk */ );
+	/*
+	 * We have to do these after the above store in done: in some systems
+	 * like aix they'll need to flush all the vm dirty pages to the disk via
+	 * the strategy routine. During that all procedure (done under no avc
+	 * locks) opens, refcounts would be zero, since it didn't reach the
+	 * afs_{rd,wr} routines which means the vcache is a perfect candidate
+	 * for flushing!
+	 */
+     } else if (AFS_IS_DISCON_RW) {
+	afs_DisconAddDirty(avc, VDisconWriteClose, 0);
+     }		/* if not disconnected */
+
 #if defined(AFS_SGI_ENV)
     osi_Assert(avc->opens > 0 && avc->execsOrWriters > 0);
 #endif
+
     avc->opens--;
     avc->execsOrWriters--;
     return code;
 }
 
-
-
 int
 afs_MemWrite(register struct vcache *avc, struct uio *auio, int aio,
-	     struct AFS_UCRED *acred, int noLock)
+	     afs_ucred_t *acred, int noLock)
 {
     afs_size_t totalLength;
     afs_size_t transferLength;
@@ -130,7 +135,7 @@ afs_MemWrite(register struct vcache *avc, struct uio *auio, int aio,
     afs_Trace4(afs_iclSetp, CM_TRACE_WRITE, ICL_TYPE_POINTER, avc,
 	       ICL_TYPE_OFFSET, ICL_HANDLE_OFFSET(filePos), ICL_TYPE_OFFSET,
 	       ICL_HANDLE_OFFSET(totalLength), ICL_TYPE_OFFSET,
-	       ICL_HANDLE_OFFSET(avc->m.Length));
+	       ICL_HANDLE_OFFSET(avc->f.m.Length));
     if (!noLock) {
 	afs_MaybeWakeupTruncateDaemon();
 	ObtainWriteLock(&avc->lock, 126);
@@ -144,8 +149,8 @@ afs_MemWrite(register struct vcache *avc, struct uio *auio, int aio,
 	 * Since we are called via strategy, we need to trim the write to
 	 * the actual size of the file
 	 */
-	osi_Assert(filePos <= avc->m.Length);
-	diff = avc->m.Length - filePos;
+	osi_Assert(filePos <= avc->f.m.Length);
+	diff = avc->f.m.Length - filePos;
 	AFS_UIO_SETRESID(auio, MIN(totalLength, diff));
 	totalLength = AFS_UIO_RESID(auio);
     }
@@ -155,7 +160,7 @@ afs_MemWrite(register struct vcache *avc, struct uio *auio, int aio,
 #if	defined(AFS_SUN56_ENV)
 	auio->uio_loffset = 0;
 #endif
-	filePos = avc->m.Length;
+	filePos = avc->f.m.Length;
 	AFS_UIO_SETOFFSET(auio, filePos);
     }
 #endif
@@ -163,7 +168,7 @@ afs_MemWrite(register struct vcache *avc, struct uio *auio, int aio,
      * Note that we use startDate rather than calling osi_Time() here.
      * This is to avoid counting lock-waiting time in file date (for ranlib).
      */
-    avc->m.Date = startDate;
+    avc->f.m.Date = startDate;
 
 #if	defined(AFS_HPUX_ENV)
 #if	defined(AFS_HPUX101_ENV)
@@ -183,77 +188,24 @@ afs_MemWrite(register struct vcache *avc, struct uio *auio, int aio,
      * high-level write op.
      */
     if (avc->execsOrWriters <= 0) {
-	printf("WARNING: afs_ufswr vp=%lx, exOrW=%d\n", (unsigned long)avc,
+	afs_warn("WARNING: afs_ufswr vp=%lx, exOrW=%d\n", (unsigned long)avc,
 	       avc->execsOrWriters);
     }
 #else
     afs_FakeOpen(avc);
 #endif
-    avc->states |= CDirty;
+    avc->f.states |= CDirty;
 #ifndef AFS_DARWIN80_ENV
     tvec = (struct iovec *)osi_AllocSmallSpace(sizeof(struct iovec));
 #endif
     while (totalLength > 0) {
-	/* 
-	 *  The following line is necessary because afs_GetDCache with
-	 *  flag == 4 expects the length field to be filled. It decides
-	 *  from this whether it's necessary to fetch data into the chunk
-	 *  before writing or not (when the whole chunk is overwritten!).
-	 */
-	len = totalLength;	/* write this amount by default */
-	if (noLock) {
-	    tdc = afs_FindDCache(avc, filePos);
-	    if (tdc)
-		ObtainWriteLock(&tdc->lock, 653);
-	} else if (afs_blocksUsed >
-		   PERCENT(CM_WAITFORDRAINPCT, afs_cacheBlocks)) {
-	    tdc = afs_FindDCache(avc, filePos);
-	    if (tdc) {
-		ObtainWriteLock(&tdc->lock, 654);
-		if (!hsame(tdc->f.versionNo, avc->m.DataVersion)
-		    || (tdc->dflags & DFFetching)) {
-		    ReleaseWriteLock(&tdc->lock);
-		    afs_PutDCache(tdc);
-		    tdc = NULL;
-		}
-	    }
-	    if (!tdc) {
-		afs_MaybeWakeupTruncateDaemon();
-		while (afs_blocksUsed >
-		       PERCENT(CM_WAITFORDRAINPCT, afs_cacheBlocks)) {
-		    ReleaseWriteLock(&avc->lock);
-		    if (afs_blocksUsed - afs_blocksDiscarded >
-			PERCENT(CM_WAITFORDRAINPCT, afs_cacheBlocks)) {
-			afs_WaitForCacheDrain = 1;
-			afs_osi_Sleep(&afs_WaitForCacheDrain);
-		    }
-		    afs_MaybeFreeDiscardedDCache();
-		    afs_MaybeWakeupTruncateDaemon();
-		    ObtainWriteLock(&avc->lock, 506);
-		}
-		avc->states |= CDirty;
-		tdc = afs_GetDCache(avc, filePos, &treq, &offset, &len, 4);
-		if (tdc)
-		    ObtainWriteLock(&tdc->lock, 655);
-	    }
-	} else {
-	    tdc = afs_GetDCache(avc, filePos, &treq, &offset, &len, 4);
-	    if (tdc)
-		ObtainWriteLock(&tdc->lock, 656);
-	}
+	tdc = afs_ObtainDCacheForWriting(avc, filePos, totalLength, &treq, 
+					 noLock);
 	if (!tdc) {
 	    error = EIO;
 	    break;
 	}
-	if (!(afs_indexFlags[tdc->index] & IFDataMod)) {
-	    afs_stats_cmperf.cacheCurrDirtyChunks++;
-	    afs_indexFlags[tdc->index] |= IFDataMod;	/* so it doesn't disappear */
-	}
-	if (!(tdc->f.states & DWriting)) {
-	    /* don't mark entry as mod if we don't have to */
-	    tdc->f.states |= DWriting;
-	    tdc->dflags |= DFEntryMod;
-	}
+
 	len = totalLength;	/* write this amount by default */
 	offset = filePos - AFS_CHUNKTOBASE(tdc->f.chunk);
 	max = AFS_CHUNKTOSIZE(tdc->f.chunk);	/* max size of this chunk */
@@ -276,12 +228,12 @@ afs_MemWrite(register struct vcache *avc, struct uio *auio, int aio,
 #endif
 	AFS_UIO_SETOFFSET(tuiop, offset);
 
-	code = afs_MemWriteUIO(tdc->f.inode, tuiop);
+	code = afs_MemWriteUIO(&tdc->f.inode, tuiop);
 	if (code) {
 	    void *mep;		/* XXX in prototype world is struct memCacheEntry * */
 	    error = code;
 	    ZapDCE(tdc);	/* bad data */
-	    mep = afs_MemCacheOpen(tdc->f.inode);
+	    mep = afs_MemCacheOpen(&tdc->f.inode);
 	    afs_MemCacheTruncate(mep, 0);
 	    afs_MemCacheClose(mep);
 	    afs_stats_cmperf.cacheCurrDirtyChunks--;
@@ -306,14 +258,18 @@ afs_MemWrite(register struct vcache *avc, struct uio *auio, int aio,
 	filePos += len;
 #if defined(AFS_SGI_ENV)
 	/* afs_xwrite handles setting m.Length */
-	osi_Assert(filePos <= avc->m.Length);
+	osi_Assert(filePos <= avc->f.m.Length);
 #else
-	if (filePos > avc->m.Length) {
+	if (filePos > avc->f.m.Length) {
+#if defined(AFS_DISCON_ENV)
+	    if (AFS_IS_DISCON_RW)
+   		afs_PopulateDCache(avc, filePos, &treq);
+#endif
 	    afs_Trace4(afs_iclSetp, CM_TRACE_SETLENGTH, ICL_TYPE_STRING,
 		       __FILE__, ICL_TYPE_LONG, __LINE__, ICL_TYPE_OFFSET,
-		       ICL_HANDLE_OFFSET(avc->m.Length), ICL_TYPE_OFFSET,
+		       ICL_HANDLE_OFFSET(avc->f.m.Length), ICL_TYPE_OFFSET,
 		       ICL_HANDLE_OFFSET(filePos));
-	    avc->m.Length = filePos;
+	    avc->f.m.Length = filePos;
 	}
 #endif
 	ReleaseWriteLock(&tdc->lock);
@@ -352,7 +308,7 @@ afs_MemWrite(register struct vcache *avc, struct uio *auio, int aio,
 /* called on writes */
 int
 afs_UFSWrite(register struct vcache *avc, struct uio *auio, int aio,
-	     struct AFS_UCRED *acred, int noLock)
+	     afs_ucred_t *acred, int noLock)
 {
     afs_size_t totalLength;
     afs_size_t transferLength;
@@ -382,6 +338,9 @@ afs_UFSWrite(register struct vcache *avc, struct uio *auio, int aio,
     if (avc->vc_error)
 	return avc->vc_error;
 
+    if (AFS_IS_DISCONNECTED && !AFS_IS_DISCON_RW)
+	return ENETDOWN;
+    
     startDate = osi_Time();
     if ((code = afs_InitReq(&treq, acred)))
 	return code;
@@ -393,7 +352,7 @@ afs_UFSWrite(register struct vcache *avc, struct uio *auio, int aio,
     afs_Trace4(afs_iclSetp, CM_TRACE_WRITE, ICL_TYPE_POINTER, avc,
 	       ICL_TYPE_OFFSET, ICL_HANDLE_OFFSET(filePos), ICL_TYPE_OFFSET,
 	       ICL_HANDLE_OFFSET(totalLength), ICL_TYPE_OFFSET,
-	       ICL_HANDLE_OFFSET(avc->m.Length));
+	       ICL_HANDLE_OFFSET(avc->f.m.Length));
     if (!noLock) {
 	afs_MaybeWakeupTruncateDaemon();
 	ObtainWriteLock(&avc->lock, 556);
@@ -407,8 +366,8 @@ afs_UFSWrite(register struct vcache *avc, struct uio *auio, int aio,
 	 * Since we are called via strategy, we need to trim the write to
 	 * the actual size of the file
 	 */
-	osi_Assert(filePos <= avc->m.Length);
-	diff = avc->m.Length - filePos;
+	osi_Assert(filePos <= avc->f.m.Length);
+	diff = avc->f.m.Length - filePos;
 	AFS_UIO_SETRESID(auio, MIN(totalLength, diff));
 	totalLength = AFS_UIO_RESID(auio);
     }
@@ -418,15 +377,15 @@ afs_UFSWrite(register struct vcache *avc, struct uio *auio, int aio,
 #if     defined(AFS_SUN56_ENV)
 	auio->uio_loffset = 0;
 #endif
-	filePos = avc->m.Length;
-	AFS_UIO_SETOFFSET(auio, avc->m.Length);
+	filePos = avc->f.m.Length;
+	AFS_UIO_SETOFFSET(auio, avc->f.m.Length);
     }
 #endif
     /*
      * Note that we use startDate rather than calling osi_Time() here.
      * This is to avoid counting lock-waiting time in file date (for ranlib).
      */
-    avc->m.Date = startDate;
+    avc->f.m.Date = startDate;
 
 #if	defined(AFS_HPUX_ENV)
 #if 	defined(AFS_HPUX101_ENV)
@@ -446,79 +405,24 @@ afs_UFSWrite(register struct vcache *avc, struct uio *auio, int aio,
      * high-level write op.
      */
     if (avc->execsOrWriters <= 0) {
-	printf("WARNING: afs_ufswr vcp=%lx, exOrW=%d\n", (unsigned long)avc,
+	afs_warn("WARNING: afs_ufswr vcp=%lx, exOrW=%d\n", (unsigned long)avc,
 	       avc->execsOrWriters);
     }
 #else
     afs_FakeOpen(avc);
 #endif
-    avc->states |= CDirty;
+    avc->f.states |= CDirty;
 #ifndef AFS_DARWIN80_ENV
     tvec = (struct iovec *)osi_AllocSmallSpace(sizeof(struct iovec));
 #endif
     while (totalLength > 0) {
-	/* 
-	 *  The following line is necessary because afs_GetDCache with
-	 *  flag == 4 expects the length field to be filled. It decides
-	 *  from this whether it's necessary to fetch data into the chunk
-	 *  before writing or not (when the whole chunk is overwritten!).
-	 */
-	len = totalLength;	/* write this amount by default */
-	/* read the cached info */
-	if (noLock) {
-	    tdc = afs_FindDCache(avc, filePos);
-	    if (tdc)
-		ObtainWriteLock(&tdc->lock, 657);
-	} else if (afs_blocksUsed >
-		   PERCENT(CM_WAITFORDRAINPCT, afs_cacheBlocks)) {
-	    tdc = afs_FindDCache(avc, filePos);
-	    if (tdc) {
-		ObtainWriteLock(&tdc->lock, 658);
-		if (!hsame(tdc->f.versionNo, avc->m.DataVersion)
-		    || (tdc->dflags & DFFetching)) {
-		    ReleaseWriteLock(&tdc->lock);
-		    afs_PutDCache(tdc);
-		    tdc = NULL;
-		}
-	    }
-	    if (!tdc) {
-		afs_MaybeWakeupTruncateDaemon();
-		while (afs_blocksUsed >
-		       PERCENT(CM_WAITFORDRAINPCT, afs_cacheBlocks)) {
-		    ReleaseWriteLock(&avc->lock);
-		    if (afs_blocksUsed - afs_blocksDiscarded >
-			PERCENT(CM_WAITFORDRAINPCT, afs_cacheBlocks)) {
-			afs_WaitForCacheDrain = 1;
-			afs_osi_Sleep(&afs_WaitForCacheDrain);
-		    }
-		    afs_MaybeFreeDiscardedDCache();
-		    afs_MaybeWakeupTruncateDaemon();
-		    ObtainWriteLock(&avc->lock, 509);
-		}
-		avc->states |= CDirty;
-		tdc = afs_GetDCache(avc, filePos, &treq, &offset, &len, 4);
-		if (tdc)
-		    ObtainWriteLock(&tdc->lock, 659);
-	    }
-	} else {
-	    tdc = afs_GetDCache(avc, filePos, &treq, &offset, &len, 4);
-	    if (tdc)
-		ObtainWriteLock(&tdc->lock, 660);
-	}
+	tdc = afs_ObtainDCacheForWriting(avc, filePos, totalLength, &treq, 
+					 noLock);
 	if (!tdc) {
 	    error = EIO;
 	    break;
 	}
-	if (!(afs_indexFlags[tdc->index] & IFDataMod)) {
-	    afs_stats_cmperf.cacheCurrDirtyChunks++;
-	    afs_indexFlags[tdc->index] |= IFDataMod;	/* so it doesn't disappear */
-	}
-	if (!(tdc->f.states & DWriting)) {
-	    /* don't mark entry as mod if we don't have to */
-	    tdc->f.states |= DWriting;
-	    tdc->dflags |= DFEntryMod;
-	}
-	tfile = (struct osi_file *)osi_UFSOpen(tdc->f.inode);
+	tfile = (struct osi_file *)osi_UFSOpen(&tdc->f.inode);
 	len = totalLength;	/* write this amount by default */
 	offset = filePos - AFS_CHUNKTOBASE(tdc->f.chunk);
 	max = AFS_CHUNKTOSIZE(tdc->f.chunk);	/* max size of this chunk */
@@ -574,22 +478,12 @@ afs_UFSWrite(register struct vcache *avc, struct uio *auio, int aio,
 		("\n\n\n*** Cache partition is full - decrease cachesize!!! ***\n\n\n");
 #elif defined(AFS_SGI_ENV)
 	AFS_GUNLOCK();
-	avc->states |= CWritingUFS;
+	avc->f.states |= CWritingUFS;
 	AFS_VOP_RWLOCK(tfile->vnode, VRWLOCK_WRITE);
 	AFS_VOP_WRITE(tfile->vnode, &tuio, IO_ISLOCKED, afs_osi_credp, code);
 	AFS_VOP_RWUNLOCK(tfile->vnode, VRWLOCK_WRITE);
-	avc->states &= ~CWritingUFS;
+	avc->f.states &= ~CWritingUFS;
 	AFS_GLOCK();
-#elif defined(AFS_OSF_ENV)
-	{
-	    struct ucred *tmpcred = u.u_cred;
-	    u.u_cred = afs_osi_credp;
-	    tuio.uio_rw = UIO_WRITE;
-	    AFS_GUNLOCK();
-	    VOP_WRITE(tfile->vnode, &tuio, 0, afs_osi_credp, code);
-	    AFS_GLOCK();
-	    u.u_cred = tmpcred;
-	}
 #elif defined(AFS_HPUX100_ENV)
 	{
 	    AFS_GUNLOCK();
@@ -610,6 +504,12 @@ afs_UFSWrite(register struct vcache *avc, struct uio *auio, int aio,
 	code = VOP_WRITE(tfile->vnode, &tuio, 0, afs_osi_credp);
 	VOP_UNLOCK(tfile->vnode, 0, current_proc());
 	AFS_GLOCK();
+#elif defined(AFS_FBSD80_ENV)
+	AFS_GUNLOCK();
+	VOP_LOCK(tfile->vnode, LK_EXCLUSIVE);
+	code = VOP_WRITE(tfile->vnode, &tuio, 0, afs_osi_credp);
+	VOP_UNLOCK(tfile->vnode, 0);
+	AFS_GLOCK();
 #elif defined(AFS_FBSD50_ENV)
 	AFS_GUNLOCK();
 	VOP_LOCK(tfile->vnode, LK_EXCLUSIVE, curthread);
@@ -656,14 +556,18 @@ afs_UFSWrite(register struct vcache *avc, struct uio *auio, int aio,
 	filePos += len;
 #if defined(AFS_SGI_ENV)
 	/* afs_xwrite handles setting m.Length */
-	osi_Assert(filePos <= avc->m.Length);
+	osi_Assert(filePos <= avc->f.m.Length);
 #else
-	if (filePos > avc->m.Length) {
+	if (filePos > avc->f.m.Length) {
+#if defined(AFS_DISCON_ENV)
+	    if (AFS_IS_DISCON_RW)
+		afs_PopulateDCache(avc, filePos, &treq);
+#endif
 	    afs_Trace4(afs_iclSetp, CM_TRACE_SETLENGTH, ICL_TYPE_STRING,
 		       __FILE__, ICL_TYPE_LONG, __LINE__, ICL_TYPE_OFFSET,
-		       ICL_HANDLE_OFFSET(avc->m.Length), ICL_TYPE_OFFSET,
+		       ICL_HANDLE_OFFSET(avc->f.m.Length), ICL_TYPE_OFFSET,
 		       ICL_HANDLE_OFFSET(filePos));
-	    avc->m.Length = filePos;
+	    avc->f.m.Length = filePos;
 	}
 #endif
 	osi_UFSClose(tfile);
@@ -729,11 +633,13 @@ afs_DoPartialWrite(register struct vcache *avc, struct vrequest *areq)
     register afs_int32 code;
 
     if (afs_stats_cmperf.cacheCurrDirtyChunks <=
-	afs_stats_cmperf.cacheMaxDirtyChunks)
+	afs_stats_cmperf.cacheMaxDirtyChunks
+	|| AFS_IS_DISCONNECTED)
 	return 0;		/* nothing to do */
     /* otherwise, call afs_StoreDCache (later try to do this async, if possible) */
     afs_Trace2(afs_iclSetp, CM_TRACE_PARTIALWRITE, ICL_TYPE_POINTER, avc,
-	       ICL_TYPE_OFFSET, ICL_HANDLE_OFFSET(avc->m.Length));
+	       ICL_TYPE_OFFSET, ICL_HANDLE_OFFSET(avc->f.m.Length));
+
 #if	defined(AFS_SUN5_ENV)
     code = afs_StoreAllSegments(avc, areq, AFS_ASYNC | AFS_VMSYNC_INVAL);
 #else
@@ -742,105 +648,23 @@ afs_DoPartialWrite(register struct vcache *avc, struct vrequest *areq)
     return code;
 }
 
-#ifdef AFS_OSF_ENV
-#ifdef AFS_DUX50_ENV
-#define vno_close(X) vn_close((X), 0, NOCRED)
-#elif defined(AFS_DUX40_ENV)
-#define      vno_close       vn_close
-#endif
-/* We don't need this for AIX since: 
- * (1) aix doesn't use fileops and it call close directly intead
- * (where the unlocking should be done) and 
- * (2) temporarily, the aix lockf isn't supported yet.
- *
- *  this stupid routine is used to release the flocks held on a
- *  particular file descriptor.  Sun doesn't pass file descr. info
- *  through to the vnode layer, and yet we must unlock flocked files
- *  on the *appropriate* (not first, as in System V) close call.  Thus
- *  this code.
- * How does this code get invoked? The afs AFS_FLOCK plugs in the new afs
- * file ops structure into any afs file when it gets flocked. 
- * N.B: Intercepting close syscall doesn't trap aborts or exit system
- * calls.
-*/
-int
-afs_closex(register struct file *afd)
-{
-    struct vrequest treq;
-    struct vcache *tvc;
-    afs_int32 flags;
-    int closeDone;
-    afs_int32 code = 0;
-    struct afs_fakestat_state fakestat;
-
-    AFS_STATCNT(afs_closex);
-    /* setup the credentials */
-    if ((code = afs_InitReq(&treq, u.u_cred)))
-	return code;
-    afs_InitFakeStat(&fakestat);
-
-    closeDone = 0;
-    /* we're the last one.  If we're an AFS vnode, clear the flags,
-     * close the file and release the lock when done.  Otherwise, just
-     * let the regular close code work.      */
-    if (afd->f_type == DTYPE_VNODE) {
-	tvc = VTOAFS(afd->f_data);
-	if (IsAfsVnode(AFSTOV(tvc))) {
-	    code = afs_EvalFakeStat(&tvc, &fakestat, &treq);
-	    if (code) {
-		afs_PutFakeStat(&fakestat);
-		return code;
-	    }
-	    VN_HOLD(AFSTOV(tvc));
-	    flags = afd->f_flag & (FSHLOCK | FEXLOCK);
-	    afd->f_flag &= ~(FSHLOCK | FEXLOCK);
-	    code = vno_close(afd);
-	    if (flags)
-		HandleFlock(tvc, LOCK_UN, &treq, u.u_procp->p_pid,
-			    1 /*onlymine */ );
-	    AFS_RELE(AFSTOV(tvc));
-	    closeDone = 1;
-	}
-    }
-    /* now, if close not done, do it */
-    if (!closeDone) {
-	code = vno_close(afd);
-    }
-    afs_PutFakeStat(&fakestat);
-    return code;		/* return code from vnode layer */
-}
-#endif
-
-
 /* handle any closing cleanup stuff */
 int
-#ifdef	AFS_SGI_ENV
-afs_close(OSI_VC_ARG(avc), aflags, lastclose,
-#if !defined(AFS_SGI65_ENV)
-	  offset,
-#endif
-	  acred
-#if defined(AFS_SGI64_ENV) && !defined(AFS_SGI65_ENV)
-	  , flp
-#endif
-    )
-     lastclose_t lastclose;
-#if !defined(AFS_SGI65_ENV)
-     off_t offset;
-#if defined(AFS_SGI64_ENV)
-     struct flid *flp;
-#endif
-#endif
+#if defined(AFS_SGI65_ENV)
+afs_close(OSI_VC_DECL(avc), afs_int32 aflags, lastclose_t lastclose,
+	  afs_ucred_t *acred)
+#elif defined(AFS_SGI64_ENV)
+afs_close(OSI_VC_DECL(avc), afs_int32 aflags, lastclose_t lastclose,
+	  off_t offset, afs_ucred_t *acred, struct flid *flp)
+#elif defined(AFS_SGI_ENV)
+afs_close(OSI_VC_DECL(avc), afs_int32 aflags, lastclose_t lastclose
+	  off_t offset, afs_ucred_t *acred)
 #elif defined(AFS_SUN5_ENV)
-afs_close(OSI_VC_ARG(avc), aflags, count, offset, acred)
-     offset_t offset;
-     int count;
+afs_close(OSI_VC_DECL(avc), afs_int32 aflags, int count, offset_t offset, 
+	 afs_ucred_t *acred)
 #else
-afs_close(OSI_VC_ARG(avc), aflags, acred)
+afs_close(OSI_VC_DECL(avc), afs_int32 aflags, afs_ucred_t *acred)
 #endif
-     OSI_VC_DECL(avc);
-     afs_int32 aflags;
-     struct AFS_UCRED *acred;
 {
     register afs_int32 code;
     register struct brequest *tb;
@@ -863,6 +687,7 @@ afs_close(OSI_VC_ARG(avc), aflags, acred)
 	afs_PutFakeStat(&fakestat);
 	return code;
     }
+    AFS_DISCON_LOCK();
 #ifdef	AFS_SUN5_ENV
     if (avc->flockCount) {
 	HandleFlock(avc, LOCK_UN, &treq, 0, 1 /*onlymine */ );
@@ -871,41 +696,41 @@ afs_close(OSI_VC_ARG(avc), aflags, acred)
 #if defined(AFS_SGI_ENV)
     if (!lastclose) {
 	afs_PutFakeStat(&fakestat);
+        AFS_DISCON_UNLOCK();
 	return 0;
     }
     /* unlock any locks for pid - could be wrong for child .. */
     AFS_RWLOCK((vnode_t *) avc, VRWLOCK_WRITE);
-#ifdef AFS_SGI65_ENV
+# ifdef AFS_SGI65_ENV
     get_current_flid(&flid);
     cleanlocks((vnode_t *) avc, flid.fl_pid, flid.fl_sysid);
     HandleFlock(avc, LOCK_UN, &treq, flid.fl_pid, 1 /*onlymine */ );
-#else
-#ifdef AFS_SGI64_ENV
+# else
+#  ifdef AFS_SGI64_ENV
     cleanlocks((vnode_t *) avc, flp);
-#else /* AFS_SGI64_ENV */
+#  else /* AFS_SGI64_ENV */
     cleanlocks((vnode_t *) avc, u.u_procp->p_epid, u.u_procp->p_sysid);
-#endif /* AFS_SGI64_ENV */
+#  endif /* AFS_SGI64_ENV */
     HandleFlock(avc, LOCK_UN, &treq, OSI_GET_CURRENT_PID(), 1 /*onlymine */ );
-#endif /* AFS_SGI65_ENV */
+# endif /* AFS_SGI65_ENV */
     /* afs_chkpgoob will drop and re-acquire the global lock. */
-    afs_chkpgoob(&avc->v, btoc(avc->m.Length));
-#elif	defined(AFS_SUN5_ENV)
+    afs_chkpgoob(&avc->v, btoc(avc->f.m.Length));
+#elif defined(AFS_SUN5_ENV)
     if (count > 1) {
-	/* The vfs layer may call this repeatedly with higher "count"; only on the last close (i.e. count = 1) we should actually proceed with the close. */
+	/* The vfs layer may call this repeatedly with higher "count"; only
+	 * on the last close (i.e. count = 1) we should actually proceed
+	 * with the close. */
 	afs_PutFakeStat(&fakestat);
+	AFS_DISCON_UNLOCK();
 	return 0;
     }
-#else /* AFS_SGI_ENV */
-    if (avc->flockCount) {	/* Release Lock */
-#if	defined(AFS_OSF_ENV) 
-	HandleFlock(avc, LOCK_UN, &treq, u.u_procp->p_pid, 1 /*onlymine */ );
 #else
+    if (avc->flockCount) {	/* Release Lock */
 	HandleFlock(avc, LOCK_UN, &treq, 0, 1 /*onlymine */ );
-#endif
     }
-#endif /* AFS_SGI_ENV */
+#endif
     if (aflags & (FWRITE | FTRUNC)) {
-	if (afs_BBusy() || (AFS_NFSXLATORREQ(acred))) {
+	if (afs_BBusy() || (AFS_NFSXLATORREQ(acred)) || AFS_IS_DISCONNECTED) {
 	    /* do it yourself if daemons are all busy */
 	    ObtainWriteLock(&avc->lock, 124);
 	    code = afs_StoreOnLastReference(avc, &treq);
@@ -920,8 +745,8 @@ afs_close(OSI_VC_ARG(avc), aflags, acred)
 	    /* at least one daemon is idle, so ask it to do the store.
 	     * Also, note that  we don't lock it any more... */
 	    tb = afs_BQueue(BOP_STORE, avc, 0, 1, acred,
-			    (afs_size_t) acred->cr_uid, (afs_size_t) 0,
-			    (void *)0);
+			    (afs_size_t) afs_cr_uid(acred), (afs_size_t) 0,
+			    (void *)0, (void *)0, (void *)0);
 	    /* sleep waiting for the store to start, then retrieve error code */
 	    while ((tb->flags & BUVALID) == 0) {
 		tb->flags |= BUWAIT;
@@ -946,6 +771,7 @@ afs_close(OSI_VC_ARG(avc), aflags, acred)
 #ifdef AFS_AIX32_ENV
 	    osi_ReleaseVM(avc, acred);
 #endif
+	    /* printf("avc->vc_error=%d\n", avc->vc_error); */
 	    code = avc->vc_error;
 	    avc->vc_error = 0;
 	}
@@ -990,11 +816,7 @@ afs_close(OSI_VC_ARG(avc), aflags, acred)
 	avc->opens--;
 	ReleaseWriteLock(&avc->lock);
     }
-#ifdef	AFS_OSF_ENV
-    if ((VREFCOUNT(avc) <= 2) && (avc->states & CUnlinked)) {
-	afs_remunlink(avc, 1);	/* ignore any return code */
-    }
-#endif
+    AFS_DISCON_UNLOCK();
     afs_PutFakeStat(&fakestat);
     code = afs_CheckCode(code, &treq, 5);
     return code;
@@ -1002,19 +824,15 @@ afs_close(OSI_VC_ARG(avc), aflags, acred)
 
 
 int
-#ifdef	AFS_OSF_ENV
-afs_fsync(OSI_VC_DECL(avc), int fflags, struct AFS_UCRED *acred, int waitfor)
-#else				/* AFS_OSF_ENV */
 #if defined(AFS_SGI_ENV) || defined(AFS_SUN53_ENV)
-afs_fsync(OSI_VC_DECL(avc), int flag, struct AFS_UCRED *acred
-#ifdef AFS_SGI65_ENV
+afs_fsync(OSI_VC_DECL(avc), int flag, afs_ucred_t *acred
+# ifdef AFS_SGI65_ENV
 	  , off_t start, off_t stop
-#endif /* AFS_SGI65_ENV */
+# endif /* AFS_SGI65_ENV */
     )
-#else /* !OSF && !SUN53 && !SGI */
-afs_fsync(OSI_VC_DECL(avc), struct AFS_UCRED *acred)
+#else /* !SUN53 && !SGI */
+afs_fsync(OSI_VC_DECL(avc), afs_ucred_t *acred)
 #endif 
-#endif
 {
     register afs_int32 code;
     struct vrequest treq;
@@ -1033,7 +851,7 @@ afs_fsync(OSI_VC_DECL(avc), struct AFS_UCRED *acred)
     afs_Trace1(afs_iclSetp, CM_TRACE_FSYNC, ICL_TYPE_POINTER, avc);
     if ((code = afs_InitReq(&treq, acred)))
 	return code;
-
+    AFS_DISCON_LOCK();
 #if defined(AFS_SGI_ENV)
     AFS_RWLOCK((vnode_t *) avc, VRWLOCK_WRITE);
     if (flag & FSYNC_INVAL)
@@ -1043,11 +861,25 @@ afs_fsync(OSI_VC_DECL(avc), struct AFS_UCRED *acred)
     ObtainSharedLock(&avc->lock, 18);
     code = 0;
     if (avc->execsOrWriters > 0) {
-	/* put the file back */
-	UpgradeSToWLock(&avc->lock, 41);
-	code = afs_StoreAllSegments(avc, &treq, AFS_SYNC);
-	ConvertWToSLock(&avc->lock);
-    }
+
+    	if (!AFS_IS_DISCONNECTED && !AFS_IS_DISCON_RW) {
+		/* Your average flush. */
+
+		/* put the file back */
+		UpgradeSToWLock(&avc->lock, 41);
+		code = afs_StoreAllSegments(avc, &treq, AFS_SYNC);
+		ConvertWToSLock(&avc->lock);
+
+#if defined(AFS_DISCON_ENV)
+	} else {
+
+	    UpgradeSToWLock(&avc->lock, 711);
+	    afs_DisconAddDirty(avc, VDisconWriteFlush, 1);
+	    ConvertWToSLock(&avc->lock);
+#endif
+    	}		/* if not disconnected */
+    }			/* if (avc->execsOrWriters > 0) */
+
 #if defined(AFS_SGI_ENV)
     AFS_RWUNLOCK((vnode_t *) avc, VRWLOCK_WRITE);
     if (code == VNOVNODE) {
@@ -1057,7 +889,7 @@ afs_fsync(OSI_VC_DECL(avc), struct AFS_UCRED *acred)
 	code = ENOENT;
     }
 #endif
-
+    AFS_DISCON_UNLOCK();
     code = afs_CheckCode(code, &treq, 33);
     ReleaseSharedLock(&avc->lock);
     return code;
