@@ -13,8 +13,6 @@
 #include <afsconfig.h>
 #include "afs/param.h"
 
-RCSID
-    ("$Header: /cvs/openafs/src/afs/afs_analyze.c,v 1.22.14.8 2008/06/29 03:39:12 shadow Exp $");
 
 #include "afs/stds.h"
 #include "afs/sysincludes.h"	/* Standard vendor system headers */
@@ -28,7 +26,7 @@ RCSID
 #ifdef AFS_SGI62_ENV
 #include "h/hashing.h"
 #endif
-#if !defined(AFS_HPUX110_ENV) && !defined(AFS_LINUX20_ENV) && !defined(AFS_FBSD_ENV) && !defined(AFS_DARWIN60_ENV)
+#if !defined(AFS_HPUX110_ENV) && !defined(AFS_LINUX20_ENV) && !defined(AFS_FBSD_ENV) && !defined(AFS_DARWIN_ENV)
 #include <netinet/in_var.h>
 #endif
 #endif /* !UKERNEL */
@@ -93,7 +91,7 @@ static int
 VLDB_Same(struct VenusFid *afid, struct vrequest *areq)
 {
     struct vrequest treq;
-    struct conn *tconn;
+    struct afs_conn *tconn;
     int i, type = 0;
     union {
 	struct vldbentry tve;
@@ -250,26 +248,27 @@ afs_BlackListOnce(struct vrequest *areq, struct VenusFid *afid,
     if (afid) {
 	tvp = afs_FindVolume(afid, READ_LOCK);
 	if (tvp) {
-	    for (i = 0; i < MAXHOSTS; i++) {
+	    for (i = 0; i < AFS_MAXHOSTS; i++) {
 		if (tvp->serverHost[i] == tsp) {
 		    areq->skipserver[i] = 1;
 		}
 		if (tvp->serverHost[i] &&
-		    !(tvp->serverHost[i]->addr->sa_flags & 
+		    (tvp->serverHost[i]->addr->sa_flags & 
 		      SRVR_ISDOWN)) {
 		    areq->skipserver[i] = 1;
 		}
 	    }
 	    afs_PutVolume(tvp, READ_LOCK);
+	    for (i = 0; i < AFS_MAXHOSTS; i++) {
+	        if (tvp->serverHost[i] && areq->skipserver[i] == 0) {
+		    serversleft = 1;
+		    break;
+		}
+	    }
+	    return serversleft;
 	}
     }
-    for (i = 0; i < MAXHOSTS; i++) {
-	if (areq->skipserver[i] == 0) {
-	    serversleft = 1;
-	    break;
-	}
-    }
-    return serversleft;
+    return 1;
 }
 
 
@@ -306,22 +305,24 @@ afs_BlackListOnce(struct vrequest *areq, struct VenusFid *afid,
  *	if this is a temporary or permanent error.
  *------------------------------------------------------------------------*/
 int
-afs_Analyze(register struct conn *aconn, afs_int32 acode,
+afs_Analyze(register struct afs_conn *aconn, afs_int32 acode,
 	    struct VenusFid *afid, register struct vrequest *areq, int op,
 	    afs_int32 locktype, struct cell *cellp)
 {
     afs_int32 i;
     struct srvAddr *sa;
     struct server *tsp;
-    struct volume *tvp;
+    struct volume *tvp = NULL;
     afs_int32 shouldRetry = 0;
     afs_int32 serversleft = 1;
     struct afs_stats_RPCErrors *aerrP;
     afs_int32 markeddown;
 
  
-    if (AFS_IS_DISCONNECTED) {
-	/* SXW - This may get very tired after a while. We should try and
+ 
+    if (AFS_IS_DISCONNECTED && !AFS_IN_SYNC) {
+	/* On reconnection, act as connected. XXX: for now.... */
+        /* SXW - This may get very tired after a while. We should try and
 	 *       intercept all RPCs before they get here ... */
 	/*printf("afs_Analyze: disconnected\n");*/
 	afs_FinalizeReq(areq);
@@ -332,7 +333,7 @@ afs_Analyze(register struct conn *aconn, afs_int32 acode,
 	}
 	return 0;
     }
-  
+    
     AFS_STATCNT(afs_Analyze);
     afs_Trace4(afs_iclSetp, CM_TRACE_ANALYZE, ICL_TYPE_INT32, op,
 	       ICL_TYPE_POINTER, aconn, ICL_TYPE_INT32, acode, ICL_TYPE_LONG,
@@ -355,7 +356,7 @@ afs_Analyze(register struct conn *aconn, afs_int32 acode,
 			   && tvp->serverHost[0]->cell) ? tvp->serverHost[0]->
 			  cell->cellName : ""));
 
-	    for (i = 0; i < MAXHOSTS; i++) {
+	    for (i = 0; i < AFS_MAXHOSTS; i++) {
 		if (tvp->status[i] != not_busy && tvp->status[i] != offline) {
 		    tvp->status[i] = not_busy;
 		}
@@ -382,7 +383,7 @@ afs_Analyze(register struct conn *aconn, afs_int32 acode,
 	return shouldRetry;	/* should retry */
     }
 
-    if (!aconn) {
+    if (!aconn || !aconn->srvr) {
 	if (!areq->volumeError) {
 	    if (aerrP)
 		(aerrP->err_Network)++;
@@ -434,7 +435,7 @@ afs_Analyze(register struct conn *aconn, afs_int32 acode,
 	if (areq->volumeError) {
 	    tvp = afs_FindVolume(afid, READ_LOCK);
 	    if (tvp) {
-		for (i = 0; i < MAXHOSTS; i++) {
+		for (i = 0; i < AFS_MAXHOSTS; i++) {
 		    if (tvp->serverHost[i] == tsp) {
 			tvp->status[i] = not_busy;
 		    }
@@ -457,12 +458,18 @@ afs_Analyze(register struct conn *aconn, afs_int32 acode,
     if ((acode < 0) && (acode != VRESTARTING)) {
 	if (acode == RX_CALL_TIMEOUT) {
 	    serversleft = afs_BlackListOnce(areq, afid, tsp);
-	    areq->idleError++;
-	    if (serversleft) {
-		shouldRetry = 1;
-	    } else {
+	    if (afid)
+		tvp = afs_FindVolume(afid, READ_LOCK);
+	    if (!afid || !tvp || (tvp->states & VRO))
+		areq->idleError++;
+	    if ((serversleft == 0) && tvp &&
+		((tvp->states & VRO) || (tvp->states & VBackup))) {
 		shouldRetry = 0;
+	    } else {
+		shouldRetry = 1;
 	    }
+	    if (tvp)
+		afs_PutVolume(tvp, READ_LOCK);
 	    /* By doing this, we avoid ever marking a server down
 	     * in an idle timeout case. That's because the server is 
 	     * still responding and may only be letting a single vnode
@@ -494,7 +501,7 @@ afs_Analyze(register struct conn *aconn, afs_int32 acode,
 
 	tvp = afs_FindVolume(afid, READ_LOCK);
 	if (tvp) {
-	    for (i = 0; i < MAXHOSTS; i++) {
+	    for (i = 0; i < AFS_MAXHOSTS; i++) {
 		if (tvp->serverHost[i] == tsp) {
 		    tvp->status[i] = rdwr_busy;	/* can't tell which yet */
 		    /* to tell which, have to look at the op code. */
@@ -526,6 +533,7 @@ afs_Analyze(register struct conn *aconn, afs_int32 acode,
 	    } else if (acode == RXKADEXPIRED) {
 		aconn->forceConnectFS = 0;	/* don't check until new tokens set */
 		aconn->user->states |= UTokensBad;
+		afs_NotifyUser(tu, UTokensDropped);
 		afs_warnuser
 		    ("afs: Tokens for user of AFS id %d for cell %s have expired\n",
 		     tu->vid, aconn->srvr->server->cell->cellName);
@@ -542,6 +550,7 @@ afs_Analyze(register struct conn *aconn, afs_int32 acode,
 		    areq->tokenError = 0;
 		    aconn->forceConnectFS = 0;	/* don't check until new tokens set */
 		    aconn->user->states |= UTokensBad;
+		    afs_NotifyUser(tu, UTokensDropped);
 		    afs_warnuser
 			("afs: Tokens for user of AFS id %d for cell %s are discarded (rxkad error=%d)\n",
 			 tu->vid, aconn->srvr->server->cell->cellName, acode);
@@ -555,12 +564,14 @@ afs_Analyze(register struct conn *aconn, afs_int32 acode,
 	    } else if (acode == RXKADEXPIRED) {
 		aconn->forceConnectFS = 0;	/* don't check until new tokens set */
 		aconn->user->states |= UTokensBad;
+		afs_NotifyUser(tu, UTokensDropped);
 		afs_warnuser
 		    ("afs: Tokens for user %d for cell %s have expired\n",
 		     areq->uid, aconn->srvr->server->cell->cellName);
 	    } else {
 		aconn->forceConnectFS = 0;	/* don't check until new tokens set */
 		aconn->user->states |= UTokensBad;
+		afs_NotifyUser(tu, UTokensDropped);
 		afs_warnuser
 		    ("afs: Tokens for user %d for cell %s are discarded (rxkad error = %d)\n",
 		     areq->uid, aconn->srvr->server->cell->cellName, acode);
@@ -600,7 +611,7 @@ afs_Analyze(register struct conn *aconn, afs_int32 acode,
 	    same = VLDB_Same(afid, areq);
 	    tvp = afs_FindVolume(afid, READ_LOCK);
 	    if (tvp) {
-		for (i = 0; i < MAXHOSTS && tvp->serverHost[i]; i++) {
+		for (i = 0; i < AFS_MAXHOSTS && tvp->serverHost[i]; i++) {
 		    if (tvp->serverHost[i] == tsp) {
 			if (tvp->status[i] == end_not_busy)
 			    tvp->status[i] = offline;
