@@ -16,8 +16,6 @@
 #include <afsconfig.h>
 #include "afs/param.h"
 
-RCSID
-    ("$Header: /cvs/openafs/src/afs/afs_callback.c,v 1.39.2.3 2007/11/23 14:32:56 shadow Exp $");
 
 #include "afs/sysincludes.h"	/*Standard vendor system headers */
 #include "afsincludes.h"	/*AFS-based standard headers */
@@ -62,9 +60,12 @@ static struct ltable {
       "afs_xsrvAddr", (char *)&afs_xsrvAddr},
     {
       "afs_xvreclaim", (char *)&afs_xvreclaim},
-#ifdef AFS_AFSDB_ENV
     { "afsdb_client_lock", (char *)&afsdb_client_lock},
     { "afsdb_req_lock", (char *)&afsdb_req_lock},
+#ifdef AFS_DISCON_ENV
+    { "afs_discon_lock", (char *)&afs_discon_lock},
+    { "afs_disconDirtyLock", (char *)&afs_disconDirtyLock},
+    { "afs_discon_vc_dirty", (char *)&afs_xvcdirty},
 #endif
 };
 unsigned long lastCallBack_vnode;
@@ -131,10 +132,10 @@ SRXAFSCB_GetCE(struct rx_call *a_call, afs_int32 a_index,
      * Copy out the located entry.
      */
     a_result->addr = afs_data_pointer_to_int32(tvc);
-    a_result->cell = tvc->fid.Cell;
-    a_result->netFid.Volume = tvc->fid.Fid.Volume;
-    a_result->netFid.Vnode = tvc->fid.Fid.Vnode;
-    a_result->netFid.Unique = tvc->fid.Fid.Unique;
+    a_result->cell = tvc->f.fid.Cell;
+    a_result->netFid.Volume = tvc->f.fid.Fid.Volume;
+    a_result->netFid.Vnode = tvc->f.fid.Fid.Vnode;
+    a_result->netFid.Unique = tvc->f.fid.Fid.Unique;
     a_result->lock.waitStates = tvc->lock.wait_states;
     a_result->lock.exclLocked = tvc->lock.excl_locked;
     a_result->lock.readersReading = tvc->lock.readers_reading;
@@ -148,16 +149,16 @@ SRXAFSCB_GetCE(struct rx_call *a_call, afs_int32 a_index,
     a_result->lock.pid_last_reader = 0;
     a_result->lock.pid_writer = 0;
     a_result->lock.src_indicator = 0;
-#endif /* AFS_OSF20_ENV */
+#endif /* INSTRUMENT_LOCKS */
 #ifdef AFS_64BIT_CLIENT
-    a_result->Length = (afs_int32) tvc->m.Length & 0xffffffff;
+    a_result->Length = (afs_int32) tvc->f.m.Length & 0xffffffff;
 #else /* AFS_64BIT_CLIENT */
-    a_result->Length = tvc->m.Length;
+    a_result->Length = tvc->f.m.Length;
 #endif /* AFS_64BIT_CLIENT */
-    a_result->DataVersion = hgetlo(tvc->m.DataVersion);
+    a_result->DataVersion = hgetlo(tvc->f.m.DataVersion);
     a_result->callback = afs_data_pointer_to_int32(tvc->callback);	/* XXXX Now a pointer; change it XXXX */
     a_result->cbExpires = tvc->cbExpires;
-    if (tvc->states & CVInit) {
+    if (tvc->f.states & CVInit) {
         a_result->refCount = 1;
     } else {
 #ifdef AFS_DARWIN80_ENV
@@ -169,7 +170,7 @@ SRXAFSCB_GetCE(struct rx_call *a_call, afs_int32 a_index,
     a_result->opens = tvc->opens;
     a_result->writers = tvc->execsOrWriters;
     a_result->mvstat = tvc->mvstat;
-    a_result->states = tvc->states;
+    a_result->states = tvc->f.states;
     code = 0;
 
     /*
@@ -217,10 +218,10 @@ SRXAFSCB_GetCE64(struct rx_call *a_call, afs_int32 a_index,
      * Copy out the located entry.
      */
     a_result->addr = afs_data_pointer_to_int32(tvc);
-    a_result->cell = tvc->fid.Cell;
-    a_result->netFid.Volume = tvc->fid.Fid.Volume;
-    a_result->netFid.Vnode = tvc->fid.Fid.Vnode;
-    a_result->netFid.Unique = tvc->fid.Fid.Unique;
+    a_result->cell = tvc->f.fid.Cell;
+    a_result->netFid.Volume = tvc->f.fid.Fid.Volume;
+    a_result->netFid.Vnode = tvc->f.fid.Fid.Vnode;
+    a_result->netFid.Unique = tvc->f.fid.Fid.Unique;
     a_result->lock.waitStates = tvc->lock.wait_states;
     a_result->lock.exclLocked = tvc->lock.excl_locked;
     a_result->lock.readersReading = tvc->lock.readers_reading;
@@ -234,17 +235,17 @@ SRXAFSCB_GetCE64(struct rx_call *a_call, afs_int32 a_index,
     a_result->lock.pid_last_reader = 0;
     a_result->lock.pid_writer = 0;
     a_result->lock.src_indicator = 0;
-#endif /* AFS_OSF20_ENV */
+#endif /* INSTRUMENT_LOCKS */
 #if !defined(AFS_64BIT_ENV) 
     a_result->Length.high = 0;
-    a_result->Length.low = tvc->m.Length;
+    a_result->Length.low = tvc->f.m.Length;
 #else
-    a_result->Length = tvc->m.Length;
+    a_result->Length = tvc->f.m.Length;
 #endif
-    a_result->DataVersion = hgetlo(tvc->m.DataVersion);
+    a_result->DataVersion = hgetlo(tvc->f.m.DataVersion);
     a_result->callback = afs_data_pointer_to_int32(tvc->callback);	/* XXXX Now a pointer; change it XXXX */
     a_result->cbExpires = tvc->cbExpires;
-    if (tvc->states & CVInit) {
+    if (tvc->f.states & CVInit) {
         a_result->refCount = 1;
     } else {
 #ifdef AFS_DARWIN80_ENV
@@ -256,7 +257,7 @@ SRXAFSCB_GetCE64(struct rx_call *a_call, afs_int32 a_index,
     a_result->opens = tvc->opens;
     a_result->writers = tvc->execsOrWriters;
     a_result->mvstat = tvc->mvstat;
-    a_result->states = tvc->states;
+    a_result->states = tvc->f.states;
     code = 0;
 
     /*
@@ -444,27 +445,27 @@ loop1:
 		for (tq = afs_vhashTV[i].prev; tq != &afs_vhashTV[i]; tq = uq) {
 		    uq = QPrev(tq);
 		    tvc = QTOVH(tq);      
-		    if (tvc->fid.Fid.Volume == a_fid->Volume) {
+		    if (tvc->f.fid.Fid.Volume == a_fid->Volume) {
 			tvc->callback = NULL;
 			if (!localFid.Cell)
-			    localFid.Cell = tvc->fid.Cell;
+			    localFid.Cell = tvc->f.fid.Cell;
 			tvc->dchint = NULL;	/* invalidate hints */
-			if (tvc->states & CVInit) {
+			if (tvc->f.states & CVInit) {
 			    ReleaseReadLock(&afs_xvcache);
-			    afs_osi_Sleep(&tvc->states);
+			    afs_osi_Sleep(&tvc->f.states);
 			    goto loop1;
 			}
-#ifdef AFS_DARWIN80_ENV
-			if (tvc->states & CDeadVnode) {
-			    ReleaseReadLock(&afs_xvcache);
-			    afs_osi_Sleep(&tvc->states);
-			    goto loop1;
-			}
-#endif
-#if     defined(AFS_SGI_ENV) || defined(AFS_OSF_ENV)  || defined(AFS_SUN5_ENV)  || defined(AFS_HPUX_ENV) || defined(AFS_LINUX20_ENV)
+#if     defined(AFS_SGI_ENV) || defined(AFS_SUN5_ENV)  || defined(AFS_HPUX_ENV) || defined(AFS_LINUX20_ENV)
 			VN_HOLD(AFSTOV(tvc));
 #else
 #ifdef AFS_DARWIN80_ENV
+			if (tvc->f.states & CDeadVnode) {
+			    if (!(tvc->f.states & CBulkFetching)) {
+				ReleaseReadLock(&afs_xvcache);
+				afs_osi_Sleep(&tvc->f.states);
+				goto loop1;
+			    }
+			}
 			vp = AFSTOV(tvc);
 			if (vnode_get(vp))
 			    continue;
@@ -474,29 +475,30 @@ loop1:
 			    AFS_GLOCK();
 			    continue;
 			}
+			if (tvc->f.states & (CBulkFetching|CDeadVnode)) {
+			    AFS_GUNLOCK();
+			    vnode_recycle(AFSTOV(tvc));
+			    AFS_GLOCK();
+			}
 #else
-#if defined(AFS_DARWIN_ENV) || defined(AFS_XBSD_ENV)
-			osi_vnhold(tvc, 0);
-#else
-			VREFCOUNT_INC(tvc); /* AIX, apparently */
-#endif
+			AFS_FAST_HOLD(tvc);
 #endif
 #endif
 			ReleaseReadLock(&afs_xvcache);
 			ObtainWriteLock(&afs_xcbhash, 449);
 			afs_DequeueCallback(tvc);
-			tvc->states &= ~(CStatd | CUnique | CBulkFetching);
+			tvc->f.states &= ~(CStatd | CUnique | CBulkFetching);
 			afs_allCBs++;
-			if (tvc->fid.Fid.Vnode & 1)
+			if (tvc->f.fid.Fid.Vnode & 1)
 			    afs_oddCBs++;
 			else
 			    afs_evenCBs++;
 			ReleaseWriteLock(&afs_xcbhash);
-			if ((tvc->fid.Fid.Vnode & 1 || (vType(tvc) == VDIR)))
+			if ((tvc->f.fid.Fid.Vnode & 1 || (vType(tvc) == VDIR)))
 			    osi_dnlc_purgedp(tvc);
 			afs_Trace3(afs_iclSetp, CM_TRACE_CALLBACK,
 				   ICL_TYPE_POINTER, tvc, ICL_TYPE_INT32,
-				   tvc->states, ICL_TYPE_INT32,
+				   tvc->f.states, ICL_TYPE_INT32,
 				   a_fid->Volume);
 #ifdef AFS_DARWIN80_ENV
 			vnode_put(AFSTOV(tvc));
@@ -504,9 +506,9 @@ loop1:
 			ObtainReadLock(&afs_xvcache);
 			uq = QPrev(tq);
 			AFS_FAST_RELE(tvc);
-		    } else if ((tvc->states & CMValid)
+		    } else if ((tvc->f.states & CMValid)
 			       && (tvc->mvid->Fid.Volume == a_fid->Volume)) {
-			tvc->states &= ~CMValid;
+			tvc->f.states &= ~CMValid;
 			if (!localFid.Cell)
 			    localFid.Cell = tvc->mvid->Cell;
 		    }
@@ -538,27 +540,27 @@ loop2:
 	    i = VCHash(&localFid);
 	    for (tvc = afs_vhashT[i]; tvc; tvc = uvc) {
 		uvc = tvc->hnext;
-		if (tvc->fid.Fid.Vnode == a_fid->Vnode
-		    && tvc->fid.Fid.Volume == a_fid->Volume
-		    && tvc->fid.Fid.Unique == a_fid->Unique) {
+		if (tvc->f.fid.Fid.Vnode == a_fid->Vnode
+		    && tvc->f.fid.Fid.Volume == a_fid->Volume
+		    && tvc->f.fid.Fid.Unique == a_fid->Unique) {
 		    tvc->callback = NULL;
 		    tvc->dchint = NULL;	/* invalidate hints */
-		    if (tvc->states & CVInit) {
+		    if (tvc->f.states & CVInit) {
 			ReleaseReadLock(&afs_xvcache);
-			afs_osi_Sleep(&tvc->states);
+			afs_osi_Sleep(&tvc->f.states);
 			goto loop2;
 		    }
-#ifdef AFS_DARWIN80_ENV
-		    if (tvc->states & CDeadVnode) {
-			ReleaseReadLock(&afs_xvcache);
-			afs_osi_Sleep(&tvc->states);
-			goto loop2;
-		    }
-#endif
-#if     defined(AFS_SGI_ENV) || defined(AFS_OSF_ENV)  || defined(AFS_SUN5_ENV)  || defined(AFS_HPUX_ENV) || defined(AFS_LINUX20_ENV)
+#if     defined(AFS_SGI_ENV) || defined(AFS_SUN5_ENV)  || defined(AFS_HPUX_ENV) || defined(AFS_LINUX20_ENV)
 		    VN_HOLD(AFSTOV(tvc));
 #else
 #ifdef AFS_DARWIN80_ENV
+		    if (tvc->f.states & CDeadVnode) {
+			if (!(tvc->f.states & CBulkFetching)) {
+			    ReleaseReadLock(&afs_xvcache);
+			    afs_osi_Sleep(&tvc->f.states);
+			    goto loop2;
+			}
+		    }
 		    vp = AFSTOV(tvc);
 		    if (vnode_get(vp))
 			continue;
@@ -568,24 +570,25 @@ loop2:
 			AFS_GLOCK();
 			continue;
 		    }
+		    if (tvc->f.states & (CBulkFetching|CDeadVnode)) {
+			AFS_GUNLOCK();
+			vnode_recycle(AFSTOV(tvc));
+			AFS_GLOCK();
+		    }
 #else
-#if defined(AFS_DARWIN_ENV) || defined(AFS_XBSD_ENV)
-		    osi_vnhold(tvc, 0);
-#else
-		    VREFCOUNT_INC(tvc); /* AIX, apparently */
-#endif
+		    AFS_FAST_HOLD(tvc);
 #endif
 #endif
 		    ReleaseReadLock(&afs_xvcache);
 		    ObtainWriteLock(&afs_xcbhash, 450);
 		    afs_DequeueCallback(tvc);
-		    tvc->states &= ~(CStatd | CUnique | CBulkFetching);
+		    tvc->f.states &= ~(CStatd | CUnique | CBulkFetching);
 		    ReleaseWriteLock(&afs_xcbhash);
-		    if ((tvc->fid.Fid.Vnode & 1 || (vType(tvc) == VDIR)))
+		    if ((tvc->f.fid.Fid.Vnode & 1 || (vType(tvc) == VDIR)))
 			osi_dnlc_purgedp(tvc);
 		    afs_Trace3(afs_iclSetp, CM_TRACE_CALLBACK,
 			       ICL_TYPE_POINTER, tvc, ICL_TYPE_INT32,
-			       tvc->states, ICL_TYPE_LONG, 0);
+			       tvc->f.states, ICL_TYPE_LONG, 0);
 #ifdef CBDEBUG
 		    lastCallBack_vnode = afid->Vnode;
 		    lastCallBack_dv = tvc->mstat.DataVersion.low;
@@ -764,7 +767,7 @@ SRXAFSCB_InitCallBackState(struct rx_call *a_call)
 			ObtainWriteLock(&afs_xcbhash, 451);
 			afs_DequeueCallback(tvc);
 			tvc->callback = NULL;
-			tvc->states &= ~(CStatd | CUnique | CBulkFetching);
+			tvc->f.states &= ~(CStatd | CUnique | CBulkFetching);
 			ReleaseWriteLock(&afs_xcbhash);
 		    }
 		}
@@ -779,7 +782,7 @@ SRXAFSCB_InitCallBackState(struct rx_call *a_call)
 
 	    for (i = 0; i < NVOLS; i++)
 		for (tv = afs_volumes[i]; tv; tv = tv->next) {
-		    for (j = 0; j < MAXHOSTS; j++)
+		    for (j = 0; j < AFS_MAXHOSTS; j++)
 			if (tv->serverHost[j] == ts)
 			    afs_ResetVolumeInfo(tv);
 		}
@@ -1009,7 +1012,7 @@ afs_RXCallBackServer(void)
     /*
      * Donate this process to Rx.
      */
-    rx_ServerProc();
+    rx_ServerProc(NULL);
     return (0);
 
 }				/*afs_RXCallBackServer */
@@ -1629,7 +1632,7 @@ SRXAFSCB_TellMeAboutYourself(struct rx_call *a_call,
 {
     int i;
     int code = 0;
-    afs_int32 *dataBuffP;
+    afs_uint32 *dataBuffP;
     afs_int32 dataBytes;
 
     RX_AFS_GLOCK();
@@ -1651,10 +1654,10 @@ SRXAFSCB_TellMeAboutYourself(struct rx_call *a_call,
 
     RX_AFS_GUNLOCK();
 
-    dataBytes = 1 * sizeof(afs_int32);
-    dataBuffP = (afs_int32 *) afs_osi_Alloc(dataBytes);
+    dataBytes = 1 * sizeof(afs_uint32);
+    dataBuffP = (afs_uint32 *) afs_osi_Alloc(dataBytes);
     dataBuffP[0] = CLIENT_CAPABILITY_ERRORTRANS;
-    capabilities->Capabilities_len = dataBytes / sizeof(afs_int32);
+    capabilities->Capabilities_len = dataBytes / sizeof(afs_uint32);
     capabilities->Capabilities_val = dataBuffP;
 
     return code;
@@ -1723,14 +1726,9 @@ resume:
 }
 #endif
 
-int SRXAFSCB_GetDE(a_call, a_index, addr, inode, flags, time, fileName)
-    struct rx_call *a_call;
-    afs_int32 a_index;
-    afs_int32 *addr;
-    afs_int32 *inode;
-    afs_int32 *flags;
-    afs_int32 *time;
-    char ** fileName;
+int SRXAFSCB_GetDE(struct rx_call *a_call, afs_int32 a_index, afs_int32 *addr,
+		   afs_int32 *inode, afs_int32 *flags, afs_int32 *time,
+		   char ** fileName)
 { /*SRXAFSCB_GetDE*/
     int code = 0;				/*Return code*/
 #if 0 && defined(AFS_LINUX24_ENV)
