@@ -118,11 +118,11 @@ afs_CacheInit(afs_int32 astatSize, afs_int32 afiles, afs_int32 ablocks,
 
 #ifdef AFS_MAXVCOUNT_ENV
     afsd_dynamic_vcaches = dynamic_vcaches;
-    printf("%s dynamically allocated vcaches\n",
-	   ( afsd_dynamic_vcaches ? "enabling" : "disabling" ));
+    afs_warn("%s dynamically allocated vcaches\n",
+	     ( afsd_dynamic_vcaches ? "enabling" : "disabling" ));
 #endif
 
-    printf("Starting AFS cache scan...");
+    afs_warn("Starting AFS cache scan...");
     if (afs_cacheinit_flag)
 	return 0;
     afs_cacheinit_flag = 1;
@@ -458,7 +458,7 @@ afs_InitCacheInfo(register char *afile)
     afs_cacheVfsp = filevp->v_vfsp;
 #endif
 #else
-#if defined(AFS_SGI62_ENV) || defined(AFS_HAVE_VXFS) || defined(AFS_DARWIN_ENV)
+#if defined(AFS_HAVE_VXFS) || defined(AFS_DARWIN_ENV)
     afs_InitDualFSCacheOps(filevp);
 #endif
 #ifndef AFS_CACHE_VNODE_PATH
@@ -694,8 +694,11 @@ shutdown_cache(void)
 	pag_epoch = 0;
 	pagCounter = 0;
 #if defined(AFS_XBSD_ENV)
-	vrele(volumeVnode);	/* let it go, finally. */
-	volumeVnode = NULL;
+	/* memcache never sets this, so don't panic on shutdown */
+	if (volumeVnode != NULL) {
+	    vrele(volumeVnode);	/* let it go, finally. */
+	    volumeVnode = NULL;
+	}
 	if (cacheDev.held_vnode) {
 	    vrele(cacheDev.held_vnode);
 	    cacheDev.held_vnode = NULL;
@@ -712,7 +715,6 @@ shutdown_cache(void)
 	cacheInfoModTime = 0;
 
 	afs_fsfragsize = 1023;
-	memset(&afs_stats_cmperf, 0, sizeof(afs_stats_cmperf));
 	memset(&cacheDev, 0, sizeof(struct osi_dev));
 	osi_dnlc_shutdown();
     }
