@@ -49,15 +49,15 @@ afs_InitMemCache(int blkCount, int blkSize, int flags)
 	(memCache + index)->data = blk;
 	memset((memCache + index)->data, 0, memCacheBlkSize);
     }
-#if defined(AFS_SGI62_ENV) || defined(AFS_HAVE_VXFS)
+#if defined(AFS_HAVE_VXFS)
     afs_InitDualFSCacheOps((struct vnode *)0);
 #endif
 
     return 0;
 
   nomem:
-    printf("afsd:  memCache allocation failure at %d KB.\n",
-	   (index * memCacheBlkSize) / 1024);
+    afs_warn("afsd:  memCache allocation failure at %d KB.\n",
+	     (index * memCacheBlkSize) / 1024);
     while (--index >= 0) {
 	afs_osi_Free((memCache + index)->data, memCacheBlkSize);
 	(memCache + index)->data = NULL;
