@@ -10,6 +10,10 @@
 #include <afsconfig.h>
 #include "afs/param.h"
 
+#ifdef IGNORE_SOME_GCC_WARNINGS
+# pragma GCC diagnostic warning "-Wold-style-definition"
+# pragma GCC diagnostic warning "-Wstrict-prototypes"
+#endif
 
 #include "afs/sysincludes.h"	/* Standard vendor system headers */
 #include "afsincludes.h"	/* Afs-based standard headers */
@@ -20,7 +24,7 @@
 #ifdef AFS_SGI62_ENV
 #include "h/hashing.h"
 #endif
-#if !defined(AFS_HPUX110_ENV) && !defined(AFS_DARWIN60_ENV)
+#if !defined(AFS_HPUX110_ENV) && !defined(AFS_DARWIN_ENV)
 #include "netinet/in_var.h"
 #endif
 #endif /* !defined(UKERNEL) */
@@ -337,13 +341,22 @@ struct iparam32 {
 
 
 #if defined(AFS_HPUX_64BIT_ENV) || defined(AFS_SUN57_64BIT_ENV) || (defined(AFS_LINUX_64BIT_KERNEL) && !defined(AFS_ALPHA_LINUX20_ENV) && !defined(AFS_IA64_LINUX20_ENV)) || defined(NEED_IOCTL32)
+#if SIZEOF_VOID_P == SIZEOF_UNSIGNED_INT
+# define uintptrsz unsigned int
+#elif SIZEOF_VOID_P == SIZEOF_UNSIGNED_LONG
+# define uintptrsz unsigned long
+#elif SIZEOF_VOID_P == SIZEOF_UNSIGNED_LONG_LONG
+# define uintptrsz unsigned long long
+#else
+# error "Unable to determine casting for pointers"
+#endif
 static void
 iparam32_to_iparam(const struct iparam32 *src, struct iparam *dst)
 {
-    dst->param1 = src->param1;
-    dst->param2 = src->param2;
-    dst->param3 = src->param3;
-    dst->param4 = src->param4;
+    dst->param1 = (iparmtype)(uintptrsz)src->param1;
+    dst->param2 = (iparmtype)(uintptrsz)src->param2;
+    dst->param3 = (iparmtype)(uintptrsz)src->param3;
+    dst->param4 = (iparmtype)(uintptrsz)src->param4;
 }
 #endif
 
@@ -435,7 +448,7 @@ extern int afs_sinited;
  * to accomodate both, *long* is used instead of afs_int32
  */
 
-#ifdef AFS_SUN57_ENV
+# ifdef AFS_SUN57_ENV
 struct afssysa {
     long syscall;
     long parm1;
@@ -445,7 +458,7 @@ struct afssysa {
     long parm5;
     long parm6;
 };
-#else
+# else
 struct afssysa {
     afs_int32 syscall;
     afs_int32 parm1;
@@ -455,13 +468,12 @@ struct afssysa {
     afs_int32 parm5;
     afs_int32 parm6;
 };
-#endif
+# endif
 
 Afs_syscall(register struct afssysa *uap, rval_t * rvp)
 {
     int *retval = &rvp->r_val1;
-#else /* AFS_SUN5_ENV */
-#ifdef AFS_DARWIN100_ENV
+#elif defined(AFS_DARWIN100_ENV)
 struct afssysa {
     afs_int32 syscall;
     afs_int32 parm1;
@@ -481,18 +493,18 @@ struct afssysa64 {
     afs_int32 syscall;
 };
 int
-afs3_syscall(struct proc *p, void *args, unsigned int *retval)
+afs3_syscall(afs_proc_t *p, void *args, unsigned int *retval)
 {
     struct afssysa64 *uap64 = NULL;
     struct afssysa *uap = NULL;
-#elif	defined(AFS_OSF_ENV) || defined(AFS_DARWIN_ENV) || defined(AFS_XBSD_ENV)
+#elif defined(AFS_DARWIN_ENV) || defined(AFS_XBSD_ENV)
 int
 afs3_syscall(p, args, retval)
-#ifdef AFS_FBSD50_ENV
+# ifdef AFS_FBSD50_ENV
      struct thread *p;
-#else
-     struct proc *p;
-#endif
+# else
+     afs_proc_t *p;
+# endif
      void *args;
      long *retval;
 {
@@ -505,8 +517,7 @@ afs3_syscall(p, args, retval)
 	long parm5;
 	long parm6;
     } *uap = (struct a *)args;
-#else /* AFS_OSF_ENV */
-#ifdef AFS_LINUX20_ENV
+#elif defined(AFS_LINUX20_ENV)
 struct afssysargs {
     long syscall;
     long parm1;
@@ -524,12 +535,25 @@ afs_syscall(long syscall, long parm1, long parm2, long parm3, long parm4)
     long linux_ret = 0;
     long *retval = &linux_ret;
     long eparm[4];		/* matches AFSCALL_ICL in fstrace.c */
-#ifdef AFS_SPARC64_LINUX24_ENV
+# ifdef AFS_SPARC64_LINUX24_ENV
     afs_int32 eparm32[4];
-#endif
+# endif
     /* eparm is also used by AFSCALL_CALL in afsd.c */
 #else
-#if defined(UKERNEL)
+# if defined(UKERNEL)
+int
+Afs_syscall()
+{
+    register struct a {
+	long syscall;
+	long parm1;
+	long parm2;
+	long parm3;
+	long parm4;
+	long parm5;
+	long parm6;
+    } *uap = (struct a *)get_user_struct()->u_ap;
+# else /* UKERNEL */
 int
 Afs_syscall()
 {
@@ -542,28 +566,15 @@ Afs_syscall()
 	long parm5;
 	long parm6;
     } *uap = (struct a *)u.u_ap;
-#else /* UKERNEL */
-int
-Afs_syscall()
-{
-    register struct a {
-	long syscall;
-	long parm1;
-	long parm2;
-	long parm3;
-	long parm4;
-	long parm5;
-	long parm6;
-    } *uap = (struct a *)u.u_ap;
-#endif /* UKERNEL */
-#if defined(AFS_HPUX_ENV)
+# endif /* UKERNEL */
+# if defined(AFS_HPUX_ENV)
     long *retval = &u.u_rval1;
-#else
+# elif defined(UKERNEL)
+    int *retval = &(get_user_struct()->u_rval1);
+# else
     int *retval = &u.u_rval1;
+# endif
 #endif
-#endif /* AFS_LINUX20_ENV */
-#endif /* AFS_OSF_ENV */
-#endif /* AFS_SUN5_ENV */
     register int code = 0;
 
     AFS_STATCNT(afs_syscall);
@@ -627,6 +638,11 @@ Afs_syscall()
 	    code =
 		afs_syscall64_call(uap64->parm1, uap64->parm2, uap64->parm3,
 				   uap64->parm4, uap64->parm5, uap64->parm6);
+	    /* pass back the code as syscall retval */
+	    if (code < 0) {
+		*retval = code;
+		code = 0;
+	    }
 	} else if (uap64->syscall == AFSCALL_SETPAG) {
 	    AFS_GLOCK();
 	    code = afs_setpag(p, args, retval);
@@ -665,6 +681,13 @@ Afs_syscall()
 	    code =
 		afs_syscall_call(uap->parm1, uap->parm2, uap->parm3,
 				 uap->parm4, uap->parm5, uap->parm6);
+#ifdef AFS_DARWIN_ENV
+	    /* pass back the code as syscall retval */
+	    if (code < 0) {
+		*retval = code;
+		code = 0;
+	    }
+#endif
 	} else if (uap->syscall == AFSCALL_SETPAG) {
 #ifdef	AFS_SUN5_ENV
 	    register proc_t *procp;
@@ -675,9 +698,9 @@ Afs_syscall()
 	    AFS_GUNLOCK();
 #else
 	    AFS_GLOCK();
-#if	defined(AFS_OSF_ENV) || defined(AFS_DARWIN_ENV) || defined(AFS_XBSD_ENV)
+#if	defined(AFS_DARWIN_ENV) || defined(AFS_XBSD_ENV)
 	    code = afs_setpag(p, args, retval);
-#else /* AFS_OSF_ENV */
+#else /* AFS_DARWIN_ENV || AFS_XBSD_ENV */
 	    code = afs_setpag();
 #endif
 	    AFS_GUNLOCK();
@@ -727,7 +750,7 @@ Afs_syscall()
 		    afs_syscall_icreate(uap->parm1, uap->parm2, iparams.param1,
 					iparams.param2, iparams.param3,
 					iparams.param4
-#if defined(AFS_OSF_ENV) || defined(AFS_DARWIN_ENV) || defined(AFS_XBSD_ENV)
+#if defined(AFS_DARWIN_ENV) || defined(AFS_XBSD_ENV)
 					, retval
 #endif
 			);
@@ -740,7 +763,7 @@ Afs_syscall()
 				  CRED());
 #else
 	    code = afs_syscall_iopen(uap->parm1, uap->parm2, uap->parm3
-#if defined(AFS_OSF_ENV) || defined(AFS_DARWIN_ENV) || defined(AFS_XBSD_ENV)
+#if defined(AFS_DARWIN_ENV) || defined(AFS_XBSD_ENV)
 				     , retval
 #endif
 		);
@@ -763,7 +786,7 @@ Afs_syscall()
 	    AFS_GLOCK();
 	    code =
 		Afscall_icl(uap->parm1, uap->parm2, uap->parm3, uap->parm4,
-			    uap->parm5, retval);
+			    uap->parm5, (long *)retval);
 	    AFS_GUNLOCK();
 #ifdef AFS_LINUX20_ENV
 	    if (!code) {
