@@ -15,8 +15,6 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
-RCSID
-    ("$Header: /cvs/openafs/src/budb/db_dump.c,v 1.7.14.2 2007/11/26 21:08:41 shadow Exp $");
 
 #ifdef AFS_NT40_ENV
 #include <winsock2.h>
@@ -35,6 +33,7 @@ RCSID
 #include "globals.h"
 #include "error_macros.h"
 #include "budb_errs.h"
+#include "budb_internal.h"
 #include "afs/audit.h"
 
 
@@ -61,10 +60,11 @@ dumpSyncP dumpSyncPtr = &dumpSync;
  */
 
 afs_int32
-canWrite(fid)
-     int fid;
+canWrite(int fid)
 {
+#ifndef AFS_PTHREAD_ENV
     afs_int32 code = 0;
+#endif
     extern dumpSyncP dumpSyncPtr;
 
     ObtainWriteLock(&dumpSyncPtr->ds_lock);
@@ -73,13 +73,23 @@ canWrite(fid)
     while (dumpSyncPtr->ds_bytes > 0) {
 	if (dumpSyncPtr->ds_readerStatus == DS_WAITING) {
 	    dumpSyncPtr->ds_readerStatus = 0;
+#ifdef AFS_PTHREAD_ENV
+	    assert(pthread_cond_broadcast(&dumpSyncPtr->ds_readerStatus_cond) == 0);
+#else
 	    code = LWP_SignalProcess(&dumpSyncPtr->ds_readerStatus);
 	    if (code)
 		LogError(code, "canWrite: Signal delivery failed\n");
+#endif
 	}
 	dumpSyncPtr->ds_writerStatus = DS_WAITING;
 	ReleaseWriteLock(&dumpSyncPtr->ds_lock);
+#ifdef AFS_PTHREAD_ENV
+	assert(pthread_mutex_lock(&dumpSyncPtr->ds_writerStatus_mutex) == 0);
+	assert(pthread_cond_wait(&dumpSyncPtr->ds_writerStatus_cond, &dumpSyncPtr->ds_writerStatus_mutex) == 0);
+	assert(pthread_mutex_unlock(&dumpSyncPtr->ds_writerStatus_mutex) == 0);
+#else
 	LWP_WaitProcess(&dumpSyncPtr->ds_writerStatus);
+#endif
 	ObtainWriteLock(&dumpSyncPtr->ds_lock);
     }
     return (1);
@@ -94,18 +104,23 @@ canWrite(fid)
  */
 
 void
-haveWritten(nbytes)
-     afs_int32 nbytes;
+haveWritten(afs_int32 nbytes)
 {
+#ifndef AFS_PTHREAD_ENV
     afs_int32 code = 0;
+#endif
     extern dumpSyncP dumpSyncPtr;
 
     dumpSyncPtr->ds_bytes += nbytes;
     if (dumpSyncPtr->ds_readerStatus == DS_WAITING) {
 	dumpSyncPtr->ds_readerStatus = 0;
+#ifdef AFS_PTHREAD_ENV
+	assert(pthread_cond_broadcast(&dumpSyncPtr->ds_readerStatus_cond) == 0);
+#else
 	code = LWP_SignalProcess(&dumpSyncPtr->ds_readerStatus);
 	if (code)
 	    LogError(code, "haveWritten: Signal delivery failed\n");
+#endif
     }
     ReleaseWriteLock(&dumpSyncPtr->ds_lock);
 }
@@ -116,10 +131,11 @@ haveWritten(nbytes)
  */
 
 void
-doneWriting(error)
-     afs_int32 error;
+doneWriting(afs_int32 error)
 {
+#ifndef AFS_PTHREAD_ENV
     afs_int32 code = 0;
+#endif
 
     /* wait for the reader */
     ObtainWriteLock(&dumpSyncPtr->ds_lock);
@@ -127,7 +143,13 @@ doneWriting(error)
 	LogDebug(4, "doneWriting: waiting for Reader\n");
 	dumpSyncPtr->ds_writerStatus = DS_WAITING;
 	ReleaseWriteLock(&dumpSyncPtr->ds_lock);
+#ifdef AFS_PTHREAD_ENV
+	assert(pthread_mutex_lock(&dumpSyncPtr->ds_writerStatus_mutex) == 0);
+	assert(pthread_cond_wait(&dumpSyncPtr->ds_writerStatus_cond, &dumpSyncPtr->ds_writerStatus_mutex) == 0);
+	assert(pthread_mutex_unlock(&dumpSyncPtr->ds_writerStatus_mutex) == 0);
+#else
 	LWP_WaitProcess(&dumpSyncPtr->ds_writerStatus);
+#endif
 	ObtainWriteLock(&dumpSyncPtr->ds_lock);
     }
 
@@ -139,9 +161,13 @@ doneWriting(error)
     else
 	dumpSyncPtr->ds_writerStatus = DS_DONE;
     dumpSyncPtr->ds_readerStatus = 0;
+#ifdef AFS_PTHREAD_ENV
+    assert(pthread_cond_broadcast(&dumpSyncPtr->ds_readerStatus_cond) == 0);
+#else
     code = LWP_NoYieldSignal(&dumpSyncPtr->ds_readerStatus);
     if (code)
 	LogError(code, "doneWriting: Signal delivery failed\n");
+#endif
     ReleaseWriteLock(&dumpSyncPtr->ds_lock);
 }
 
@@ -154,9 +180,7 @@ doneWriting(error)
  */
 
 afs_int32
-writeStructHeader(fid, type)
-     int fid;
-     afs_int32 type;
+writeStructHeader(int fid, afs_int32 type)
 {
     struct structDumpHeader hostDumpHeader, netDumpHeader;
 
@@ -207,9 +231,7 @@ writeStructHeader(fid, type)
  */
 
 afs_int32
-writeTextHeader(fid, type)
-     int fid;
-     afs_int32 type;
+writeTextHeader(int fid, afs_int32 type)
 {
     struct structDumpHeader hostDumpHeader, netDumpHeader;
 
@@ -249,8 +271,7 @@ writeTextHeader(fid, type)
 }
 
 afs_int32
-writeDbHeader(fid)
-     int fid;
+writeDbHeader(int fid)
 {
     struct DbHeader header;
     afs_int32 curtime;
@@ -293,9 +314,7 @@ writeDbHeader(fid)
  */
 
 afs_int32
-writeDump(fid, dumpPtr)
-     int fid;
-     dbDumpP dumpPtr;
+writeDump(int fid, dbDumpP dumpPtr)
 {
     struct budb_dumpEntry dumpEntry;
     afs_int32 code = 0, tcode;
@@ -318,10 +337,7 @@ writeDump(fid, dumpPtr)
 }
 
 afs_int32
-writeTape(fid, tapePtr, dumpid)
-     int fid;
-     struct tape *tapePtr;
-     afs_int32 dumpid;
+writeTape(int fid, struct tape *tapePtr, afs_int32 dumpid)
 {
     struct budb_tapeEntry tapeEntry;
     afs_int32 code = 0, tcode;
@@ -349,13 +365,8 @@ writeTape(fid, tapePtr, dumpid)
 /* combines volFragment and volInfo */
 
 afs_int32
-writeVolume(ut, fid, volFragmentPtr, volInfoPtr, dumpid, tapeName)
-     struct ubik_trans *ut;
-     int fid;
-     struct volFragment *volFragmentPtr;
-     struct volInfo *volInfoPtr;
-     afs_int32 dumpid;
-     char *tapeName;
+writeVolume(struct ubik_trans *ut, int fid, struct volFragment *volFragmentPtr,
+	    struct volInfo *volInfoPtr, afs_int32 dumpid, char *tapeName)
 {
     struct budb_volumeEntry budbVolume;
     afs_int32 code = 0;
@@ -392,8 +403,7 @@ writeVolume(ut, fid, volFragmentPtr, volInfoPtr, dumpid, tapeName)
  */
 
 afs_int32
-checkLock(textType)
-     afs_int32 textType;
+checkLock(afs_int32 textType)
 {
     db_lockP lockPtr;
 
@@ -411,9 +421,8 @@ checkLock(textType)
  *	check the integrity of the specified text type
  */
 
-checkText(ut, textType)
-     struct ubik_trans *ut;
-     afs_int32 textType;
+int
+checkText(struct ubik_trans *ut, afs_int32 textType)
 {
     struct textBlock *tbPtr;
     afs_int32 nBytes = 0;	/* accumulated actual size */
@@ -458,10 +467,7 @@ checkText(ut, textType)
  */
 
 afs_int32
-writeText(ut, fid, textType)
-     struct ubik_trans *ut;
-     int fid;
-     int textType;
+writeText(struct ubik_trans *ut, int fid, int textType)
 {
     struct textBlock *tbPtr;
     afs_int32 textSize, writeSize;
@@ -518,9 +524,7 @@ writeText(ut, fid, textType)
 #define MAXAPPENDS 200
 
 afs_int32
-writeDatabase(ut, fid)
-     struct ubik_trans *ut;
-     int fid;
+writeDatabase(struct ubik_trans *ut, int fid)
 {
     dbadr dbAddr, dbAppAddr;
     struct dump diskDump, apDiskDump;
@@ -774,8 +778,7 @@ writeDatabase(ut, fid)
 #ifdef notdef
 
 afs_int32
-canWrite(fid)
-     int fid;
+canWrite(int fid)
 {
     afs_int32 in, out, except;
     struct timeval tp;
