@@ -10,9 +10,6 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
-RCSID
-    ("$Header: /cvs/openafs/src/butc/dump.c,v 1.18.6.2 2008/03/10 22:32:33 shadow Exp $");
-
 #include <sys/types.h>
 #ifdef AFS_NT40_ENV
 #include <winsock2.h>
@@ -25,6 +22,9 @@ RCSID
 #endif
 #include <stdlib.h>
 #include <string.h>
+#ifdef HAVE_STDINT_H
+# include <stdint.h>
+#endif
 #include <rx/xdr.h>
 #include <rx/rx.h>
 #include <lwp.h>
@@ -33,18 +33,21 @@ RCSID
 #include <afs/tcdata.h>
 #include <afs/bubasics.h>
 #include <afs/budb_client.h>
+#include <afs/butm_prototypes.h>
 #include <afs/vldbint.h>
 #include <afs/ktime.h>
 #include <afs/vlserver.h>
 #include <afs/volser.h>
+#include <afs/volser_prototypes.h>
 #include <afs/volint.h>
 #include <afs/cellconfig.h>
+#include <afs/bucoord_prototypes.h>
 
+#include "butc_internal.h"
 #include "error_macros.h"
 #include "butc_xbsa.h"
 #include "afs/butx.h"
 
-
 /* GLOBAL CONFIGURATION PARAMETERS */
 extern int dump_namecheck;
 extern int queryoperator;
@@ -55,11 +58,6 @@ extern struct ubik_client *cstruct;
 dlqlinkT savedEntries;
 dlqlinkT entries_to_flush;
 
-afs_int32 flushSavedEntries(), finishDump(), finishTape(), useTape(),
-addVolume();
-
-extern struct rx_connection *UV_Bind();
-
 extern afs_int32 groupId;
 extern afs_int32 BufferSize;
 extern afs_int32 statusSize;
@@ -102,7 +100,7 @@ afs_int32 tapeblocks;		/* Number of 16K tape datablocks in buffer (!CONF_XBSA) *
    else \
      sprintf(dumpname, "%s (DumpId %u)", name, dbDumpId);
 
-#if defined(AFS_NT40_ENV) || (defined(AFS_DARWIN_ENV) && !defined(AFS_DARWIN60_ENV)) || defined(AFS_SUN4_ENV)
+#if defined(AFS_NT40_ENV) || defined(AFS_SUN4_ENV)
 int
 localtime_r(time_t * t, struct tm *tm)
 {
@@ -138,6 +136,15 @@ struct dumpRock {
     struct dumpNode *node;
 };
 
+/* Forward declarations */
+
+int makeVolumeHeader(struct volumeHeader *, struct dumpRock *, int);
+int volumeHeader_hton(struct volumeHeader *, struct volumeHeader *);
+char retryPrompt(char *, afs_int32, afs_uint32);
+int getDumpTape(struct dumpRock *, int, afs_int32);
+int getXBSATape(struct dumpRock *);
+afs_int32 createDump(struct dumpRock *);
+
 /* configuration variables */
 #define HITEOT(code) ((code == BUTM_IO) || (code == BUTM_EOT) || (code == BUTM_IOCTL))
 extern int autoQuery;
@@ -152,7 +159,6 @@ afs_int32
 calcExpirationDate(afs_int32 expType, afs_int32 expDate, afs_int32 createTime)
 {
     struct ktime_date kd;
-    afs_int32 Add_RelDate_to_Time();
 
     switch (expType) {
     case BC_REL_EXPDATE:
@@ -293,7 +299,7 @@ dumpVolume(struct tc_dumpDesc * curDump, struct dumpRock * dparamsPtr)
 	/* Create and Write the volume header */
 	makeVolumeHeader(&hostVolumeHeader, dparamsPtr, fragmentNumber);
 	hostVolumeHeader.contd = ((fragmentNumber == 1) ? 0 : TC_VOLCONTD);
-	volumeHeader_hton(&hostVolumeHeader, buffer);
+	volumeHeader_hton(&hostVolumeHeader, (struct volumeHeader *)buffer);
 
 	rc = butm_WriteFileData(tapeInfoPtr, buffer, 1,
 				sizeof(hostVolumeHeader));
@@ -358,6 +364,13 @@ dumpVolume(struct tc_dumpDesc * curDump, struct dumpRock * dparamsPtr)
 		}
 	    }
 
+#ifdef xbsa
+	    /* Set aside space for the trailing volume header when using large buffers. */
+	    if (XBSAMAXBUFFER < toread + sizeof(hostVolumeHeader)) {
+		toread = XBSAMAXBUFFER - sizeof(hostVolumeHeader);
+	    }
+#endif
+
 	    /* Read some volume data. */
 	    if (fragmentvolume) {
 		bytesread = 0;
@@ -380,7 +393,7 @@ dumpVolume(struct tc_dumpDesc * curDump, struct dumpRock * dparamsPtr)
 		hostVolumeHeader.contd = (endofvolume ? 0 : TC_VOLCONTD);
 		hostVolumeHeader.magic = TC_VOLENDMAGIC;
 		hostVolumeHeader.endTime = (endofvolume ? time(0) : 0);
-		volumeHeader_hton(&hostVolumeHeader, &buffer[bytesread]);
+		volumeHeader_hton(&hostVolumeHeader, (struct volumeHeader *)&buffer[bytesread]);
 		bytesread += sizeof(hostVolumeHeader);
 	    }
 
@@ -631,10 +644,10 @@ xbsaDumpVolume(struct tc_dumpDesc * curDump, struct dumpRock * dparamsPtr)
     /* Create and Write the volume header */
     makeVolumeHeader(&hostVolumeHeader, dparamsPtr, 1);
     hostVolumeHeader.contd = 0;
-    volumeHeader_hton(&hostVolumeHeader, buffer);
+    volumeHeader_hton(&hostVolumeHeader, (struct volumeHeader *)buffer);
 
-    rc = xbsa_WriteObjectData(&butxInfo, buffer, sizeof(struct volumeHeader),
-			      &bytesWritten);
+    rc = xbsa_WriteObjectData(&butxInfo, (struct volumeHeader *)buffer,
+			      sizeof(struct volumeHeader), &bytesWritten);
     if (rc != XBSA_SUCCESS) {
 	ErrorLog(1, taskId, rc, 0,
 		 "Unable to write VolumeHeader data to the server\n");
@@ -849,7 +862,6 @@ dumpPass(struct dumpRock * dparamsPtr, int passNumber)
     int action, e;
     afs_int32 code = 0, tcode, dvcode;
     char ch;
-    char retryPrompt();
     struct vldbentry vldbEntry;
     struct sockaddr_in server;
     afs_int32 tapepos;
@@ -1143,7 +1155,6 @@ Dumper(void *param)
 
     extern struct deviceSyncNode *deviceLatch;
     extern struct tapeConfig globalTapeConfig;
-    extern afs_int32 createDump();
 
     taskId = nodePtr->taskID;	/* Get task Id */
     setStatus(taskId, DRIVE_WAIT);
@@ -1268,7 +1279,7 @@ Dumper(void *param)
     if (!CONF_XBSA) {
 	code = butm_WriteEOT(&tapeInfo);
 	if (code)
-	    TapeLog(taskId, code, tapeInfo.error,
+	    TapeLog(0, taskId, code, tapeInfo.error,
 		    "Warning: Can't write end-of-dump on tape\n");
     }
 
@@ -1353,14 +1364,14 @@ Dumper(void *param)
 	sprintf(line,
 		"%-5d  %02d/%02d/%04d %02d:%02d:%02d  "
 		"%02d/%02d/%04d %02d:%02d:%02d  " "%02d:%02d:%02d  "
-		"%s %d of %d volumes dumped (%ld KB)\n", taskId,
+		"%s %d of %d volumes dumped (%lu KB)\n", taskId,
 		tmstart.tm_mon + 1, tmstart.tm_mday, tmstart.tm_year + 1900,
 		tmstart.tm_hour, tmstart.tm_min, tmstart.tm_sec,
 		tmend.tm_mon + 1, tmend.tm_mday, tmend.tm_year + 1900,
 		tmend.tm_hour, tmend.tm_min, tmend.tm_sec, hrs, min, sec,
 		nodePtr->volumeSetName, dumpedvolumes,
 		dumpedvolumes + failedvolumes,
-		dparams.tapeInfoPtr->kBytes + 1);
+		afs_printable_uint32_lu(dparams.tapeInfoPtr->kBytes + 1));
 
 	fwrite(line, strlen(line), 1, centralLogIO);
 	fflush(centralLogIO);
@@ -1370,7 +1381,7 @@ Dumper(void *param)
 
     FreeNode(taskId);		/* free the dump node */
     LeaveDeviceQueue(deviceLatch);
-    return (void *)(code);
+    return (void *)(intptr_t)(code);
 }
 
 #define BELLTIME 60		/* 60 seconds before a bell rings */
@@ -1405,7 +1416,7 @@ retryPrompt(char *volumeName, afs_int32 volumeId, afs_uint32 taskId)
     printf("a - abort, the entire dump\n");
 
     while (1) {
-	FFlushInput(stdin);
+	FFlushInput();
 	putchar(BELLCHAR);
 	fflush(stdout);
 
@@ -1528,7 +1539,6 @@ getDumpTape(struct dumpRock *dparamsPtr, int interactiveFlag,
     afs_int32 tapepos, lastpos;
 
     extern struct tapeConfig globalTapeConfig;
-    extern struct udbHandleS udbHandle;
 
     askForTape = interactiveFlag;
     dparamsPtr->wroteLabel = 0;
@@ -2215,6 +2225,6 @@ DeleteDump(void *param)
 	code = BUTX_DELETENOVOL;
 	setStatus(taskId, TASK_ERROR);
     }
-    return (code);
+    return (void *)(code);
 }
 #endif
