@@ -21,6 +21,9 @@
 #include <netinet/in.h>
 #include <netdb.h>
 #endif
+#ifdef HAVE_STDINT_H
+# include <stdint.h>
+#endif
 
 #include <afs/procmgmt.h>
 #include <rx/xdr.h>
@@ -32,7 +35,9 @@
 #include <afs/bubasics.h>	/* PA */
 #include <afs/budb_client.h>
 #include <afs/bucoord_prototypes.h>
+#include <afs/butm_prototypes.h>
 #include <afs/volser.h>
+#include <afs/volser_prototypes.h>
 #include <afs/com_err.h>
 #include "error_macros.h"
 #include <afs/afsutil.h>
@@ -138,14 +143,11 @@ static struct TapeBlock tapeBlock;
 char tapeVolumeHT[sizeof(struct volumeHeader) + 2 * sizeof(char)];
 
 void
-PrintLog(log, error1, error2, str, a, b, c, d, e, f, g, h, i, j)
-     FILE *log;
-     afs_int32 error1, error2;
-     char *str, *a, *b, *c, *d, *e, *f, *g, *h, *i, *j;
+PrintLogStr(FILE *log, afs_int32 error1, afs_int32 error2, char *str)
 {
     char *err1, *err2;
 
-    fprintf(log, str, a, b, c, d, e, f, g, h, i, j);
+    fprintf(log, "%s", str);
     if (error1) {
 	err2 = "vols";
 	switch (error1) {
@@ -199,10 +201,8 @@ PrintLog(log, error1, error2, str, a, b, c, d, e, f, g, h, i, j)
 }
 
 void
-TapeLog(debug, task, error1, error2, str, a, b, c, d, e, f, g, h, i, j)
-     int debug;
-     afs_int32 task, error1, error2;
-     char *str, *a, *b, *c, *d, *e, *f, *g, *h, *i, *j;
+TapeLogStr(int debug, afs_int32 task, afs_int32 error1, afs_int32 error2,
+	   char *str)
 {
     time_t now;
     char tbuffer[32], *timestr;
@@ -214,35 +214,51 @@ TapeLog(debug, task, error1, error2, str, a, b, c, d, e, f, g, h, i, j)
     fprintf(logIO, "%s: ", timestr);
     if (task)
 	fprintf(logIO, "Task %u: ", task);
-    PrintLog(logIO, error1, error2, str, a, b, c, d, e, f, g, h, i, j);
+    PrintLogStr(logIO, error1, error2, str);
 
     if (lastPass && lastLogIO) {
 	fprintf(lastLogIO, "%s: ", timestr);
 	if (task)
 	    fprintf(lastLogIO, "Task %u: ", task);
-	PrintLog(lastLogIO, error1, error2, str, a, b, c, d, e, f, g, h, i,
-		 j);
+	PrintLogStr(lastLogIO, error1, error2, str);
     }
 
     /* Now print to the screen if debug level requires */
     if (debug <= debugLevel)
-	PrintLog(stdout, error1, error2, str, a, b, c, d, e, f, g, h, i, j);
+	PrintLogStr(stdout, error1, error2, str);
 }
 
 void
-TLog(task, str, a, b, c, d, e, f, g, h, i, j)
-     afs_int32 task;
-     char *str, *a, *b, *c, *d, *e, *f, *g, *h, *i, *j;
+TapeLog(int debug, afs_int32 task, afs_int32 error1, afs_int32 error2,
+	char *fmt, ...)
 {
-    /* Sends message to TapeLog and stdout */
-    TapeLog(0, task, 0, 0, str, a, b, c, d, e, f, g, h, i, j);
+    char tmp[1024];
+    va_list ap;
+
+    va_start(ap, fmt);
+    afs_vsnprintf(tmp, sizeof(tmp), fmt, ap);
+    va_end(ap);
+
+    TapeLogStr(debug, task, error1, error2, tmp);
 }
 
 void
-ErrorLog(debug, task, error1, error2, str, a, b, c, d, e, f, g, h, i, j)
-     int debug;
-     afs_int32 task, error1, error2;
-     char *str, *a, *b, *c, *d, *e, *f, *g, *h, *i, *j;
+TLog(afs_int32 task, char *fmt, ...)
+{
+    char tmp[1024];
+    va_list ap;
+
+    va_start(ap, fmt);
+    afs_vsnprintf(tmp, sizeof(tmp), fmt, ap);
+    va_end(ap);
+
+    /* Sends message to TapeLog and stdout */
+    TapeLogStr(0, task, 0, 0, tmp);
+}
+
+void
+ErrorLogStr(int debug, afs_int32 task, afs_int32 error1, afs_int32 error2,
+	    char *errStr)
 {
     time_t now;
     char tbuffer[32], *timestr;
@@ -255,18 +271,38 @@ ErrorLog(debug, task, error1, error2, str, a, b, c, d, e, f, g, h, i, j)
     /* Print the time and task number */
     if (task)
 	fprintf(ErrorlogIO, "Task %u: ", task);
-    PrintLog(ErrorlogIO, error1, error2, str, a, b, c, d, e, f, g, h, i, j);
 
-    TapeLog(debug, task, error1, error2, str, a, b, c, d, e, f, g, h, i, j);
+    PrintLogStr(ErrorlogIO, error1, error2, errStr);
+    TapeLogStr(debug, task, error1, error2, errStr);
 }
 
 void
-ELog(task, str, a, b, c, d, e, f, g, h, i, j)
-     afs_int32 task;
-     char *str, *a, *b, *c, *d, *e, *f, *g, *h, *i, *j;
+ErrorLog(int debug, afs_int32 task, afs_int32 error1, afs_int32 error2,
+	 char *fmt, ...)
 {
+    char tmp[1024];
+    va_list ap;
+
+    va_start(ap, fmt);
+    afs_vsnprintf(tmp, sizeof(tmp), fmt, ap);
+    va_end(ap);
+
+    ErrorLogStr(debug, task, error1, error2, tmp);
+
+}
+
+void
+ELog(afs_int32 task, char *fmt, ...)
+{
+    char tmp[1024];
+    va_list ap;
+
+    va_start(ap, fmt);
+    afs_vsnprintf(tmp, sizeof(tmp), fmt, ap);
+    va_end(ap);
+
     /* Sends message to ErrorLog, TapeLog and stdout */
-    ErrorLog(0, task, 0, 0, str, a, b, c, d, e, f, g, h, i, j);
+    ErrorLog(0, task, 0, 0, "%s", tmp);
 }
 
 /* first proc called by anybody who intends to use the device */
@@ -1069,10 +1105,11 @@ GetRestoreTape(afs_int32 taskId, struct butm_tapeInfo *tapeInfoPtr,
 }
 
 afs_int32
-xbsaRestoreVolumeData(struct rx_call *call, struct restoreParams *rparamsPtr)
+xbsaRestoreVolumeData(struct rx_call *call, void *rock)
 {
     afs_int32 code = 0;
 #ifdef xbsa
+    struct restoreParams *rparamsPtr = (struct restoreParams *)rock;
     afs_int32 curChunk, rc;
     afs_uint32 totalWritten;
     afs_int32 headBytes, tailBytes, w;
@@ -1220,8 +1257,9 @@ xbsaRestoreVolumeData(struct rx_call *call, struct restoreParams *rparamsPtr)
  */
 
 afs_int32
-restoreVolumeData(struct rx_call *call, struct restoreParams *rparamsPtr)
+restoreVolumeData(struct rx_call *call, void *rock)
 {
+    struct restoreParams *rparamsPtr = (struct restoreParams *)rock;
     afs_int32 curChunk;
     afs_uint32 totalWritten = 0;
     afs_int32 code = 0;
@@ -1896,7 +1934,7 @@ Restorer(void *param) {
 
     FreeNode(taskId);
     LeaveDeviceQueue(deviceLatch);
-    return (void *)(code);
+    return (void *)(intptr_t)(code);
 }
 
 /* this is just scaffolding, creates new tape label with name <tapeName> */
@@ -2245,7 +2283,7 @@ Labeller(void *param)
 
     free(labelIfPtr);
     LeaveDeviceQueue(deviceLatch);
-    return (void *)(code);
+    return (void *)(intptr_t)(code);
 }
 
 /* PrintTapeLabel
@@ -2420,7 +2458,9 @@ readVolumeHeader(char *buffer,		/* in - buffer to read header from */
 	/* Handle Case 1 */
 	memset(&vhptr, 0, sizeof(struct volumeHeader));
 	memcpy(&vhptr, buffer + bufloc, firstSplice);
-	memcpy(&vhptr + firstSplice, buffer + bufloc + firstSplice + padLen,
+	/* probably GCC bug 37060; however, no guarantee on length of buffer */
+	tempvhptr = (struct volumeHeader *)(buffer + firstSplice);
+	memcpy(tempvhptr, buffer + bufloc + firstSplice + padLen,
 	       nextSplice);
 	HEADER_CHECKS(vhptr, header);
 
