@@ -13,18 +13,16 @@
  * without license from the U.S. Commerce department.
  */
 
-#include <mit-cpyright.h>
+#include "mit-cpyright.h"
 
 #include <afsconfig.h>
 #include <afs/param.h>
 
-RCSID
-    ("$Header: /cvs/openafs/src/des/new_rnd_key.c,v 1.13.2.2 2005/05/30 04:57:34 shadow Exp $");
 
 #ifndef KERNEL
 #include <stdio.h>
 #endif
-#include <des.h>
+#include "des.h"
 #include "des_internal.h"
 #include "des_prototypes.h"
 
@@ -38,7 +36,6 @@ RCSID
 #include <strings.h>
 #endif
 #endif
-#include "stats.h"
 
 static afs_int32 des_set_sequence_number(des_cblock new_sequence_number);
 static afs_int32 des_generate_random_block(des_cblock block);
@@ -53,7 +50,11 @@ static int is_inited = 0;
  */
 
 #include <assert.h>
-pthread_mutex_t des_init_mutex;
+pthread_mutex_t des_init_mutex
+#ifdef PTHREAD_MUTEX_INITIALIZER
+= PTHREAD_MUTEX_INITIALIZER
+#endif
+;
 #define LOCK_INIT assert(pthread_mutex_lock(&des_init_mutex)==0)
 #define UNLOCK_INIT assert(pthread_mutex_unlock(&des_init_mutex)==0)
 #else
@@ -102,7 +103,7 @@ des_random_key(des_cblock key)
  *
  * Note: this routine calls des_set_random_generator_seed.
  */
-#if !defined(BSDUNIX) && !defined(AFS_SGI_ENV) && !defined(AFS_NT40_ENV) && !defined(AFS_LINUX20_ENV) && !defined(AFS_DARWIN_ENV) && !defined(AFS_DJGPP_ENV)
+#if !defined(BSDUNIX) && !defined(AFS_SGI_ENV) && !defined(AFS_NT40_ENV) && !defined(AFS_LINUX20_ENV) && !defined(AFS_DARWIN_ENV) 
 you lose ... (aka, you get to implement an analog of this for your system ...)
 #else
 
@@ -130,7 +131,7 @@ des_init_random_number_generator(des_cblock key)
      * use a host id and process id in generating the seed to ensure
      * that different servers have different streams:
      */
-#if !defined(AFS_HPUX_ENV) && !defined(AFS_NT40_ENV) && !defined(AFS_DJGPP_ENV)
+#if !defined(AFS_HPUX_ENV) && !defined(AFS_NT40_ENV)
     seed.host_id = gethostid();
 #endif
     seed.process_id = getpid();
@@ -192,7 +193,11 @@ static unsigned char sequence_number[8];
  */
 
 #include <assert.h>
-pthread_mutex_t des_random_mutex;
+pthread_mutex_t des_random_mutex
+#ifdef PTHREAD_MUTEX_INITIALIZER
+= PTHREAD_MUTEX_INITIALIZER
+#endif
+;
 #define LOCK_RANDOM assert(pthread_mutex_lock(&des_random_mutex)==0)
 #define UNLOCK_RANDOM assert(pthread_mutex_unlock(&des_random_mutex)==0)
 #else
@@ -257,7 +262,6 @@ des_generate_random_block(des_cblock block)
 {
     int i;
 
-    INC_RXKAD_STATS(des_randoms);
     /*
      * Encrypt the sequence number to get the new random block:
      */
