@@ -10,12 +10,10 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
-RCSID("$Header: /cvs/openafs/src/finale/translate_et.c,v 1.5.4.2 2001/10/13 04:21:30 shadow Exp $");
 
 #include <afs/com_err.h>
 #include <rx/rxkad.h>
 #include <afs/kautils.h>
-#include <afs/auth.h>
 #include <afs/cellconfig.h>
 #include <afs/cmd.h>
 #include <afs/vlserver.h>
@@ -28,18 +26,17 @@ RCSID("$Header: /cvs/openafs/src/finale/translate_et.c,v 1.5.4.2 2001/10/13 04:2
 #endif
 
 
-#define ERRCODE_RANGE 8			/* from error_table.h */
+#define ERRCODE_RANGE 8		/* from error_table.h */
 
 #include "AFS_component_version_number.c"
 
 int
-main (argc, argv)
-  int   argc;
-  char *argv[];
+main(int argc, char *argv[])
 {
-    int  i;
+    int i;
     afs_int32 code;
     afs_int32 offset;
+    char msgbuf[BUFSIZ];
 
 #ifdef	AFS_AIX32_ENV
     /*
@@ -49,7 +46,7 @@ main (argc, argv)
      * generated which, in many cases, isn't too useful.
      */
     struct sigaction nsa;
-    
+
     sigemptyset(&nsa.sa_mask);
     nsa.sa_handler = SIG_DFL;
     nsa.sa_flags = SA_FULLDUMP;
@@ -67,17 +64,16 @@ main (argc, argv)
     initialize_VOLS_error_table();
 
     if (argc < 2) {
-	fprintf (stderr, "Usage is: %s [<code>]+\n", argv[0]);
-	exit (1);
+	fprintf(stderr, "Usage is: %s [<code>]+\n", argv[0]);
+	exit(1);
     }
 
-    for (i=1; i<argc; i++) {
+    for (i = 1; i < argc; i++) {
 	code = atoi(argv[i]);
-	offset = code & ((1<<ERRCODE_RANGE)-1);
+	offset = code & ((1 << ERRCODE_RANGE) - 1);
 
-	printf ("%d (%s).%d = %s\n", (int) code, error_table_name (code), (int) offset,
-		error_message (code));
+	printf("%d (%s).%d = %s\n", (int)code, afs_error_table_name(code),
+	       (int)offset, afs_error_message_localize(code, msgbuf, BUFSIZ));
     }
     return 0;
 }
-
