@@ -16,8 +16,6 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
-RCSID
-    ("$Header: /cvs/openafs/src/gtx/lightobject.c,v 1.6.14.1 2007/10/30 15:16:39 shadow Exp $");
 
 #include "gtxlightobj.h"	/*Interface for this module */
 #include <stdio.h>		/*Standard I/O stuff */
@@ -60,10 +58,7 @@ static char mn[] = "gator_lightobject";	/*Module name */
  *------------------------------------------------------------------------*/
 
 int
-gator_light_create(light_onp, params)
-     struct onode *light_onp;
-     struct onode_createparams *params;
-
+gator_light_create(struct onode *light_onp, struct onode_createparams *params)
 {				/*gator_light_create */
 
     static char rn[] = "gator_light_create";	/*Routine name */
@@ -90,7 +85,7 @@ gator_light_create(light_onp, params)
 	(struct gator_lightobj *)malloc(sizeof(struct gator_lightobj));
     if (light_data == (struct gator_lightobj *)0) {
 	fprintf(stderr,
-		"[%s:%s] Can't allocate %d bytes for light object private data region, errno is %d\n",
+		"[%s:%s] Can't allocate %" AFS_SIZET_FMT " bytes for light object private data region, errno is %d\n",
 		mn, rn, sizeof(struct gator_lightobj), errno);
 	return (errno);
     }
@@ -99,7 +94,7 @@ gator_light_create(light_onp, params)
 	(struct gwin_strparams *)malloc(sizeof(struct gwin_strparams));
     if (light_strparams == (struct gwin_strparams *)0) {
 	fprintf(stderr,
-		"[%s:%s] Can't allocate %d bytes for light object label in private data region, errno is %d\n",
+		"[%s:%s] Can't allocate %" AFS_SIZET_FMT " bytes for light object label in private data region, errno is %d\n",
 		mn, rn, sizeof(struct gwin_strparams), errno);
 	free(light_data);
 	return (errno);
@@ -149,9 +144,7 @@ gator_light_create(light_onp, params)
  *------------------------------------------------------------------------*/
 
 int
-gator_light_destroy(onp)
-     struct onode *onp;
-
+gator_light_destroy(struct onode *onp)
 {				/*gator_light_destroy */
 
     /*
@@ -185,9 +178,7 @@ gator_light_destroy(onp)
  *------------------------------------------------------------------------*/
 
 int
-gator_light_display(onp)
-     struct onode *onp;
-
+gator_light_display(struct onode *onp)
 {				/*gator_light_display */
 
     static char rn[] = "gator_light_display";	/*Routine name */
@@ -231,9 +222,7 @@ gator_light_display(onp)
  *------------------------------------------------------------------------*/
 
 int
-gator_light_release(onp)
-     struct onode *onp;
-
+gator_light_release(struct onode *onp)
 {				/*gator_light_release */
 
     /*
@@ -267,10 +256,7 @@ gator_light_release(onp)
  *------------------------------------------------------------------------*/
 
 int
-gator_light_set(onp, setting)
-     struct onode *onp;
-     int setting;
-
+gator_light_set(struct onode *onp, int setting)
 {				/*gator_light_set */
 
     static char rn[] = "gator_light_set";	/*Routine name */
@@ -284,7 +270,7 @@ gator_light_set(onp, setting)
     light_data = (struct gator_lightobj *)(onp->o_data);
     label_strparams = (struct gwin_strparams *)(light_data->llrock);
     if (objects_debug)
-	fprintf(stderr, "[%s:%s] Setting light object at 0x%x to %d (%s)", mn,
+	fprintf(stderr, "[%s:%s] Setting light object at %p to %d (%s)", mn,
 		rn, onp, setting, (setting ? "ON" : "OFF"));
     light_data->setting = setting;
     label_strparams->highlight = setting;
