@@ -16,7 +16,7 @@
 
 
 #if defined(UKERNEL)
-#ifdef HAVE_UNISTD_H
+#if defined(HAVE_UNISTD_H) && !defined(__USE_XOPEN)
 #define __USE_XOPEN
 #endif
 #include "afs/sysincludes.h"
@@ -42,7 +42,9 @@
 #endif
 #include <string.h>
 #ifdef HAVE_UNISTD_H
+#if !defined(__USE_XOPEN)
 #define __USE_XOPEN
+#endif
 #include <unistd.h>
 #endif
 #include <afs/cellconfig.h>
