@@ -31,6 +31,7 @@
 
 #define TABLE_TOKENS_LIST 1
 #define TABLE_CELL_LIST 2
+#define TABLE_LINK_LIST 3
 
 #define TAB_TOKENS 1
 #define TAB_CELL_SERV_DB 2
@@ -43,6 +44,11 @@
 #define CELLSRVDB_TABLE_NAME_COLUMN				2
 #define CELLSRVDB_TABLE_DESCRIPTION_COLUMN		3
 
+//Link Table
+#define TABLE_COLUMN_LINK_NAME	0
+#define TABLE_COLUMN_LINK_PATH	1
+
+
 @implementation AFSCommanderPref
 
 // -------------------------------------------------------------------------------
@@ -130,10 +136,10 @@
 	afsProperty = [[AFSPropertyManager alloc] init];
 	
 	// register preference pane to detect menuextra killed by user
-/*	[[NSDistributedNotificationCenter defaultCenter] addObserver:self 
-														selector:@selector(mextraChangeActivation:) 
-															name:kAfsCommanderID 
-														  object:kMExtraClosedNotification];*/
+	[[NSDistributedNotificationCenter defaultCenter] addObserver:self
+														selector:@selector(refreshTokensNotify:)
+															name:kAfsCommanderID
+														  object:kMExtraTokenOperation];
 	 
 	[[NSDistributedNotificationCenter defaultCenter] addObserver:self 
 														selector:@selector(refreshGui:) 
@@ -150,9 +156,9 @@
 															   name:NSWorkspaceDidUnmountNotification object:nil];
 	
 	// set self as table data source
-	[((NSTableView*)cellList) setDataSource:self];
-	[((NSTableView*)tokensTable) setDataSource:self];
-	
+	[cellList setDataSource:self];
+	[tokensTable setDataSource:self];
+	//[tableViewLink setDataSource:self];
 	//check the afs state
 	[self setAfsStatus];
 	
@@ -211,11 +217,14 @@
 	[self writePreferenceFile];
 	
 	// unregister preference pane to detect menuextra killed by user
-	[[NSDistributedNotificationCenter defaultCenter] removeObserver:self 
-															   name:kAfsCommanderID 
+	[[NSDistributedNotificationCenter defaultCenter] removeObserver:self
+															   name:kAfsCommanderID
 															 object:kMExtraClosedNotification];
-	[[NSDistributedNotificationCenter defaultCenter] removeObserver:self 
-															   name:kAfsCommanderID 
+	[[NSDistributedNotificationCenter defaultCenter] removeObserver:self
+															   name:kAfsCommanderID
+															 object:kMExtraTokenOperation];
+	[[NSDistributedNotificationCenter defaultCenter] removeObserver:self
+															   name:kAfsCommanderID
 															 object:kMenuExtraEventOccured];
 	[[[NSWorkspace sharedWorkspace] notificationCenter] removeObserver:self 
 																  name:NSWorkspaceDidMountNotification object:nil];
@@ -256,7 +265,6 @@
 // -------------------------------------------------------------------------------
 - (void) readPreferenceFile
 {
-	
 	// read the preference for aklog use
 	NSNumber *useAklogPrefValue = (NSNumber*)CFPreferencesCopyValue((CFStringRef)PREFERENCE_USE_AKLOG, (CFStringRef)kAfsCommanderID,  
 																	kCFPreferencesCurrentUser, kCFPreferencesAnyHost);
@@ -270,9 +278,9 @@
 	[installKRB5AuthAtLoginButton setState:[PListManager checkKrb5AtLoginTimeLaunchdEnable]];
 
 	//check for AFS enable at startup
-	NSNumber *afsEnableStartupTime = (NSNumber*)CFPreferencesCopyValue((CFStringRef)PREFERENCE_START_AFS_AT_STARTUP, 
+	NSNumber *afsEnableStartupTime = (NSNumber*)CFPreferencesCopyValue((CFStringRef)PREFERENCE_START_AFS_AT_STARTUP,
 																	   (CFStringRef)kAfsCommanderID,  kCFPreferencesAnyUser, kCFPreferencesAnyHost);
-	if(afsEnableStartupTime) 
+	if(afsEnableStartupTime)
 		startAFSAtLogin = [afsEnableStartupTime boolValue];
 	else 
 		startAFSAtLogin = false;
@@ -284,6 +292,34 @@
 	
 	//backgrounder state
 	[backgrounderActivationCheck setState:[PListManager launchdJobState:BACKGROUNDER_P_FILE]];
+	
+	//link enabled status
+	NSNumber *linkEnabledStatus =  (NSNumber*)CFPreferencesCopyValue((CFStringRef)PREFERENCE_USE_LINK,  (CFStringRef)kAfsCommanderID,  kCFPreferencesCurrentUser, kCFPreferencesAnyHost);
+	[checkEnableLink setState:[linkEnabledStatus boolValue]];
+	
+	//check the user preference for manage the renew
+	NSNumber *checkRenew =  (NSNumber*)CFPreferencesCopyValue((CFStringRef)PREFERENCE_KRB5_CHECK_ENABLE,  (CFStringRef)kAfsCommanderID,  kCFPreferencesCurrentUser, kCFPreferencesAnyHost);
+	if(checkRenew)[nsButtonEnableDisableKrb5RenewCheck setState:[checkRenew intValue]];
+
+	NSNumber *renewTime = (NSNumber*)CFPreferencesCopyValue((CFStringRef)PREFERENCE_KRB5_RENEW_TIME,  (CFStringRef)kAfsCommanderID,  kCFPreferencesCurrentUser, kCFPreferencesAnyHost);
+	if(renewTime && [renewTime intValue])[nsTextFieldKrb5RenewTime setIntValue:[renewTime intValue]];
+	else [nsTextFieldKrb5RenewTime setIntValue:PREFERENCE_KRB5_RENEW_TIME_DEFAULT_VALUE];
+
+	NSNumber *renewCheckTimeInterval = (NSNumber*)CFPreferencesCopyValue((CFStringRef)PREFERENCE_KRB5_RENEW_CHECK_TIME_INTERVALL,  (CFStringRef)kAfsCommanderID,  kCFPreferencesCurrentUser, kCFPreferencesAnyHost);
+	if(renewCheckTimeInterval && [renewCheckTimeInterval intValue])[nsTextFieldKrb5RenewCheckIntervall setIntValue:[renewCheckTimeInterval intValue]];
+	else [nsTextFieldKrb5RenewCheckIntervall setIntValue:PREFERENCE_KRB5_RENEW_CHECK_TIME_INTERVALL_DEFAULT_VALUE];
+
+	NSNumber *expireTimeForRenew = (NSNumber*)CFPreferencesCopyValue((CFStringRef)PREFERENCE_KRB5_SEC_TO_EXPIRE_TIME_FOR_RENEW,  (CFStringRef)kAfsCommanderID,  kCFPreferencesCurrentUser, kCFPreferencesAnyHost);
+	if(expireTimeForRenew && [expireTimeForRenew intValue])[nsTextFieldKrb5SecToExpireDateForRenew setIntValue:[expireTimeForRenew intValue]];
+	else [nsTextFieldKrb5SecToExpireDateForRenew setIntValue:PREFERENCE_KRB5_SEC_TO_EXPIRE_TIME_FOR_RENEW_DEFAULT_VALUE];
+
+	//link configuration
+	NSData *prefData = (NSData*)CFPreferencesCopyValue((CFStringRef)PREFERENCE_LINK_CONFIGURATION,  (CFStringRef)kAfsCommanderID,  kCFPreferencesCurrentUser, kCFPreferencesAnyHost);
+	linkConfiguration = (NSMutableDictionary*)[NSPropertyListSerialization propertyListFromData:prefData
+																			   mutabilityOption:NSPropertyListMutableContainers
+																						 format:nil
+																			   errorDescription:nil];
+	
 }
 
 // -------------------------------------------------------------------------------
@@ -292,10 +328,6 @@
 - (void) writePreferenceFile
 {
 	//Set the preference for afs path
-	/*CFPreferencesSetValue((CFStringRef)PREFERENCE_AFS_SYS_PAT, 
-						  (CFStringRef)[((NSTextField*) installationPathTextField ) stringValue], 
-						  (CFStringRef)kAfsCommanderID, kCFPreferencesAnyUser, kCFPreferencesAnyHost);*/
-	
 	//Set the preference for aklog use
 	CFPreferencesSetValue((CFStringRef)PREFERENCE_USE_AKLOG, 
 						  (CFNumberRef)[NSNumber numberWithInt:[useAklogCheck state]], 
@@ -313,12 +345,31 @@
 	
 	//set aklog at login
 	CFPreferencesSetValue((CFStringRef)PREFERENCE_SHOW_STATUS_MENU, 
-						  (CFNumberRef)[NSNumber numberWithBool:[afsMenucheckBox state]], 
+						  (CFNumberRef)[NSNumber numberWithBool:[(NSButton*)afsMenucheckBox state]],
 						  (CFStringRef)kAfsCommanderID, kCFPreferencesCurrentUser, kCFPreferencesAnyHost);
 	
+	//preference for link
+	CFPreferencesSetValue((CFStringRef)PREFERENCE_USE_LINK,
+						  (CFNumberRef)[NSNumber numberWithBool:[checkEnableLink state]], 
+						  (CFStringRef)kAfsCommanderID, kCFPreferencesCurrentUser, kCFPreferencesAnyHost);
+	
+		//preference for renew time
+	CFPreferencesSetValue((CFStringRef)PREFERENCE_KRB5_RENEW_TIME,
+						  (CFNumberRef)[NSNumber numberWithInt:[nsTextFieldKrb5RenewTime intValue]],
+						  (CFStringRef)kAfsCommanderID,  kCFPreferencesCurrentUser, kCFPreferencesAnyHost);
+
+		//expire time for renew
+	CFPreferencesSetValue((CFStringRef)PREFERENCE_KRB5_SEC_TO_EXPIRE_TIME_FOR_RENEW,
+						  (CFNumberRef)[NSNumber numberWithInt:[nsTextFieldKrb5SecToExpireDateForRenew intValue]],
+						  (CFStringRef)kAfsCommanderID,  kCFPreferencesCurrentUser, kCFPreferencesAnyHost);
+
+		//sec to expiretime for renew job
+	CFPreferencesSetValue((CFStringRef)PREFERENCE_KRB5_RENEW_CHECK_TIME_INTERVALL,
+						  (CFNumberRef)[NSNumber numberWithInt:[nsTextFieldKrb5RenewCheckIntervall intValue]],
+						  (CFStringRef)kAfsCommanderID,  kCFPreferencesCurrentUser, kCFPreferencesAnyHost);
+
 	CFPreferencesSynchronize((CFStringRef)kAfsCommanderID,  kCFPreferencesAnyUser, kCFPreferencesAnyHost);
 	CFPreferencesSynchronize((CFStringRef)kAfsCommanderID,  kCFPreferencesCurrentUser, kCFPreferencesAnyHost);
-	
 	[[NSDistributedNotificationCenter defaultCenter] postNotificationName:kAFSMenuExtraID object:kPrefChangeNotification];
 }
 
@@ -576,46 +627,18 @@
 // -------------------------------------------------------------------------------
 - (IBAction) startStopAfs:(id) sender
 {
-	OSStatus status = noErr;
-	NSString *afsdPath = [TaskUtil searchExecutablePath:@"afsd"];
-	//NSString *startStopScript = nil;
-	NSString *rootHelperApp = nil;
 	BOOL currentAfsState = NO;
-	
 	@try {
-		if(afsdPath == nil) return;
 		currentAfsState = [afsProperty checkAfsStatus];
-		rootHelperApp = [[self bundle] pathForResource:@"afshlp" ofType:@""];
-			
-		//[startStopScript setString: resourcePath];
-		NSLog(@"Launch repair HelperTool");
-		//Check helper app
-		[self repairHelperTool];
-		
 		// make the parameter to call the root helper app
-		status = [[AuthUtil shared] autorize];
-		if(status == noErr){
-			if(currentAfsState){
-				//shutdown afs
-				NSLog(@"Shutting down afs");
-				NSMutableString *afsKextPath = [[NSMutableString alloc] initWithCapacity:256];
-				[afsKextPath setString:[afsProperty path]];
-				[afsKextPath appendString:@"/etc/afs.kext"];
-				
-				//Make the array for arguments
-				NSLog(@"executeTaskWithAuth");
-				const char *stopAfsArgs[] = {"stop_afs", [afsKextPath  UTF8String], [afsdPath UTF8String], 0L};
-				[[AuthUtil shared] execUnixCommand:[rootHelperApp UTF8String] 
-											  args:stopAfsArgs
-											output:nil];
-				
-			} else {
-				NSLog(@"Starting up afs");
-				const char *startAfsArgs[] = {[[[self bundle] pathForResource:@"start_afs" ofType:@"sh"]  UTF8String], [[afsProperty path]  UTF8String], [afsdPath UTF8String], 0L};
-				[[AuthUtil shared] execUnixCommand:[rootHelperApp UTF8String] 
-											  args:startAfsArgs
-											output:nil];
-			}
+		if(currentAfsState){
+			//shutdown afs
+			NSLog(@"Shutting down afs");
+			[afsProperty shutdown];
+		} else {
+			//Start afs
+			NSLog(@"Starting up afs");
+			[afsProperty startup];
 		}
 		[self refreshGui:nil];
 	}
@@ -633,9 +656,16 @@
 - (void) refreshGui:(NSNotification *)notification{
 	BOOL afsIsUp = [afsProperty checkAfsStatus];
 	[self setAfsStatus];
-	[self refreshTokens:nil];
 	[tokensButton setEnabled:afsIsUp];
 	[unlogButton setEnabled:afsIsUp];
+
+}
+
+// -------------------------------------------------------------------------------
+//  -(void) refreshTokensNotify:(NSNotification*)notification
+// -------------------------------------------------------------------------------
+-(void) refreshTokensNotify:(NSNotification*)notification {
+	[self refreshTokens:nil];
 }
 
 // -------------------------------------------------------------------------------
@@ -654,7 +684,7 @@
 // -------------------------------------------------------------------------------
 - (IBAction) info:(id) sender
 {
-	[((InfoController*) infoController) showHtmlResource:[[self bundle] pathForResource:@"licenza" ofType:@"rtf"]];
+	[((InfoController*) infoController) showHtmlResource:[[self bundle] pathForResource:@"license" ofType:@"rtf"]];
 
 	[NSApp beginSheet: infoSheet
 	   modalForWindow: [[self mainView] window]
@@ -760,32 +790,13 @@
 - (IBAction) afsStartupSwitchEvent:(id) sender {
 	NSString *rootHelperApp = [[self bundle] pathForResource:@"afshlp" ofType:@""];
 	//get the new state
-	NSString *afsdPath = [TaskUtil searchExecutablePath:@"afsd"];
-	NSString *afsStartupScriptPath = [[self bundle] pathForResource:@"start_afs" ofType:@"sh"];
 	startAFSAtLogin = [checkButtonAfsAtBootTime state];
-	const char			*startAfsArgs[] = {"load", "-w", [AFS_STARTUP_CONTROL_FILE UTF8String], 0L};
-	const char			*stopAfsArgs[] = {"unload", "-w", [AFS_STARTUP_CONTROL_FILE UTF8String], 0L};
-	const char			*launchctlExecutable = "/bin/launchctl";
-	/*	[PListManager manageAfsStartupLaunchdFile:startAFSAtLogin 
-	 afsStartupScript:afsStartupScriptPath 
-	 afsBasePath:[afsProperty path] afsdPath:afsdPath];
-	 */
-	const char *startupConfigureOption[] = {"start_afs_at_startup", [afsStartupScriptPath  UTF8String],  [afsdPath UTF8String], [[afsProperty path]  UTF8String], 0L};
+	const char *startupConfigureOption[] = {"start_afs_at_startup", startAFSAtLogin?"enable":"disable", 0L};
 	if([[AuthUtil shared] autorize] == noErr) {
-		if(startAFSAtLogin) {
-			[[AuthUtil shared] execUnixCommand:[rootHelperApp UTF8String] 
+			//now disable the launchd configuration
+			[[AuthUtil shared] execUnixCommand:[rootHelperApp UTF8String]
 										  args:startupConfigureOption
 										output:nil];
-			
-			[[AuthUtil shared] execUnixCommand:launchctlExecutable 
-										  args:startAfsArgs
-										output:nil];
-		} else {
-			//now disable the launchd configuration
-			[[AuthUtil shared] execUnixCommand:launchctlExecutable
-										  args:stopAfsArgs
-										output:nil];
-		}
 	}
 }
 
@@ -816,7 +827,7 @@
 -(IBAction) afsMenuActivationEvent:(id) sender
 {
 	CFPreferencesSetValue((CFStringRef)PREFERENCE_SHOW_STATUS_MENU, 
-						  (CFNumberRef)[NSNumber numberWithBool:[afsMenucheckBox state]], 
+						  (CFNumberRef)[NSNumber numberWithBool:[(NSButton*)afsMenucheckBox state]],
 						  (CFStringRef)kAfsCommanderID, kCFPreferencesCurrentUser, kCFPreferencesAnyHost);
 	
 	CFPreferencesSynchronize((CFStringRef)kAfsCommanderID,  kCFPreferencesAnyUser, kCFPreferencesAnyHost);
@@ -911,20 +922,35 @@
 	[alert release];
 }
 
-// -------------------------------------------------------------------------------
-//  manageButtonState:
-// -------------------------------------------------------------------------------
--(void) manageButtonState:(int) rowSelected {
-	[((NSControl*) cellIpButton) setEnabled:rowSelected >= 0];
-	[((NSControl*) removeCellButton) setEnabled:rowSelected >= 0];
-}
-
 // -------------------------------------------------------------------------------
 //  setAfsStatus:
 // -------------------------------------------------------------------------------
 -(void) setAfsStatus
 {
+	
 	BOOL afsIsUp = [afsProperty checkAfsStatus];
+	BOOL afsEnabledAtStartup = NO;
+	
+	NSMutableString *commandOutput = [NSMutableString stringWithCapacity:20];
+	NSString *rootHelperApp = [[self bundle] pathForResource:@"afshlp" ofType:@""];
+		
+#if 0
+			// make the parameter to call the root helper app
+	const char *checkAFSDaemonParam[] = {"check_afs_daemon",  0L};
+	if([[AuthUtil shared] autorize] == noErr) {
+				//now disable the launchd configuration
+		[[AuthUtil shared] execUnixCommand:[rootHelperApp UTF8String]
+									  args:checkAFSDaemonParam
+									output:commandOutput];
+		afsEnabledAtStartup = [commandOutput rangeOfString:@"afshlp:afs daemon registration result:1"].location!=NSNotFound;
+	}
+#else
+	afsEnabledAtStartup = 1;
+#endif
+	
+	
+
+
 	[((NSButton *)startStopButton) setTitle: (afsIsUp?kAfsButtonShutdown:kAfsButtonStartup)];
 	
 	NSMutableAttributedString *colorTitle =[[NSMutableAttributedString alloc] initWithAttributedString:[((NSButton *)startStopButton) attributedTitle]];
@@ -935,7 +961,7 @@
                        range:titleRange];
 	
     [((NSButton *)startStopButton) setAttributedTitle:colorTitle];
-	
+	[checkButtonAfsAtBootTime setState:afsEnabledAtStartup];
 	if(afsIsUp) {
 		[self startTimer];
 	} else {
@@ -976,14 +1002,40 @@
 //  removeExtra:
 // -------------------------------------------------------------------------------
 - (IBAction) removeLink:(id) sender {
+	if(!linkConfiguration) return;
+	int index = 0;
+	NSArray *keys = [linkConfiguration allKeys];
+	NSIndexSet *linkToRemove = [tableViewLink selectedRowIndexes];
+	if( [linkToRemove count] > 0) {
+		index = [linkToRemove firstIndex];
+		do {
+			[linkConfiguration removeObjectForKey:[keys objectAtIndex:index]];
+		} while ((index = [linkToRemove indexGreaterThanIndex:index]) != -1);
+	}
 	
+	//write the new configuration
+	NSData *prefData = nil;
+	if([linkConfiguration count] > 0) {
+		prefData = [NSPropertyListSerialization dataWithPropertyList:linkConfiguration
+															  format:NSPropertyListXMLFormat_v1_0
+															 options:0
+															   error:nil];
+	}
+	CFPreferencesSetValue((CFStringRef)PREFERENCE_LINK_CONFIGURATION,
+						  (CFDataRef)prefData,
+						  (CFStringRef)kAfsCommanderID,  kCFPreferencesCurrentUser, kCFPreferencesAnyHost);
+	
+	CFPreferencesSynchronize((CFStringRef)kAfsCommanderID,  kCFPreferencesCurrentUser, kCFPreferencesAnyHost);
+	
+	//reload the new data
+	[tableViewLink reloadData];
 }
 
 // -------------------------------------------------------------------------------
 //  removeExtra:
 // -------------------------------------------------------------------------------
 - (IBAction) enableLink:(id) sender {
-	
+	[self writePreferenceFile];
 }
 
 // -------------------------------------------------------------------------------
@@ -998,6 +1050,12 @@
 	[backgrounderActivationCheck setState:[PListManager launchdJobState:BACKGROUNDER_P_FILE]];
 }
 
+// -------------------------------------------------------------------------------
+//  tableViewLinkPerformClick:
+// -------------------------------------------------------------------------------
+- (IBAction) tableViewLinkPerformClick:(id) sender {
+	NSLog(@"tableViewLinkPerformClick");
+}
 
 // -------------------------------------------------------------------------------
 //  - (void)tabView:(NSTabView *)tabView willSelectTabViewItem: (NSTabViewItem *)tabViewItem
@@ -1007,11 +1065,51 @@
 	//check to see if the cache param tab is the tab that will be selected
 	if([((NSString*)[tabViewItem identifier]) intValue] == TAB_LINK)
 	{
-		[ViewUtility enbleDisableControlView:[tabViewItem view]
-								controlState:NO];
+		[tableViewLink reloadData];
 	}
 }
+// -------------------------------------------------------------------------------
+//  tableViewLinkPerformClick:
+// -------------------------------------------------------------------------------
+- (IBAction) enableDisableKrb5RenewCheck:(id) sender {
+	//NSLog(@"enableDisableKrb5RenewCheck");
+	CFPreferencesSetValue((CFStringRef)PREFERENCE_KRB5_CHECK_ENABLE,
+						  (CFNumberRef) [NSNumber numberWithInt:[(NSButton*)sender intValue]],
+						  (CFStringRef)kAfsCommanderID,  kCFPreferencesCurrentUser, kCFPreferencesAnyHost);
+	CFPreferencesSynchronize((CFStringRef)kAfsCommanderID,  kCFPreferencesCurrentUser, kCFPreferencesAnyHost);
+		//notify the backgrounder
+	[[NSDistributedNotificationCenter defaultCenter] postNotificationName:kAFSMenuExtraID object:kPrefChangeNotification];
+}
+// -------------------------------------------------------------------------------
+//  tableViewLinkPerformClick:
+// -------------------------------------------------------------------------------
+- (IBAction) krb5RenewParamChange:(id) sender {
+	//NSLog(@"krb5RenewParamChange %@", [sender description]);
+	CFStringRef prefStr = 0L;
+	NSNumber *newNumberValue = [NSNumber numberWithInt:[(NSButton*)sender intValue]];
 
+	switch([(NSControl*)sender tag]){
+		case 1:{
+			prefStr = (CFStringRef)PREFERENCE_KRB5_RENEW_TIME;
+		}
+		break;
+
+		case 2:{
+			prefStr = (CFStringRef)PREFERENCE_KRB5_SEC_TO_EXPIRE_TIME_FOR_RENEW;
+		}
+		break;
+
+		case 3:{
+			prefStr = (CFStringRef)PREFERENCE_KRB5_RENEW_CHECK_TIME_INTERVALL;
+		}
+		break;
+	}
+
+		/*CFPreferencesSetValue(prefStr,
+						  (CFNumberRef)newNumberValue,
+						  (CFStringRef)kAfsCommanderID,  kCFPreferencesCurrentUser, kCFPreferencesAnyHost);
+	CFPreferencesSynchronize((CFStringRef)kAfsCommanderID,  kCFPreferencesCurrentUser, kCFPreferencesAnyHost);*/
+}
 @end
 
 @implementation AFSCommanderPref (NSTableDataSource)
@@ -1072,6 +1170,11 @@
 			//We are refreshing cell db table
 			result = [self getTableCelListValue:[identifier intValue] row:rowIndex];
 			break;
+			
+		case TABLE_LINK_LIST:
+			result = [self getTableLinkValue:[identifier intValue] row:rowIndex];
+			break;
+
 		
 	}
 	return result;  
@@ -1121,6 +1224,26 @@
 	return result;
 }
 
+// -------------------------------------------------------------------------------
+//  getTableCelListValue:
+// -------------------------------------------------------------------------------
+- (id)getTableLinkValue:(int) colId row:(int)row
+{
+	id result = nil;
+	NSArray *allKey = [linkConfiguration allKeys];
+	switch(colId){
+		case TABLE_COLUMN_LINK_NAME:
+			result = [allKey objectAtIndex:row];
+			break;
+			
+		case TABLE_COLUMN_LINK_PATH:
+			result = [linkConfiguration objectForKey:[allKey objectAtIndex:row]];
+			break;
+	}
+	return result;
+}
+
+
 // -------------------------------------------------------------------------------
 //  numberOfRowsInTableView:
 // -------------------------------------------------------------------------------
@@ -1138,6 +1261,11 @@
 			if(filteredCellDB)  rowCount = [filteredCellDB count];
 			break;
 			
+		case TABLE_LINK_LIST:
+			if(linkConfiguration)  rowCount = [linkConfiguration count];
+			break;
+
+			
 	}	
 	return rowCount;  
 }
@@ -1150,7 +1278,21 @@
 // -------------------------------------------------------------------------------
 - (BOOL)selectionShouldChangeInTableView:(NSTableView *)aTable
 {
-	[self manageButtonState:[aTable selectedRow]];
+	switch([aTable tag]){
+		case TABLE_TOKENS_LIST:
+			
+			break;
+			
+		case TABLE_CELL_LIST:
+			[self tableViewCellmanageButtonState:[aTable selectedRow]];
+			break;
+			
+		case TABLE_LINK_LIST:
+			break;
+			
+			
+	}	
+	
 	return YES;
 }
 
@@ -1159,10 +1301,57 @@
 // -------------------------------------------------------------------------------
 - (BOOL)tableView:(NSTableView *)aTable shouldSelectRow:(int)aRow
 {
-	[self manageButtonState:aRow];
+	switch([aTable tag]){
+		case TABLE_TOKENS_LIST:
+			
+			break;
+			
+		case TABLE_CELL_LIST:
+			[self tableViewCellmanageButtonState:aRow];
+			break;
+			
+		case TABLE_LINK_LIST:
+			break;
+			
+			
+	}
+	
 	return YES;
 }
 
+// -------------------------------------------------------------------------------
+//  tableView:
+// -------------------------------------------------------------------------------
+- (void)tableViewSelectionDidChange:(NSNotification *)aNotification {
+	NSTableView *aTable = [aNotification object];
+	switch([aTable tag]){
+		case TABLE_TOKENS_LIST:
+			break;
+			
+		case TABLE_CELL_LIST:
+			break;
+			
+		case TABLE_LINK_LIST:
+			[self tableViewLinkmanageButtonState:[aTable selectedRowIndexes]];
+			break;
+			
+			
+	}
+}
+// -------------------------------------------------------------------------------
+//  manageButtonState:
+// -------------------------------------------------------------------------------
+-(void) tableViewCellmanageButtonState:(int) rowSelected  {
+	[((NSControl*) cellIpButton) setEnabled:rowSelected >= 0];
+	[((NSControl*) removeCellButton) setEnabled:rowSelected >= 0];
+}
+
+// -------------------------------------------------------------------------------
+//  manageButtonState:
+// -------------------------------------------------------------------------------
+-(void) tableViewLinkmanageButtonState:(NSIndexSet *) rowsSelectedIndex {
+	[buttonRemoveLink setEnabled:[rowsSelectedIndex count]>0];
+}
 @end
 
 
@@ -1211,5 +1400,8 @@
 - (void)didEndSymlinkSheet:(NSWindow *)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo
 {
 	[lyncCreationSheet orderOut:self];
+	[self readPreferenceFile];
+	[tableViewLink reloadData];
+	
 }
 @end
