@@ -10,8 +10,6 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
-RCSID
-    ("$Header: /cvs/openafs/src/ptserver/ptclient.c,v 1.12.2.2 2006/07/31 21:51:21 shadow Exp $");
 
 #ifdef	AFS_AIX32_ENV
 #include <signal.h>
@@ -32,7 +30,9 @@ RCSID
 #include <afs/com_err.h>
 #include <afs/cellconfig.h>
 #include "ptclient.h"
+#include "ptuser.h"
 #include "pterror.h"
+#include "display.h"
 #include <afs/afsutil.h>
 
 
@@ -43,6 +43,8 @@ char *whoami;
 
 #ifndef AFS_PTHREAD_ENV
 extern struct ubik_client *pruclient;
+static void skip(char **);
+static void PrintHelp(void);
 #endif
 
 static int ignoreExist = 0;
@@ -64,9 +66,7 @@ osi_audit()
 #endif /* !AFS_PTHREAD_ENV */
 
 int
-GetToken(format, l)
-     char *format;
-     afs_int32 *l;
+GetToken(char *format, afs_int32 *l)
 {
     int c;
 
@@ -87,9 +87,7 @@ GetToken(format, l)
 #define GetXInt32(l) GetToken ("%x", l)
 
 int
-GetString(s, slen)
-     char *s;
-     int slen;
+GetString(char *s, int slen)
 {
     char *beg;
     int l;
@@ -123,8 +121,7 @@ GetString(s, slen)
 }
 
 int
-CodeOk(code)
-     afs_int32 code;
+CodeOk(afs_int32 code)
 {
     if (!ignoreExist)
 	return code;
@@ -132,14 +129,13 @@ CodeOk(code)
 }
 
 int
-PrintEntry(ea, e, indent)
-     afs_int32 ea;
-     struct prentry *e;
-     int indent;
+PrintEntry(afs_int32 ea, struct prentry *e, int indent)
 {
     /* handle screwed up versions of DumpEntry */
     if (e->flags & PRCONT) {
-	afs_int32 id = *(afs_int32 *) (e->name);
+	afs_int32 id;
+
+	memcpy(&id, e->name, sizeof(id));
 	if ((id != PRBADID) && ((id > (1 << 24)) || (id < -(1 << 24)))) {
 	    /* assume server incorrectly swapped these bytes... */
 	    int i = 0;
@@ -164,16 +160,16 @@ PrintEntry(ea, e, indent)
 
 #include "AFS_component_version_number.c"
 
-main(argc, argv)
-     int argc;
-     char **argv;
+int
+main(int argc, char **argv)
 {
     register afs_int32 code;
     char op[8];
     char name[PR_MAXNAMELEN];
     afs_int32 id, oid = ANONYMOUSID, gid;
     afs_int32 pos;
-    int i;
+    unsigned int i;
+    int n;
     struct prentry entry;
     prlist alist;
     idlist lid;
@@ -205,30 +201,30 @@ main(argc, argv)
 
     strcpy(confdir, AFSDIR_CLIENT_ETC_DIRPATH);
     cell = 0;
-    i = 1;
-    while (i < argc) {
-	int arglen = strlen(argv[i]);
+    n = 1;
+    while (n < argc) {
+	int arglen = strlen(argv[n]);
 	char arg[256];
-	lcstring(arg, argv[i], sizeof(arg));
+	lcstring(arg, argv[n], sizeof(arg));
 #define IsArg(a) (strncmp (arg,a, arglen) == 0)
 	if (IsArg("-testconfdir"))
-	    strncpy(confdir, argv[++i], sizeof(confdir));
+	    strncpy(confdir, argv[++n], sizeof(confdir));
 	else if (IsArg("client"))
 	    strncpy(confdir, AFSDIR_CLIENT_ETC_DIRPATH, sizeof(confdir));
 	else if (IsArg("server"))
 	    strncpy(confdir, AFSDIR_SERVER_ETC_DIRPATH, sizeof(confdir));
 	else if (IsArg("0") || IsArg("1") || IsArg("2"))
-	    security = atoi(argv[i]);
+	    security = atoi(argv[n]);
 	else if (IsArg("-ignoreexist"))
 	    ignoreExist++;
 	else if (IsArg("-cell"))
-	    cell = argv[++i];
+	    cell = argv[++n];
 	else {
 	    printf
 		("Usage is: 'prclient [-testconfdir <dir> | server | client] [0 | 1 | 2] [-ignoreExist] [-cell <cellname>]\n");
 	    exit(1);
 	}
-	i++;
+	n++;
     }
 
     printf("Using CellServDB file in %s\n", confdir);
@@ -237,7 +233,7 @@ main(argc, argv)
 
     code = pr_Initialize(security, confdir, cell);
     if (code) {
-	com_err(whoami, code, "Couldn't initialize protection library");
+	afs_com_err(whoami, code, "Couldn't initialize protection library");
 	exit(1);
     }
 
@@ -252,7 +248,7 @@ main(argc, argv)
 
 	code = GetString(op, sizeof(op));
 	if (code) {
-	    com_err(whoami, PRBADARG,
+	    afs_com_err(whoami, PRBADARG,
 		    "error reading opcode in line '%s', got '%.*s'", line,
 		    sizeof(op), op);
 	    exit(1);
@@ -269,7 +265,7 @@ main(argc, argv)
 	    else
 		code = ubik_PR_INewEntry(pruclient, 0, name, id, oid);
 	    if (CodeOk(code))
-		com_err(whoami, code, "on %s %s %d %d", op, name, id, oid);
+		afs_com_err(whoami, code, "on %s %s %d %d", op, name, id, oid);
 	} else if (!strcmp(op, "sf")) {
 	    afs_int32 mask, access, gq, uq;
 	    if (GetInt32(&id) || GetXInt32(&mask) || GetXInt32(&access)
@@ -280,7 +276,7 @@ main(argc, argv)
 		    ubik_PR_SetFieldsEntry(pruclient, 0, id, mask,
 			      access, gq, uq, 0, 0);
 	    if (CodeOk(code))
-		com_err(whoami, code, "on %s %d %x %x %d %d", op, id, mask,
+		afs_com_err(whoami, code, "on %s %d %x %x %d %d", op, id, mask,
 			access, gq, uq);
 	} else if (!strcmp(op, "ce")) {
 	    char newname[PR_MAXNAMELEN];
@@ -293,7 +289,7 @@ main(argc, argv)
 		    ubik_PR_ChangeEntry(pruclient, 0, id, newname, oid,
 			      newid);
 	    if (CodeOk(code))
-		com_err(whoami, code, "on %s %d %s %d %d", op, id, newname,
+		afs_com_err(whoami, code, "on %s %d %s %d %d", op, id, newname,
 			oid, newid);
 	} else if (!strcmp(op, "wh")) {
 	    /* scanf("%d",&id); */
@@ -311,7 +307,7 @@ main(argc, argv)
 	    if (GetInt32(&pos))
 		code = PRBADARG;
 	    else
-		code = ubik_PR_DumpEntry(pruclient, 0, pos, &entry);
+		code = ubik_PR_DumpEntry(pruclient, 0, pos, (struct prdebugentry *)&entry);
 	    if (CodeOk(code))
 		printf("%s\n", pr_ErrorMsg(code));
 	    if (code == PRSUCCESS) {
@@ -334,7 +330,7 @@ main(argc, argv)
 	    else
 		code = ubik_PR_AddToGroup(pruclient, 0, id, gid);
 	    if (CodeOk(code))
-		com_err(whoami, code, "on %s %d %d", op, id, gid);
+		afs_com_err(whoami, code, "on %s %d %d", op, id, gid);
 	} else if (!strcmp(op, "iton")) {
 	    lid.idlist_val = (afs_int32 *) malloc(20 * sizeof(afs_int32));
 	    ptr = lid.idlist_val;
@@ -477,7 +473,7 @@ main(argc, argv)
 	    else if (!(hostinfo = gethostbyname(name)))
 		code = PRBADARG;
 	    else {
-		hostaddr = hostinfo->h_addr_list[0];
+		hostaddr = (struct in_addr *)hostinfo->h_addr_list[0];
 		id = ntohl(hostaddr->s_addr);
 		code =
 		    ubik_PR_GetHostCPS(pruclient, 0, id, &alist, &over);
@@ -648,7 +644,7 @@ main(argc, argv)
 	else if (!strcmp(op, "fih")) {
 	    char tname[128];
 	    struct PrUpdateEntry uentry;
-	    bzero(&uentry, sizeof(uentry));
+	    memset(&uentry, 0, sizeof(uentry));
 	    /* scanf("%s",name); */
 	    if (GetString(name, sizeof(name))) {
 		code = PRBADARG;
@@ -675,7 +671,7 @@ main(argc, argv)
 	} else if (!strcmp(op, "fnh")) {
 	    int tid;
 	    struct PrUpdateEntry uentry;
-	    bzero(&uentry, sizeof(uentry));
+	    memset(&uentry, 0, sizeof(uentry));
 	    /* scanf("%d", &id); */
 	    if (GetInt32(&id)) {
 		code = PRBADARG;
@@ -713,7 +709,8 @@ main(argc, argv)
 }
 
 
-PrintHelp()
+static void
+PrintHelp(void)
 {
     printf("cr name id owner - create entry with name and id.\n");
     printf("wh id  - what is the offset into database for id?\n");
@@ -742,9 +739,8 @@ PrintHelp()
     printf("q - quit.\n?- this message.\n");
 }
 
-
-skip(s)
-     char **s;
+static void
+skip(char **s)
 {
     while (**s != ' ' && **s != '\0')
 	(*s)++;
