@@ -10,8 +10,6 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
-RCSID
-    ("$Header: /cvs/openafs/src/ptserver/testpt.c,v 1.14.14.4 2007/10/31 04:09:32 shadow Exp $");
 
 #include <ctype.h>
 #include <errno.h>
@@ -25,6 +23,7 @@ RCSID
 #include <netinet/in.h>
 #endif
 #include <string.h>
+#include <math.h>
 #ifdef HAVE_FCNTL_H
 #include <fcntl.h>
 #endif
@@ -42,6 +41,8 @@ RCSID
 #include <afs/cellconfig.h>
 #include "ptclient.h"
 #include "pterror.h"
+#include "ptuser.h"
+#include "ptprototypes.h"
 #include <afs/afsutil.h>
 #include <afs/com_err.h>
 
@@ -178,11 +179,11 @@ afs_int32 *groupOwners;		/* ids of owners of groups */
 int nUsers, nGroups, nAdds, nRems, nUDels, nGDels;
 
 int
-IdCmp(afs_int32 *a, afs_int32 *b)
+IdCmp(const void *a, const void *b)
 {
-    if (*a > *b) {
+    if (*(afs_int32 *)a > *(afs_int32 *)b) {
 	return 1;
-    } else if (*a == *b) {
+    } else if (*(afs_int32 *)a == *(afs_int32 *)b) {
 	return 0;
     } else /* (*a < *b) */ {
 	return -1;
@@ -199,7 +200,6 @@ static int
 GetGroupLimit(int N, int x)
 {
     int y;
-    double sqrt();
 
     if ((x >= N) || (x < 0)) {
 	printf("GetGroupLimit: input value out of range %d (%d)\n", x, N);
@@ -271,9 +271,8 @@ CreateGroup(int g)
     afs_int32 code;
     char name[16];
     afs_int32 id = 0;
-    afs_int32 flags = PRGRP;
-    afs_int32 owner;
-    char *ownerName;
+    afs_int32 owner = 0;
+    char *ownerName = NULL;
     int ownerType;		/* type of ownership */
     static char *lastGroupPrefix;	/* prefix used for type==2 */
 
@@ -344,7 +343,7 @@ DeleteRandomId(afs_int32 *list)
     k = random();		/* random starting point */
     for (j = 0; j < number; j++) {	/* find an undeleted id */
 	m = (k + j) % number;
-	if (id = list[m]) {
+	if ((id = list[m])) {
 	    code = ubik_PR_Delete(pruclient, 0, id);
 	    if (code) {
 		afs_com_err(whoami, code, "Couldn't delete %di", id);
@@ -582,21 +581,23 @@ TestManyMembers(struct cmd_syndesc *as, void *arock)
 	if (ui) {
 	    int i;
 	    int ng;		/* number groups */
-	    int (*proc) ();	/* membership listing procedure */
 	    int over;
+	    int (*proc)(struct ubik_client *, afs_int32, afs_int32, prlist *,
+	    		afs_int32 *);
 	    prlist alist;
 
 	    alist.prlist_len = 0;
 	    alist.prlist_val = 0;
-	    if (random() & 4)
-		proc = PR_ListElements;
-	    else
-		proc = PR_GetCPS;
-	    code = ubik_Call(proc, pruclient, 0, ui, &alist, &over);
+	    if (random() & 4) {
+	    	proc = ubik_PR_ListElements;
+	    } else {
+	    	proc = ubik_PR_GetCPS;
+	    }
+	    code = (*proc)(pruclient, 0, ui, &alist, &over);
 	    if (code) {
 		afs_com_err(whoami, code,
 			"getting membership list of (%di) using %s", ui,
-			((proc == PR_GetCPS) ? "GetCPR" : "ListElements"));
+			(proc == ubik_PR_ListElements?"ListElements":"GetCPS"));
 		exit(24);
 	    }
 	    if (over) {
@@ -607,7 +608,7 @@ TestManyMembers(struct cmd_syndesc *as, void *arock)
 		if (population[u * number + i])
 		    glist[ng++] = groups[i];
 	    qsort(glist, ng, sizeof(afs_int32), IdCmp);
-	    if (ng != (alist.prlist_len - ((proc == PR_GetCPS) ? 3 : 0))) {
+	    if (ng != (alist.prlist_len - ((proc == ubik_PR_GetCPS) ? 3 : 0))) {
 		fprintf(stderr,
 			"Membership list for %di of unexpected length: was %d but expected %d\n",
 			ui, alist.prlist_len, ng);
@@ -624,7 +625,7 @@ TestManyMembers(struct cmd_syndesc *as, void *arock)
 		}
 	    if (code)
 		exit(21);
-	    if (proc == PR_GetCPS) {
+	    if (proc == ubik_PR_GetCPS) {
 		if ((alist.prlist_val[i /* =ng */ ] != AUTHUSERID) ||
 		    (alist.prlist_val[++i] != ANYUSERID)
 		    || (alist.prlist_val[++i] != ui)) {
@@ -852,7 +853,7 @@ TestPrServ(struct cmd_syndesc *as, void *arock)
     strcat(name, ":abcdefghijklmnopqrstuvwxyz");
     name[0] = 1;		/* bash the owner name */
     id = 0;
-    code = pr_CreateGroup(name, &id);
+    code = pr_CreateGroup(name, creator, &id);
     if (code != PRNOENT) {	/* owner doesn't exist */
 	afs_com_err(whoami, code, "succeeded creating %s", name);
 	exit(9);
@@ -1067,7 +1068,7 @@ add_std_args(register struct cmd_syndesc *ts)
 }
 
 int
-osi_audit()
+osi_audit(void)
 {
 /* OK, this REALLY sucks bigtime, but I can't tell who is calling
  * afsconf_CheckAuth easily, and only *SERVERS* should be calling osi_audit
