@@ -25,8 +25,6 @@
 #include <afs/param.h>
 #endif
 
-RCSID
-    ("$Header: /cvs/openafs/src/rx/rx_globals.c,v 1.9.8.3 2006/12/19 03:19:08 shadow Exp $");
 
 /* Enable data initialization when the header file is included */
 #define GLOBALSINIT(stuff) = stuff
@@ -47,3 +45,60 @@ RCSID
 #endif /* KERNEL */
 
 #include "rx_globals.h"
+
+void rx_SetMaxReceiveWindow(int packets)
+{
+    if (packets > rx_maxWindow)
+	packets = rx_maxWindow;
+
+    rx_maxReceiveWindow = packets;
+}
+
+int rx_GetMaxReceiveWindow(void)
+{
+    return rx_maxReceiveWindow;
+}
+
+void rx_SetMaxSendWindow(int packets)
+{
+    if (packets > rx_maxWindow)
+	packets = rx_maxWindow;
+
+    rx_maxSendWindow = packets;
+}
+
+int rx_GetMaxSendWindow(void)
+{
+    return rx_maxSendWindow;
+}
+
+void rx_SetMinPeerTimeout(int timeo)
+{
+    if (timeo >= 1 && timeo < 1000)
+        rx_minPeerTimeout = timeo;
+}
+
+int rx_GetMinPeerTimeout(void)
+{
+    return rx_minPeerTimeout;
+}
+
+#ifdef AFS_NT40_ENV
+
+void rx_SetRxDeadTime(int seconds)
+{
+    rx_connDeadTime = seconds;
+}
+
+int rx_GetMinUdpBufSize(void)
+{
+    return 64*1024;
+}
+
+void rx_SetUdpBufSize(int x)
+{
+    if (x > rx_GetMinUdpBufSize())
+        rx_UdpBufSize = x;
+}
+
+#endif /* AFS_NT40_ENV */
