@@ -454,7 +454,7 @@ rxi_syscall(afs_uint32 a3, afs_uint32 a4, void *a5)
     old = signal(SIGSYS, SIG_IGN);
 
 #if defined(AFS_SGI_ENV)
-    rcode = afs_syscall(a3, a4, a5);
+    rcode = afs_syscall(AFS_SYSCALL, 28, a3, a4, a5);
 #else
     rcode = syscall(AFS_SYSCALL, 28 /* AFSCALL_CALL */ , a3, a4, a5);
 #endif /* AFS_SGI_ENV */
