@@ -11,131 +11,115 @@
  * xdr_rx.c.  XDR using RX. 
  */
 
+#include <afsconfig.h>
 #ifdef	KERNEL
 #include "afs/param.h"
 #else
 #include <afs/param.h>
 #endif
-#include <afsconfig.h>
 
-RCSID
-    ("$Header: /cvs/openafs/src/rx/xdr_rx.c,v 1.9 2003/07/15 23:16:13 shadow Exp $");
 
 #ifdef KERNEL
-#ifndef UKERNEL
-#include "h/types.h"
-#include "h/uio.h"
-#ifdef	AFS_OSF_ENV
-#include <net/net_globals.h>
-#endif /* AFS_OSF_ENV */
-#ifdef AFS_LINUX20_ENV
-#include "h/socket.h"
-#else
-#include "rpc/types.h"
-#endif
-#ifdef  AFS_ALPHA_ENV
-#undef kmem_alloc
-#undef kmem_free
-#undef mem_alloc
-#undef mem_free
-#undef register
-#endif /* AFS_ALPHA_ENV */
-#ifdef AFS_LINUX22_ENV
-#ifndef quad_t
-#define quad_t __quad_t
-#define u_quad_t __u_quad_t
-#endif
-#endif
-#include "rx/xdr.h"
-#include "netinet/in.h"
-#else /* !UKERNEL */
-#include "afs/sysincludes.h"
-#include "rpc/types.h"
-#include "rpc/xdr.h"
-#endif /* !UKERNEL */
-#include "rx/rx.h"
-
-#include "afs/longc_procs.h"
+# include "afs/sysincludes.h"
+# ifndef UKERNEL
+#  include "h/types.h"
+#  include "h/uio.h"
+#  ifdef AFS_OSF_ENV
+#   include <net/net_globals.h>
+#  endif /* AFS_OSF_ENV */
+#  ifdef AFS_LINUX20_ENV
+#   include "h/socket.h"
+#  else
+#   include "rpc/types.h"
+#  endif
+#  ifdef  AFS_OSF_ENV
+#   undef kmem_alloc
+#   undef kmem_free
+#   undef mem_alloc
+#   undef mem_free
+#   undef register
+#  endif /* AFS_OSF_ENV */
+#  ifdef AFS_LINUX22_ENV
+#   ifndef quad_t
+#    define quad_t __quad_t
+#    define u_quad_t __u_quad_t
+#   endif
+#  endif
+#  include "rx/xdr.h"
+#  include "netinet/in.h"
+# endif /* !UKERNEL */
+# include "rx/xdr.h"
+# include "rx/rx.h"
 
 #else /* KERNEL */
-#include <sys/types.h>
-#include <stdio.h>
-#ifndef AFS_NT40_ENV
-#include <netinet/in.h>
-#endif
-#include "rx.h"
-#include "xdr.h"
-#endif /* KERNEL */
-
-/*
- * This section should really go in the param.*.h files.
- */
-#if defined(KERNEL)
-/*
- * kernel version needs to agree with <rpc/xdr.h>
- */
-# if defined(AFS_SUN57_ENV)
-#  define AFS_RPC_INLINE_T rpc_inline_t
-# elif defined(AFS_DUX40_ENV)
-#  define AFS_RPC_INLINE_T int
-# else
-#  define AFS_RPC_INLINE_T long
+# include <sys/types.h>
+# include <stdio.h>
+# ifndef AFS_NT40_ENV
+#  include <netinet/in.h>
 # endif
-#else /* KERNEL */
-/*
- * user version needs to agree with "xdr.h", i.e. <rx/xdr.h>
- */
-#  define AFS_RPC_INLINE_T afs_int32
+# include "rx.h"
+# include "xdr.h"
 #endif /* KERNEL */
 
 /* Static prototypes */
-#if (defined(AFS_SGI61_ENV) && (_MIPS_SZLONG != _MIPS_SZINT)) || defined(AFS_HPUX_64BIT_ENV)
-static bool_t xdrrx_getint64(XDR * xdrs, long *lp);
-static bool_t xdrrx_putint64(XDR * xdrs, long *lp);
-#endif /* (defined(AFS_SGI61_ENV) && (_MIPS_SZLONG != _MIPS_SZINT)) || defined(AFS_HPUX_64BIT_ENV) */
+#ifdef AFS_XDR_64BITOPS
+static bool_t xdrrx_getint64(XDR *axdrs, long *lp);
+static bool_t xdrrx_putint64(XDR *axdrs, long *lp);
+#endif /* AFS_XDR_64BITOPS */
 
-static bool_t xdrrx_getint32(XDR * xdrs, afs_int32 * lp);
-static bool_t xdrrx_putint32(register XDR * xdrs, register afs_int32 * lp);
-static bool_t xdrrx_getbytes(register XDR * xdrs, register caddr_t addr,
-			     register u_int len);
-static bool_t xdrrx_putbytes(register XDR * xdrs, register caddr_t addr,
-			     register u_int len);
-static AFS_RPC_INLINE_T *xdrrx_inline(register XDR * xdrs,
-				      register u_int len);
+static bool_t xdrrx_getint32(XDR *axdrs, afs_int32 * lp);
+static bool_t xdrrx_putint32(XDR *axdrs, afs_int32 * lp);
+static bool_t xdrrx_getbytes(XDR *axdrs, caddr_t addr,
+			     u_int len);
+static bool_t xdrrx_putbytes(XDR *axdrs, caddr_t addr,
+			     u_int len);
+static afs_int32 *xdrrx_inline(XDR *axdrs, u_int len);
 
 
 /*
  * Ops vector for stdio type XDR
  */
 static struct xdr_ops xdrrx_ops = {
-#if defined(KERNEL) && ((defined(AFS_SGI61_ENV) && (_MIPS_SZLONG != _MIPS_SZINT)) || defined(AFS_HPUX_64BIT_ENV))
-    xdrrx_getint64,
-    xdrrx_putint64,
-#endif /* defined(KERNEL) && ((defined(AFS_SGI61_ENV) && (_MIPS_SZLONG != _MIPS_SZINT)) || defined(AFS_HPUX_64BIT_ENV)) */
-#if !(defined(KERNEL) && defined(AFS_SUN57_ENV))
-    xdrrx_getint32,		/* deserialize an afs_int32 */
-    xdrrx_putint32,		/* serialize an afs_int32 */
+#if defined(AFS_NT40_ENV) || (defined(AFS_SGI_ENV) && !defined(__c99))
+#ifdef AFS_XDR_64BITOPS
+    xdrrx_getint64,     /* deserialize an afs_int64 */
+    xdrrx_putint64,     /* serialize an afs_int64 */
 #endif
-    xdrrx_getbytes,		/* deserialize counted bytes */
-    xdrrx_putbytes,		/* serialize counted bytes */
-    0,				/* get offset in the stream: not supported. */
-    0,				/* set offset in the stream: not supported. */
-    xdrrx_inline,		/* prime stream for inline macros */
-    0,				/* destroy stream */
+    /* Windows does not support labeled assigments */
+    xdrrx_getint32,	/* deserialize an afs_int32 */
+    xdrrx_putint32,	/* serialize an afs_int32 */
+    xdrrx_getbytes,	/* deserialize counted bytes */
+    xdrrx_putbytes,	/* serialize counted bytes */
+    NULL,		/* get offset in the stream: not supported. */
+    NULL,		/* set offset in the stream: not supported. */
+    xdrrx_inline,	/* prime stream for inline macros */
+    NULL		/* destroy stream */
+#else
+#ifdef AFS_XDR_64BITOPS
+    .x_getint64 = xdrrx_getint64,
+    .x_putint64 = xdrrx_putint64,
+#endif /* AFS_XDR_64BITOPS */
+    .x_getint32 = xdrrx_getint32,	/* deserialize an afs_int32 */
+    .x_putint32 = xdrrx_putint32,	/* serialize an afs_int32 */
+    .x_getbytes = xdrrx_getbytes,	/* deserialize counted bytes */
+    .x_putbytes = xdrrx_putbytes,	/* serialize counted bytes */
+    .x_getpostn = NULL,		/* get offset in the stream: not supported. */
+    .x_setpostn = NULL,		/* set offset in the stream: not supported. */
+    .x_inline = xdrrx_inline,		/* prime stream for inline macros */
+    .x_destroy = NULL,			/* destroy stream */
 #if defined(KERNEL) && defined(AFS_SUN57_ENV)
-    0,
-    xdrrx_getint32,		/* deserialize an afs_int32 */
-    xdrrx_putint32,		/* serialize an afs_int32 */
+    .x_control = NULL,
+#endif
 #endif
 };
 
 /*
  * Initialize an rx xdr handle, for a given rx call.  op must be XDR_ENCODE or XDR_DECODE.
- * Call must have been returned by rx_MakeCall or rx_GetCall.  Stream should be a pointer to a local rx_stream structure.
+ * Call must have been returned by rx_MakeCall or rx_GetCall.
  */
 void
-xdrrx_create(register XDR * xdrs, register struct rx_call *call,
-	     register enum xdr_op op)
+xdrrx_create(XDR * xdrs, struct rx_call *call,
+	     enum xdr_op op)
 {
     xdrs->x_op = op;
     xdrs->x_ops = &xdrrx_ops;
@@ -147,11 +131,12 @@ xdrrx_create(register XDR * xdrs, register struct rx_call *call,
 int rx_pin_failed = 0;
 #endif
 
-#if (defined(AFS_SGI61_ENV) && (_MIPS_SZLONG != _MIPS_SZINT)) || defined(AFS_HPUX_64BIT_ENV)
+#ifdef AFS_XDR_64BITOPS
 static bool_t
-xdrrx_getint64(XDR * xdrs, long *lp)
+xdrrx_getint64(XDR *axdrs, long *lp)
 {
-    register struct rx_call *call = ((struct rx_call *)(xdrs)->x_private);
+    XDR * xdrs = (XDR *)axdrs;
+    struct rx_call *call = ((struct rx_call *)(xdrs)->x_private);
     afs_int32 i;
 
     if (rx_Read32(call, &i) == sizeof(i)) {
@@ -162,21 +147,23 @@ xdrrx_getint64(XDR * xdrs, long *lp)
 }
 
 static bool_t
-xdrrx_putint64(XDR * xdrs, long *lp)
+xdrrx_putint64(XDR *axdrs, long *lp)
 {
+    XDR * xdrs = (XDR *)axdrs;
     afs_int32 code, i = htonl(*lp);
-    register struct rx_call *call = ((struct rx_call *)(xdrs)->x_private);
+    struct rx_call *call = ((struct rx_call *)(xdrs)->x_private);
 
     code = (rx_Write32(call, &i) == sizeof(i));
     return code;
 }
-#endif /* (defined(AFS_SGI61_ENV) && (_MIPS_SZLONG != _MIPS_SZINT)) || defined(AFS_HPUX_64BIT_ENV) */
+#endif /* AFS_XDR_64BITOPS */
 
 static bool_t
-xdrrx_getint32(XDR * xdrs, afs_int32 * lp)
+xdrrx_getint32(XDR *axdrs, afs_int32 * lp)
 {
     afs_int32 l;
-    register struct rx_call *call = ((struct rx_call *)(xdrs)->x_private);
+    XDR * xdrs = (XDR *)axdrs;
+    struct rx_call *call = ((struct rx_call *)(xdrs)->x_private);
 #if	defined(KERNEL) && defined(AFS_AIX32_ENV)
     char *saddr = (char *)&l;
     saddr -= STACK_TO_PIN;
@@ -211,10 +198,11 @@ xdrrx_getint32(XDR * xdrs, afs_int32 * lp)
 }
 
 static bool_t
-xdrrx_putint32(register XDR * xdrs, register afs_int32 * lp)
+xdrrx_putint32(XDR *axdrs, afs_int32 * lp)
 {
     afs_int32 code, l = htonl(*lp);
-    register struct rx_call *call = ((struct rx_call *)(xdrs)->x_private);
+    XDR * xdrs = (XDR *)axdrs;
+    struct rx_call *call = ((struct rx_call *)(xdrs)->x_private);
 #if	defined(KERNEL) && defined(AFS_AIX32_ENV)
     char *saddr = (char *)&code;
     saddr -= STACK_TO_PIN;
@@ -241,10 +229,11 @@ xdrrx_putint32(register XDR * xdrs, register afs_int32 * lp)
 }
 
 static bool_t
-xdrrx_getbytes(register XDR * xdrs, register caddr_t addr, register u_int len)
+xdrrx_getbytes(XDR *axdrs, caddr_t addr, u_int len)
 {
     afs_int32 code;
-    register struct rx_call *call = ((struct rx_call *)(xdrs)->x_private);
+    XDR * xdrs = (XDR *)axdrs;
+    struct rx_call *call = ((struct rx_call *)(xdrs)->x_private);
 #if	defined(KERNEL) && defined(AFS_AIX32_ENV)
     char *saddr = (char *)&code;
     saddr -= STACK_TO_PIN;
@@ -272,10 +261,11 @@ xdrrx_getbytes(register XDR * xdrs, register caddr_t addr, register u_int len)
 }
 
 static bool_t
-xdrrx_putbytes(register XDR * xdrs, register caddr_t addr, register u_int len)
+xdrrx_putbytes(XDR *axdrs, caddr_t addr, u_int len)
 {
     afs_int32 code;
-    register struct rx_call *call = ((struct rx_call *)(xdrs)->x_private);
+    XDR * xdrs = (XDR *)axdrs;
+    struct rx_call *call = ((struct rx_call *)(xdrs)->x_private);
 #if	defined(KERNEL) && defined(AFS_AIX32_ENV)
     char *saddr = (char *)&code;
     saddr -= STACK_TO_PIN;
@@ -304,22 +294,22 @@ xdrrx_putbytes(register XDR * xdrs, register caddr_t addr, register u_int len)
 
 #ifdef undef			/* not used */
 static u_int
-xdrrx_getpos(register XDR * xdrs)
+xdrrx_getpos(XDR * xdrs)
 {
     /* Not supported.  What error code should we return? (It doesn't matter:  it will never be called, anyway!) */
     return -1;
 }
 
 static bool_t
-xdrrx_setpos(register XDR * xdrs, u_int pos)
+xdrrx_setpos(XDR * xdrs, u_int pos)
 {
     /* Not supported */
     return FALSE;
 }
 #endif
 
-static AFS_RPC_INLINE_T *
-xdrrx_inline(register XDR * xdrs, register u_int len)
+static afs_int32 *
+xdrrx_inline(XDR *axdrs, u_int len)
 {
     /* I don't know what this routine is supposed to do, but the stdio module returns null, so we will, too */
     return (0);
