@@ -9,9 +9,8 @@
 
 #include <afsconfig.h>
 #include <afs/param.h>
+#include <afs/com_err.h>
 
-RCSID
-    ("$Header: /cvs/openafs/src/tsm41/aix_aklog.c,v 1.1.2.6 2008/01/03 18:03:46 shadow Exp $");
 
 #if defined(AFS_AIX51_ENV)
 #include <sys/types.h>
@@ -23,7 +22,9 @@ RCSID
 #include <netdb.h>
 #include <sys/socket.h>
 #include <sys/file.h>
+#ifdef HAVE_SYS_PAG_H
 #include <sys/pag.h>
+#endif
 #include <errno.h>
 #include <usersec.h>
 #include <syslog.h>
@@ -155,9 +156,18 @@ aklog_authenticate(char *userName, char *response, int *reenter, char **message)
     status = auth_to_cell(context, userName, NULL, NULL);
     
     if (status) {
+	char *str = afs_error_message(status);
 	*message = (char *)malloc(1024);
-	sprintf(*message, "Unable to obtain AFS tokens: %s.\n",
-		afs_error_message(status));
+#ifdef HAVE_KRB5_SVC_GET_MSG
+	if (strncmp(str, "unknown", strlen("unknown")) == 0) {
+	    krb5_svc_get_msg(status,&str);
+	    sprintf(*message, "Unable to obtain AFS tokens: %s.\n",
+                    str);
+	    krb5_free_string(str);
+	} else
+#endif
+	    sprintf(*message, "Unable to obtain AFS tokens: %s.\n",
+		    str);
 	return AUTH_FAILURE; /* NOTFOUND? */
     }
     
@@ -205,7 +215,7 @@ static krb5_error_code get_credv5(krb5_context context, char *user,
     static krb5_principal client_principal = 0;
     char *str;
     
-    memset((char *)&increds, 0, sizeof(increds));
+    memset(&increds, 0, sizeof(increds));
     /* instance may be ptr to a null string. Pass null then */
     if ((r = krb5_build_principal(context, &increds.server,
 				  strlen(realm), realm,
@@ -286,7 +296,7 @@ static int get_cellconfig(char *cell, struct afsconf_cell *cellconfig, char *loc
     struct afsconf_dir *configdir;
     
     memset(local_cell, 0, sizeof(local_cell));
-    memset((char *)cellconfig, 0, sizeof(*cellconfig));
+    memset(cellconfig, 0, sizeof(*cellconfig));
     
     if (!(configdir = afsconf_Open(AFSDIR_CLIENT_ETC_DIRPATH))) {
 	return AFSCONF_NODB;
