@@ -10,8 +10,6 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
-RCSID
-    ("$Header: /cvs/openafs/src/usd/usd_file.c,v 1.14 2004/03/03 22:41:05 rees Exp $");
 
 #include <errno.h>
 #include <fcntl.h>
@@ -25,21 +23,21 @@ RCSID
 #ifdef AFS_DARWIN_ENV
 #include <sys/ioccom.h>
 #endif
-#if defined(AFS_DUX40_ENV) || defined(AFS_OBSD_ENV) || defined(AFS_NBSD_ENV)
+#if defined(AFS_DUX40_ENV) || defined(AFS_OBSD_ENV) || defined(AFS_NBSD_ENV) || (defined(AFS_DARWIN_ENV) && !defined(AFS_DARWIN100_ENV))
 #include <sys/ioctl.h>
 #endif
+#ifndef AFS_DARWIN100_ENV
 #include <sys/mtio.h>
+#endif
 #endif /* AFS_AIX_ENV */
 
-#ifdef HAVE_STRING_H
 #include <string.h>
-#else
-#ifdef HAVE_STRINGS_H
-#include <strings.h>
-#endif
-#endif
 #include <stdlib.h>
 
+#ifdef HAVE_STDINT_H
+# include <stdint.h>
+#endif
+
 #include <afs/debug.h>
 #include "usd.h"
 
@@ -83,7 +81,7 @@ static int
 usd_FileRead(usd_handle_t usd, char *buf, afs_uint32 nbytes,
 	     afs_uint32 * xferdP)
 {
-    int fd = (int)(usd->handle);
+    int fd = (intptr_t)(usd->handle);
     int got;
 
     got = read(fd, buf, nbytes);
@@ -101,7 +99,7 @@ static int
 usd_FileWrite(usd_handle_t usd, char *buf, afs_uint32 nbytes,
 	      afs_uint32 * xferdP)
 {
-    int fd = (int)(usd->handle);
+    int fd = (intptr_t)(usd->handle);
     int sent;
 
     sent = write(fd, buf, nbytes);
@@ -121,7 +119,7 @@ static int
 usd_FileSeek(usd_handle_t usd, afs_hyper_t reqOff, int whence,
 	     afs_hyper_t * curOffP)
 {
-    int fd = (int)(usd->handle);
+    int fd = (intptr_t)(usd->handle);
     osi_lloff_t lloff;
 
     if (!osi_hFitsInOff(reqOff, lloff))
@@ -140,7 +138,7 @@ usd_FileSeek(usd_handle_t usd, afs_hyper_t reqOff, int whence,
 static int
 usd_FileIoctl(usd_handle_t usd, int req, void *arg)
 {
-    int fd = (int)(usd->handle);
+    int fd = (intptr_t)(usd->handle);
 #ifdef O_LARGEFILE
     struct stat64 info;
 #else /* O_LARGEFILE */
@@ -217,6 +215,9 @@ usd_FileIoctl(usd_handle_t usd, int req, void *arg)
 
     case USD_IOCTL_TAPEOPERATION:
 	{
+#ifdef AFS_DARWIN100_ENV
+	    code = EOPNOTSUPP;
+#else
 	    usd_tapeop_t *tapeOpp = (usd_tapeop_t *) arg;
 #if defined(AFS_AIX_ENV)
 	    struct stop os_tapeop;
@@ -263,7 +264,7 @@ usd_FileIoctl(usd_handle_t usd, int req, void *arg)
 
 	    code = ioctl(fd, MTIOCTOP, &os_tapeop);
 #endif /* AFS_AIX_ENV */
-
+#endif
 	    if (code == -1) {
 		code = errno;
 	    } else {
@@ -293,7 +294,7 @@ usd_FileIoctl(usd_handle_t usd, int req, void *arg)
 static int
 usd_FileClose(usd_handle_t usd)
 {
-    int fd = (int)(usd->handle);
+    int fd = (intptr_t)(usd->handle);
     int code = 0;
     int ccode;
 
@@ -354,7 +355,7 @@ usd_FileOpen(const char *path, int flags, int mode, usd_handle_t * usdP)
 
     usd = (usd_handle_t) malloc(sizeof(*usd));
     memset(usd, 0, sizeof(*usd));
-    usd->handle = (void *)fd;
+    usd->handle = (void *)(intptr_t)fd;
     usd->read = usd_FileRead;
     usd->write = usd_FileWrite;
     usd->seek = usd_FileSeek;
