@@ -7,8 +7,6 @@
  * directory or online at http://www.openafs.org/dl/license10.html
  */
 
-#ifndef lint
-#endif
 /*
 	System:		VICE-TWO
 	Module:		common.c
@@ -18,35 +16,50 @@
 
 #include <sys/types.h>
 #include <stdio.h>
+#include <stdlib.h>
+#include <afsconfig.h>
 #include <afs/param.h>
+
+
 #include <afs/afsutil.h>
 
 int Statistics = 0;
 
-/* VARARGS */
-Log (a,b,c,d,e,f,g,h,i,j,k)
-    char *a, *b, *c, *d, *e, *f, *g, *h, *i, *j, *k;
+/*@printflike@*/ void
+Log(const char *format, ...)
 {
     int level;
+    va_list args;
 
     if (Statistics)
 	level = -1;
     else
 	level = 0;
-    ViceLog(level,(a,b,c,d,e,f,g,h,i,j,k));
+
+    va_start(args, format);
+    vViceLog(level, (format, args));
+    va_end(args);
 }
 
-Abort(s,a,b,c,d,e,f,g,h,i,j) 
-    char *s, *a, *b, *c, *d, *e, *f, *g, *h, *i, *j;
+/*@printflike@*/ void
+Abort(const char *format, ...)
 {
+    va_list args;
+
     ViceLog(0, ("Program aborted: "));
-    ViceLog(0, (s,a,b,c,d,e,f,g,h,i,j));
+    va_start(args, format);
+    vViceLog(0, (format, args));
+    va_end(args);
     abort();
 }
 
-Quit(s,a,b,c,d,e,f,g,h,i,j) 
-    char *s, *a, *b, *c, *d, *e, *f, *g, *h, *i, *j;
+/*@printflike@*/ void
+Quit(const char *format, ...)
 {
-    ViceLog(0, (s,a,b,c,d,e,f,g,h,i,j));
+    va_list args;
+
+    va_start(args, format);
+    vViceLog(0, (format, args));
+    va_end(args);
     exit(1);
 }
