@@ -10,16 +10,16 @@
 #include <sys/param.h>
 #include <afsconfig.h>
 
-RCSID("$Header: /cvs/openafs/src/vol/devname.c,v 1.7.2.2 2001/10/13 04:22:11 shadow Exp $");
 
 #include <afs/param.h>
 #include <rx/xdr.h>
 #include <afs/afsint.h>
+#include <string.h>
 #include <ctype.h>
 #if !defined(AFS_SGI_ENV)
 #ifdef	AFS_OSF_ENV
 #include <ufs/fs.h>
-#else	/* AFS_OSF_ENV */
+#else /* AFS_OSF_ENV */
 #ifdef AFS_VFSINCL_ENV
 #define VFS
 #ifdef	AFS_SUN5_ENV
@@ -37,7 +37,7 @@ RCSID("$Header: /cvs/openafs/src/vol/devname.c,v 1.7.2.2 2001/10/13 04:22:11 sha
 #include <sys/fs.h>
 #endif
 #endif /* AFS_VFSINCL_ENV */
-#endif	/* AFS_OSF_ENV */
+#endif /* AFS_OSF_ENV */
 #endif /* AFS_SGI_ENV */
 #include <sys/errno.h>
 #include <sys/types.h>
@@ -63,10 +63,6 @@ RCSID("$Header: /cvs/openafs/src/vol/devname.c,v 1.7.2.2 2001/10/13 04:22:11 sha
 #if defined(AFS_SGI_ENV)
 #include <fcntl.h>
 #include <mntent.h>
-#ifdef AFS_SGI_EFS_IOPS_ENV
-#define ROOTINO EFS_ROOTINO
-#include "../sgiefs/efs.h"
-#endif
 #else
 #ifdef AFS_LINUX22_ENV
 #include <mntent.h>
@@ -84,22 +80,17 @@ RCSID("$Header: /cvs/openafs/src/vol/devname.c,v 1.7.2.2 2001/10/13 04:22:11 sha
 #ifndef ITIMER_REAL
 #include <sys/time.h>
 #endif /* ITIMER_REAL */
+#include "ihandle.h"
 #include "partition.h"
-#ifdef AFS_LINUX22_ENV
-#include <asm/types.h>
-#include <linux/ext2_fs.h>
-#define ROOTINO EXT2_ROOT_INO /* Assuming we do this on ext2, of course. */
-#endif
 
 /* ensure that we don't have a "/" instead of a "/dev/rxd0a" type of device.
  * returns pointer to static storage; copy it out quickly!
  */
-char *vol_DevName(adev, wpath)
-char *wpath;
-dev_t adev; {
+char *
+vol_DevName(dev_t adev, char *wpath)
+{
     static char pbuffer[128];
     char pbuf[128], *ptr;
-    int code, i;
 #ifdef	AFS_SUN5_ENV
     struct mnttab mnt;
     FILE *mntfile;
@@ -117,15 +108,17 @@ dev_t adev; {
 #endif
 
 #ifdef	AFS_AIX_ENV
-    if ((nmounts = getmount(&vmountp)) <= 0)	{   
-	return (char *)0;
+    if ((nmounts = getmount(&vmountp)) <= 0) {
+	return NULL;
     }
-    for (; nmounts; nmounts--, vmountp = (struct vmount *)((int)vmountp + vmountp->vmt_length)) {
+    for (; nmounts;
+	 nmounts--, vmountp =
+	 (struct vmount *)((int)vmountp + vmountp->vmt_length)) {
 	char *part = vmt2dataptr(vmountp, VMT_STUB);
 #else
 #ifdef	AFS_SUN5_ENV
     if (!(mntfile = fopen(MNTTAB, "r"))) {
-	return (char *)0;
+	return NULL;
     }
     while (!getmntent(mntfile, &mnt)) {
 	char *part = mnt.mnt_mountp;
@@ -134,41 +127,44 @@ dev_t adev; {
 #ifdef AFS_LINUX22_ENV
     if ((mfd = setmntent("/proc/mounts", "r")) == NULL) {
 	if ((mfd = setmntent("/etc/mtab", "r")) == NULL) {
-	    return (char *)0;
+	    return NULL;
 	}
     }
 #else
-    if ((mfd = setmntent(MOUNTED/*MNTTAB*/, "r")) == NULL) {
-	return (char *)0;
+    if ((mfd = setmntent(MOUNTED /*MNTTAB*/, "r")) == NULL) {
+	return NULL;
     }
 #endif
-    while (mntent = getmntent(mfd)) {
+    while ((mntent = getmntent(mfd))) {
 	char *part = mntent->mnt_dir;
 #else
     setfsent();
-    while (fsent = getfsent()) {
+    while ((fsent = getfsent())) {
 	char *part = fsent->fs_file;
 #endif
 #endif /* AFS_SGI_ENV */
 #endif
 	struct stat status;
 #ifdef	AFS_AIX_ENV
-	if (vmountp->vmt_flags & (MNT_READONLY|MNT_REMOVABLE|MNT_REMOTE)) continue; /* Ignore any "special" partitions */
+	if (vmountp->vmt_flags & (MNT_READONLY | MNT_REMOVABLE | MNT_REMOTE))
+	    continue;		/* Ignore any "special" partitions */
 #else
 #ifdef	AFS_SUN5_ENV
 	/* Ignore non ufs or non read/write partitions */
-	if ((strcmp(mnt.mnt_fstype, "ufs") !=0) ||
-	    (strncmp(mnt.mnt_mntopts, "ro,ignore",9) ==0)) 
-	    continue; 
+	if ((strcmp(mnt.mnt_fstype, "ufs") != 0)
+	    || (strncmp(mnt.mnt_mntopts, "ro,ignore", 9) == 0))
+	    continue;
 #else
 #if defined(AFS_LINUX22_ENV)
 	if (strcmp(mntent->mnt_type, "ext2"))
 	    continue;
 #else
 #if defined(AFS_SGI_ENV) || defined(AFS_SUN_ENV) || defined(AFS_HPUX_ENV)
-	if (!hasmntopt(mntent, MNTOPT_RW)) continue;
+	if (!hasmntopt(mntent, MNTOPT_RW))
+	    continue;
 #else
-	if (strcmp(fsent->fs_type, "rw") != 0) continue; /* Ignore non read/write partitions */
+	if (strcmp(fsent->fs_type, "rw") != 0)
+	    continue;		/* Ignore non read/write partitions */
 #endif /* AFS_LINUX22_ENV */
 #endif /* AFS_SGI_ENV */
 #endif
@@ -176,12 +172,13 @@ dev_t adev; {
 	/* Only keep track of "/vicepx" partitions since it can get hairy when NFS mounts are involved.. */
 	if (strncmp(part, VICE_PARTITION_PREFIX, VICE_PREFIX_SIZE)) {
 	    continue;		/* Non /vicepx; ignore */
-        }
+	}
 	if (stat(part, &status) == -1) {
 	    continue;
 	}
-#ifndef AFS_SGI_XFS_IOPS_ENV
-	if ((status.st_ino != ROOTINO) /*|| ((status.st_mode & S_IFMT) != S_IFBLK)*/) {
+#if !defined(AFS_SGI_XFS_IOPS_ENV) && !defined(AFS_LINUX22_ENV)
+	if ((status.st_ino !=
+	     ROOTINO) /*|| ((status.st_mode & S_IFMT) != S_IFBLK) */ ) {
 	    continue;
 	}
 #endif
@@ -197,7 +194,7 @@ dev_t adev; {
 #else
 	    strcpy(pbuffer, fsent->fs_spec);
 #endif
-#endif	/* AFS_SGI_ENV */
+#endif /* AFS_SGI_ENV */
 #endif
 	    if (wpath) {
 		strcpy(pbuf, pbuffer);
@@ -206,59 +203,56 @@ dev_t adev; {
 		    *ptr = '\0';
 		    strcpy(wpath, pbuf);
 		} else
-		    return (char *)0;
+		    return NULL;
 	    }
-	    ptr = (char *)strrchr(pbuffer, '/');	    
+	    ptr = (char *)strrchr(pbuffer, '/');
 	    if (ptr) {
-		strcpy(pbuffer, ptr+1);
+		strcpy(pbuffer, ptr + 1);
 		return pbuffer;
 	    } else
-		return (char *)0;
+		return NULL;
 	}
     }
-#ifndef	AFS_AIX_ENV
 #ifdef	AFS_SUN5_ENV
-   (void) fclose(mntfile);
+    (void)fclose(mntfile);
 #else
 #if defined(AFS_SGI_ENV) || defined(AFS_SUN_ENV) || defined(AFS_HPUX_ENV) || defined(AFS_LINUX22_ENV)
     endmntent(mfd);
 #else
+#ifndef	AFS_AIX_ENV
     endfsent();
 #endif
-#endif /* AFS_SGI_ENV */
 #endif
-    return (char *)0;
+#endif /* AFS_SGI_ENV */
+    return NULL;
 }
 
-
 /* Search for the raw device name. Will put an "r" in front of each
  * directory and file entry of the pathname until we find a character
  * device.
  */
-char *afs_rawname(devfile)
-  char *devfile;
+char *
+afs_rawname(char *devfile)
 {
-  static char rawname[100];
-  struct stat statbuf;
-  char *p;
-  int code, i;
+    static char rawname[100];
+    struct stat statbuf;
+    int code, i;
 
-  i = strlen(devfile);
-  while (i >= 0) {
-     strcpy(rawname, devfile);
-     if (devfile[i] == '/') {
-        rawname[i+1] = 'r';
-        rawname[i+2] = 0;
-        strcat(rawname, &devfile[i+1]);
-     }
+    i = strlen(devfile);
+    while (i >= 0) {
+	strcpy(rawname, devfile);
+	if (devfile[i] == '/') {
+	    rawname[i + 1] = 'r';
+	    rawname[i + 2] = 0;
+	    strcat(rawname, &devfile[i + 1]);
+	}
 
-     code = stat(rawname, &statbuf);
-     if (!code && S_ISCHR(statbuf.st_mode))
-        return rawname;
-  
-     while((--i>=0) && (devfile[i] != '/'));
-  }
+	code = stat(rawname, &statbuf);
+	if (!code && S_ISCHR(statbuf.st_mode))
+	    return rawname;
 
-  return (char *)0;
+	while ((--i >= 0) && (devfile[i] != '/'));
+    }
+
+    return NULL;
 }
-
