@@ -40,6 +40,7 @@
 #include "voldefs.h"
 #include "partition.h"
 #include "fssync.h"
+#include "volume_inline.h"
 #include <afs/errors.h>
 
 /*@+fcnmacros +macrofcndecl@*/
@@ -1218,60 +1219,44 @@ VerifyDirPerms(char *path)
  * If the resultFile is NULL, then don't call the write routine.
  */
 int
-ListViceInodes(char *devname, char *mountedOn, char *resultFile,
+ListViceInodes(char *devname, char *mountedOn, FILE *inodeFile,
 	       int (*judgeInode) (struct ViceInodeInfo * info, afs_uint32 vid, void *rock),
 	       afs_uint32 singleVolumeNumber, int *forcep, int forceR, char *wpath, 
 	       void *rock)
 {
-    FILE *fp = (FILE *) - 1;
     int ninodes;
     struct afs_stat status;
 
     *forcep = 0; /* no need to salvage until further notice */
 
-    if (resultFile) {
-	fp = afs_fopen(resultFile, "w");
-	if (!fp) {
-	    Log("Unable to create inode description file %s\n", resultFile);
-	    return -1;
-	}
-    }
-
     /* Verify protections on directories. */
     mode_errors = 0;
     VerifyDirPerms(mountedOn);
 
     ninodes =
-	namei_ListAFSFiles(mountedOn, WriteInodeInfo, fp, judgeInode,
+	namei_ListAFSFiles(mountedOn, WriteInodeInfo, inodeFile, judgeInode,
 			   singleVolumeNumber, rock);
 
-    if (!resultFile)
+    if (!inodeFile)
 	return ninodes;
 
     if (ninodes < 0) {
-	fclose(fp);
 	return ninodes;
     }
 
-    if (fflush(fp) == EOF) {
+    if (fflush(inodeFile) == EOF) {
 	Log("Unable to successfully flush inode file for %s\n", mountedOn);
-	fclose(fp);
 	return -2;
     }
-    if (fsync(fileno(fp)) == -1) {
+    if (fsync(fileno(inodeFile)) == -1) {
 	Log("Unable to successfully fsync inode file for %s\n", mountedOn);
-	fclose(fp);
-	return -2;
-    }
-    if (fclose(fp) == EOF) {
-	Log("Unable to successfully close inode file for %s\n", mountedOn);
 	return -2;
     }
 
     /*
      * Paranoia:  check that the file is really the right size
      */
-    if (afs_stat(resultFile, &status) == -1) {
+    if (afs_fstat(fileno(inodeFile), &status) == -1) {
 	Log("Unable to successfully stat inode file for %s\n", mountedOn);
 	return -2;
     }
@@ -1621,6 +1606,7 @@ convertVolumeInfo(int fdr, int fdw, afs_uint32 vid)
 int
 namei_ConvertROtoRWvolume(char *pname, afs_uint32 volumeId)
 {
+    int code = 0;
 #ifdef FSSYNC_BUILD_CLIENT
     namei_t n;
     char dir_name[512], oldpath[512], newpath[512];
@@ -1633,7 +1619,7 @@ namei_ConvertROtoRWvolume(char *pname, afs_uint32 volumeId)
     char smallSeen = 0;
     char largeSeen = 0;
     char linkSeen = 0;
-    int code, fd, fd2;
+    int fd, fd2;
     char *p;
     DIR *dirp;
     Inode ino;
@@ -1641,18 +1627,33 @@ namei_ConvertROtoRWvolume(char *pname, afs_uint32 volumeId)
     struct DiskPartition64 *partP;
     struct ViceInodeInfo info;
     struct VolumeDiskHeader h;
+# ifdef AFS_DEMAND_ATTACH_FS
+    int locktype = 0;
+# endif /* AFS_DEMAND_ATTACH_FS */
 
     for (partP = DiskPartitionList; partP && strcmp(partP->name, pname);
          partP = partP->next);
     if (!partP) {
         Log("1 namei_ConvertROtoRWvolume: Couldn't find DiskPartition for %s\n", pname);
-        return EIO;
+	code = EIO;
+	goto done;
     }
 
+# ifdef AFS_DEMAND_ATTACH_FS
+    locktype = VVolLockType(V_VOLUPD, 1);
+    code = VLockVolumeByIdNB(volumeId, partP, locktype);
+    if (code) {
+	locktype = 0;
+	code = EIO;
+	goto done;
+    }
+# endif /* AFS_DEMAND_ATTACH_FS */
+
     if (VReadVolumeDiskHeader(volumeId, partP, &h)) {
 	Log("1 namei_ConvertROtoRWvolume: Couldn't read header for RO-volume %lu.\n",
 	    afs_printable_uint32_lu(volumeId));
-	return EIO;
+	code = EIO;
+	goto done;
     }
 
     FSYNC_VolOp(volumeId, pname, FSYNC_VOL_BREAKCBKS, 0, NULL);
@@ -1667,7 +1668,8 @@ namei_ConvertROtoRWvolume(char *pname, afs_uint32 volumeId)
     dirp = opendir(dir_name);
     if (!dirp) {
 	Log("1 namei_ConvertROtoRWvolume: Could not opendir(%s)\n", dir_name);
-	return EIO;
+	code = EIO;
+	goto done;
     }
 
     while ((dp = readdir(dirp))) {
@@ -1679,12 +1681,14 @@ namei_ConvertROtoRWvolume(char *pname, afs_uint32 volumeId)
 	    Log("1 namei_ConvertROtoRWvolume: DecodeInode failed for %s/%s\n",
 		dir_name, dp->d_name);
 	    closedir(dirp);
-	    return -1;
+	    code = -1;
+	    goto done;
 	}
 	if (info.u.param[1] != -1) {
 	    Log("1 namei_ConvertROtoRWvolume: found other than volume special file %s/%s\n", dir_name, dp->d_name);
 	    closedir(dirp);
-	    return -1;
+	    code = -1;
+	    goto done;
 	}
 	if (info.u.param[0] != volumeId) {
 	    if (info.u.param[0] == ih->ih_vid) {
@@ -1695,7 +1699,8 @@ namei_ConvertROtoRWvolume(char *pname, afs_uint32 volumeId)
 	    }
 	    Log("1 namei_ConvertROtoRWvolume: found special file %s/%s for volume %lu\n", dir_name, dp->d_name, info.u.param[0]);
 	    closedir(dirp);
-	    return VVOLEXISTS;
+	    code = VVOLEXISTS;
+	    goto done;
 	}
 	if (info.u.param[2] == VI_VOLINFO) {	/* volume info file */
 	    strlcpy(infoName, dp->d_name, sizeof(infoName));
@@ -1709,14 +1714,16 @@ namei_ConvertROtoRWvolume(char *pname, afs_uint32 volumeId)
 	} else {
 	    closedir(dirp);
 	    Log("1 namei_ConvertROtoRWvolume: unknown type %d of special file found : %s/%s\n", info.u.param[2], dir_name, dp->d_name);
-	    return -1;
+	    code = -1;
+	    goto done;
 	}
     }
     closedir(dirp);
 
     if (!infoSeen || !smallSeen || !largeSeen || !linkSeen) {
 	Log("1 namei_ConvertROtoRWvolume: not all special files found in %s\n", dir_name);
-	return -1;
+	code = -1;
+	goto done;
     }
 
     /*
@@ -1733,7 +1740,8 @@ namei_ConvertROtoRWvolume(char *pname, afs_uint32 volumeId)
     if (fd < 0) {
 	Log("1 namei_ConvertROtoRWvolume: could not open RO info file: %s\n",
 	    oldpath);
-	return -1;
+	code = -1;
+	goto done;
     }
     t_ih.ih_ino = namei_MakeSpecIno(ih->ih_vid, VI_VOLINFO);
     namei_HandleToName(&n, &t_ih);
@@ -1741,14 +1749,16 @@ namei_ConvertROtoRWvolume(char *pname, afs_uint32 volumeId)
     if (fd2 < 0) {
 	Log("1 namei_ConvertROtoRWvolume: could not create RW info file: %s\n", n.n_path);
 	close(fd);
-	return -1;
+	code = -1;
+	goto done;
     }
     code = convertVolumeInfo(fd, fd2, ih->ih_vid);
     close(fd);
     if (code) {
 	close(fd2);
 	unlink(n.n_path);
-	return -1;
+	code = -1;
+	goto done;
     }
     SetOGM(fd2, ih->ih_vid, 1);
     close(fd2);
@@ -1759,7 +1769,8 @@ namei_ConvertROtoRWvolume(char *pname, afs_uint32 volumeId)
     fd = afs_open(newpath, O_RDWR, 0);
     if (fd < 0) {
 	Log("1 namei_ConvertROtoRWvolume: could not open SmallIndex file: %s\n", newpath);
-	return -1;
+	code = -1;
+	goto done;
     }
     SetOGM(fd, ih->ih_vid, 2);
     close(fd);
@@ -1772,7 +1783,8 @@ namei_ConvertROtoRWvolume(char *pname, afs_uint32 volumeId)
     fd = afs_open(newpath, O_RDWR, 0);
     if (fd < 0) {
 	Log("1 namei_ConvertROtoRWvolume: could not open LargeIndex file: %s\n", newpath);
-	return -1;
+	code = -1;
+	goto done;
     }
     SetOGM(fd, ih->ih_vid, 3);
     close(fd);
@@ -1790,7 +1802,8 @@ namei_ConvertROtoRWvolume(char *pname, afs_uint32 volumeId)
     if (VCreateVolumeDiskHeader(&h, partP)) {
         Log("1 namei_ConvertROtoRWvolume: Couldn't write header for RW-volume %lu\n",
 	    afs_printable_uint32_lu(h.id));
-        return EIO;
+	code = EIO;
+	goto done;
     }
 
     if (VDestroyVolumeDiskHeader(partP, volumeId, h.parent)) {
@@ -1800,8 +1813,16 @@ namei_ConvertROtoRWvolume(char *pname, afs_uint32 volumeId)
 
     FSYNC_VolOp(volumeId, pname, FSYNC_VOL_DONE, 0, NULL);
     FSYNC_VolOp(h.id, pname, FSYNC_VOL_ON, 0, NULL);
+
+ done:
+# ifdef AFS_DEMAND_ATTACH_FS
+    if (locktype) {
+	VUnlockVolumeById(volumeId, partP);
+    }
+# endif /* AFS_DEMAND_ATTACH_FS */
 #endif
-    return 0;
+
+    return code;
 }
 
 /* PrintInode
