@@ -156,7 +156,7 @@ extern char *fileSysPath;	/* The path of the mounted partition currently
 extern char *fileSysPathName;	/* NT needs this to make name pretty in log. */
 extern IHandle_t *VGLinkH;	/* Link handle for current volume group. */
 extern int VGLinkH_cnt;	        /* # of references to lnk handle. */
-extern struct DiskPartition *fileSysPartition;	/* Partition  being salvaged */
+extern struct DiskPartition64 *fileSysPartition;	/* Partition  being salvaged */
 #ifndef AFS_NT40_ENV
 extern char *fileSysDeviceName;	/* The block device where the file system
 				 * being salvaged was mounted */
@@ -220,8 +220,8 @@ extern int canfork;
 extern void Exit(int code);
 extern int Fork(void);
 extern int Wait(char *prog);
-extern char *ToString(char *s);
-extern void AskOffline(VolumeId volumeId);
+extern char *ToString(const char *s);
+extern void AskOffline(VolumeId volumeId, char * partition);
 extern void AskOnline(VolumeId volumeId, char *partition);
 extern void CheckLogFile(char * log_path);
 #ifndef AFS_NT40_ENV
@@ -236,13 +236,14 @@ extern void CountVolumeInodes(register struct ViceInodeInfo *ip, int maxInodes,
 extern void DeleteExtraVolumeHeaderFile(register struct VolumeSummary *vsp);
 extern void DistilVnodeEssence(VolumeId vid, VnodeClass class, Inode ino,
 			       Unique * maxu);
-extern int GetInodeSummary(char *path, VolumeId singleVolumeNumber);
-extern void GetVolumeSummary(VolumeId singleVolumeNumber);
-extern void JudgeEntry(struct DirSummary *dir, char *name, VnodeId vnodeNumber,
-		       Unique unique);
+extern int GetInodeSummary(FILE *inodeFile, VolumeId singleVolumeNumber);
+extern int GetVolumeSummary(VolumeId singleVolumeNumber);
+extern int JudgeEntry(void *dirVal, char *name, afs_int32 vnodeNumber,
+		      afs_int32 unique);
 extern void MaybeZapVolume(register struct InodeSummary *isp, char *message,
 			   int deleteMe, int check);
 extern void ObtainSalvageLock(void);
+extern void ObtainSharedSalvageLock(void);
 extern void PrintInodeList(void);
 extern void PrintInodeSummary(void);
 extern void PrintVolumeSummary(void);
@@ -251,9 +252,9 @@ extern void RemoveTheForce(char *path);
 extern void SalvageDir(char *name, VolumeId rwVid, struct VnodeInfo *dirVnodeInfo,
 		       IHandle_t * alinkH, int i, struct DirSummary *rootdir,
 		       int *rootdirfound);
-extern void SalvageFileSysParallel(struct DiskPartition *partP);
-extern void SalvageFileSys(struct DiskPartition *partP, VolumeId singleVolumeNumber);
-extern void SalvageFileSys1(struct DiskPartition *partP,
+extern void SalvageFileSysParallel(struct DiskPartition64 *partP);
+extern void SalvageFileSys(struct DiskPartition64 *partP, VolumeId singleVolumeNumber);
+extern void SalvageFileSys1(struct DiskPartition64 *partP,
 			    VolumeId singleVolumeNumber);
 extern int SalvageHeader(register struct stuff *sp, struct InodeSummary *isp,
 			 int check, int *deleteMe);
