@@ -10,11 +10,12 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
-RCSID
-    ("$Header: /cvs/openafs/src/volser/volmain.c,v 1.22.2.13 2008/09/24 21:36:52 shadow Exp $");
 
 #include <sys/types.h>
 #include <string.h>
+#ifdef HAVE_STDINT_H
+# include <stdint.h>
+#endif
 #ifdef AFS_NT40_ENV
 #include <time.h>
 #include <fcntl.h>
@@ -24,6 +25,7 @@ RCSID
 #include <sys/time.h>
 #include <sys/file.h>
 #include <netinet/in.h>
+#include <unistd.h>
 #endif
 #include <rx/xdr.h>
 #include <afs/afsint.h>
@@ -47,18 +49,23 @@ RCSID
 #include <afs/volume.h>
 #include <afs/partition.h>
 #include <rx/rx.h>
+#include <rx/rxstat.h>
 #include <rx/rx_globals.h>
 #include <afs/auth.h>
 #include <afs/cellconfig.h>
 #include <afs/keys.h>
+#include <afs/dir.h>
 #include <ubik.h>
 #include <fcntl.h>
 #include <sys/stat.h>
 
-#include "volser.h"
 #include <errno.h>
 #include <afs/audit.h>
 #include <afs/afsutil.h>
+#include <lwp.h>
+#include "volser.h"
+#include "volint.h"
+#include "volser_internal.h"
 
 /*@printflike@*/ extern void Log(const char *format, ...);
 /*@printflike@*/ extern void Abort(const char *format, ...);
@@ -67,11 +74,6 @@ RCSID
 #define N_SECURITY_OBJECTS 3
 
 extern struct Lock localLock;
-extern struct volser_trans *TransList();
-#ifndef AFS_PTHREAD_ENV
-extern int (*vol_PollProc) ();
-extern int IOMGR_Poll();
-#endif
 char *GlobalNameHack = NULL;
 int hackIsIn = 0;
 afs_int32 GlobalVolCloneId, GlobalVolParentId;
@@ -79,8 +81,6 @@ int GlobalVolType;
 int VolumeChanged;		/* XXXX */
 static char busyFlags[MAXHELPERS];
 struct volser_trans *QI_GlobalWriteTrans = 0;
-extern void AFSVolExecuteRequest();
-extern void RXSTATS_ExecuteRequest();
 struct afsconf_dir *tdir;
 static afs_int32 runningCalls = 0;
 int DoLogging = 0;
@@ -101,36 +101,36 @@ afs_uint32 SHostAddrs[ADDRSPERSITE];
 		       }
 
 #if defined(AFS_PTHREAD_ENV)
-char *
+int
 threadNum(void)
 {
-    return pthread_getspecific(rx_thread_id_key);
+    return (intptr_t)pthread_getspecific(rx_thread_id_key);
 }
 #endif
 
-static afs_int32
+static void
 MyBeforeProc(struct rx_call *acall)
 {
     VTRANS_LOCK;
     runningCalls++;
     VTRANS_UNLOCK;
-    return 0;
+    return;
 }
 
-static afs_int32
+static void
 MyAfterProc(struct rx_call *acall, afs_int32 code)
 {
     VTRANS_LOCK;
     runningCalls--;
     VTRANS_UNLOCK;
-    return 0;
+    return;
 }
 
 /* Called every GCWAKEUP seconds to try to unlock all our partitions,
  * if we're idle and there are no active transactions 
  */
 static void
-TryUnlock()
+TryUnlock(void)
 {
     /* if there are no running calls, and there are no active transactions, then
      * it should be safe to release any partition locks we've accumulated */
@@ -171,6 +171,7 @@ BKGLoop(void *unused)
 
 /* Background daemon for sleeping so the volserver does not become I/O bound */
 afs_int32 TTsleep, TTrun;
+#ifndef AFS_PTHREAD_ENV
 static void *
 BKGSleep(void *unused)
 {
@@ -185,13 +186,17 @@ BKGSleep(void *unused)
 #endif
 	    VTRANS_LOCK;
 	    for (tt = TransList(); tt; tt = tt->next) {
+                VTRANS_OBJ_LOCK(tt);
 		if ((strcmp(tt->lastProcName, "DeleteVolume") == 0)
 		    || (strcmp(tt->lastProcName, "Clone") == 0)
 		    || (strcmp(tt->lastProcName, "ReClone") == 0)
 		    || (strcmp(tt->lastProcName, "Forward") == 0)
 		    || (strcmp(tt->lastProcName, "Restore") == 0)
-		    || (strcmp(tt->lastProcName, "ForwardMulti") == 0))
+		    || (strcmp(tt->lastProcName, "ForwardMulti") == 0)) {
+                    VTRANS_OBJ_UNLOCK(tt);
 		    break;
+                }
+                VTRANS_OBJ_UNLOCK(tt);
 	    }
 	    if (tt) {
 	        VTRANS_UNLOCK;
@@ -202,15 +207,16 @@ BKGSleep(void *unused)
     }
     return NULL;
 }
+#endif
 
 #ifndef AFS_NT40_ENV
 int
 volser_syscall(afs_uint32 a3, afs_uint32 a4, void *a5)
 {
     afs_uint32 rcode;
-    void (*old) ();
-
 #ifndef AFS_LINUX20_ENV
+    void (*old) (int);
+
     old = signal(SIGSYS, SIG_IGN);
 #endif
     rcode =
@@ -237,7 +243,8 @@ int
 main(int argc, char **argv)
 {
     register afs_int32 code;
-    struct rx_securityClass *(securityObjects[3]);
+    struct rx_securityClass **securityClasses;
+    afs_int32 numClasses;
     struct rx_service *service;
     struct ktc_encryptionKey tkey;
     int rxpackets = 100;
@@ -247,6 +254,8 @@ main(int argc, char **argv)
     int rxMaxMTU = -1;
     int bufSize = 0;		/* temp variable to read in udp socket buf size */
     afs_uint32 host = ntohl(INADDR_ANY);
+    char *auditFileName = NULL;
+    VolumePackageOptions opts;
 
 #ifdef	AFS_AIX32_ENV
     /*
@@ -310,35 +319,15 @@ main(int argc, char **argv)
 		lwps = MAXLWP;
 	    }
 	} else if (strcmp(argv[code], "-auditlog") == 0) {
-	    int tempfd, flags;
-	    FILE *auditout;
-	    char oldName[MAXPATHLEN];
-	    char *fileName = argv[++code];
+	    auditFileName = argv[++code];
 
-#ifndef AFS_NT40_ENV
-	    struct stat statbuf;
-	    
-	    if ((lstat(fileName, &statbuf) == 0) 
-		&& (S_ISFIFO(statbuf.st_mode))) {
-		flags = O_WRONLY | O_NONBLOCK;
-	    } else 
-#endif
-	    {
-		strcpy(oldName, fileName);
-		strcat(oldName, ".old");
-		renamefile(fileName, oldName);
-		flags = O_WRONLY | O_TRUNC | O_CREAT;
+	} else if (strcmp(argv[code], "-audit-interface") == 0) {
+	    char *interface = argv[++code];
+
+	    if (osi_audit_interface(interface)) {
+		printf("Invalid audit interface '%s'\n", interface);
+		return -1;
 	    }
-	    tempfd = open(fileName, flags, 0666);
-	    if (tempfd > -1) {
-		auditout = fdopen(tempfd, "a");
-		if (auditout) {
-		    osi_audit_file(auditout);
-		    osi_audit(VS_StartEvent, 0, AUD_END);
-		} else
-		    printf("Warning: auditlog %s not writable, ignored.\n", fileName);
-	    } else
-		printf("Warning: auditlog %s not writable, ignored.\n", fileName);
 	} else if (strcmp(argv[code], "-nojumbo") == 0) {
 	    rxJumbograms = 0;
 	} else if (strcmp(argv[code], "-jumbo") == 0) {
@@ -351,7 +340,7 @@ main(int argc, char **argv)
 	    rxMaxMTU = atoi(argv[++code]);
 	    if ((rxMaxMTU < RX_MIN_PACKET_SIZE) || 
 		(rxMaxMTU > RX_MAX_PACKET_DATA_SIZE)) {
-		printf("rxMaxMTU %d% invalid; must be between %d-%d\n",
+		printf("rxMaxMTU %d invalid; must be between %d-%" AFS_SIZET_FMT "\n",
 		       rxMaxMTU, RX_MIN_PACKET_SIZE, 
 		       RX_MAX_PACKET_DATA_SIZE);
 		exit(1);
@@ -411,6 +400,11 @@ main(int argc, char **argv)
 	    VS_EXIT(1);
 	}
     }
+
+    if (auditFileName) {
+	osi_audit_file(auditFileName);
+	osi_audit(VS_StartEvent, 0, AUD_END);
+    }
 #ifdef AFS_SGI_VNODE_GLUE
     if (afs_init_kernel_config(-1) < 0) {
 	printf
@@ -431,11 +425,15 @@ main(int argc, char **argv)
 	exit(1);
     }
 #endif
-    /* Open VolserLog and map stdout, stderr into it; VInitVolumePackage can
+    /* Open VolserLog and map stdout, stderr into it; VInitVolumePackage2 can
        log, so we need to do this here */
     OpenLog(AFSDIR_SERVER_VOLSERLOG_FILEPATH);
 
-    VInitVolumePackage(volumeUtility, 0, 0, CONNECT_FS, 0);
+    VOptDefaults(volumeServer, &opts);
+    if (VInitVolumePackage2(volumeServer, &opts)) {
+	Log("Shutting down: errors encountered initializing volume package\n");
+	exit(1);
+    }
     /* For nuke() */
     Lock_Init(&localLock);
     DInit(40);
@@ -450,7 +448,6 @@ main(int argc, char **argv)
 	rx_SetUdpBufSize(udpBufSize);	/* set the UDP buffer size for receive */
     if (rxBind) {
 	afs_int32 ccode;
-#ifndef AFS_NT40_ENV
         if (AFSDIR_SERVER_NETRESTRICT_FILEPATH || 
             AFSDIR_SERVER_NETINFO_FILEPATH) {
             char reason[1024];
@@ -459,7 +456,6 @@ main(int argc, char **argv)
                                            AFSDIR_SERVER_NETINFO_FILEPATH,
                                            AFSDIR_SERVER_NETRESTRICT_FILEPATH);
         } else 
-#endif	
 	{
             ccode = rx_getAllAddr(SHostAddrs, ADDRSPERSITE);
         }
@@ -493,7 +489,7 @@ main(int argc, char **argv)
 	assert(pthread_attr_init(&tattr) == 0);
 	assert(pthread_attr_setdetachstate(&tattr, PTHREAD_CREATE_DETACHED) == 0);
 
-	assert(pthread_create(&tid, &tattr, (void *)BKGLoop, NULL) == 0);
+	assert(pthread_create(&tid, &tattr, BKGLoop, NULL) == 0);
 #else
 	PROCESS pid;
 	LWP_CreateProcess(BKGLoop, 16*1024, 3, 0, "vol bkg daemon", &pid);
@@ -510,15 +506,12 @@ main(int argc, char **argv)
 	VS_EXIT(1);
     }
     afsconf_GetKey(tdir, 999, &tkey);
-    securityObjects[0] = rxnull_NewServerSecurityObject();
-    securityObjects[1] = (struct rx_securityClass *)0;	/* don't bother with rxvab */
-    securityObjects[2] =
-	rxkad_NewServerSecurityObject(0, tdir, afsconf_GetKey, NULL);
-    if (securityObjects[0] == (struct rx_securityClass *)0)
+    afsconf_BuildServerSecurityObjects(tdir, 0, &securityClasses, &numClasses);
+    if (securityClasses[0] == NULL)
 	Abort("rxnull_NewServerSecurityObject");
     service =
-	rx_NewServiceHost(host, 0, VOLSERVICE_ID, "VOLSER", securityObjects, 3,
-		      AFSVolExecuteRequest);
+	rx_NewServiceHost(host, 0, VOLSERVICE_ID, "VOLSER", securityClasses,
+			  numClasses, AFSVolExecuteRequest);
     if (service == (struct rx_service *)0)
 	Abort("rx_NewService");
     rx_SetBeforeProc(service, MyBeforeProc);
@@ -527,10 +520,12 @@ main(int argc, char **argv)
     if (lwps < 4)
 	lwps = 4;
     rx_SetMaxProcs(service, lwps);
-#ifdef AFS_SGI_ENV
-    rx_SetStackSize(service, 49152);
+#if defined(AFS_XBSD_ENV)
+    rx_SetStackSize(service, (128 * 1024));
+#elif defined(AFS_SGI_ENV)
+    rx_SetStackSize(service, (48 * 1024));
 #else
-    rx_SetStackSize(service, 32768);
+    rx_SetStackSize(service, (32 * 1024));
 #endif
 
     if (rxkadDisableDotCheck) {
@@ -539,8 +534,8 @@ main(int argc, char **argv)
     }
 
     service =
-	rx_NewService(0, RX_STATS_SERVICE_ID, "rpcstats", securityObjects, 3,
-		      RXSTATS_ExecuteRequest);
+	rx_NewService(0, RX_STATS_SERVICE_ID, "rpcstats", securityClasses,
+		      numClasses, RXSTATS_ExecuteRequest);
     if (service == (struct rx_service *)0)
 	Abort("rx_NewService");
     rx_SetMinProcs(service, 2);
